
/*******************
 * From src: modules/lsfilter/src/js/LSFilterHistory.js
 *******************/
var lsfilter_history = {
	on: {
		"update_ok": function(data) {
			if (!window.history.pushState) {
				/* Do update of regular link in toolbar
				 * (TODO) */
				return;
			}
			if (data.source == 'history') return;
			var order_query = '';
			if (data.order) {
				order_query = '&s=' + encodeURIComponent(data.order);
			}
			window.history.pushState(data, window.title, '?q=' + encodeURIComponent(data.query) + order_query);
		}
	},

	init: function()
	{
		if (!window.history.pushState) {
			return;
		}

		window.onpopstate = function(evt) {
			if (evt.state && evt.state.query) {
				var order = evt.state.order;
				if (!order) order = '';
				lsfilter_main.update(evt.state.query, 'history', order);
			}
		};
	}
};



/*******************
 * From src: modules/lsfilter/src/js/LSFilterInputWindow.js
 *******************/
$().ready(function() {
	$('.main-toolbar-buttons').on('click', '.filter-query-button a', function() {
		var button = $(this);
		if (!button.data('drop-down')) {
			return true;
		}
		var dropdown = $('#'+button.data('drop-down'));
		if (!dropdown.length) {
			console.error("Could not find referenced "+button.data('drop-down'));
			return;
		}
		$('.filter-query-dropdown:visible').not(dropdown).toggle(100,
			function() {$('.filter-query-button').not(button).removeClass('active');}
		);
		dropdown.toggle(100,
			function () {button.toggleClass('active');}
		);
		if(button.attr('href') === '#') {
			return false;
		}
	});
});



/*******************
 * From src: modules/lsfilter/src/js/LSFilterList.js
 *******************/
function lsfilter_list(config)
{
	// Configuration
	this.defaults = {
		per_page: 100,
		autorefresh_delay: 30000,
		autorefresh_enabled: true,
		request_url: _site_domain + _index_page + "/" + _controller_name + "/fetch_ajax",
		columns: false,
		toolbar: false,
		attach_head: false,
		notify: false,
		loading_start: function()
		{
		},
		loading_stop: function()
		{
		}
	};
	this.config = $.extend({}, this.defaults, config);

	if (isNaN(this.config.autorefresh_delay)) {
		// Invalid autorefresh_delay defaults to 30 seconds
		this.config.autorefresh_delay = 30000;
	} else if (this.config.autorefresh_delay === 0) {
		// A delay of 0 means that refresh is disabled.
		this.config.autorefresh_enabled = false;
	}

	if($.browser.msie) {
		var parts = $.browser.version.split('.');
		if( parseInt(parts[0], 10) < 8 ) {
			this.config.attach_head = false; /* Don't support attached head in ie7 */
		}
	}

	if(this.config.table)
		lsfilter_list_attach_events( this, this.config.table );
	if(this.config.totals)
		lsfilter_list_attach_events( this, this.config.totals );

	/* Bind this once and for all globally... Needed in lot of callbacks */
	var self = this;

	if( this.config.attach_head ) {
		$(window).on('resize scroll load', function(e) {
			self.update_float_header();
		});
	}
	/***************************************************************************
	 * External methods
	 **************************************************************************/
	this.on = {
		'update_ok': function(data) {
			var metadata = data.metadata;

			if (data.source && data.source == 'list') { return; }

			if (!metadata) {
				var parser = new LSFilter(new LSFilterPreprocessor(),
						new LSFilterMetadataVisitor());
				metadata = parser.parse(data.query);
			}

			if (self.request_metadata && self.request_metadata !== metadata) {
				// we're switching from one type of view to another,
				// reset unwanted state
				self.config.offset = 0;
				self.previous_obj = {};
			}

			self.request_query = data.query;
			self.request_metadata = metadata;

			self.table_desc = new lsfilter_list_table_desc(self.request_metadata, self.config.columns);

			self.sort_vis_column = false;
			self.sort_db_columns = [];
			self.sort_ascending = true;

			var order_parts = [];
			if (data.order) {
				order_parts = data.order.split(' ');
			}
			if (order_parts.length >= 1 && order_parts.length <= 2) {

				self.sort_vis_column = order_parts[0];

				self.sort_db_columns = self.table_desc.sort_cols(self.sort_vis_column);
			}
			if (order_parts.length == 2) {
				self.sort_ascending = (order_parts[1].toLowerCase() == 'asc');
			}

			self.send_request({});
		}
	};

	this.set_sort = function(vis_column)
	{
		this.config.offset = 0;
		this.previous_obj = {};

		if (this.sort_vis_column == vis_column) {
			this.sort_ascending = !this.sort_ascending;
		}
		else {
			this.sort_db_columns = this.table_desc.sort_cols(vis_column);
			this.sort_vis_column = vis_column;
			this.sort_ascending = true;
		}

		if( typeof lsfilter_main != "undefined" )
			lsfilter_main.update(false, 'list', this.sort_vis_column + (this.sort_ascending ? ' asc' : ' desc'));

		this.send_request({});
	};

	this.send_request = function(config)
	{
		if (typeof config.increment_items_in_view !== "undefined" && Boolean(config.increment_items_in_view)) {
			delete config.increment_items_in_view;
			self.config.offset += self.config.per_page;
		}

		if (this.active_ajax_request) {
			this.active_ajax_request.abort();
		}

		var db_sort_columns = [];
		for ( var i = 0; i < this.sort_db_columns.length; i++) {
			var col = this.sort_db_columns[i];
			var parts = col.split(' ');
			var col_asc = true;
			if (parts.length == 2) {
				col_asc = parts[1].toLowerCase() == 'asc';
			}
			if (!this.sort_ascending) col_asc = !col_asc;
			db_sort_columns.push(parts[0] + (col_asc ? ' asc' : ' desc'));
		}

		var loading_id = self.config.loading_start();
		var options = $.extend(
			{
				success: false
			},
			this.config,
			config
		);
		this.active_ajax_request = $
				.ajax({
					url: this.config.request_url,
					dataType: 'json',
					type: "POST",
					data: {
						"query": this.request_query,
						"sort": db_sort_columns,
						"columns": this.table_desc.db_columns,
						"limit": options.per_page,
						"offset": options.offset,
						"csrf_token": _csrf_token
					},
					success: function(data)
					{
						if(data == null) {
							self.handle_ajax_error_response({data:_("No valid response from server. Still logged in?")});
						} else if(options.success) {
							options.success(data);
						} else {
							self.handle_ajax_response(data);
						}
					},
					error: function(data, text_status)
					{
						if(!data.getAllResponseHeaders() || text_status === "abort") {
							// getAllResponseHeaders() provided by http://ilikestuffblog.com/2009/11/30/how-to-distinguish-a-user-aborted-ajax-call-from-an-error/
							// since my tests of === "abort" all are negative, keeping it if jQuery decides to follow its API sometime..


							// only continue to display error message if
							// it's an actual error, not just multiple requests
							// stacked or something silly like that
							return;
						}
						var message;
						try {
							message = JSON.parse(data.responseText);
						} catch(not_json) {
							message = _('Error reloading');
						}
						self.handle_ajax_error_response(message);
					},
					complete: function(data)
					{
						self.active_ajax_request = false;
						self.config.loading_stop(loading_id);
					}
				});
	};

	/***************************************************************************
	 * Internal veriables
	 **************************************************************************/
	this.request_query = '';
	this.request_metadata = {};
	this.resuest_timer = false;
	this.visible_count = 0;

	this.current_columns = [];

	this.active_ajax_request = false;

	this.sort_vis_column = null;
	this.sort_db_columns = [];
	this.sort_ascending = true;
	this.sort_columns_table = {};

	this.autorefresh_timer = false;

	/***************************************************************************
	 * Internal methods
	 **************************************************************************/


	this.update_float_header = function()
	{

		var thead =  $(this.config.table).find('thead');
		var header = $(thead).filter(function(){return !$(this).hasClass('floating-header');});
		var clone = $(thead).filter(function(){return $(this).hasClass('floating-header');});

		if( !header || !clone ) return;

		var header_div = $("body .container #header"),
			head = header.find("tr").children(),
			cloneHead = clone.find("tr").children(),
			index = 0;

		clone.css( 'min-width', header.outerWidth() );
		clone.css( 'top', header_div.outerHeight() + "px" );

		head.each(function() {

			var clonehead = $(cloneHead[index]);
			var thishead = $(this);

			var w = parseInt( thishead.width(), 10 ) + 1;

			index++;

			clonehead.css( 'width', w + 'px');

			clonehead.css('padding-right', thishead.css('padding-right'));
			clonehead.css('padding-top', thishead.css('padding-top'));
			clonehead.css('padding-bottom', thishead.css('padding-bottom'));
			clonehead.css('margin', thishead.css('margin'));
			clonehead.css('border', thishead.css('border'));
		});

		// Get scrollbar height and width
		var scroll = this.get_browser_scroll_size();
		$('#align_th').remove();
		if ($('.content').get(0).scrollHeight > $('.content').height()) {
			/*
			 *	Append a th to the floating header
			 *	Give it a width that matches the scrollbars width
			 *	It's not a very nice solution but at least we
			 *	don't have to rewrite the floating header at this time
			*/
			clone.find("tr")
				.append(
					$("<th>")
						.attr('id', 'align_th')
						.css('width', (scroll.width - 4) + 'px')
						.css('padding-right', 0)
				);
		}

	};

	this.update_nagbar = function(messages)
	{
		var i;
		if(this.config.notify) {
			this.config.notify.clear('nagbar');
			for(i=0;i<messages.length;i++) {
				if (typeof(messages[i]) === 'object') {
					this.config.notify.message(messages[i].message, {
						type: messages[i].type,
						nag: true
					});
				} else {
					this.config.notify.message(messages[i], {
						type: 'error',
						nag: true
					});
				}
			}
			$(window).trigger('resize');
		}
	}

	this.get_browser_scroll_size = function()
	{
		var css = {
			"border":  "none",
			"height":  "200px",
			"margin":  "0",
			"padding": "0",
			"width":   "200px"
		};

		var inner = $("<div>").css($.extend({}, css));
		var outer = $("<div>").css($.extend({
			"left":       "-2000px",
			"overflow":   "scroll",
			"position":   "absolute",
			"top":        "-2000px"
		}, css)).append(inner).appendTo("body")
		.scrollLeft(2000)
		.scrollTop(2000);

		var scrollSize = {
			"height": (outer.offset().top - inner.offset().top) || 0,
			"width": (outer.offset().left - inner.offset().left) || 0
		};

		outer.remove();
		return scrollSize;
	};

	this.handle_autorefresh = function()
	{
		this.send_request({
			offset: 0,
			per_page: this.config.offset + this.config.per_page
		});
	};

	this.start_autorefresh_timer = function() {
		if( this.config.autorefresh_enabled === false || isNaN(this.config.autorefresh_delay) || this.config.autorefresh_delay < 1 )
			return;

		if(this.autorefresh_timer !== false ) {
			clearTimeout( this.autorefresh_timer );
		}
		this.autorefresh_timer = setTimeout(function() {
			self.handle_autorefresh();
		}, this.config.autorefresh_delay);
	};

	this.handle_ajax_response = function(data)
	{
		var new_table;
		var new_totals = $('<span />');

		if(data.messages) {
			self.update_nagbar(data.messages);
		}

		if(data.data.length) {
			new_table = this.render_table(data, this.sort_vis_column, this.sort_ascending);
		} else {
			var empty_text;
			if(!(empty_text = $(this.config.table).parents('.widget').data('text-if-empty'))) {
				empty_text = _("No entries found using filter");
			}
			new_table = $('<div class="alert notice"></div>').text(empty_text);
		}
		new_totals = this.render_totals(data.table, data.totals);

		if(this.config.toolbar) {
			this.config.toolbar.find('.toolbar-buttons').replaceContent(
				$('<ul/>').append(
					$.map([$('<li id="filter_loading_status"/>')].concat(
							listview_renderer_buttons[data.table] || []
					).concat(
								listview_renderer_buttons.all || []
					),
						function(x) {
							if (typeof x == 'function') {
								x = x();
							}
							return $('<li class="filter-query-button"/>').html(x).toArray();
			})));
		}

		if (this.config.totals) {
			this.config.totals.replaceContent(new_totals);
		}
		if (this.config.table) {
			this.config.table.replaceContent(new_table);
			this.attach_header();
		}

		this.start_autorefresh_timer();
	};

	this.handle_ajax_error_response = function(data)
	{

		var alert = $('<div class="alert error" />');
		if (typeof(data.data) === 'undefined') {
			alert.html("Service Unavailable, you may attempt to refresh your browser");
		} else {
			alert.html("<strong>Error:</strong> " + data.data);
		}

		if (this.config.table) {
			this.config.table.replaceContent(alert);
		}
		this.start_autorefresh_timer();
	};

	this.render_totals = function(table, totals)
	{

		var subtitle = null;
		if ( this.config.toolbar ) {
			subtitle = this.config.toolbar.find( '.main-toolbar-subtitle' );
		}

		var container = $('<ul />');

		if ( subtitle ) {
			subtitle.html("").append( link_query('['+table+'] all')
				.text(table.charAt(0).toUpperCase() + table.slice(1))
				.css( "border", "none" )
			);
		}

		if (totals) {
			for ( var field in listview_renderer_totals) {
				if (field in totals) {

					var item = listview_renderer_totals[field](totals[field][1])
						.wrapInner(
							link_query(totals[field][0])
						);

					/* If the label is the same as the toolbar sub-header, remove it */
					if ( item.hasClass( "extra_toolbar_category" ) ) {
						var label = item.find( 'a[data-query="[' + table + '] all"]' );
						if ( label.length > 0 ) {
							var replacer = table.charAt(0).toUpperCase() + table.slice(1) + ":";
							label.html( label.html().replace( replacer, "" ) );
						}
					}

					item.find('a').attr('title', item.find('span').attr('title'));
					container.append(item);

				}
			}
		}
		return container;
	};

	this.insert_rows = function(data, tbody)
	{
		/*
		 * // #load_more is a tr>td>a node var last_row =
		 * $('#load_more').parent().parent();
		 */
		for ( var i = 0; i < data.data.length; i++) {
			var obj = data.data[i];

			var row = $('<tr />');
			row.addClass(i % 2 ? 'odd' : 'even');
			row.data('key', obj.key);

			for ( var cur_col = 0; cur_col < this.current_columns.length; cur_col++) {
				row.append(this.current_columns[cur_col]({
					obj: obj,
					last_obj: this.previous_obj,
					row: row,
					listview: this
				}));
				// .addClass( 'listview-cell-' + cur_col));
			}
			row.addClass('listview_row');

			tbody.append(row);
			this.previous_obj = obj;
		}
	};

	this.add_fill_bar = function(data, tbody)
	{
		if( this.current_columns.length <= 0 ) {
			/* Don't bother then...*/
			return;
		}

		var more_rows = data.count - tbody.children().length;
		if (more_rows > this.config.per_page) {
			more_rows = this.config.per_page;
		}
		if (more_rows > 0) {
			var loadcell = $('<td/>');
			var loadrow = $('<tr class="table_pagination" />');
			loadrow.append(loadcell);
			tbody.append(loadrow);

			loadcell.attr('colspan', this.current_columns.length);
			loadcell.append($('<a id="load_more" href="#">' + _('Load ' + more_rows + ' more rows') + '</a>'));
			loadcell.addClass('link_load_more_rows');
		}
	};

	this.load_more_rows = function(loadingcell)
	{
		loadingcell.removeClass('link_load_more_rows');
		loadingcell.text('Loading...');

		var loadrow = loadingcell.parent('tr');
		var tbody = loadrow.parent('tbody');

		this.send_request({
			success: function(data)
			{
				loadrow.remove();
				self.insert_rows(data, tbody);
				self.add_fill_bar(data, tbody);
			},
			increment_items_in_view: true
		});
	};

	this.render_table = function(data, sort_col, sort_asc)
	{
		listview_table_col_index = 0;
		listview_last_host = '';

		if (data.length === 0) {
			return $('<h2 class="lsfilter-noresult">' + _('Empty result set') + '</h2>');
		}

		/*
		 * Render table
		 */

		var table = $('<table cellspacing="0" cellpadding="0" border="0" />');
		var thead = $('<thead />');
		var tbody = $('<tbody />');
		table.append(thead);
		table.append(tbody);

		/*
		 * Render table header
		 */

		this.current_columns = [];
		var header = $('<tr />');

		for ( var key=0; key<this.table_desc.vis_columns.length; key++ ) {
			var col_name = this.table_desc.vis_columns[key];
			var col_render = this.table_desc.col_renderers[col_name];

			/*
			 * Check if column is available in current view.
			 */
			if (col_render.available) {
				if (!col_render.available({table:this.table_desc.metadata.table})) {
					continue;
				}
			}

			this.current_columns.push(col_render.cell);

			var th = $('<th />');

			if (col_render.sort) {
				var sort_dir = 0;
				if (sort_col == col_name) sort_dir = -1;
				if (sort_asc) sort_dir = -sort_dir;
				this.add_sort(th, col_name, sort_dir);
			}

			if (typeof(col_render.header) === 'function') {
				th.append(col_render.header());
			} else {
				th.append(col_render.header);
			}
			header.append(th);
		}
		thead.append(header);

		this.previous_obj = {};
		this.insert_rows(data, tbody);
		this.add_fill_bar(data, tbody);

		return table;
	};

	this.add_sort = function(element, vis_column, current)
	{
		var img = _site_domain+'application/views/icons/arrow-up-down.png';
		element.attr('title', 'Sort ascending');
		if (current > 0) {
			element.attr('title', 'Sort descending');
			img = _site_domain+'application/views/icons/arrow-up.png';
			element.addClass('current');
		}
		else if(current < 0) {
			img = _site_domain+'application/views/icons/arrow-down.png';
			element.addClass('current');
		}

		element.addClass('sortable');
		element.addClass('link_set_sort');

		element.attr('data-column', vis_column );

		element.append($('<span class="lsfilter-sort-span" />').css('background-image', 'url('+img+')'));
	};

	this.attach_header = function()
	{
		if (!this.config.attach_head) return;

		var thead =  $(this.config.table).find('thead');
		var header = $(thead).filter(function(){return !$(this).hasClass('floating-header');});
		var clone = header.clone(true);
		header.after(clone);

		clone.addClass('floating-header');
		header.find('input').remove();

		this.update_float_header();
	};
}



/*******************
 * From src: modules/lsfilter/src/js/LSFilterListEvents.js
 *******************/
function lsfilter_list_attach_events( listview, base_element ) {

	/* link_query, from lists */
	$(base_element)

	.on('click', '.query_link', function(e) {
		var query = $(this).attr('data-query');
		if( lsfilter_main ) {
			if (_controller_name === 'listview') {
				e.preventDefault();
				lsfilter_main.update( query, false, '' );
				return false;
			}
		}
	})

	.on('click', 'a.link_ajax_refresh', function(e) {
		e.preventDefault();
		var url = $(this).attr('href');
		$.ajax(url).done(function() {
			if (lsfilter_main)
				lsfilter_main.refresh();
			if (lsfilter_saved)
				lsfilter_saved.refresh_filter_list();
		});
		return false;
	})

	.on('click', '.link_set_sort', function(e) {
		e.preventDefault();
		var column = $(this).attr('data-column');
		listview.set_sort(column);
		return false;
	})

	.on('click', '.link_load_more_rows', function(e) {
		e.preventDefault();
		listview.load_more_rows($(this));
		return false;
	})

	.on('change', '.listview_multiselect_checkbox', function(e) {
		var tgt = $(this);
		if( lsfilter_multiselect ) {
			lsfilter_multiselect.box_register(tgt.attr('value'), tgt.prop('checked'));
		}
		var tr = tgt.closest('tr');
		var classname = "";
		if (tr.hasClass('odd'))
			classname = 'selected_odd';
		else
			classname = 'selected_even';
		if (tgt.prop('checked')) {
			tr.addClass(classname);
		}
		else {
			tr.removeClass(classname);
			$('#select_all').prop('checked', false);
		}
	})

	.on('change', '.listview_multiselect_checkbox_all', function(e) {
		listview.config.table.find('.listview_multiselect_checkbox').prop('checked', $(this).prop('checked')).trigger('change');
	});
}



/*******************
 * From src: modules/lsfilter/src/js/LSFilterListTableDesc.js
 *******************/
// Add isInteger browser compability.
Number.isInteger = Number.isInteger || function(value) {
	return typeof value === "number" && isFinite(value) && Math.floor(value) === value;
};

var LSColumnsPP = function() {
	/* add preprocessor as parent */
	this.parent = LSColumnsPreprocessor;
	this.parent();

	this.preprocess_string = function(value) {
		return value.substring(1, value.length - 1).replace(/\\n/g, "\n")
				.replace(/\\(.)/g, '$1');
	};

	this.preprocess_float = function(value) {
		return parseFloat(value);
	};

	this.preprocess_integer = function(value) {
		return parseInt(value, 10);
	};

};

var LSColumnsFilterListVisitor = function(all_columns, all_db_columns, metadata) {
	var column_exists = function(col_name) {
		/* Could have used indexOf if IE7 wasn't crappy... */
		var i;
		for (i = 0; i < all_columns.length; i++) {
			if (all_columns[i] == col_name) {
				return true;
			}
		}
		return false;
	};
	var db_column_exists = function(col_name) {
		if (all_db_columns[col_name]) {
			return true;
		}
		return false;
	};

	this.custom_cols = {};
	this.custom_deps = [];
	this.error_id = 1;

	// entry: definition := * column_list end
	this.visit_entry = function(column_list0, end1) {
	};

	// column_list_single: column_list := * column
	this.visit_column_list_single = function(column0) {
		if (column0.op == 'add') {
			return column0.cols;
		}
		return [];
	};

	// column_list_multi: column_list := * column_list comma column
	this.visit_column_list_multi = function(column_list0, column2) {
		var result = [];
		if (column2.op == 'add') {
			var tmpresult = column_list0.concat(column2.cols);
			/*
			 * Only save unique columns. If this wasn't IE7-compatible, this
			 * would have been a simple filter...
			 */
			for (var i = 0; i < tmpresult.length; i++) {
				var to_add = true;
				for (var j = 0; to_add && j < result.length; j++) {
					if (result[j] == tmpresult[i]) {
						to_add = false;
					}
				}
				if (to_add) {
					result.push(tmpresult[i]);
				}
			}
		} else if (column2.op == 'sub') {
			/*
			 * If we didn't have to care about IE7, this would be a simple
			 * Array.filter: result =
			 * column_list0.filter(function(el,i,a){return
			 * column2.cols.indexOf(el) < 0;});
			 */
			for (var i = 0; i < column_list0.length; i++) {
				var to_add = true;
				for (var j = 0; to_add && j < column2.cols.length; j++) {
					if (column2.cols[j] == column_list0[i]) {
						to_add = false;
					}
				}
				if (to_add) {
					result.push(column_list0[i]);
				}
			}
		} else {
			// Do nothing, pass previous list through...
			result = column_list0;
		}
		return result;
	};

	// column_all: column := * all
	this.visit_column_all = function() {
		return {
			op : 'add',
			cols : all_columns
		};
	};

	// column_default: column := * default
	this.visit_column_default = function() {
		return {
			op : 'add',
			cols : this.default_columns
		};
	};

	// column_named: column := * name
	this.visit_column_named = function(name0) {
		if (!column_exists(name0)) {
			return {
				op : 'nop'
			};
		}
		return {
			op : 'add',
			cols : [ name0 ]
		};
	};

	// column_disable: column := * minus name
	this.visit_column_disable = function(name1) {
		return {
			op : 'sub',
			cols : [ name1 ]
		};
	};

	// column_disable_str: column := * minus string
	this.visit_column_disable_str = function(name1) {
		return {
			op : 'sub',
			cols : [ name1 ]
		};
	};

	// column_custom: column := * custom_name eq expr
	this.visit_column_custom = function(custom_name0, expr2) {
		this.custom_cols[custom_name0] = {
			evaluator : function(args) {
				var value = expr2.evaluator(args);

				/* If an array, join the values */
				if (typeof value == "object") {
					value = value.join(", ");
				}

				return value;
			},
			sort : expr2.sort
		};
		return {
			op : 'add',
			cols : [ custom_name0 ]
		};
	};

	// expr_add: expr := * expr op_add expr2
	this.visit_expr_eq = function(expr0, expr2) {
		return {
			evaluator : function(args) {
				return expr0.evaluator(args) == expr2.evaluator(args);
			},
			sort : false
		};
	};

	// expr_add: expr := * expr op_add expr2
	this.visit_expr_add = function(expr0, expr2) {
		return {
			evaluator : function(args) {
				return expr0.evaluator(args) + expr2.evaluator(args);
			},
			sort : false
		};
	};

	// expr_sub: expr := * expr op_sub expr2
	this.visit_expr_sub = function(expr0, expr2) {
		return {
			evaluator : function(args) {
				return expr0.evaluator(args) - expr2.evaluator(args);
			},
			sort : false
		};
	};

	// expr_mult: expr2 := * expr2 op_mult expr3
	this.visit_expr_mult = function(expr0, expr2) {
		return {
			evaluator : function(args) {
				return expr0.evaluator(args) * expr2.evaluator(args);
			},
			sort : false
		};
	};

	// expr_div: expr2 := * expr2 op_div expr3
	this.visit_expr_div = function(expr0, expr2) {
		return {
			evaluator : function(args) {
				return expr0.evaluator(args) / expr2.evaluator(args);
			},
			sort : false
		};
	};

	// expr_neg: expr3 := * op_sub expr4
	this.visit_expr_neg = function(expr1) {
		return {
			evaluator : function(args) {
				return -expr1.evaluator(args);
			},
			sort : false
		};
	};

	// expr_var: expr3 := * var
	this.visit_expr_var = function(var0) {

		var fetchvar = var0.slice(0); /* Make a clone */
		if (fetchvar[0] == 'prev') {
			fetchvar.shift();
		}

		/*
		 * if var is an object, fetch next also This will filter out columns
		 * that might be available, so we can request them. But it does also
		 * filter out some more, which is good, because some columns might be
		 * specified in the ORM layer instead. So just dig deeper if we know
		 * it's an object
		 */
		var fetchlen = 0;

		var curtbl = metadata.table;
		while (fetchvar[fetchlen] && ninja_manifest.orm_structure[curtbl][fetchvar[fetchlen]]
				&& ninja_manifest.orm_structure[curtbl][fetchvar[fetchlen]][0] == 'object') {
			curtbl = ninja_manifest.orm_structure[curtbl][fetchvar[fetchlen]][1];
			fetchlen++;
		}
		fetchlen++;
		fetchvar = fetchvar.slice(0, fetchlen);

		this.custom_deps.push(fetchvar.join('.'));

		/*
		 * It's only possible to sort on this field if it is:
		 *
		 * a leaf object, which means, fetchvar.length == var0.length (custom
		 * variables)
		 *
		 * it is sortable, which meeans a list of types (string,int and time for
		 * now)
		 */
		var is_sortable = true;

		if (fetchvar.length != var0.length)
			is_sortable = false;

		/* Might be a non-existing column, or non-backened-column */
		if( ninja_manifest.orm_structure[curtbl][fetchvar[fetchlen - 1]] ) {
			var type = ninja_manifest.orm_structure[curtbl][fetchvar[fetchlen - 1]][0];
			if (type != 'int' && type != 'string' && type != 'time' && type != 'float')
				is_sortable = false;
		} else {
			is_sortable = false;
		}

		var sortcols = false;
		if (is_sortable)
			sortcols = [ fetchvar.join('.') ];

		if (fetchvar.length != fetchvar)

			return {
				evaluator : function(args) {
					/*
					 * By some reason, local variables is defined under args,
					 * but the object should be accessable within args.obj
					 * without prefix. Merge those namespaces
					 */
					var vars = $.extend({}, args.obj, {
						prev : args.last_obj
					}, args.local);

					for (var i = 0; i < var0.length; i++) {
						if (typeof vars != "undefined"
								&& typeof var0[i] != "undefined"
								&& typeof vars[var0[i]] != "undefined") {
							vars = vars[var0[i]];
						} else {
							return "undefined";
						}
					}

					return vars;
				},
				sort : sortcols
			};
	};

	// var_var: var := * name
	this.visit_var_var = function(name0) {
		return [ name0 ];
	};

	// var_index: var := * var sq_l integer sq_r
	this.visit_var_index = function(var0, integer2) {
		return var0.concat([ integer2 ]);
	};

	// var_attr: var := * var dot name
	this.visit_var_attr = function(var0, name2) {
		return var0.concat([ name2 ]);
	};

	// expr_string: expr2 := * string
	this.visit_expr_string = function(string0) {
		return {
			evaluator : function(args) {
				return string0;
			},
			sort : false
		};
	};

	// expr_int: expr4 := * integer
	this.visit_expr_int = function(integer0) {
		return {
			evaluator : function(args) {
				return integer0;
			},
			sort : false
		};
	};

	// expr_float: expr4 := * float
	this.visit_expr_float = function(float0) {
		return {
			evaluator : function(args) {
				return float0;
			},
			sort : false
		};
	};

	// expr_list_comp: expr4 := * sq_l expr for name in expr sq_r
	this.visit_expr_list_comp = function(expr1, name3, expr5) {
		return {
			evaluator : function(args) {
				var list = expr5.evaluator(args);
				var result = [];
				var subargs = $.extend(true, {
					local : {}
				}, args);
				for (var i = 0; i < list.length; i++) {
					subargs.local[name3] = list[i];
					result.push(expr1.evaluator(subargs));
				}
				return result;
			},
			sort : false
		};
	};

	// expr_list_comp: expr4 := * sq_l expr for name in expr if expr sq_r
	this.visit_expr_list_comp_if = function(expr1, name3, expr5, expr7) {
		return {
			evaluator : function(args) {
				var list = expr5.evaluator(args);
				var result = [];
				var subargs = $.extend(true, {
					local : {}
				}, args);
				for (var i = 0; i < list.length; i++) {
					subargs.local[name3] = list[i];
					if (expr7(subargs))
						result.push(expr1.evaluator(subargs));
				}
				return result;
			},
			sort : false
		};
	};

	// expr_func: expr4 := * name par_l expr_list par_r
	this.visit_expr_func = function(name0, expr_list2) {
		switch (name0) {
		case "implode": // implode( delimiter, array )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					/* FIXME: test variable types */
					return fargs[1].join(fargs[0]);
				},
				sort : false
			};
		case "time": // time( unixtimestamp )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					/* FIXME: test variable types */
					return format_timestamp(fargs[0]);
				},
				sort : false
			};
		case "idx": // idx( argnr, arg0, arg1, arg2 ... )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					var idx = parseInt(fargs[0], 10);
					if (!(0 <= idx && idx < fargs.length - 1)) {
						return "Unknown index " + idx;
					}
					return fargs[idx + 1];
				},
				sort : false
			};
		case "urlencode": // urlencode( string )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					return encodeURIComponent(fargs[0]);
				},
				sort : false
			};
		case "htmlescape": // htmlenscape( string )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					var el = $('<div />').text(fargs[0]);
					var text = el.html();
					el.remove(); // Make sure it's memory is freed
					return text;
				},
				sort : false
			};
		case "link": // link( relative_url, content )
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					var el = $('<a />');
					el
							.attr('href', _site_domain + _index_page + "/"
									+ fargs[0]);
					el.html(fargs[1]);
					/* Exprt as html, use a container, and get its html content */
					var cont = $('<div />').append(el);
					return cont.html();
				},
				sort : false
			};
		case "nl2br": // nl2br(string)
			return {
				evaluator : function(args) {
					var fargs = expr_list2.evaluator(args);
					var text = fargs[0].replace(/\n/g, "<br />");
					return text;
				},
				sort : false
			};
		}
		return {
			evaluator : function(args) {
				return "Unknown function " + name0;
			},
			sort : false
		};
	};

	// expr_list: expr_list := * expr comma expr_list
	this.visit_expr_list = function(expr0, expr_list2) {
		return {
			evaluator : function(args) {
				var arr = expr_list2.evaluator(args);
				arr.unshift(expr0.evaluator(args));
				return arr;
			},
			sort : false
		};
	};

	// expr_if: expr4 := * if expr then expr4 else expr4
	this.visit_expr_if = function(expr1, expr3, expr5) {
		return {
			evaluator : function(args) {
				if (expr1.evaluator(args))
					return expr3.evaluator(args);
				return expr5.evaluator(args);
			},
			sort : false
		};
	};

	// expr_list_end: expr_list := * expr
	this.visit_expr_list_end = function(expr0) {
		return {
			evaluator : function(args) {
				return [ expr0.evaluator(args) ];
			},
			sort : false
		};
	};

	this.accept = function(result) {
		return result;
	};

	/***************************************************************************
	 * Error recovery routines
	 **************************************************************************/

	var errormessage = function(stack, tokens, lexer) {
		var stacktoks = [];
		for (var i = 0; i < stack.length; i++) {
			if (stack[i][1][0] != "column_list") {
				// Only show current column
				stacktoks.push(stack[i][1]);
			}
		}

		var before = lexer.tokens_to_string(stacktoks);
		var follow = lexer.tokens_to_string(tokens);

		/* Add error notification column */
		return 'syntax error: ' + before + ' <span style="color: red;">'
				+ follow + '</span>';
	};

	/*
	 * Recover from column_list error: totally invalid column definition Tries
	 * to dig out the previous column list from the stack.
	 */
	this.error_column_list_error = function(stack, tokens, lexer) {
		var outp_list = [];
		console.log("ERROR");
		console.log(tokens);

		/* Extract column list from stack, if available */
		if (stack.length >= 1 && stack[0][1][0] == 'column_list') {
			outp_list = stack[0][1][1];
		}

		var column_name = "parse error " + this.error_id;
		outp_list.push(column_name);
		this.error_id++;

		var msg = errormessage(stack, tokens, lexer);
		this.custom_cols[column_name] = {
			evaluator : function(args) {
				return msg;
			},
			sort : false
		};

		return outp_list;
	};

	/*
	 * Recover from custom column error; the name is known, but not the
	 * definition, recover in definition
	 */
	this.error_custom_content_error = function(stack, tokens, lexer) {
		var msg = errormessage(stack, tokens, lexer);
		return {
			evaluator : function(args) {
				return msg;
			},
			sort : false
		};
	};

};

function lsfilter_list_table_desc(metadata, columndesc) {

	var self = this;

	this.metadata = metadata;
	this.vis_columns = [];
	this.col_renderers = {};
	this.db_columns = [];
	this.commands = {};

	if (!listview_renderer_table[metadata.table])
		return;

	var all_col_renderers = $.extend(
		{},
		listview_renderer_table_all,
		listview_renderer_table[metadata.table]
	);

	// Create an array to be used for sorting column names.
	var unordered_index = 0;
	var col_render_order = $.map(all_col_renderers, function(val, col) {
		if (!Number.isInteger(val.order)) {
			// For columns without "order" (or "order" set to valid number) we
			// give it an order index that will make it end up after any columns
			// with "order" set, yet keeping it's current relative order.
			return {"column" : col, "order" : 1000 + unordered_index++};
		}
		return {"column" : col, "order" : val.order};
	});

	// Sort columns by "order" property.
	col_render_order.sort(function(a, b) { return a.order - b.order; });

	// Create an array with column names in the right order.
	var all_columns = $.map(col_render_order, function(val) {
		return val.column;
	});

	var all_db_columns = ninja_manifest.orm_structure[metadata.table];

	var custom_columns = {};

	var all_command_info = {};
	if(listview_commands[metadata.table]) {
		all_command_info = listview_commands[metadata.table];
	}

	if (!columndesc) {
		// If not having a column desc, does a user-config exist?
		if (lsfilter_list_columns && lsfilter_list_columns[metadata.table]) {
			columndesc = lsfilter_list_columns[metadata.table];
		}
	} else {
		columndesc = [ columndesc ];
	}

	if (columndesc) {
		// TODO: handling of column slection description

		var columns_line_visitor = new LSColumnsFilterListVisitor(all_columns,
				all_db_columns, metadata);
		var parser = new LSColumns(new LSColumnsPP(), columns_line_visitor);
		try {
			columns_line_visitor.default_columns = all_columns;

			var cur_columns = [];
			for (var i = 0; i < columndesc.length; i++) {
				cur_columns = parser.parse(columndesc[i]);
				columns_line_visitor.default_columns = cur_columns;
			}
			this.vis_columns = cur_columns;

			custom_columns = columns_line_visitor.custom_cols;
			this.db_columns = this.db_columns
					.concat(columns_line_visitor.custom_deps);
		} catch (e) {
			this.vis_columns = all_columns;
			this.vis_columns.push('message');
			custom_columns['message'] = {
				evaluator : function(args) {
					return e.message;
				},
				sort : false
			};

			console.log(parser);
			console.log(columndesc);
			console.log(e);
			console.log(e.stack);
		}
	} else {
		this.vis_columns = all_columns;
	}

	/* Add custom column renderers */
	for ( var name in custom_columns) {
		var content = custom_columns[name];
		/* Some ugly way to bind variables... there must be a better way? */
		this.col_renderers[name] = (function(in_content) {
			return {
				"header" : name,
				"depends" : [],
				"sort" : in_content.sort,
				"cell" : function(args) {
					return $('<td />').append(in_content.evaluator(args));
				}
			};
		})(content);
	}

	for (var i = 0; i < this.vis_columns.length; i++) {
		/* Fetch column renderers */
		var column_obj = this.col_renderers[this.vis_columns[i]];
		if (!column_obj) {
			column_obj = all_col_renderers[this.vis_columns[i]];
			this.col_renderers[this.vis_columns[i]] = column_obj;
		}

		/* Fetch database column dependencies */
		for (var j = 0; j < column_obj.depends.length; j++) {
			this.db_columns.push(column_obj.depends[j]);
		}
	}

	$.each(all_command_info, function(cmd, cmdinfo) {
		var field = cmdinfo.enabled_if;
		var negate = false;

		if(field[0] == '!') {
			negate = true;
			field = field.substring(1);
		}

		(function(){
			if(!all_command_info[cmd].enabled_if) {
				self.commands[cmd] = function(obj) {
					return true;
				};
			} else {
				var myfield = field;
				if(negate) {
					self.commands[cmd] = function(obj) {
						return !obj[myfield];
					}
				} else {
					self.commands[cmd] = function(obj) {
						return obj[myfield];
					}
				}
			}
		})();

		self.db_columns.push(field);
	});

	/* Build fetch sort columns method */
	this.sort_cols = function(vis_col) {
		var sort = this.col_renderers[vis_col].sort;
		if (sort)
			return sort;
		return [];
	};
}



/*******************
 * From src: modules/lsfilter/src/js/LSFilterMain.js
 *******************/
var lsfilter_storage = {};

var lsfilter_main = {

	update_delay : 300,

	listeners: [],

	// Modules that listens for events, for example an instance of
	// "lsfilter_textarea"
	add_listener: function(listener) {
		this.listeners.push(listener);
	},

	/***************************************************************************
	 * Trigger update of ListView
	 **************************************************************************/
	// Proxy for update()
	update_delayed : function(query, source, order) {
		var self = this; // To be able to access it from within handlers
		if (this.update_timer) {
			clearTimeout(this.update_timer);
		}
		this.update_timer = setTimeout(function() {
			self.update(query, source, order);
		}, this.update_delay);
	},

	update : function(query, source, order) {
		if(this.update_query &&
			$.trim(query) === $.trim(this.update_query) &&
			$.trim(order) === $.trim(this.update_order)
			) {
			// don't issue multiple requests for the same query,
			// use refresh() if you want a brand new result set
			// of the current query
			return;
		}

		this.update_query = query;
		this.update_source = source;
		this.update_order = order;
		this.update_run();
	},

	update_timer : false,
	update_query : false,
	update_source : false,
	update_order : false,

	state : {
		query : false,
		order : false
	},

	refresh: function() {
		this.update_run();
	},

	emit: function(signal) {
		var args = Array.prototype.slice.call(arguments);
		// remove the named 'signal' argument
		args.splice(0, 1);
		for(var i = 0; i < this.listeners.length; i++) {
			var listener = this.listeners[i];
			if(typeof listener['on'] === "undefined") {
				continue;
			}
			if(typeof listener['on'][signal] === "function") {
				// Using "apply" makes sure that callbacks
				// can refer to their own instance as "this"
				listener['on'][signal].apply(listener, args);
			}
		}
	},

	// Private method, run either update(), update_delayed() or refresh()
	update_run : function() {
		var source = this.update_source;
		var query = this.update_query;
		var order = this.update_order;

		var header_height = $( "body > .container >#header" ).outerHeight() + 4;

		this.update_timer = false;

		if (query) {
			this.state.query = query;
		}
		if (typeof order == 'string') {
			this.state.order = order;
		}

		var parser = new LSFilter(new LSFilterPreprocessor(),
				new LSFilterMetadataVisitor());
		try {
			var metadata = parser.parse(this.state.query);
			//Validate custom variables filter query format
			if(metadata['columns'].indexOf('custom_variables') > -1) {
				var custom_variables_values = this.state.query.split('custom_variables')[1].split('"')[1];
				var valid = new RegExp("[\\w]+\\s[\\w.]+$");
				if(!valid.test(custom_variables_values)) {
					this.set_parse_status("Error: Invalid query, custom variables format will be 'name value', Ex: 'NOMONITORING value'");
					$('#filter-query-builder').show();
					this.emit('update_failed');
					return;
				}
			}
		} catch (ex) {
			console.log(ex);
			this.set_parse_status(ex);
			$('#filter-query-builder').show();
			this.emit('update_failed');
			return;
		}

		var data = $.extend({
			source : source,
			metadata : metadata
		}, this.state);

		this.clear_parse_status();

		$('#filter-query-builder').css( "top", header_height + "px" );

		$('#extra-dropdowns').replaceContent(
			$.map((listview_renderer_extra_objects[metadata.table] || []).concat(listview_renderer_extra_objects_all || []), function(x) {
				if (typeof x == 'function') {
					x = x();
				}
				x.addClass('filter-query-dropdown');
				x.css( "top", header_height + "px" );
				return x.toArray();
			}));
		this.emit('update_ok', data);
	},
	/***************************************************************************
	 * Initialize ListView
	 **************************************************************************/
	init : function() {
		// Add all components that make up the listview. All of them
		// receive events when the global state is updated (e.g. the
		// textarea is notified whenever a change occurs in the
		// graphical filter builder).
		this.update_page_links();

		lsfilter_history.init();
		this.add_listener(lsfilter_history);

		lsfilter_storage.list = new lsfilter_list({
			autorefresh_delay : _lv_refresh_delay * 1000,
			table : $('#filter_result'),
			toolbar: $( '.main-toolbar' ),
			totals : $('#filter_result_totals'),
			attach_head : true,
			notify : Notify,
			loading_start : function() {
				var loader = $('<span class="lsfilter-loader" />').append(
						$('<span>' + _('Loading...') + '</span>'));
				$('#filter_loading_status').append(loader);
				return loader;
			},
			loading_stop : function(loader) {
				loader.remove();
			},
			per_page: lsfilter_per_page
		});
		this.add_listener(lsfilter_storage.list);

		lsfilter_textarea.init($('#filter_query'), $('#filter_query_order'));
		this.add_listener(lsfilter_textarea);

		lsfilter_saved.init();
		this.add_listener(lsfilter_saved);

		lsfilter_visual.init($('#filter_visual'));
		this.add_listener(lsfilter_visual);

		this.update(lsfilter_query, "Sweet Sweetback's Baadasssss Song", lsfilter_query_order);
		/**
		 * Closes the filter builder IF
		 * 1. User clicks the "Close filter builder" button
		 * 2. User clicks outside of the filter builder
		 * 3. User presses ESC
		 */
		var filterBuilder = $('#filter-query-builder');
		$('#close-filter-builder').on({
			click: function() {
				// Button clicked
				filterBuilder.hide();
			}
		});
		$(document).on({
			click: function(e) {
				if ($(e.target).closest(filterBuilder).length === 0) {
					// Clicked outside of the filter builder
					$(filterBuilder).hide();
				}
			},
			keyup: function(e) {
				if (e.keyCode === 27) {
					// Pressed ESC
					$(filterBuilder).hide();
				}
			}
		});

	},

	/***************************************************************************
	 * Update status bar
	 **************************************************************************/

	set_parse_status : function(status) {
		$('#filter-query-status').text(status);
	},
	clear_parse_status : function() {
		$('#filter-query-status').html("&nbsp;");
	},

	/***************************************************************************
	 * Update links from rest of the page
	 **************************************************************************/

	update_page_links : function() {
		var self = this; // To be able to access it from within handlers
		$(
				'a[href^="' + _site_domain + _index_page + '/' + _controller_name + '"]').click(function(evt) {
			var href = $(this).attr('href');
			var args = self.parseParams(href);
			if (args.q) {

				lsfilter_main.update(args.q, 'external_link', '');

				var content_div = $( "body > .container > #content" )
				content_div.click();
				content_div.focus();

				return false;

			}
			return true;
		});
	},

	/***************************************************************************
	 * Helpers for query parsing
	 **************************************************************************/
	re : /([^&=?]+)=([^&]*)/g,
	decodeRE : /\+/g, // Regex for replacing addition symbol with a space
	decode : function(str) {
		return decodeURIComponent(str.replace(this.decodeRE, " "));
	},
	parseParams : function(query) {
		var params = {}, e;
		while (e = this.re.exec(query)) {
			var k = this.decode(e[1]), v = this.decode(e[2]);
			if (k.substring(k.length - 2) === '[]') {
				k = k.substring(0, k.length - 2);
				(params[k] || (params[k] = [])).push(v);
			} else
				params[k] = v;
		}
		return params;
	}
};



/*******************
 * From src: modules/lsfilter/src/js/LSFilterMultiselect.js
 *******************/
var lsfilter_multiselect = {
	on: {
		'update_ok': function(data) {
			if (data.source == 'multiselect')
				return;
			if (!this.elem_menu)
				return;
			if (data.metadata.table && data.metadata.table != this.selection_table) {
				this.selection_table = data.metadata.table;
				this.selection = {};



				if (listview_commands[this.selection_table]) {
					this.populate_select(listview_commands[this.selection_table]);
				} else {
					this.populate_select({});
				}

				this.elem_objtype.attr('value', this.selection_table);
			}
		}
	},
	update: function() {
		// TODO temporary alias, it's only used in extra_objects.js,
		// which should be modified somehow, by someone who
		// understands that construct
		this.on.update_ok.apply(this, arguments);
	},
	init : function(elem) {
		var self = this; // To be able to access it from within handlers
		lsfilter_main.add_listener(self);

		this.elem_menu = elem;
		this.elem_objtype = $('#listview_multi_action_obj_type');
		$(document).on('click', '.multi-action-send-link', function(e) {
			e.preventDefault();
			self.do_send($(this));
			return false;
		});
	},

	elem_menu : false,
	elem_objtype : false,

	selection : {},
	selection_table : false,

	populate_select : function(commands) {
		this.elem_menu.empty();
		var categories = {};
		var cmd_count = 0;

		for ( var cmdname in commands) {
			var cmd = commands[cmdname];
			// Redirect commands can't be applied in multi aciton
			if(!cmd['redirect']) {
				if(!categories[cmd.category]) {
					categories[cmd.category] = {}
				}
				categories[cmd.category][cmdname] = cmd;

				cmd_count++;
			}
		}

		if(cmd_count == 0) {
			this.elem_menu.empty().text("No commands available");
		} else {
			for ( var category in categories) {
				var category_commands = categories[category];
				var cat_list = $('<ul />');
				cat_list.append($('<li />').text(category).addClass('multi-action-title'));

				for ( var cmdname in category_commands) {
					var cmd = category_commands[cmdname];
					var opt = $('<a href="#"/>')

					opt.attr('data-multi-action-command', cmdname);
					opt.text(cmd.name);
					opt.prepend($('<span />').addClass('icon-16').addClass('x16-' + cmd.icon));
					opt.addClass('multi-action-send-link');

					cat_list.append($('<li />').append(opt));
				}
				this.elem_menu.append(cat_list);
			}
		}
	},

	do_send : function(link) {
		var action = link.attr('data-multi-action-command');
		var selcount = $('.listview_multiselect_checkbox:checked').length;
		if (selcount == 0) {
			Notify.message('No items selected');
		} else if (!action) {
			Notify.message('No action selected');
		} else {
			$('#listview_multi_action_obj_action').attr('value', action);
			$('#listview_multi_action_form').submit();
		}
	},
	box_register : function(key, value) {
		this.selection[key] = value;
	},
	box_selected : function(key) {
		if (this.selection[key])
			return true;
		return false;
	}
};



/*******************
 * From src: modules/lsfilter/src/js/LSFilterSaved.js
 *******************/
var lsfilter_saved = {

	on: {
		'update_ok': function(data) {
			if (data.source == 'saved')
				return;
			this.last_query = data.query;
		}
	},
	init : function() {
		var self = this; // To be able to access it from within handlers

		$(document).on('click', '#lsfilter_save_filter', function() {
			$(this).addClass('saving').text(_('Saving...'));
			self.save($('#filter_query').val());
		});
	},

	icons : {
		hosts : '<span class="icon-menu menu-host"></span>',
		services : '<span class="icon-menu menu-service"></span>',
		hostgroups : '<span class="icon-menu menu-hostgroupsummary"></span>',
		servicegroups : '<span class="icon-menu menu-servicegroupsummary"></span>',
		other : '<span class="icon-menu menu-eventlog"></span>'
	},
	last_query : false,

	refresh_filter_list : function() {
		var self = this; // To be able to access it from within handlers
		var ajax_obj = {
			data : {
				'type' : 'lsfilters_saved',
				'page' : 'listview'
			},
			type : 'GET',
			success : function(data) {
				var list = $("#saved-filters-menu").empty();
				for ( var filter in data.data) {
					/* Someone broke the Array prototype, aaarrrrgggghhhh!!! */
					if (isNaN(parseInt(filter)))
						continue;
					(function() {
						var save = data.data[filter];
						var current_icon = self.icons.other;

						var table = 'hosts';
						if (save.table)
							table = save.table;

						if (self.icons[table])
							current_icon = self.icons[table];

						var link = link_query(save.query).addClass(
								'ninja_menu_links');

						link.append(current_icon);
						link.append($('<span class="nav-seg-span" />').text(
								save.name));

						list.append($('<li class="nav-seg" />').append(link));
					})();
				}
				var link = link_query('[saved_filters] all').addClass(
						'ninja_menu_links');
				link.append(self.icons['other']);
				link.append($('<span class="nav-seg-span" />').text(
						'Manage and view filters'));
				list.prepend($('<li class="nav-seg" />').append(link));
			}
		};
		$.ajax(_site_domain + _index_page + '/listview/fetch_saved_filters',
				ajax_obj);
	},

	save : function() {
		var self = this; // To be able to access it from within handlers

		var basepath = _site_domain + _index_page;
		var save = {
			"query" : this.last_query,
			"scope" : "user"
		};
		var name = $('#lsfilter_save_filter_name').val();

		if (!this.last_query)
			return;

		if (name) {

			if ($('#lsfilter_save_filter_global').attr('checked')) {
				save["scope"] = "global";
			}

			save['name'] = name;

			$.ajax(basepath + '/listview/save_filter', {
				data : save,
				type : 'GET',
				complete : function(xhr) {
					$('#lsfilter_save_filter').removeClass().text(_('Save'));
					self.refresh_filter_list();
					lsfilter_main.refresh();
				}
			});

		} else {
			Notify.message(_('You must give the filter a name!'));
			$('#lsfilter_save_filter').removeClass().text(_('Save'));
		}
	}
};

$(document).ready(function() {
	lsfilter_saved.refresh_filter_list();
});



/*******************
 * From src: modules/lsfilter/src/js/LSFilterTextarea.js
 *******************/
var lsfilter_textarea = {
	// Configuration

	// External methods
	init: function(element, orderelement)
	{
		var self = this; // To be able to access it from within handlers

		this.element = element;
		this.orderelement = orderelement;
		this.element.bind('keyup paste cut', function(evt)
		{
			var query = $.trim((self.element.val()).toString());
			lsfilter_main.update_delayed(query, 'textarea');
		});
	},
	on: {
		'update_failed': function() {
			this.element.css("border", "2px solid #f40");
		},
		'update_ok': function(data) {
			if (!this.element)
				return;
			this.element.css("border", "2px solid #5d2");
			if (data.source == 'textarea') return;
			this.element.val(data.query);
		}
	},
	load: function()
	{
		var query = this.element.val();
		var order = this.orderelement.val();

		lsfilter_main.update(query, 'textarea', order);
	},

	// Internal veriables
	element: false,
	orderelement: false,
};



/*******************
 * From src: modules/lsfilter/src/js/LSFilterVisitors.js
 *******************/
var LSFilterPP = function LSFilterPP() {
	this.parent = LSFilterPreprocessor;
	this.parent();

	this.preprocess_string = function(value) {
		return value.substring(1,value.length-1).replace(/\\(.)/g, '$1');
	};
};

var LSFilterMetadataVisitor = function LSFilterMetadataVisitor(){
	this.visit_entry            = function(query0)                    { return query0; };
	this.visit_query            = function(table_def1, search_query3) {
		var metadata = table_def1;
		metadata['columns'] = search_query3;
		return metadata;
		};
	this.visit_table_def_simple = function(name0)                     { return {'table':name0}; };
	this.visit_search_query     = function(filter0)                   { return filter0; };
	this.visit_filter_or        = function(filter0, filter2)          { return filter0.concat(filter2); };
	this.visit_filter_and       = function(filter0, filter2)          { return filter0.concat(filter2); };
	this.visit_filter_not       = function(filter1)                   { return filter1; };
	this.visit_filter_ok        = function(match0)                    { return match0; };
	this.visit_match_all        = function()                          { return []; };
	this.visit_match_in         = function(set_descr1)                { return []; };
	this.visit_match_field_in   = function(field0, set_descr2)        { return [field0]; };
	this.visit_match_not_re_ci  = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_not_re_cs  = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_re_ci      = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_re_cs      = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_not_eq_ci  = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_eq_ci      = function(field0, arg_string2)       { return [field0]; };
	this.visit_match_not_eq     = function(field0, arg_num_string2)   { return [field0]; };
	this.visit_match_gt_eq      = function(field0, arg_num2)          { return [field0]; };
	this.visit_match_lt_eq      = function(field0, arg_num2)          { return [field0]; };
	this.visit_match_gt         = function(field0, arg_num2)          { return [field0]; };
	this.visit_match_lt         = function(field0, arg_num2)          { return [field0]; };
	this.visit_match_eq         = function(field0, arg_num_string2)   { return [field0]; };
	this.visit_set_descr_name   = function(string0)                   { return null; };
	this.visit_field_name       = function(name0)                     { return name0; };
	this.visit_field_obj        = function(name0, field2)             { return name0+"."+field2; };
	this.visit_arg_num_func     = function(name0, arglist2)           { return arglist2 ; };
	this.visit_arg_list         = function(arg_num0, comma, arg_list2) { return arg_num0 + comma + arg_list2; };
	this.visit_arg_list_end     = function(arg_num0)                  { return arg_num0 ; };
	this.accept                 = function(result)                    { return result; };
};


var LSFilterASTVisitor = function LSFilterASTVisit() {
	this.visit_entry            = function(query0)                    { return query0; };
	this.visit_query            = function(table_def1, search_query3) { return {'obj':'query', 'table':table_def1, 'query':search_query3}; };
	this.visit_table_def_simple = function(name0)                     { return name0; };
	this.visit_search_query     = function(filter0)                   { return filter0; };
	this.visit_filter_or        = function(filter0, filter2)          { return this.addto('or',[filter0,filter2]); };
	this.visit_filter_and       = function(filter0, filter2)          { return this.addto('and',[filter0,filter2]); };
	this.visit_filter_not       = function(filter1)                   { if(filter1.obj=='not') return filter1.sub; return {'obj':'not','sub':filter1}; };
	this.visit_filter_ok        = function(match0)                    { return match0; };
	this.visit_match_all        = function()                          { return {'obj':'match',                 'value':'',              'op':'all'};       };
	this.visit_match_in         = function(set_descr1)                { return {'obj':'match',                 'value':set_descr1,      'op':'in'};        };
	this.visit_match_field_in   = function(field0, set_descr2)        { return {'obj':'match', 'field':field0, 'value':set_descr2,      'op':'in'};        };
	this.visit_match_not_re_ci  = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'not_re_ci'}; };
	this.visit_match_not_re_cs  = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'not_re_cs'}; };
	this.visit_match_re_ci      = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'re_ci'};     };
	this.visit_match_re_cs      = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'re_cs'};     };
	this.visit_match_not_eq_ci  = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'not_eq_ci'}; };
	this.visit_match_eq_ci      = function(field0, arg_string2)       { return {'obj':'match', 'field':field0, 'value':arg_string2,     'op':'eq_ci'};     };
	this.visit_match_not_eq     = function(field0, arg_num_string2)   { return {'obj':'match', 'field':field0, 'value':arg_num_string2, 'op':'not_eq'};    };
	this.visit_match_gt_eq      = function(field0, arg_num2)          { return {'obj':'match', 'field':field0, 'value':arg_num2,        'op':'gt_eq'};     };
	this.visit_match_lt_eq      = function(field0, arg_num2)          { return {'obj':'match', 'field':field0, 'value':arg_num2,        'op':'lt_eq'};     };
	this.visit_match_gt         = function(field0, arg_num2)          { return {'obj':'match', 'field':field0, 'value':arg_num2,        'op':'gt'};        };
	this.visit_match_lt         = function(field0, arg_num2)          { return {'obj':'match', 'field':field0, 'value':arg_num2,        'op':'lt'};        };
	this.visit_match_eq         = function(field0, arg_num_string2)   { return {'obj':'match', 'field':field0, 'value':arg_num_string2, 'op':'eq'};        };
	this.visit_set_descr_name   = function(string0)                   { return string0; };
	this.visit_field_name       = function(name0)                     { return name0; };
	this.visit_field_obj        = function(name0, field2)             { return name0+"."+field2; };
	this.visit_arg_num_func     = function(name0, arglist2)           { return arglist2 ; };
	this.visit_arg_list         = function(arg_num0, comma, arg_list2) { return arg_num0 + comma + arg_list2; };
	this.visit_arg_list_end     = function(arg_num0)                  { return arg_num0 ; };
	this.accept                 = function(result)                    { return result; };

	this.addto = function( obj, fields ) {
		var list = [];
		for( var i in fields ) {
			if( fields[i].obj == obj ) {
				list = list.concat(fields[i].sub);
			} else {
				list.push(fields[i]);
			}
		}
		return {'obj':obj, 'sub':list};
	};
};

function LSFilterASTVisit( obj, visitor ) {
	var method_name = 'visit_'+obj.obj;
	return visitor[method_name](obj);
}



/*******************
 * From src: modules/lsfilter/src/js/LSFilterVisual.js
 *******************/
/*******************************************************************************
 * Visitor to add extra and/or blocks around innermost objects so all objects is
 * expandable
 ******************************************************************************/
var lsfilter_visual_ast_preproc = {
	visit_query : function(obj) {
		var query = obj.query;

		if (query.obj != 'and') {
			/* If the outermost query isn't an and statement, enforce it */
			query = {
				'obj' : 'and',
				'sub' : [ query ]
			};
		} else {
			/*
			 * If it is an and statement, we want to wrap all
			 * non-group-statements in an and-clause, so it won't become a huge
			 * amount of and-clauses later. To make it easier to look at, keep
			 * the order, and just group following clauses
			 */

			var clauses = [];
			var current_and = false;
			for ( var i in query.sub) {
				var nextobj = query.sub[i];
				while (nextobj.obj == 'not') {
					nextobj = nextobj.sub;
				}
				if (nextobj.obj != 'and' && nextobj.obj != 'or') {
					if (current_and) {
						current_and.push(query.sub[i]);
					} else {
						current_and = [ query.sub[i] ];
						clauses.push({
							'obj' : 'and',
							'sub' : current_and
						});
					}
				} else {
					current_and = false;
					clauses.push(query.sub[i]);
				}
			}
			query = {
				'obj' : 'and',
				'sub' : clauses
			};
		}

		/*
		 * Make sure second level objects is group statmenets. No single rules
		 * should be floating around, and outermost and group isn't a real and
		 * group
		 */
		for ( var i in query.sub) {
			var nextobj = query.sub[i];

			/* Skip over not-nodes, because negated groups are still groups */
			while (nextobj.obj == 'not') {
				nextobj = nextobj.sub;
			}
			if (nextobj.obj != 'and' && nextobj.obj != 'or') {
				query.sub[i] = {
					'obj' : 'and',
					'sub' : [ query.sub[i] ]
				};
			}
		}

		/*
		 * Reduce the tree. This shouldn't remove operator levels, but just
		 * unused operators, like stray "all" and empty groups
		 */
		query = this.visit(query, 'query');

		/*
		 * If everything was reduced, just create a simple empty and statment as
		 * outermost container, to ensure a "Add * group" link exists, mark it
		 * outermost
		 */
		if (!query) {
			query = {
				'obj' : 'and',
				'sub' : []
			};
		}

		/*
		 * We now know that query root is an and statement. Mark it as
		 * outermost, to remove headers, "Add rule" and "Negate"
		 */
		query.outermost = true;

		return {
			'obj' : 'query',
			'table' : obj.table,
			'query' : query,
		};
	},

	visit_and : function(obj) {
		var list = [];

		for ( var i in obj.sub) {
			var sub = this.visit(obj.sub[i]);

			if (sub && !(sub.obj == 'match' && sub.op == 'all'))
				list.push(sub);
		}
		if (list.length == 0)
			return false;
		return {
			'obj' : 'and',
			'sub' : list
		};
	},
	visit_or : function(obj) {
		var list = [];

		for ( var i in obj.sub) {
			var sub = this.visit(obj.sub[i]);
			if (sub)
				list.push(sub);
		}
		if (list.length == 0)
			return {
				'obj' : 'match',
				'op' : 'all',
				'value' : ''
			};
		return {
			'obj' : 'or',
			'sub' : list
		};
	},
	visit_match : function(obj) {
		return obj;
	},
	visit_not : function(obj) {
		return {
			'obj' : 'not',
			'sub' : this.visit(obj.sub)
		};
	},

	visit : function(obj) {
		return LSFilterASTVisit(obj, this);
	}
};

/*******************************************************************************
 * Convert a lsfilter AST to html
 ******************************************************************************/
var lsfilter_graphics_visitor = {
	fields : null,
	operators : {
		"string" : {
			'eq' : 'equals',
			'not_eq' : 'not equals',
			'eq_ci' : 'equals, case insensitive',
			'not_eq_ci' : 'not equals, case insensitive',
			're_cs' : 'matching regexp',
			're_ci' : 'matching regexp, case insensitive',
			'not_re_cs' : 'not matching regexp',
			'not_re_ci' : 'not matching regexp, case insensitive'
		},
		"int" : {
			'eq' : 'equal',
			'not_eq' : 'not equals',
			'lt' : 'less than',
			'lt_eq' : 'less than or equal',
			'gt' : 'greater than',
			'gt_eq' : 'greater than or equals'
		},
		"float" : {
			'eq' : 'equals',
			'not_eq' : 'not equals',
			'lt' : 'less than',
			'lt_eq' : 'less than or equals',
			'gt' : 'greater than',
			'gt_eq' : 'greater than or equals'
		},
		"time" : {
			'eq' : 'at',
			'not_eq' : 'not at',
			'lt' : 'earlier than',
			'lt_eq' : 'earlier than or at',
			'gt' : 'later than',
			'gt_eq' : 'later than or at'
		},
		"object" : {
			'all' : 'all',
			'in' : 'in named set'
		},
		"list" : {
			'eq' : 'equals',
			'not_eq' : 'not equals',
			'gt_eq' : 'contains'
		},
		/* FIXME: This is ugly... better grammar for custom_variables */
		"dict" : {
			'eq' : 'equals',
			'not_eq' : 'not equals',
			'eq_ci' : 'equals, case insensitive',
			'not_eq_ci' : 'not equals, case insensitive',
			're_cs' : 'matching regexp',
			'not_re_cs' : 'not matching regexp',
			're_ci' : 'matching regexp, case insensitive',
			'not_re_ci' : 'not matching regexp, case insensitive'
		}
	},

	visit_query : function(obj) {
		this.fields = lsfilter_visual.fields_for_table(obj.table);
		var container = $('<div />');

		var tablesel = $('<select />');
		tablesel.addClass('lsfilter_visual_table_select');
		var tables = Object.keys(listview_renderer_table).sort();
		var table;
		for (var i = 0; i < tables.length; i++) {
			table = tables[i];
			var opt = $('<option />');
			opt.attr('value', table);
			opt.text(table);
			tablesel.append(opt);
		}
		tablesel.val(obj.table);
		container.append(tablesel);

		container.append(this.visit(obj.query));
		return container;
	},

	visit_and : function(obj) {
		return this.visit_andor(obj, 'and');
	},

	visit_or : function(obj) {
		return this.visit_andor(obj, 'or');
	},

	visit_andor : function(obj, op) {
		var list = $('<div />');

		var cmplop = (op == 'and') ? ('or') : ('and');

		list.addClass('lsfilter_visual_node');
		list.addClass('lsfilter_visual_group');
		list.addClass('lsfilter_visual_group_' + op);

		list.attr('data-op', op);

		var header = $("<div />");
		header.addClass('lsfilter_visual_group_header');

		list.append(header);

		if (!obj.outermost) {
			header.append($('<a class="lsfilter_visual_node_remove" />')
					.append(icon12('cross')));

			header.append($('<span />').text(_(op + " filter group")));
		}

		for ( var i in obj.sub) {
			list.append(this.visit(obj.sub[i]));
		}

		var footer = $('<div />');
		footer.addClass('lsfilter_visual_newmarker');
		footer.addClass('lsfilter_visual_group_footer');

		list.append(footer);

		var link;

		if (!obj.outermost) {
			link = $('<a />');
			link.addClass('lsfilter_visual_node_addrule');
			link.text(_('Add rule'));
			footer.append(link);
		}

		link = $('<a />');
		link.addClass('lsfilter_visual_node_addgroup');
		link.attr('data-op', 'and');
		link.text(_('Add and group'));
		footer.append(link);

		link = $('<a />');
		link.addClass('lsfilter_visual_node_addgroup');
		link.attr('data-op', 'or');
		link.text(_('Add or group'));
		footer.append(link);

		if (!obj.outermost) {
			link = $('<a />');
			link.addClass('lsfilter_visual_node_negate');
			link.text(_('Negate group'));
			footer.append(link);
		}

		return list;

	},

	visit_not : function(obj) {
		var result = this.visit(obj.sub);
		result.toggleClass('lsfilter_visual_not');
		return result;
	},

	visit_match : function(obj) {
		var result = $('<div />');

		result.addClass('lsfilter_visual_node');
		result.addClass('lsfilter_visual_match');

		result.append($('<a class="lsfilter_visual_node_remove" />').append(
				icon12('cross')));

		if (obj.op == 'all' || (obj.op == 'in' && !obj.field)) {
			obj.field = 'this';
		}

		/* Attribute select box */
		var attrsel = $('<select />');
		attrsel.addClass('lsfilter_visual_field_select');
		for ( var f in this.fields) {
			var opt = $('<option />');
			opt.attr('value', f);
			opt.text(f);
			opt.attr('data-type', this.fields[f].join(";"));
			attrsel.append(opt);
		}
		if (!this.fields[obj.field]) {
			console.log("Unknown field " + obj.field);
			var opt = $('<option />');
			opt.attr('value', obj.field);
			opt.text(_("Unknown column: ") + obj.field);
			opt.attr('data-type', "string");
			attrsel.append(opt);
		}
		attrsel.val(obj.field);
		result.append(attrsel);

		/* Operator select box */
		var opsel = $('<select />');
		opsel.addClass('lsfilter_visual_operator_select');
		this.fill_operator_select(opsel,
				this.fields[obj.field] ? this.fields[obj.field] : [ "string" ]);
		opsel.val(obj.op);
		result.append(opsel);

		/* Input text field */
		var valueinp = $('<input />');
		valueinp.addClass('lsfilter_visual_value_field');
		valueinp.val(obj.value);
		result.append(valueinp);

		var link = $('<a />');
		link.addClass('lsfilter_visual_node_negate');
		link.text(_('Negate'));
		result.append(link);

		return result;
	},

	visit : function(obj) {
		return LSFilterASTVisit(obj, this);
	},

	fill_operator_select : function(select, type) {
		select.empty();
		var ops = [];
		if (type[0]) {
			ops = this.operators[type[0]];
		}
		for ( var op in ops) {
			var opt = $('<option />');
			opt.attr('value', op);
			opt.text(_(ops[op]));
			select.append(opt);
		}
	}
};

/*******************************************************************************
 * Parse DOM structure down to a query
 ******************************************************************************/
var lsfilter_dom_to_query = {
	op_prio : {
		'or' : 1,
		'and' : 2
	},
	operators : {
		'not_re_ci' : '!~~',
		'not_re_cs' : '!~',
		'not_eq_ci' : '!=~',
		'not_eq' : '!=',
		're_ci' : '~~',
		're_cs' : '~',
		'eq_ci' : '=~',
		'gt_eq' : '>=',
		'lt_eq' : '<=',
		'eq' : '=',
		'gt' : '>',
		'lt' : '<',
		'in' : 'in',
		'all' : 'all'
	},

	visit : function(node, prio) {
		node = $(node);
		var result;
		if (node.attr('id') == 'filter_visual') {
			result = this.visit_query(node, prio);
		} else if (node.hasClass('lsfilter_visual_group')) {
			result = this.visit_binary(node, prio);
		} else if (node.hasClass('lsfilter_visual_match')) {
			result = this.visit_match(node, prio);
		}
		if (node.hasClass('lsfilter_visual_not'))
			result = "not (" + result + ")";
		return result;
	},

	/**
	 * Visit all children nodes, either direct or indirect, with class
	 * lsfilter_visual_node. Might be nodes between visited node and root, but
	 * never a node with class lsfilter_visual_node.
	 */
	visit_children : function(root, prio) {
		var self = this;
		var result = [];
		root.children().each(function() {
			if ($(this).hasClass('lsfilter_visual_node')) {
				result.push(self.visit($(this), prio));
			} else {
				result = result.concat(self.visit_children($(this), prio));
			}
		});
		return result;
	},
	visit_query : function(node, prio) {
		var table = node.find('.lsfilter_visual_table_select').val();
		var subq = this.visit_children(node, prio);
		if (subq.length != 1) {
			return "";
		}
		return "[" + table + "] " + subq[0];
	},
	visit_binary : function(node, prio) {
		var op = node.attr('data-op');
		var op_prio = this.op_prio[op];

		var result = this.visit_children(node, op_prio);

		if (prio > op_prio && result.length > 1) {
			return "(" + result.join(" " + op + " ") + ")";
		} else if (result.length == 0) {
			if (op == "and") {
				return "all";
			} else {
				return "not all";
			}
		}
		return result.join(" " + op + " ");

	},
	visit_match : function(node, prio) {
		var field_el = node.find('.lsfilter_visual_field_select');
		var field = field_el.val();

		var opt = field_el.children().filter(function(i) {
			return $(this).attr('value') == field;
		}).last();
		var type = opt.attr('data-type').split(';');

		var op = node.find('.lsfilter_visual_operator_select').val();
		var op_query = this.operators[op];

		var value_el = node.find('.lsfilter_visual_value_field');
		var value = value_el.val();

		if (type[0] == 'int') {
			value = parseInt(value);
			if (isNaN(value))
				value = 0;
		} else if (type[0] == 'float') {
			value = parseFloat(value);
		} else if (type[0] == 'time') {
			value = 'date("'+value+'")'
		} else {
			value = '"' + value.replace(/([\\"'])/g, "\\$1") + '"';
		}

		if (field == 'this')
			field = "";
		else
			field = field + " ";

		if (op == 'all')
			return 'all';

		return field + op_query + " " + value;
	}
};

/*******************************************************************************
 * Main object for graphical visualization
 ******************************************************************************/
var lsfilter_visual = {

	type_timeout : 500,
	current_timeout : false,

	on: {
		'update_ok': function(data) {
			if (data.source == 'visual')
				return;
			if (!this.filter_visual)
				return;
			var parser = new LSFilter(new LSFilterPP(), new LSFilterASTVisitor());
			try {
				var ast = parser.parse(data.query);
				ast = lsfilter_visual_ast_preproc.visit(ast);
				var result = lsfilter_graphics_visitor.visit(ast);
				this.filter_visual.empty().append(result);
				this.update_depths();
				this.update_binary_delimiters();
			} catch (ex) {
				console.log(ex.stack);
				console.log(data.query);
			}
		}
	},
	init : function(filter_visual) {
		this.filter_visual = filter_visual;
		var onnode = function(fnc) {
			return function(e) {
				e.preventDefault();
				fnc($(this).closest('.lsfilter_visual_node'), $(this));

				lsfilter_visual.validate_top_integrity();
				lsfilter_visual.update_depths();
				lsfilter_visual.update_binary_delimiters();
				lsfilter_visual.update_query_delayed();
				return false;
			};
		};

		this.filter_visual

		.on('click', '.lsfilter_visual_node_addrule', onnode(function(n, el) {
			var marker = n.children('.lsfilter_visual_newmarker');
			var newobj = lsfilter_graphics_visitor.visit({
				'obj' : 'match',
				'field' : 'this',
				'value' : '',
				'op' : 'all'
			});
			marker.before(newobj);
		}))

		.on('click', '.lsfilter_visual_node_addgroup', onnode(function(n, el) {
			var marker = n.children('.lsfilter_visual_newmarker');
			var op = el.attr('data-op');
			var newobj = lsfilter_graphics_visitor.visit({
				obj : op,
				sub : []
			});
			marker.before(newobj);
		}))

		.on('click', '.lsfilter_visual_node_remove', onnode(function(n, el) {
			/* Never remove outermost group */
			if (n.closest('.lsfilter_visual_node')) {
				n.remove();
			}
		}))

		.on('click', '.lsfilter_visual_node_negate', onnode(function(n, el) {
			n.toggleClass('lsfilter_visual_not');
		}))

		.on('change', '.lsfilter_visual_field_select', onnode(function(n, el) {
			var opt = el.children().filter(function(i) {
				return $(this).attr('value') == el.val();
			}).last();
			var type = opt.attr('data-type').split(';');
			var opselect = n.find('.lsfilter_visual_operator_select');
			if (type)
				lsfilter_graphics_visitor.fill_operator_select(opselect, type);
		}))

		.on('change', '.lsfilter_visual_operator_select',
				onnode(function(n, el) {
					/* Well... don't do anything here. onnode does what's needed */
				}))

		.on('change keyup', '.lsfilter_visual_value_field',
				onnode(function(n, el) {
					/* Well... don't do anything here. onnode does what's needed */
				}))

		.on('change', '.lsfilter_visual_table_select', function(e) {
			e.preventDefault();
			var n = $(this);
			var table = n.val();

			var ast = {
				'obj' : 'query',
				'table' : table,
				'query' : {
					'obj' : 'match',
					'field' : 'this',
					'value' : '',
					'op' : 'all'
				}
			};
			ast = lsfilter_visual_ast_preproc.visit(ast);
			var result = lsfilter_graphics_visitor.visit(ast);
			$('#filter_visual').empty().append(result);

			lsfilter_visual.update_query_delayed();
			return false;
		});
	},

	fields : null,

	update_query : function() {
		var query = lsfilter_dom_to_query.visit($('#filter_visual'), 0);

		lsfilter_main.update(query, 'visual', false);
	},

	update_query_delayed : function() {
		var self = this;
		if (this.current_timeout) {
			clearTimeout(this.current_timeout);
		}
		this.current_timeout = setTimeout(function() {
			self.current_timeout = false;
			self.update_query();
		}, this.type_timeout);
	},

	/* Color nodes after depth */
	update_depths : function() {
		$('#filter_visual').find('.lsfilter_visual_node').each(function() {
			var depth = $(this).parents('.lsfilter_visual_node').length;
			$(this).addClass('lsfilter_visual_node_' + depth);
		});
	},

	update_binary_delimiters : function() {
		var do_update = function(delimiter, operator) {
			var cls = '.lsfilter_visual_group_' + operator;
			delimiter.addClass('lsfilter_visual_delimiter');
			$('#filter_visual')
					.find(cls + ' > .lsfilter_visual_node')
					.filter(
							function(i) {
								return $(this).prev('.lsfilter_visual_node').length > 0;
							}).before(delimiter);
		};
		$('#filter_visual').find('.lsfilter_visual_delimiter').remove();
		do_update($('<div>OR</div>'), 'or');
		do_update($('<div>AND</div>'), 'and');
	},

	/* Validate that there exist at least one top object */
	validate_top_integrity : function() {
		if ($('#filter_visual').find('.lsfilter_visual_node').length == 0) {
			var table = $('#filter_visual').find(
					'.lsfilter_visual_table_select').val();
			var ast = {
				'obj' : 'query',
				'table' : table,
				'query' : {
					'obj' : 'and',
					'sub' : []
				}
			};
			ast = lsfilter_visual_ast_preproc.visit(ast);
			var result = lsfilter_graphics_visitor.visit(ast);
			$('#filter_visual').empty().append(result);
		}
	},

	fields_for_table : function(table) {
		/* Clone to not modify original structure */
		var fields = $.extend({}, ninja_manifest.orm_structure[table]);

		var subtables = [];
		var key;

		for (key in fields) {
			if (fields[key][0] == 'object') {
				subtables.push({
					field : key,
					table : fields[key][1]
				});
			}
		}

		for (key in subtables) {
			var j;
			var ref = subtables[key];
			for (j in ninja_manifest.orm_structure[ref.table]) {
				if (ninja_manifest.orm_structure[ref.table][j][0] != 'object') {
					fields[ref.field + '.' + j] = ninja_manifest.orm_structure[ref.table][j];
				}
			}
		}

		fields['this'] = [ 'object', table ];
		return fields;
	}
};



/*******************
 * From src: modules/lsfilter/src/js/codemirror/codemirror.js
 *******************/
// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE

// This is CodeMirror (http://codemirror.net), a code editor
// implemented in JavaScript on top of the browser's DOM.
//
// You can find some technical background for some of the code below
// at http://marijnhaverbeke.nl/blog/#cm-internals .

(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    module.exports = mod();
  else if (typeof define == "function" && define.amd) // AMD
    return define([], mod);
  else // Plain browser env
    (this || window).CodeMirror = mod();
})(function() {
  "use strict";

  // BROWSER SNIFFING

  // Kludges for bugs and behavior differences that can't be feature
  // detected are enabled based on userAgent etc sniffing.
  var userAgent = navigator.userAgent;
  var platform = navigator.platform;

  var gecko = /gecko\/\d/i.test(userAgent);
  var ie_upto10 = /MSIE \d/.test(userAgent);
  var ie_11up = /Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(userAgent);
  var ie = ie_upto10 || ie_11up;
  var ie_version = ie && (ie_upto10 ? document.documentMode || 6 : ie_11up[1]);
  var webkit = /WebKit\//.test(userAgent);
  var qtwebkit = webkit && /Qt\/\d+\.\d+/.test(userAgent);
  var chrome = /Chrome\//.test(userAgent);
  var presto = /Opera\//.test(userAgent);
  var safari = /Apple Computer/.test(navigator.vendor);
  var mac_geMountainLion = /Mac OS X 1\d\D([8-9]|\d\d)\D/.test(userAgent);
  var phantom = /PhantomJS/.test(userAgent);

  var ios = /AppleWebKit/.test(userAgent) && /Mobile\/\w+/.test(userAgent);
  // This is woefully incomplete. Suggestions for alternative methods welcome.
  var mobile = ios || /Android|webOS|BlackBerry|Opera Mini|Opera Mobi|IEMobile/i.test(userAgent);
  var mac = ios || /Mac/.test(platform);
  var chromeOS = /\bCrOS\b/.test(userAgent);
  var windows = /win/i.test(platform);

  var presto_version = presto && userAgent.match(/Version\/(\d*\.\d*)/);
  if (presto_version) presto_version = Number(presto_version[1]);
  if (presto_version && presto_version >= 15) { presto = false; webkit = true; }
  // Some browsers use the wrong event properties to signal cmd/ctrl on OS X
  var flipCtrlCmd = mac && (qtwebkit || presto && (presto_version == null || presto_version < 12.11));
  var captureRightClick = gecko || (ie && ie_version >= 9);

  // Optimize some code when these features are not used.
  var sawReadOnlySpans = false, sawCollapsedSpans = false;

  // EDITOR CONSTRUCTOR

  // A CodeMirror instance represents an editor. This is the object
  // that user code is usually dealing with.

  function CodeMirror(place, options) {
    if (!(this instanceof CodeMirror)) return new CodeMirror(place, options);

    this.options = options = options ? copyObj(options) : {};
    // Determine effective options based on given values and defaults.
    copyObj(defaults, options, false);
    setGuttersForLineNumbers(options);

    var doc = options.value;
    if (typeof doc == "string") doc = new Doc(doc, options.mode, null, options.lineSeparator);
    this.doc = doc;

    var input = new CodeMirror.inputStyles[options.inputStyle](this);
    var display = this.display = new Display(place, doc, input);
    display.wrapper.CodeMirror = this;
    updateGutters(this);
    themeChanged(this);
    if (options.lineWrapping)
      this.display.wrapper.className += " CodeMirror-wrap";
    if (options.autofocus && !mobile) display.input.focus();
    initScrollbars(this);

    this.state = {
      keyMaps: [],  // stores maps added by addKeyMap
      overlays: [], // highlighting overlays, as added by addOverlay
      modeGen: 0,   // bumped when mode/overlay changes, used to invalidate highlighting info
      overwrite: false,
      delayingBlurEvent: false,
      focused: false,
      suppressEdits: false, // used to disable editing during key handlers when in readOnly mode
      pasteIncoming: false, cutIncoming: false, // help recognize paste/cut edits in input.poll
      selectingText: false,
      draggingText: false,
      highlight: new Delayed(), // stores highlight worker timeout
      keySeq: null,  // Unfinished key sequence
      specialChars: null
    };

    var cm = this;

    // Override magic textarea content restore that IE sometimes does
    // on our hidden textarea on reload
    if (ie && ie_version < 11) setTimeout(function() { cm.display.input.reset(true); }, 20);

    registerEventHandlers(this);
    ensureGlobalHandlers();

    startOperation(this);
    this.curOp.forceUpdate = true;
    attachDoc(this, doc);

    if ((options.autofocus && !mobile) || cm.hasFocus())
      setTimeout(bind(onFocus, this), 20);
    else
      onBlur(this);

    for (var opt in optionHandlers) if (optionHandlers.hasOwnProperty(opt))
      optionHandlers[opt](this, options[opt], Init);
    maybeUpdateLineNumberWidth(this);
    if (options.finishInit) options.finishInit(this);
    for (var i = 0; i < initHooks.length; ++i) initHooks[i](this);
    endOperation(this);
    // Suppress optimizelegibility in Webkit, since it breaks text
    // measuring on line wrapping boundaries.
    if (webkit && options.lineWrapping &&
        getComputedStyle(display.lineDiv).textRendering == "optimizelegibility")
      display.lineDiv.style.textRendering = "auto";
  }

  // DISPLAY CONSTRUCTOR

  // The display handles the DOM integration, both for input reading
  // and content drawing. It holds references to DOM nodes and
  // display-related state.

  function Display(place, doc, input) {
    var d = this;
    this.input = input;

    // Covers bottom-right square when both scrollbars are present.
    d.scrollbarFiller = elt("div", null, "CodeMirror-scrollbar-filler");
    d.scrollbarFiller.setAttribute("cm-not-content", "true");
    // Covers bottom of gutter when coverGutterNextToScrollbar is on
    // and h scrollbar is present.
    d.gutterFiller = elt("div", null, "CodeMirror-gutter-filler");
    d.gutterFiller.setAttribute("cm-not-content", "true");
    // Will contain the actual code, positioned to cover the viewport.
    d.lineDiv = elt("div", null, "CodeMirror-code");
    // Elements are added to these to represent selection and cursors.
    d.selectionDiv = elt("div", null, null, "position: relative; z-index: 1");
    d.cursorDiv = elt("div", null, "CodeMirror-cursors");
    // A visibility: hidden element used to find the size of things.
    d.measure = elt("div", null, "CodeMirror-measure");
    // When lines outside of the viewport are measured, they are drawn in this.
    d.lineMeasure = elt("div", null, "CodeMirror-measure");
    // Wraps everything that needs to exist inside the vertically-padded coordinate system
    d.lineSpace = elt("div", [d.measure, d.lineMeasure, d.selectionDiv, d.cursorDiv, d.lineDiv],
                      null, "position: relative; outline: none");
    // Moved around its parent to cover visible view.
    d.mover = elt("div", [elt("div", [d.lineSpace], "CodeMirror-lines")], null, "position: relative");
    // Set to the height of the document, allowing scrolling.
    d.sizer = elt("div", [d.mover], "CodeMirror-sizer");
    d.sizerWidth = null;
    // Behavior of elts with overflow: auto and padding is
    // inconsistent across browsers. This is used to ensure the
    // scrollable area is big enough.
    d.heightForcer = elt("div", null, null, "position: absolute; height: " + scrollerGap + "px; width: 1px;");
    // Will contain the gutters, if any.
    d.gutters = elt("div", null, "CodeMirror-gutters");
    d.lineGutter = null;
    // Actual scrollable element.
    d.scroller = elt("div", [d.sizer, d.heightForcer, d.gutters], "CodeMirror-scroll");
    d.scroller.setAttribute("tabIndex", "-1");
    // The element in which the editor lives.
    d.wrapper = elt("div", [d.scrollbarFiller, d.gutterFiller, d.scroller], "CodeMirror");

    // Work around IE7 z-index bug (not perfect, hence IE7 not really being supported)
    if (ie && ie_version < 8) { d.gutters.style.zIndex = -1; d.scroller.style.paddingRight = 0; }
    if (!webkit && !(gecko && mobile)) d.scroller.draggable = true;

    if (place) {
      if (place.appendChild) place.appendChild(d.wrapper);
      else place(d.wrapper);
    }

    // Current rendered range (may be bigger than the view window).
    d.viewFrom = d.viewTo = doc.first;
    d.reportedViewFrom = d.reportedViewTo = doc.first;
    // Information about the rendered lines.
    d.view = [];
    d.renderedView = null;
    // Holds info about a single rendered line when it was rendered
    // for measurement, while not in view.
    d.externalMeasured = null;
    // Empty space (in pixels) above the view
    d.viewOffset = 0;
    d.lastWrapHeight = d.lastWrapWidth = 0;
    d.updateLineNumbers = null;

    d.nativeBarWidth = d.barHeight = d.barWidth = 0;
    d.scrollbarsClipped = false;

    // Used to only resize the line number gutter when necessary (when
    // the amount of lines crosses a boundary that makes its width change)
    d.lineNumWidth = d.lineNumInnerWidth = d.lineNumChars = null;
    // Set to true when a non-horizontal-scrolling line widget is
    // added. As an optimization, line widget aligning is skipped when
    // this is false.
    d.alignWidgets = false;

    d.cachedCharWidth = d.cachedTextHeight = d.cachedPaddingH = null;

    // Tracks the maximum line length so that the horizontal scrollbar
    // can be kept static when scrolling.
    d.maxLine = null;
    d.maxLineLength = 0;
    d.maxLineChanged = false;

    // Used for measuring wheel scrolling granularity
    d.wheelDX = d.wheelDY = d.wheelStartX = d.wheelStartY = null;

    // True when shift is held down.
    d.shift = false;

    // Used to track whether anything happened since the context menu
    // was opened.
    d.selForContextMenu = null;

    d.activeTouch = null;

    input.init(d);
  }

  // STATE UPDATES

  // Used to get the editor into a consistent state again when options change.

  function loadMode(cm) {
    cm.doc.mode = CodeMirror.getMode(cm.options, cm.doc.modeOption);
    resetModeState(cm);
  }

  function resetModeState(cm) {
    cm.doc.iter(function(line) {
      if (line.stateAfter) line.stateAfter = null;
      if (line.styles) line.styles = null;
    });
    cm.doc.frontier = cm.doc.first;
    startWorker(cm, 100);
    cm.state.modeGen++;
    if (cm.curOp) regChange(cm);
  }

  function wrappingChanged(cm) {
    if (cm.options.lineWrapping) {
      addClass(cm.display.wrapper, "CodeMirror-wrap");
      cm.display.sizer.style.minWidth = "";
      cm.display.sizerWidth = null;
    } else {
      rmClass(cm.display.wrapper, "CodeMirror-wrap");
      findMaxLine(cm);
    }
    estimateLineHeights(cm);
    regChange(cm);
    clearCaches(cm);
    setTimeout(function(){updateScrollbars(cm);}, 100);
  }

  // Returns a function that estimates the height of a line, to use as
  // first approximation until the line becomes visible (and is thus
  // properly measurable).
  function estimateHeight(cm) {
    var th = textHeight(cm.display), wrapping = cm.options.lineWrapping;
    var perLine = wrapping && Math.max(5, cm.display.scroller.clientWidth / charWidth(cm.display) - 3);
    return function(line) {
      if (lineIsHidden(cm.doc, line)) return 0;

      var widgetsHeight = 0;
      if (line.widgets) for (var i = 0; i < line.widgets.length; i++) {
        if (line.widgets[i].height) widgetsHeight += line.widgets[i].height;
      }

      if (wrapping)
        return widgetsHeight + (Math.ceil(line.text.length / perLine) || 1) * th;
      else
        return widgetsHeight + th;
    };
  }

  function estimateLineHeights(cm) {
    var doc = cm.doc, est = estimateHeight(cm);
    doc.iter(function(line) {
      var estHeight = est(line);
      if (estHeight != line.height) updateLineHeight(line, estHeight);
    });
  }

  function themeChanged(cm) {
    cm.display.wrapper.className = cm.display.wrapper.className.replace(/\s*cm-s-\S+/g, "") +
      cm.options.theme.replace(/(^|\s)\s*/g, " cm-s-");
    clearCaches(cm);
  }

  function guttersChanged(cm) {
    updateGutters(cm);
    regChange(cm);
    setTimeout(function(){alignHorizontally(cm);}, 20);
  }

  // Rebuild the gutter elements, ensure the margin to the left of the
  // code matches their width.
  function updateGutters(cm) {
    var gutters = cm.display.gutters, specs = cm.options.gutters;
    removeChildren(gutters);
    for (var i = 0; i < specs.length; ++i) {
      var gutterClass = specs[i];
      var gElt = gutters.appendChild(elt("div", null, "CodeMirror-gutter " + gutterClass));
      if (gutterClass == "CodeMirror-linenumbers") {
        cm.display.lineGutter = gElt;
        gElt.style.width = (cm.display.lineNumWidth || 1) + "px";
      }
    }
    gutters.style.display = i ? "" : "none";
    updateGutterSpace(cm);
  }

  function updateGutterSpace(cm) {
    var width = cm.display.gutters.offsetWidth;
    cm.display.sizer.style.marginLeft = width + "px";
  }

  // Compute the character length of a line, taking into account
  // collapsed ranges (see markText) that might hide parts, and join
  // other lines onto it.
  function lineLength(line) {
    if (line.height == 0) return 0;
    var len = line.text.length, merged, cur = line;
    while (merged = collapsedSpanAtStart(cur)) {
      var found = merged.find(0, true);
      cur = found.from.line;
      len += found.from.ch - found.to.ch;
    }
    cur = line;
    while (merged = collapsedSpanAtEnd(cur)) {
      var found = merged.find(0, true);
      len -= cur.text.length - found.from.ch;
      cur = found.to.line;
      len += cur.text.length - found.to.ch;
    }
    return len;
  }

  // Find the longest line in the document.
  function findMaxLine(cm) {
    var d = cm.display, doc = cm.doc;
    d.maxLine = getLine(doc, doc.first);
    d.maxLineLength = lineLength(d.maxLine);
    d.maxLineChanged = true;
    doc.iter(function(line) {
      var len = lineLength(line);
      if (len > d.maxLineLength) {
        d.maxLineLength = len;
        d.maxLine = line;
      }
    });
  }

  // Make sure the gutters options contains the element
  // "CodeMirror-linenumbers" when the lineNumbers option is true.
  function setGuttersForLineNumbers(options) {
    var found = indexOf(options.gutters, "CodeMirror-linenumbers");
    if (found == -1 && options.lineNumbers) {
      options.gutters = options.gutters.concat(["CodeMirror-linenumbers"]);
    } else if (found > -1 && !options.lineNumbers) {
      options.gutters = options.gutters.slice(0);
      options.gutters.splice(found, 1);
    }
  }

  // SCROLLBARS

  // Prepare DOM reads needed to update the scrollbars. Done in one
  // shot to minimize update/measure roundtrips.
  function measureForScrollbars(cm) {
    var d = cm.display, gutterW = d.gutters.offsetWidth;
    var docH = Math.round(cm.doc.height + paddingVert(cm.display));
    return {
      clientHeight: d.scroller.clientHeight,
      viewHeight: d.wrapper.clientHeight,
      scrollWidth: d.scroller.scrollWidth, clientWidth: d.scroller.clientWidth,
      viewWidth: d.wrapper.clientWidth,
      barLeft: cm.options.fixedGutter ? gutterW : 0,
      docHeight: docH,
      scrollHeight: docH + scrollGap(cm) + d.barHeight,
      nativeBarWidth: d.nativeBarWidth,
      gutterWidth: gutterW
    };
  }

  function NativeScrollbars(place, scroll, cm) {
    this.cm = cm;
    var vert = this.vert = elt("div", [elt("div", null, null, "min-width: 1px")], "CodeMirror-vscrollbar");
    var horiz = this.horiz = elt("div", [elt("div", null, null, "height: 100%; min-height: 1px")], "CodeMirror-hscrollbar");
    place(vert); place(horiz);

    on(vert, "scroll", function() {
      if (vert.clientHeight) scroll(vert.scrollTop, "vertical");
    });
    on(horiz, "scroll", function() {
      if (horiz.clientWidth) scroll(horiz.scrollLeft, "horizontal");
    });

    this.checkedZeroWidth = false;
    // Need to set a minimum width to see the scrollbar on IE7 (but must not set it on IE8).
    if (ie && ie_version < 8) this.horiz.style.minHeight = this.vert.style.minWidth = "18px";
  }

  NativeScrollbars.prototype = copyObj({
    update: function(measure) {
      var needsH = measure.scrollWidth > measure.clientWidth + 1;
      var needsV = measure.scrollHeight > measure.clientHeight + 1;
      var sWidth = measure.nativeBarWidth;

      if (needsV) {
        this.vert.style.display = "block";
        this.vert.style.bottom = needsH ? sWidth + "px" : "0";
        var totalHeight = measure.viewHeight - (needsH ? sWidth : 0);
        // A bug in IE8 can cause this value to be negative, so guard it.
        this.vert.firstChild.style.height =
          Math.max(0, measure.scrollHeight - measure.clientHeight + totalHeight) + "px";
      } else {
        this.vert.style.display = "";
        this.vert.firstChild.style.height = "0";
      }

      if (needsH) {
        this.horiz.style.display = "block";
        this.horiz.style.right = needsV ? sWidth + "px" : "0";
        this.horiz.style.left = measure.barLeft + "px";
        var totalWidth = measure.viewWidth - measure.barLeft - (needsV ? sWidth : 0);
        this.horiz.firstChild.style.width =
          (measure.scrollWidth - measure.clientWidth + totalWidth) + "px";
      } else {
        this.horiz.style.display = "";
        this.horiz.firstChild.style.width = "0";
      }

      if (!this.checkedZeroWidth && measure.clientHeight > 0) {
        if (sWidth == 0) this.zeroWidthHack();
        this.checkedZeroWidth = true;
      }

      return {right: needsV ? sWidth : 0, bottom: needsH ? sWidth : 0};
    },
    setScrollLeft: function(pos) {
      if (this.horiz.scrollLeft != pos) this.horiz.scrollLeft = pos;
      if (this.disableHoriz) this.enableZeroWidthBar(this.horiz, this.disableHoriz);
    },
    setScrollTop: function(pos) {
      if (this.vert.scrollTop != pos) this.vert.scrollTop = pos;
      if (this.disableVert) this.enableZeroWidthBar(this.vert, this.disableVert);
    },
    zeroWidthHack: function() {
      var w = mac && !mac_geMountainLion ? "12px" : "18px";
      this.horiz.style.height = this.vert.style.width = w;
      this.horiz.style.pointerEvents = this.vert.style.pointerEvents = "none";
      this.disableHoriz = new Delayed;
      this.disableVert = new Delayed;
    },
    enableZeroWidthBar: function(bar, delay) {
      bar.style.pointerEvents = "auto";
      function maybeDisable() {
        // To find out whether the scrollbar is still visible, we
        // check whether the element under the pixel in the bottom
        // left corner of the scrollbar box is the scrollbar box
        // itself (when the bar is still visible) or its filler child
        // (when the bar is hidden). If it is still visible, we keep
        // it enabled, if it's hidden, we disable pointer events.
        var box = bar.getBoundingClientRect();
        var elt = document.elementFromPoint(box.left + 1, box.bottom - 1);
        if (elt != bar) bar.style.pointerEvents = "none";
        else delay.set(1000, maybeDisable);
      }
      delay.set(1000, maybeDisable);
    },
    clear: function() {
      var parent = this.horiz.parentNode;
      parent.removeChild(this.horiz);
      parent.removeChild(this.vert);
    }
  }, NativeScrollbars.prototype);

  function NullScrollbars() {}

  NullScrollbars.prototype = copyObj({
    update: function() { return {bottom: 0, right: 0}; },
    setScrollLeft: function() {},
    setScrollTop: function() {},
    clear: function() {}
  }, NullScrollbars.prototype);

  CodeMirror.scrollbarModel = {"native": NativeScrollbars, "null": NullScrollbars};

  function initScrollbars(cm) {
    if (cm.display.scrollbars) {
      cm.display.scrollbars.clear();
      if (cm.display.scrollbars.addClass)
        rmClass(cm.display.wrapper, cm.display.scrollbars.addClass);
    }

    cm.display.scrollbars = new CodeMirror.scrollbarModel[cm.options.scrollbarStyle](function(node) {
      cm.display.wrapper.insertBefore(node, cm.display.scrollbarFiller);
      // Prevent clicks in the scrollbars from killing focus
      on(node, "mousedown", function() {
        if (cm.state.focused) setTimeout(function() { cm.display.input.focus(); }, 0);
      });
      node.setAttribute("cm-not-content", "true");
    }, function(pos, axis) {
      if (axis == "horizontal") setScrollLeft(cm, pos);
      else setScrollTop(cm, pos);
    }, cm);
    if (cm.display.scrollbars.addClass)
      addClass(cm.display.wrapper, cm.display.scrollbars.addClass);
  }

  function updateScrollbars(cm, measure) {
    if (!measure) measure = measureForScrollbars(cm);
    var startWidth = cm.display.barWidth, startHeight = cm.display.barHeight;
    updateScrollbarsInner(cm, measure);
    for (var i = 0; i < 4 && startWidth != cm.display.barWidth || startHeight != cm.display.barHeight; i++) {
      if (startWidth != cm.display.barWidth && cm.options.lineWrapping)
        updateHeightsInViewport(cm);
      updateScrollbarsInner(cm, measureForScrollbars(cm));
      startWidth = cm.display.barWidth; startHeight = cm.display.barHeight;
    }
  }

  // Re-synchronize the fake scrollbars with the actual size of the
  // content.
  function updateScrollbarsInner(cm, measure) {
    var d = cm.display;
    var sizes = d.scrollbars.update(measure);

    d.sizer.style.paddingRight = (d.barWidth = sizes.right) + "px";
    d.sizer.style.paddingBottom = (d.barHeight = sizes.bottom) + "px";
    d.heightForcer.style.borderBottom = sizes.bottom + "px solid transparent"

    if (sizes.right && sizes.bottom) {
      d.scrollbarFiller.style.display = "block";
      d.scrollbarFiller.style.height = sizes.bottom + "px";
      d.scrollbarFiller.style.width = sizes.right + "px";
    } else d.scrollbarFiller.style.display = "";
    if (sizes.bottom && cm.options.coverGutterNextToScrollbar && cm.options.fixedGutter) {
      d.gutterFiller.style.display = "block";
      d.gutterFiller.style.height = sizes.bottom + "px";
      d.gutterFiller.style.width = measure.gutterWidth + "px";
    } else d.gutterFiller.style.display = "";
  }

  // Compute the lines that are visible in a given viewport (defaults
  // the the current scroll position). viewport may contain top,
  // height, and ensure (see op.scrollToPos) properties.
  function visibleLines(display, doc, viewport) {
    var top = viewport && viewport.top != null ? Math.max(0, viewport.top) : display.scroller.scrollTop;
    top = Math.floor(top - paddingTop(display));
    var bottom = viewport && viewport.bottom != null ? viewport.bottom : top + display.wrapper.clientHeight;

    var from = lineAtHeight(doc, top), to = lineAtHeight(doc, bottom);
    // Ensure is a {from: {line, ch}, to: {line, ch}} object, and
    // forces those lines into the viewport (if possible).
    if (viewport && viewport.ensure) {
      var ensureFrom = viewport.ensure.from.line, ensureTo = viewport.ensure.to.line;
      if (ensureFrom < from) {
        from = ensureFrom;
        to = lineAtHeight(doc, heightAtLine(getLine(doc, ensureFrom)) + display.wrapper.clientHeight);
      } else if (Math.min(ensureTo, doc.lastLine()) >= to) {
        from = lineAtHeight(doc, heightAtLine(getLine(doc, ensureTo)) - display.wrapper.clientHeight);
        to = ensureTo;
      }
    }
    return {from: from, to: Math.max(to, from + 1)};
  }

  // LINE NUMBERS

  // Re-align line numbers and gutter marks to compensate for
  // horizontal scrolling.
  function alignHorizontally(cm) {
    var display = cm.display, view = display.view;
    if (!display.alignWidgets && (!display.gutters.firstChild || !cm.options.fixedGutter)) return;
    var comp = compensateForHScroll(display) - display.scroller.scrollLeft + cm.doc.scrollLeft;
    var gutterW = display.gutters.offsetWidth, left = comp + "px";
    for (var i = 0; i < view.length; i++) if (!view[i].hidden) {
      if (cm.options.fixedGutter && view[i].gutter)
        view[i].gutter.style.left = left;
      var align = view[i].alignable;
      if (align) for (var j = 0; j < align.length; j++)
        align[j].style.left = left;
    }
    if (cm.options.fixedGutter)
      display.gutters.style.left = (comp + gutterW) + "px";
  }

  // Used to ensure that the line number gutter is still the right
  // size for the current document size. Returns true when an update
  // is needed.
  function maybeUpdateLineNumberWidth(cm) {
    if (!cm.options.lineNumbers) return false;
    var doc = cm.doc, last = lineNumberFor(cm.options, doc.first + doc.size - 1), display = cm.display;
    if (last.length != display.lineNumChars) {
      var test = display.measure.appendChild(elt("div", [elt("div", last)],
                                                 "CodeMirror-linenumber CodeMirror-gutter-elt"));
      var innerW = test.firstChild.offsetWidth, padding = test.offsetWidth - innerW;
      display.lineGutter.style.width = "";
      display.lineNumInnerWidth = Math.max(innerW, display.lineGutter.offsetWidth - padding) + 1;
      display.lineNumWidth = display.lineNumInnerWidth + padding;
      display.lineNumChars = display.lineNumInnerWidth ? last.length : -1;
      display.lineGutter.style.width = display.lineNumWidth + "px";
      updateGutterSpace(cm);
      return true;
    }
    return false;
  }

  function lineNumberFor(options, i) {
    return String(options.lineNumberFormatter(i + options.firstLineNumber));
  }

  // Computes display.scroller.scrollLeft + display.gutters.offsetWidth,
  // but using getBoundingClientRect to get a sub-pixel-accurate
  // result.
  function compensateForHScroll(display) {
    return display.scroller.getBoundingClientRect().left - display.sizer.getBoundingClientRect().left;
  }

  // DISPLAY DRAWING

  function DisplayUpdate(cm, viewport, force) {
    var display = cm.display;

    this.viewport = viewport;
    // Store some values that we'll need later (but don't want to force a relayout for)
    this.visible = visibleLines(display, cm.doc, viewport);
    this.editorIsHidden = !display.wrapper.offsetWidth;
    this.wrapperHeight = display.wrapper.clientHeight;
    this.wrapperWidth = display.wrapper.clientWidth;
    this.oldDisplayWidth = displayWidth(cm);
    this.force = force;
    this.dims = getDimensions(cm);
    this.events = [];
  }

  DisplayUpdate.prototype.signal = function(emitter, type) {
    if (hasHandler(emitter, type))
      this.events.push(arguments);
  };
  DisplayUpdate.prototype.finish = function() {
    for (var i = 0; i < this.events.length; i++)
      signal.apply(null, this.events[i]);
  };

  function maybeClipScrollbars(cm) {
    var display = cm.display;
    if (!display.scrollbarsClipped && display.scroller.offsetWidth) {
      display.nativeBarWidth = display.scroller.offsetWidth - display.scroller.clientWidth;
      display.heightForcer.style.height = scrollGap(cm) + "px";
      display.sizer.style.marginBottom = -display.nativeBarWidth + "px";
      display.sizer.style.borderRightWidth = scrollGap(cm) + "px";
      display.scrollbarsClipped = true;
    }
  }

  // Does the actual updating of the line display. Bails out
  // (returning false) when there is nothing to be done and forced is
  // false.
  function updateDisplayIfNeeded(cm, update) {
    var display = cm.display, doc = cm.doc;

    if (update.editorIsHidden) {
      resetView(cm);
      return false;
    }

    // Bail out if the visible area is already rendered and nothing changed.
    if (!update.force &&
        update.visible.from >= display.viewFrom && update.visible.to <= display.viewTo &&
        (display.updateLineNumbers == null || display.updateLineNumbers >= display.viewTo) &&
        display.renderedView == display.view && countDirtyView(cm) == 0)
      return false;

    if (maybeUpdateLineNumberWidth(cm)) {
      resetView(cm);
      update.dims = getDimensions(cm);
    }

    // Compute a suitable new viewport (from & to)
    var end = doc.first + doc.size;
    var from = Math.max(update.visible.from - cm.options.viewportMargin, doc.first);
    var to = Math.min(end, update.visible.to + cm.options.viewportMargin);
    if (display.viewFrom < from && from - display.viewFrom < 20) from = Math.max(doc.first, display.viewFrom);
    if (display.viewTo > to && display.viewTo - to < 20) to = Math.min(end, display.viewTo);
    if (sawCollapsedSpans) {
      from = visualLineNo(cm.doc, from);
      to = visualLineEndNo(cm.doc, to);
    }

    var different = from != display.viewFrom || to != display.viewTo ||
      display.lastWrapHeight != update.wrapperHeight || display.lastWrapWidth != update.wrapperWidth;
    adjustView(cm, from, to);

    display.viewOffset = heightAtLine(getLine(cm.doc, display.viewFrom));
    // Position the mover div to align with the current scroll position
    cm.display.mover.style.top = display.viewOffset + "px";

    var toUpdate = countDirtyView(cm);
    if (!different && toUpdate == 0 && !update.force && display.renderedView == display.view &&
        (display.updateLineNumbers == null || display.updateLineNumbers >= display.viewTo))
      return false;

    // For big changes, we hide the enclosing element during the
    // update, since that speeds up the operations on most browsers.
    var focused = activeElt();
    if (toUpdate > 4) display.lineDiv.style.display = "none";
    patchDisplay(cm, display.updateLineNumbers, update.dims);
    if (toUpdate > 4) display.lineDiv.style.display = "";
    display.renderedView = display.view;
    // There might have been a widget with a focused element that got
    // hidden or updated, if so re-focus it.
    if (focused && activeElt() != focused && focused.offsetHeight) focused.focus();

    // Prevent selection and cursors from interfering with the scroll
    // width and height.
    removeChildren(display.cursorDiv);
    removeChildren(display.selectionDiv);
    display.gutters.style.height = display.sizer.style.minHeight = 0;

    if (different) {
      display.lastWrapHeight = update.wrapperHeight;
      display.lastWrapWidth = update.wrapperWidth;
      startWorker(cm, 400);
    }

    display.updateLineNumbers = null;

    return true;
  }

  function postUpdateDisplay(cm, update) {
    var viewport = update.viewport;

    for (var first = true;; first = false) {
      if (!first || !cm.options.lineWrapping || update.oldDisplayWidth == displayWidth(cm)) {
        // Clip forced viewport to actual scrollable area.
        if (viewport && viewport.top != null)
          viewport = {top: Math.min(cm.doc.height + paddingVert(cm.display) - displayHeight(cm), viewport.top)};
        // Updated line heights might result in the drawn area not
        // actually covering the viewport. Keep looping until it does.
        update.visible = visibleLines(cm.display, cm.doc, viewport);
        if (update.visible.from >= cm.display.viewFrom && update.visible.to <= cm.display.viewTo)
          break;
      }
      if (!updateDisplayIfNeeded(cm, update)) break;
      updateHeightsInViewport(cm);
      var barMeasure = measureForScrollbars(cm);
      updateSelection(cm);
      updateScrollbars(cm, barMeasure);
      setDocumentHeight(cm, barMeasure);
    }

    update.signal(cm, "update", cm);
    if (cm.display.viewFrom != cm.display.reportedViewFrom || cm.display.viewTo != cm.display.reportedViewTo) {
      update.signal(cm, "viewportChange", cm, cm.display.viewFrom, cm.display.viewTo);
      cm.display.reportedViewFrom = cm.display.viewFrom; cm.display.reportedViewTo = cm.display.viewTo;
    }
  }

  function updateDisplaySimple(cm, viewport) {
    var update = new DisplayUpdate(cm, viewport);
    if (updateDisplayIfNeeded(cm, update)) {
      updateHeightsInViewport(cm);
      postUpdateDisplay(cm, update);
      var barMeasure = measureForScrollbars(cm);
      updateSelection(cm);
      updateScrollbars(cm, barMeasure);
      setDocumentHeight(cm, barMeasure);
      update.finish();
    }
  }

  function setDocumentHeight(cm, measure) {
    cm.display.sizer.style.minHeight = measure.docHeight + "px";
    cm.display.heightForcer.style.top = measure.docHeight + "px";
    cm.display.gutters.style.height = (measure.docHeight + cm.display.barHeight + scrollGap(cm)) + "px";
  }

  // Read the actual heights of the rendered lines, and update their
  // stored heights to match.
  function updateHeightsInViewport(cm) {
    var display = cm.display;
    var prevBottom = display.lineDiv.offsetTop;
    for (var i = 0; i < display.view.length; i++) {
      var cur = display.view[i], height;
      if (cur.hidden) continue;
      if (ie && ie_version < 8) {
        var bot = cur.node.offsetTop + cur.node.offsetHeight;
        height = bot - prevBottom;
        prevBottom = bot;
      } else {
        var box = cur.node.getBoundingClientRect();
        height = box.bottom - box.top;
      }
      var diff = cur.line.height - height;
      if (height < 2) height = textHeight(display);
      if (diff > .001 || diff < -.001) {
        updateLineHeight(cur.line, height);
        updateWidgetHeight(cur.line);
        if (cur.rest) for (var j = 0; j < cur.rest.length; j++)
          updateWidgetHeight(cur.rest[j]);
      }
    }
  }

  // Read and store the height of line widgets associated with the
  // given line.
  function updateWidgetHeight(line) {
    if (line.widgets) for (var i = 0; i < line.widgets.length; ++i)
      line.widgets[i].height = line.widgets[i].node.parentNode.offsetHeight;
  }

  // Do a bulk-read of the DOM positions and sizes needed to draw the
  // view, so that we don't interleave reading and writing to the DOM.
  function getDimensions(cm) {
    var d = cm.display, left = {}, width = {};
    var gutterLeft = d.gutters.clientLeft;
    for (var n = d.gutters.firstChild, i = 0; n; n = n.nextSibling, ++i) {
      left[cm.options.gutters[i]] = n.offsetLeft + n.clientLeft + gutterLeft;
      width[cm.options.gutters[i]] = n.clientWidth;
    }
    return {fixedPos: compensateForHScroll(d),
            gutterTotalWidth: d.gutters.offsetWidth,
            gutterLeft: left,
            gutterWidth: width,
            wrapperWidth: d.wrapper.clientWidth};
  }

  // Sync the actual display DOM structure with display.view, removing
  // nodes for lines that are no longer in view, and creating the ones
  // that are not there yet, and updating the ones that are out of
  // date.
  function patchDisplay(cm, updateNumbersFrom, dims) {
    var display = cm.display, lineNumbers = cm.options.lineNumbers;
    var container = display.lineDiv, cur = container.firstChild;

    function rm(node) {
      var next = node.nextSibling;
      // Works around a throw-scroll bug in OS X Webkit
      if (webkit && mac && cm.display.currentWheelTarget == node)
        node.style.display = "none";
      else
        node.parentNode.removeChild(node);
      return next;
    }

    var view = display.view, lineN = display.viewFrom;
    // Loop over the elements in the view, syncing cur (the DOM nodes
    // in display.lineDiv) with the view as we go.
    for (var i = 0; i < view.length; i++) {
      var lineView = view[i];
      if (lineView.hidden) {
      } else if (!lineView.node || lineView.node.parentNode != container) { // Not drawn yet
        var node = buildLineElement(cm, lineView, lineN, dims);
        container.insertBefore(node, cur);
      } else { // Already drawn
        while (cur != lineView.node) cur = rm(cur);
        var updateNumber = lineNumbers && updateNumbersFrom != null &&
          updateNumbersFrom <= lineN && lineView.lineNumber;
        if (lineView.changes) {
          if (indexOf(lineView.changes, "gutter") > -1) updateNumber = false;
          updateLineForChanges(cm, lineView, lineN, dims);
        }
        if (updateNumber) {
          removeChildren(lineView.lineNumber);
          lineView.lineNumber.appendChild(document.createTextNode(lineNumberFor(cm.options, lineN)));
        }
        cur = lineView.node.nextSibling;
      }
      lineN += lineView.size;
    }
    while (cur) cur = rm(cur);
  }

  // When an aspect of a line changes, a string is added to
  // lineView.changes. This updates the relevant part of the line's
  // DOM structure.
  function updateLineForChanges(cm, lineView, lineN, dims) {
    for (var j = 0; j < lineView.changes.length; j++) {
      var type = lineView.changes[j];
      if (type == "text") updateLineText(cm, lineView);
      else if (type == "gutter") updateLineGutter(cm, lineView, lineN, dims);
      else if (type == "class") updateLineClasses(lineView);
      else if (type == "widget") updateLineWidgets(cm, lineView, dims);
    }
    lineView.changes = null;
  }

  // Lines with gutter elements, widgets or a background class need to
  // be wrapped, and have the extra elements added to the wrapper div
  function ensureLineWrapped(lineView) {
    if (lineView.node == lineView.text) {
      lineView.node = elt("div", null, null, "position: relative");
      if (lineView.text.parentNode)
        lineView.text.parentNode.replaceChild(lineView.node, lineView.text);
      lineView.node.appendChild(lineView.text);
      if (ie && ie_version < 8) lineView.node.style.zIndex = 2;
    }
    return lineView.node;
  }

  function updateLineBackground(lineView) {
    var cls = lineView.bgClass ? lineView.bgClass + " " + (lineView.line.bgClass || "") : lineView.line.bgClass;
    if (cls) cls += " CodeMirror-linebackground";
    if (lineView.background) {
      if (cls) lineView.background.className = cls;
      else { lineView.background.parentNode.removeChild(lineView.background); lineView.background = null; }
    } else if (cls) {
      var wrap = ensureLineWrapped(lineView);
      lineView.background = wrap.insertBefore(elt("div", null, cls), wrap.firstChild);
    }
  }

  // Wrapper around buildLineContent which will reuse the structure
  // in display.externalMeasured when possible.
  function getLineContent(cm, lineView) {
    var ext = cm.display.externalMeasured;
    if (ext && ext.line == lineView.line) {
      cm.display.externalMeasured = null;
      lineView.measure = ext.measure;
      return ext.built;
    }
    return buildLineContent(cm, lineView);
  }

  // Redraw the line's text. Interacts with the background and text
  // classes because the mode may output tokens that influence these
  // classes.
  function updateLineText(cm, lineView) {
    var cls = lineView.text.className;
    var built = getLineContent(cm, lineView);
    if (lineView.text == lineView.node) lineView.node = built.pre;
    lineView.text.parentNode.replaceChild(built.pre, lineView.text);
    lineView.text = built.pre;
    if (built.bgClass != lineView.bgClass || built.textClass != lineView.textClass) {
      lineView.bgClass = built.bgClass;
      lineView.textClass = built.textClass;
      updateLineClasses(lineView);
    } else if (cls) {
      lineView.text.className = cls;
    }
  }

  function updateLineClasses(lineView) {
    updateLineBackground(lineView);
    if (lineView.line.wrapClass)
      ensureLineWrapped(lineView).className = lineView.line.wrapClass;
    else if (lineView.node != lineView.text)
      lineView.node.className = "";
    var textClass = lineView.textClass ? lineView.textClass + " " + (lineView.line.textClass || "") : lineView.line.textClass;
    lineView.text.className = textClass || "";
  }

  function updateLineGutter(cm, lineView, lineN, dims) {
    if (lineView.gutter) {
      lineView.node.removeChild(lineView.gutter);
      lineView.gutter = null;
    }
    if (lineView.gutterBackground) {
      lineView.node.removeChild(lineView.gutterBackground);
      lineView.gutterBackground = null;
    }
    if (lineView.line.gutterClass) {
      var wrap = ensureLineWrapped(lineView);
      lineView.gutterBackground = elt("div", null, "CodeMirror-gutter-background " + lineView.line.gutterClass,
                                      "left: " + (cm.options.fixedGutter ? dims.fixedPos : -dims.gutterTotalWidth) +
                                      "px; width: " + dims.gutterTotalWidth + "px");
      wrap.insertBefore(lineView.gutterBackground, lineView.text);
    }
    var markers = lineView.line.gutterMarkers;
    if (cm.options.lineNumbers || markers) {
      var wrap = ensureLineWrapped(lineView);
      var gutterWrap = lineView.gutter = elt("div", null, "CodeMirror-gutter-wrapper", "left: " +
                                             (cm.options.fixedGutter ? dims.fixedPos : -dims.gutterTotalWidth) + "px");
      cm.display.input.setUneditable(gutterWrap);
      wrap.insertBefore(gutterWrap, lineView.text);
      if (lineView.line.gutterClass)
        gutterWrap.className += " " + lineView.line.gutterClass;
      if (cm.options.lineNumbers && (!markers || !markers["CodeMirror-linenumbers"]))
        lineView.lineNumber = gutterWrap.appendChild(
          elt("div", lineNumberFor(cm.options, lineN),
              "CodeMirror-linenumber CodeMirror-gutter-elt",
              "left: " + dims.gutterLeft["CodeMirror-linenumbers"] + "px; width: "
              + cm.display.lineNumInnerWidth + "px"));
      if (markers) for (var k = 0; k < cm.options.gutters.length; ++k) {
        var id = cm.options.gutters[k], found = markers.hasOwnProperty(id) && markers[id];
        if (found)
          gutterWrap.appendChild(elt("div", [found], "CodeMirror-gutter-elt", "left: " +
                                     dims.gutterLeft[id] + "px; width: " + dims.gutterWidth[id] + "px"));
      }
    }
  }

  function updateLineWidgets(cm, lineView, dims) {
    if (lineView.alignable) lineView.alignable = null;
    for (var node = lineView.node.firstChild, next; node; node = next) {
      var next = node.nextSibling;
      if (node.className == "CodeMirror-linewidget")
        lineView.node.removeChild(node);
    }
    insertLineWidgets(cm, lineView, dims);
  }

  // Build a line's DOM representation from scratch
  function buildLineElement(cm, lineView, lineN, dims) {
    var built = getLineContent(cm, lineView);
    lineView.text = lineView.node = built.pre;
    if (built.bgClass) lineView.bgClass = built.bgClass;
    if (built.textClass) lineView.textClass = built.textClass;

    updateLineClasses(lineView);
    updateLineGutter(cm, lineView, lineN, dims);
    insertLineWidgets(cm, lineView, dims);
    return lineView.node;
  }

  // A lineView may contain multiple logical lines (when merged by
  // collapsed spans). The widgets for all of them need to be drawn.
  function insertLineWidgets(cm, lineView, dims) {
    insertLineWidgetsFor(cm, lineView.line, lineView, dims, true);
    if (lineView.rest) for (var i = 0; i < lineView.rest.length; i++)
      insertLineWidgetsFor(cm, lineView.rest[i], lineView, dims, false);
  }

  function insertLineWidgetsFor(cm, line, lineView, dims, allowAbove) {
    if (!line.widgets) return;
    var wrap = ensureLineWrapped(lineView);
    for (var i = 0, ws = line.widgets; i < ws.length; ++i) {
      var widget = ws[i], node = elt("div", [widget.node], "CodeMirror-linewidget");
      if (!widget.handleMouseEvents) node.setAttribute("cm-ignore-events", "true");
      positionLineWidget(widget, node, lineView, dims);
      cm.display.input.setUneditable(node);
      if (allowAbove && widget.above)
        wrap.insertBefore(node, lineView.gutter || lineView.text);
      else
        wrap.appendChild(node);
      signalLater(widget, "redraw");
    }
  }

  function positionLineWidget(widget, node, lineView, dims) {
    if (widget.noHScroll) {
      (lineView.alignable || (lineView.alignable = [])).push(node);
      var width = dims.wrapperWidth;
      node.style.left = dims.fixedPos + "px";
      if (!widget.coverGutter) {
        width -= dims.gutterTotalWidth;
        node.style.paddingLeft = dims.gutterTotalWidth + "px";
      }
      node.style.width = width + "px";
    }
    if (widget.coverGutter) {
      node.style.zIndex = 5;
      node.style.position = "relative";
      if (!widget.noHScroll) node.style.marginLeft = -dims.gutterTotalWidth + "px";
    }
  }

  // POSITION OBJECT

  // A Pos instance represents a position within the text.
  var Pos = CodeMirror.Pos = function(line, ch) {
    if (!(this instanceof Pos)) return new Pos(line, ch);
    this.line = line; this.ch = ch;
  };

  // Compare two positions, return 0 if they are the same, a negative
  // number when a is less, and a positive number otherwise.
  var cmp = CodeMirror.cmpPos = function(a, b) { return a.line - b.line || a.ch - b.ch; };

  function copyPos(x) {return Pos(x.line, x.ch);}
  function maxPos(a, b) { return cmp(a, b) < 0 ? b : a; }
  function minPos(a, b) { return cmp(a, b) < 0 ? a : b; }

  // INPUT HANDLING

  function ensureFocus(cm) {
    if (!cm.state.focused) { cm.display.input.focus(); onFocus(cm); }
  }

  // This will be set to a {lineWise: bool, text: [string]} object, so
  // that, when pasting, we know what kind of selections the copied
  // text was made out of.
  var lastCopied = null;

  function applyTextInput(cm, inserted, deleted, sel, origin) {
    var doc = cm.doc;
    cm.display.shift = false;
    if (!sel) sel = doc.sel;

    var paste = cm.state.pasteIncoming || origin == "paste";
    var textLines = doc.splitLines(inserted), multiPaste = null
    // When pasing N lines into N selections, insert one line per selection
    if (paste && sel.ranges.length > 1) {
      if (lastCopied && lastCopied.text.join("\n") == inserted) {
        if (sel.ranges.length % lastCopied.text.length == 0) {
          multiPaste = [];
          for (var i = 0; i < lastCopied.text.length; i++)
            multiPaste.push(doc.splitLines(lastCopied.text[i]));
        }
      } else if (textLines.length == sel.ranges.length) {
        multiPaste = map(textLines, function(l) { return [l]; });
      }
    }

    // Normal behavior is to insert the new text into every selection
    for (var i = sel.ranges.length - 1; i >= 0; i--) {
      var range = sel.ranges[i];
      var from = range.from(), to = range.to();
      if (range.empty()) {
        if (deleted && deleted > 0) // Handle deletion
          from = Pos(from.line, from.ch - deleted);
        else if (cm.state.overwrite && !paste) // Handle overwrite
          to = Pos(to.line, Math.min(getLine(doc, to.line).text.length, to.ch + lst(textLines).length));
        else if (lastCopied && lastCopied.lineWise && lastCopied.text.join("\n") == inserted)
          from = to = Pos(from.line, 0)
      }
      var updateInput = cm.curOp.updateInput;
      var changeEvent = {from: from, to: to, text: multiPaste ? multiPaste[i % multiPaste.length] : textLines,
                         origin: origin || (paste ? "paste" : cm.state.cutIncoming ? "cut" : "+input")};
      makeChange(cm.doc, changeEvent);
      signalLater(cm, "inputRead", cm, changeEvent);
    }
    if (inserted && !paste)
      triggerElectric(cm, inserted);

    ensureCursorVisible(cm);
    cm.curOp.updateInput = updateInput;
    cm.curOp.typing = true;
    cm.state.pasteIncoming = cm.state.cutIncoming = false;
  }

  function handlePaste(e, cm) {
    var pasted = e.clipboardData && e.clipboardData.getData("text/plain");
    if (pasted) {
      e.preventDefault();
      if (!cm.isReadOnly() && !cm.options.disableInput)
        runInOp(cm, function() { applyTextInput(cm, pasted, 0, null, "paste"); });
      return true;
    }
  }

  function triggerElectric(cm, inserted) {
    // When an 'electric' character is inserted, immediately trigger a reindent
    if (!cm.options.electricChars || !cm.options.smartIndent) return;
    var sel = cm.doc.sel;

    for (var i = sel.ranges.length - 1; i >= 0; i--) {
      var range = sel.ranges[i];
      if (range.head.ch > 100 || (i && sel.ranges[i - 1].head.line == range.head.line)) continue;
      var mode = cm.getModeAt(range.head);
      var indented = false;
      if (mode.electricChars) {
        for (var j = 0; j < mode.electricChars.length; j++)
          if (inserted.indexOf(mode.electricChars.charAt(j)) > -1) {
            indented = indentLine(cm, range.head.line, "smart");
            break;
          }
      } else if (mode.electricInput) {
        if (mode.electricInput.test(getLine(cm.doc, range.head.line).text.slice(0, range.head.ch)))
          indented = indentLine(cm, range.head.line, "smart");
      }
      if (indented) signalLater(cm, "electricInput", cm, range.head.line);
    }
  }

  function copyableRanges(cm) {
    var text = [], ranges = [];
    for (var i = 0; i < cm.doc.sel.ranges.length; i++) {
      var line = cm.doc.sel.ranges[i].head.line;
      var lineRange = {anchor: Pos(line, 0), head: Pos(line + 1, 0)};
      ranges.push(lineRange);
      text.push(cm.getRange(lineRange.anchor, lineRange.head));
    }
    return {text: text, ranges: ranges};
  }

  function disableBrowserMagic(field) {
    field.setAttribute("autocorrect", "off");
    field.setAttribute("autocapitalize", "off");
    field.setAttribute("spellcheck", "false");
  }

  // TEXTAREA INPUT STYLE

  function TextareaInput(cm) {
    this.cm = cm;
    // See input.poll and input.reset
    this.prevInput = "";

    // Flag that indicates whether we expect input to appear real soon
    // now (after some event like 'keypress' or 'input') and are
    // polling intensively.
    this.pollingFast = false;
    // Self-resetting timeout for the poller
    this.polling = new Delayed();
    // Tracks when input.reset has punted to just putting a short
    // string into the textarea instead of the full selection.
    this.inaccurateSelection = false;
    // Used to work around IE issue with selection being forgotten when focus moves away from textarea
    this.hasSelection = false;
    this.composing = null;
  };

  function hiddenTextarea() {
    var te = elt("textarea", null, null, "position: absolute; padding: 0; width: 1px; height: 1em; outline: none");
    var div = elt("div", [te], null, "overflow: hidden; position: relative; width: 3px; height: 0px;");
    // The textarea is kept positioned near the cursor to prevent the
    // fact that it'll be scrolled into view on input from scrolling
    // our fake cursor out of view. On webkit, when wrap=off, paste is
    // very slow. So make the area wide instead.
    if (webkit) te.style.width = "1000px";
    else te.setAttribute("wrap", "off");
    // If border: 0; -- iOS fails to open keyboard (issue #1287)
    if (ios) te.style.border = "1px solid black";
    disableBrowserMagic(te);
    return div;
  }

  TextareaInput.prototype = copyObj({
    init: function(display) {
      var input = this, cm = this.cm;

      // Wraps and hides input textarea
      var div = this.wrapper = hiddenTextarea();
      // The semihidden textarea that is focused when the editor is
      // focused, and receives input.
      var te = this.textarea = div.firstChild;
      display.wrapper.insertBefore(div, display.wrapper.firstChild);

      // Needed to hide big blue blinking cursor on Mobile Safari (doesn't seem to work in iOS 8 anymore)
      if (ios) te.style.width = "0px";

      on(te, "input", function() {
        if (ie && ie_version >= 9 && input.hasSelection) input.hasSelection = null;
        input.poll();
      });

      on(te, "paste", function(e) {
        if (signalDOMEvent(cm, e) || handlePaste(e, cm)) return

        cm.state.pasteIncoming = true;
        input.fastPoll();
      });

      function prepareCopyCut(e) {
        if (signalDOMEvent(cm, e)) return
        if (cm.somethingSelected()) {
          lastCopied = {lineWise: false, text: cm.getSelections()};
          if (input.inaccurateSelection) {
            input.prevInput = "";
            input.inaccurateSelection = false;
            te.value = lastCopied.text.join("\n");
            selectInput(te);
          }
        } else if (!cm.options.lineWiseCopyCut) {
          return;
        } else {
          var ranges = copyableRanges(cm);
          lastCopied = {lineWise: true, text: ranges.text};
          if (e.type == "cut") {
            cm.setSelections(ranges.ranges, null, sel_dontScroll);
          } else {
            input.prevInput = "";
            te.value = ranges.text.join("\n");
            selectInput(te);
          }
        }
        if (e.type == "cut") cm.state.cutIncoming = true;
      }
      on(te, "cut", prepareCopyCut);
      on(te, "copy", prepareCopyCut);

      on(display.scroller, "paste", function(e) {
        if (eventInWidget(display, e) || signalDOMEvent(cm, e)) return;
        cm.state.pasteIncoming = true;
        input.focus();
      });

      // Prevent normal selection in the editor (we handle our own)
      on(display.lineSpace, "selectstart", function(e) {
        if (!eventInWidget(display, e)) e_preventDefault(e);
      });

      on(te, "compositionstart", function() {
        var start = cm.getCursor("from");
        if (input.composing) input.composing.range.clear()
        input.composing = {
          start: start,
          range: cm.markText(start, cm.getCursor("to"), {className: "CodeMirror-composing"})
        };
      });
      on(te, "compositionend", function() {
        if (input.composing) {
          input.poll();
          input.composing.range.clear();
          input.composing = null;
        }
      });
    },

    prepareSelection: function() {
      // Redraw the selection and/or cursor
      var cm = this.cm, display = cm.display, doc = cm.doc;
      var result = prepareSelection(cm);

      // Move the hidden textarea near the cursor to prevent scrolling artifacts
      if (cm.options.moveInputWithCursor) {
        var headPos = cursorCoords(cm, doc.sel.primary().head, "div");
        var wrapOff = display.wrapper.getBoundingClientRect(), lineOff = display.lineDiv.getBoundingClientRect();
        result.teTop = Math.max(0, Math.min(display.wrapper.clientHeight - 10,
                                            headPos.top + lineOff.top - wrapOff.top));
        result.teLeft = Math.max(0, Math.min(display.wrapper.clientWidth - 10,
                                             headPos.left + lineOff.left - wrapOff.left));
      }

      return result;
    },

    showSelection: function(drawn) {
      var cm = this.cm, display = cm.display;
      removeChildrenAndAdd(display.cursorDiv, drawn.cursors);
      removeChildrenAndAdd(display.selectionDiv, drawn.selection);
      if (drawn.teTop != null) {
        this.wrapper.style.top = drawn.teTop + "px";
        this.wrapper.style.left = drawn.teLeft + "px";
      }
    },

    // Reset the input to correspond to the selection (or to be empty,
    // when not typing and nothing is selected)
    reset: function(typing) {
      if (this.contextMenuPending) return;
      var minimal, selected, cm = this.cm, doc = cm.doc;
      if (cm.somethingSelected()) {
        this.prevInput = "";
        var range = doc.sel.primary();
        minimal = hasCopyEvent &&
          (range.to().line - range.from().line > 100 || (selected = cm.getSelection()).length > 1000);
        var content = minimal ? "-" : selected || cm.getSelection();
        this.textarea.value = content;
        if (cm.state.focused) selectInput(this.textarea);
        if (ie && ie_version >= 9) this.hasSelection = content;
      } else if (!typing) {
        this.prevInput = this.textarea.value = "";
        if (ie && ie_version >= 9) this.hasSelection = null;
      }
      this.inaccurateSelection = minimal;
    },

    getField: function() { return this.textarea; },

    supportsTouch: function() { return false; },

    focus: function() {
      if (this.cm.options.readOnly != "nocursor" && (!mobile || activeElt() != this.textarea)) {
        try { this.textarea.focus(); }
        catch (e) {} // IE8 will throw if the textarea is display: none or not in DOM
      }
    },

    blur: function() { this.textarea.blur(); },

    resetPosition: function() {
      this.wrapper.style.top = this.wrapper.style.left = 0;
    },

    receivedFocus: function() { this.slowPoll(); },

    // Poll for input changes, using the normal rate of polling. This
    // runs as long as the editor is focused.
    slowPoll: function() {
      var input = this;
      if (input.pollingFast) return;
      input.polling.set(this.cm.options.pollInterval, function() {
        input.poll();
        if (input.cm.state.focused) input.slowPoll();
      });
    },

    // When an event has just come in that is likely to add or change
    // something in the input textarea, we poll faster, to ensure that
    // the change appears on the screen quickly.
    fastPoll: function() {
      var missed = false, input = this;
      input.pollingFast = true;
      function p() {
        var changed = input.poll();
        if (!changed && !missed) {missed = true; input.polling.set(60, p);}
        else {input.pollingFast = false; input.slowPoll();}
      }
      input.polling.set(20, p);
    },

    // Read input from the textarea, and update the document to match.
    // When something is selected, it is present in the textarea, and
    // selected (unless it is huge, in which case a placeholder is
    // used). When nothing is selected, the cursor sits after previously
    // seen text (can be empty), which is stored in prevInput (we must
    // not reset the textarea when typing, because that breaks IME).
    poll: function() {
      var cm = this.cm, input = this.textarea, prevInput = this.prevInput;
      // Since this is called a *lot*, try to bail out as cheaply as
      // possible when it is clear that nothing happened. hasSelection
      // will be the case when there is a lot of text in the textarea,
      // in which case reading its value would be expensive.
      if (this.contextMenuPending || !cm.state.focused ||
          (hasSelection(input) && !prevInput && !this.composing) ||
          cm.isReadOnly() || cm.options.disableInput || cm.state.keySeq)
        return false;

      var text = input.value;
      // If nothing changed, bail.
      if (text == prevInput && !cm.somethingSelected()) return false;
      // Work around nonsensical selection resetting in IE9/10, and
      // inexplicable appearance of private area unicode characters on
      // some key combos in Mac (#2689).
      if (ie && ie_version >= 9 && this.hasSelection === text ||
          mac && /[\uf700-\uf7ff]/.test(text)) {
        cm.display.input.reset();
        return false;
      }

      if (cm.doc.sel == cm.display.selForContextMenu) {
        var first = text.charCodeAt(0);
        if (first == 0x200b && !prevInput) prevInput = "\u200b";
        if (first == 0x21da) { this.reset(); return this.cm.execCommand("undo"); }
      }
      // Find the part of the input that is actually new
      var same = 0, l = Math.min(prevInput.length, text.length);
      while (same < l && prevInput.charCodeAt(same) == text.charCodeAt(same)) ++same;

      var self = this;
      runInOp(cm, function() {
        applyTextInput(cm, text.slice(same), prevInput.length - same,
                       null, self.composing ? "*compose" : null);

        // Don't leave long text in the textarea, since it makes further polling slow
        if (text.length > 1000 || text.indexOf("\n") > -1) input.value = self.prevInput = "";
        else self.prevInput = text;

        if (self.composing) {
          self.composing.range.clear();
          self.composing.range = cm.markText(self.composing.start, cm.getCursor("to"),
                                             {className: "CodeMirror-composing"});
        }
      });
      return true;
    },

    ensurePolled: function() {
      if (this.pollingFast && this.poll()) this.pollingFast = false;
    },

    onKeyPress: function() {
      if (ie && ie_version >= 9) this.hasSelection = null;
      this.fastPoll();
    },

    onContextMenu: function(e) {
      var input = this, cm = input.cm, display = cm.display, te = input.textarea;
      var pos = posFromMouse(cm, e), scrollPos = display.scroller.scrollTop;
      if (!pos || presto) return; // Opera is difficult.

      // Reset the current text selection only if the click is done outside of the selection
      // and 'resetSelectionOnContextMenu' option is true.
      var reset = cm.options.resetSelectionOnContextMenu;
      if (reset && cm.doc.sel.contains(pos) == -1)
        operation(cm, setSelection)(cm.doc, simpleSelection(pos), sel_dontScroll);

      var oldCSS = te.style.cssText, oldWrapperCSS = input.wrapper.style.cssText;
      input.wrapper.style.cssText = "position: absolute"
      var wrapperBox = input.wrapper.getBoundingClientRect()
      te.style.cssText = "position: absolute; width: 30px; height: 30px; top: " + (e.clientY - wrapperBox.top - 5) +
        "px; left: " + (e.clientX - wrapperBox.left - 5) + "px; z-index: 1000; background: " +
        (ie ? "rgba(255, 255, 255, .05)" : "transparent") +
        "; outline: none; border-width: 0; outline: none; overflow: hidden; opacity: .05; filter: alpha(opacity=5);";
      if (webkit) var oldScrollY = window.scrollY; // Work around Chrome issue (#2712)
      display.input.focus();
      if (webkit) window.scrollTo(null, oldScrollY);
      display.input.reset();
      // Adds "Select all" to context menu in FF
      if (!cm.somethingSelected()) te.value = input.prevInput = " ";
      input.contextMenuPending = true;
      display.selForContextMenu = cm.doc.sel;
      clearTimeout(display.detectingSelectAll);

      // Select-all will be greyed out if there's nothing to select, so
      // this adds a zero-width space so that we can later check whether
      // it got selected.
      function prepareSelectAllHack() {
        if (te.selectionStart != null) {
          var selected = cm.somethingSelected();
          var extval = "\u200b" + (selected ? te.value : "");
          te.value = "\u21da"; // Used to catch context-menu undo
          te.value = extval;
          input.prevInput = selected ? "" : "\u200b";
          te.selectionStart = 1; te.selectionEnd = extval.length;
          // Re-set this, in case some other handler touched the
          // selection in the meantime.
          display.selForContextMenu = cm.doc.sel;
        }
      }
      function rehide() {
        input.contextMenuPending = false;
        input.wrapper.style.cssText = oldWrapperCSS
        te.style.cssText = oldCSS;
        if (ie && ie_version < 9) display.scrollbars.setScrollTop(display.scroller.scrollTop = scrollPos);

        // Try to detect the user choosing select-all
        if (te.selectionStart != null) {
          if (!ie || (ie && ie_version < 9)) prepareSelectAllHack();
          var i = 0, poll = function() {
            if (display.selForContextMenu == cm.doc.sel && te.selectionStart == 0 &&
                te.selectionEnd > 0 && input.prevInput == "\u200b")
              operation(cm, commands.selectAll)(cm);
            else if (i++ < 10) display.detectingSelectAll = setTimeout(poll, 500);
            else display.input.reset();
          };
          display.detectingSelectAll = setTimeout(poll, 200);
        }
      }

      if (ie && ie_version >= 9) prepareSelectAllHack();
      if (captureRightClick) {
        e_stop(e);
        var mouseup = function() {
          off(window, "mouseup", mouseup);
          setTimeout(rehide, 20);
        };
        on(window, "mouseup", mouseup);
      } else {
        setTimeout(rehide, 50);
      }
    },

    readOnlyChanged: function(val) {
      if (!val) this.reset();
    },

    setUneditable: nothing,

    needsContentAttribute: false
  }, TextareaInput.prototype);

  // CONTENTEDITABLE INPUT STYLE

  function ContentEditableInput(cm) {
    this.cm = cm;
    this.lastAnchorNode = this.lastAnchorOffset = this.lastFocusNode = this.lastFocusOffset = null;
    this.polling = new Delayed();
    this.gracePeriod = false;
  }

  ContentEditableInput.prototype = copyObj({
    init: function(display) {
      var input = this, cm = input.cm;
      var div = input.div = display.lineDiv;
      disableBrowserMagic(div);

      on(div, "paste", function(e) {
        if (!signalDOMEvent(cm, e)) handlePaste(e, cm);
      })

      on(div, "compositionstart", function(e) {
        var data = e.data;
        input.composing = {sel: cm.doc.sel, data: data, startData: data};
        if (!data) return;
        var prim = cm.doc.sel.primary();
        var line = cm.getLine(prim.head.line);
        var found = line.indexOf(data, Math.max(0, prim.head.ch - data.length));
        if (found > -1 && found <= prim.head.ch)
          input.composing.sel = simpleSelection(Pos(prim.head.line, found),
                                                Pos(prim.head.line, found + data.length));
      });
      on(div, "compositionupdate", function(e) {
        input.composing.data = e.data;
      });
      on(div, "compositionend", function(e) {
        var ours = input.composing;
        if (!ours) return;
        if (e.data != ours.startData && !/\u200b/.test(e.data))
          ours.data = e.data;
        // Need a small delay to prevent other code (input event,
        // selection polling) from doing damage when fired right after
        // compositionend.
        setTimeout(function() {
          if (!ours.handled)
            input.applyComposition(ours);
          if (input.composing == ours)
            input.composing = null;
        }, 50);
      });

      on(div, "touchstart", function() {
        input.forceCompositionEnd();
      });

      on(div, "input", function() {
        if (input.composing) return;
        if (cm.isReadOnly() || !input.pollContent())
          runInOp(input.cm, function() {regChange(cm);});
      });

      function onCopyCut(e) {
        if (signalDOMEvent(cm, e)) return
        if (cm.somethingSelected()) {
          lastCopied = {lineWise: false, text: cm.getSelections()};
          if (e.type == "cut") cm.replaceSelection("", null, "cut");
        } else if (!cm.options.lineWiseCopyCut) {
          return;
        } else {
          var ranges = copyableRanges(cm);
          lastCopied = {lineWise: true, text: ranges.text};
          if (e.type == "cut") {
            cm.operation(function() {
              cm.setSelections(ranges.ranges, 0, sel_dontScroll);
              cm.replaceSelection("", null, "cut");
            });
          }
        }
        // iOS exposes the clipboard API, but seems to discard content inserted into it
        if (e.clipboardData && !ios) {
          e.preventDefault();
          e.clipboardData.clearData();
          e.clipboardData.setData("text/plain", lastCopied.text.join("\n"));
        } else {
          // Old-fashioned briefly-focus-a-textarea hack
          var kludge = hiddenTextarea(), te = kludge.firstChild;
          cm.display.lineSpace.insertBefore(kludge, cm.display.lineSpace.firstChild);
          te.value = lastCopied.text.join("\n");
          var hadFocus = document.activeElement;
          selectInput(te);
          setTimeout(function() {
            cm.display.lineSpace.removeChild(kludge);
            hadFocus.focus();
          }, 50);
        }
      }
      on(div, "copy", onCopyCut);
      on(div, "cut", onCopyCut);
    },

    prepareSelection: function() {
      var result = prepareSelection(this.cm, false);
      result.focus = this.cm.state.focused;
      return result;
    },

    showSelection: function(info, takeFocus) {
      if (!info || !this.cm.display.view.length) return;
      if (info.focus || takeFocus) this.showPrimarySelection();
      this.showMultipleSelections(info);
    },

    showPrimarySelection: function() {
      var sel = window.getSelection(), prim = this.cm.doc.sel.primary();
      var curAnchor = domToPos(this.cm, sel.anchorNode, sel.anchorOffset);
      var curFocus = domToPos(this.cm, sel.focusNode, sel.focusOffset);
      if (curAnchor && !curAnchor.bad && curFocus && !curFocus.bad &&
          cmp(minPos(curAnchor, curFocus), prim.from()) == 0 &&
          cmp(maxPos(curAnchor, curFocus), prim.to()) == 0)
        return;

      var start = posToDOM(this.cm, prim.from());
      var end = posToDOM(this.cm, prim.to());
      if (!start && !end) return;

      var view = this.cm.display.view;
      var old = sel.rangeCount && sel.getRangeAt(0);
      if (!start) {
        start = {node: view[0].measure.map[2], offset: 0};
      } else if (!end) { // FIXME dangerously hacky
        var measure = view[view.length - 1].measure;
        var map = measure.maps ? measure.maps[measure.maps.length - 1] : measure.map;
        end = {node: map[map.length - 1], offset: map[map.length - 2] - map[map.length - 3]};
      }

      try { var rng = range(start.node, start.offset, end.offset, end.node); }
      catch(e) {} // Our model of the DOM might be outdated, in which case the range we try to set can be impossible
      if (rng) {
        if (!gecko && this.cm.state.focused) {
          sel.collapse(start.node, start.offset);
          if (!rng.collapsed) sel.addRange(rng);
        } else {
          sel.removeAllRanges();
          sel.addRange(rng);
        }
        if (old && sel.anchorNode == null) sel.addRange(old);
        else if (gecko) this.startGracePeriod();
      }
      this.rememberSelection();
    },

    startGracePeriod: function() {
      var input = this;
      clearTimeout(this.gracePeriod);
      this.gracePeriod = setTimeout(function() {
        input.gracePeriod = false;
        if (input.selectionChanged())
          input.cm.operation(function() { input.cm.curOp.selectionChanged = true; });
      }, 20);
    },

    showMultipleSelections: function(info) {
      removeChildrenAndAdd(this.cm.display.cursorDiv, info.cursors);
      removeChildrenAndAdd(this.cm.display.selectionDiv, info.selection);
    },

    rememberSelection: function() {
      var sel = window.getSelection();
      this.lastAnchorNode = sel.anchorNode; this.lastAnchorOffset = sel.anchorOffset;
      this.lastFocusNode = sel.focusNode; this.lastFocusOffset = sel.focusOffset;
    },

    selectionInEditor: function() {
      var sel = window.getSelection();
      if (!sel.rangeCount) return false;
      var node = sel.getRangeAt(0).commonAncestorContainer;
      return contains(this.div, node);
    },

    focus: function() {
      if (this.cm.options.readOnly != "nocursor") this.div.focus();
    },
    blur: function() { this.div.blur(); },
    getField: function() { return this.div; },

    supportsTouch: function() { return true; },

    receivedFocus: function() {
      var input = this;
      if (this.selectionInEditor())
        this.pollSelection();
      else
        runInOp(this.cm, function() { input.cm.curOp.selectionChanged = true; });

      function poll() {
        if (input.cm.state.focused) {
          input.pollSelection();
          input.polling.set(input.cm.options.pollInterval, poll);
        }
      }
      this.polling.set(this.cm.options.pollInterval, poll);
    },

    selectionChanged: function() {
      var sel = window.getSelection();
      return sel.anchorNode != this.lastAnchorNode || sel.anchorOffset != this.lastAnchorOffset ||
        sel.focusNode != this.lastFocusNode || sel.focusOffset != this.lastFocusOffset;
    },

    pollSelection: function() {
      if (!this.composing && !this.gracePeriod && this.selectionChanged()) {
        var sel = window.getSelection(), cm = this.cm;
        this.rememberSelection();
        var anchor = domToPos(cm, sel.anchorNode, sel.anchorOffset);
        var head = domToPos(cm, sel.focusNode, sel.focusOffset);
        if (anchor && head) runInOp(cm, function() {
          setSelection(cm.doc, simpleSelection(anchor, head), sel_dontScroll);
          if (anchor.bad || head.bad) cm.curOp.selectionChanged = true;
        });
      }
    },

    pollContent: function() {
      var cm = this.cm, display = cm.display, sel = cm.doc.sel.primary();
      var from = sel.from(), to = sel.to();
      if (from.line < display.viewFrom || to.line > display.viewTo - 1) return false;

      var fromIndex;
      if (from.line == display.viewFrom || (fromIndex = findViewIndex(cm, from.line)) == 0) {
        var fromLine = lineNo(display.view[0].line);
        var fromNode = display.view[0].node;
      } else {
        var fromLine = lineNo(display.view[fromIndex].line);
        var fromNode = display.view[fromIndex - 1].node.nextSibling;
      }
      var toIndex = findViewIndex(cm, to.line);
      if (toIndex == display.view.length - 1) {
        var toLine = display.viewTo - 1;
        var toNode = display.lineDiv.lastChild;
      } else {
        var toLine = lineNo(display.view[toIndex + 1].line) - 1;
        var toNode = display.view[toIndex + 1].node.previousSibling;
      }

      var newText = cm.doc.splitLines(domTextBetween(cm, fromNode, toNode, fromLine, toLine));
      var oldText = getBetween(cm.doc, Pos(fromLine, 0), Pos(toLine, getLine(cm.doc, toLine).text.length));
      while (newText.length > 1 && oldText.length > 1) {
        if (lst(newText) == lst(oldText)) { newText.pop(); oldText.pop(); toLine--; }
        else if (newText[0] == oldText[0]) { newText.shift(); oldText.shift(); fromLine++; }
        else break;
      }

      var cutFront = 0, cutEnd = 0;
      var newTop = newText[0], oldTop = oldText[0], maxCutFront = Math.min(newTop.length, oldTop.length);
      while (cutFront < maxCutFront && newTop.charCodeAt(cutFront) == oldTop.charCodeAt(cutFront))
        ++cutFront;
      var newBot = lst(newText), oldBot = lst(oldText);
      var maxCutEnd = Math.min(newBot.length - (newText.length == 1 ? cutFront : 0),
                               oldBot.length - (oldText.length == 1 ? cutFront : 0));
      while (cutEnd < maxCutEnd &&
             newBot.charCodeAt(newBot.length - cutEnd - 1) == oldBot.charCodeAt(oldBot.length - cutEnd - 1))
        ++cutEnd;

      newText[newText.length - 1] = newBot.slice(0, newBot.length - cutEnd);
      newText[0] = newText[0].slice(cutFront);

      var chFrom = Pos(fromLine, cutFront);
      var chTo = Pos(toLine, oldText.length ? lst(oldText).length - cutEnd : 0);
      if (newText.length > 1 || newText[0] || cmp(chFrom, chTo)) {
        replaceRange(cm.doc, newText, chFrom, chTo, "+input");
        return true;
      }
    },

    ensurePolled: function() {
      this.forceCompositionEnd();
    },
    reset: function() {
      this.forceCompositionEnd();
    },
    forceCompositionEnd: function() {
      if (!this.composing || this.composing.handled) return;
      this.applyComposition(this.composing);
      this.composing.handled = true;
      this.div.blur();
      this.div.focus();
    },
    applyComposition: function(composing) {
      if (this.cm.isReadOnly())
        operation(this.cm, regChange)(this.cm)
      else if (composing.data && composing.data != composing.startData)
        operation(this.cm, applyTextInput)(this.cm, composing.data, 0, composing.sel);
    },

    setUneditable: function(node) {
      node.contentEditable = "false"
    },

    onKeyPress: function(e) {
      e.preventDefault();
      if (!this.cm.isReadOnly())
        operation(this.cm, applyTextInput)(this.cm, String.fromCharCode(e.charCode == null ? e.keyCode : e.charCode), 0);
    },

    readOnlyChanged: function(val) {
      this.div.contentEditable = String(val != "nocursor")
    },

    onContextMenu: nothing,
    resetPosition: nothing,

    needsContentAttribute: true
  }, ContentEditableInput.prototype);

  function posToDOM(cm, pos) {
    var view = findViewForLine(cm, pos.line);
    if (!view || view.hidden) return null;
    var line = getLine(cm.doc, pos.line);
    var info = mapFromLineView(view, line, pos.line);

    var order = getOrder(line), side = "left";
    if (order) {
      var partPos = getBidiPartAt(order, pos.ch);
      side = partPos % 2 ? "right" : "left";
    }
    var result = nodeAndOffsetInLineMap(info.map, pos.ch, side);
    result.offset = result.collapse == "right" ? result.end : result.start;
    return result;
  }

  function badPos(pos, bad) { if (bad) pos.bad = true; return pos; }

  function domToPos(cm, node, offset) {
    var lineNode;
    if (node == cm.display.lineDiv) {
      lineNode = cm.display.lineDiv.childNodes[offset];
      if (!lineNode) return badPos(cm.clipPos(Pos(cm.display.viewTo - 1)), true);
      node = null; offset = 0;
    } else {
      for (lineNode = node;; lineNode = lineNode.parentNode) {
        if (!lineNode || lineNode == cm.display.lineDiv) return null;
        if (lineNode.parentNode && lineNode.parentNode == cm.display.lineDiv) break;
      }
    }
    for (var i = 0; i < cm.display.view.length; i++) {
      var lineView = cm.display.view[i];
      if (lineView.node == lineNode)
        return locateNodeInLineView(lineView, node, offset);
    }
  }

  function locateNodeInLineView(lineView, node, offset) {
    var wrapper = lineView.text.firstChild, bad = false;
    if (!node || !contains(wrapper, node)) return badPos(Pos(lineNo(lineView.line), 0), true);
    if (node == wrapper) {
      bad = true;
      node = wrapper.childNodes[offset];
      offset = 0;
      if (!node) {
        var line = lineView.rest ? lst(lineView.rest) : lineView.line;
        return badPos(Pos(lineNo(line), line.text.length), bad);
      }
    }

    var textNode = node.nodeType == 3 ? node : null, topNode = node;
    if (!textNode && node.childNodes.length == 1 && node.firstChild.nodeType == 3) {
      textNode = node.firstChild;
      if (offset) offset = textNode.nodeValue.length;
    }
    while (topNode.parentNode != wrapper) topNode = topNode.parentNode;
    var measure = lineView.measure, maps = measure.maps;

    function find(textNode, topNode, offset) {
      for (var i = -1; i < (maps ? maps.length : 0); i++) {
        var map = i < 0 ? measure.map : maps[i];
        for (var j = 0; j < map.length; j += 3) {
          var curNode = map[j + 2];
          if (curNode == textNode || curNode == topNode) {
            var line = lineNo(i < 0 ? lineView.line : lineView.rest[i]);
            var ch = map[j] + offset;
            if (offset < 0 || curNode != textNode) ch = map[j + (offset ? 1 : 0)];
            return Pos(line, ch);
          }
        }
      }
    }
    var found = find(textNode, topNode, offset);
    if (found) return badPos(found, bad);

    // FIXME this is all really shaky. might handle the few cases it needs to handle, but likely to cause problems
    for (var after = topNode.nextSibling, dist = textNode ? textNode.nodeValue.length - offset : 0; after; after = after.nextSibling) {
      found = find(after, after.firstChild, 0);
      if (found)
        return badPos(Pos(found.line, found.ch - dist), bad);
      else
        dist += after.textContent.length;
    }
    for (var before = topNode.previousSibling, dist = offset; before; before = before.previousSibling) {
      found = find(before, before.firstChild, -1);
      if (found)
        return badPos(Pos(found.line, found.ch + dist), bad);
      else
        dist += after.textContent.length;
    }
  }

  function domTextBetween(cm, from, to, fromLine, toLine) {
    var text = "", closing = false, lineSep = cm.doc.lineSeparator();
    function recognizeMarker(id) { return function(marker) { return marker.id == id; }; }
    function walk(node) {
      if (node.nodeType == 1) {
        var cmText = node.getAttribute("cm-text");
        if (cmText != null) {
          if (cmText == "") cmText = node.textContent.replace(/\u200b/g, "");
          text += cmText;
          return;
        }
        var markerID = node.getAttribute("cm-marker"), range;
        if (markerID) {
          var found = cm.findMarks(Pos(fromLine, 0), Pos(toLine + 1, 0), recognizeMarker(+markerID));
          if (found.length && (range = found[0].find()))
            text += getBetween(cm.doc, range.from, range.to).join(lineSep);
          return;
        }
        if (node.getAttribute("contenteditable") == "false") return;
        for (var i = 0; i < node.childNodes.length; i++)
          walk(node.childNodes[i]);
        if (/^(pre|div|p)$/i.test(node.nodeName))
          closing = true;
      } else if (node.nodeType == 3) {
        var val = node.nodeValue;
        if (!val) return;
        if (closing) {
          text += lineSep;
          closing = false;
        }
        text += val;
      }
    }
    for (;;) {
      walk(from);
      if (from == to) break;
      from = from.nextSibling;
    }
    return text;
  }

  CodeMirror.inputStyles = {"textarea": TextareaInput, "contenteditable": ContentEditableInput};

  // SELECTION / CURSOR

  // Selection objects are immutable. A new one is created every time
  // the selection changes. A selection is one or more non-overlapping
  // (and non-touching) ranges, sorted, and an integer that indicates
  // which one is the primary selection (the one that's scrolled into
  // view, that getCursor returns, etc).
  function Selection(ranges, primIndex) {
    this.ranges = ranges;
    this.primIndex = primIndex;
  }

  Selection.prototype = {
    primary: function() { return this.ranges[this.primIndex]; },
    equals: function(other) {
      if (other == this) return true;
      if (other.primIndex != this.primIndex || other.ranges.length != this.ranges.length) return false;
      for (var i = 0; i < this.ranges.length; i++) {
        var here = this.ranges[i], there = other.ranges[i];
        if (cmp(here.anchor, there.anchor) != 0 || cmp(here.head, there.head) != 0) return false;
      }
      return true;
    },
    deepCopy: function() {
      for (var out = [], i = 0; i < this.ranges.length; i++)
        out[i] = new Range(copyPos(this.ranges[i].anchor), copyPos(this.ranges[i].head));
      return new Selection(out, this.primIndex);
    },
    somethingSelected: function() {
      for (var i = 0; i < this.ranges.length; i++)
        if (!this.ranges[i].empty()) return true;
      return false;
    },
    contains: function(pos, end) {
      if (!end) end = pos;
      for (var i = 0; i < this.ranges.length; i++) {
        var range = this.ranges[i];
        if (cmp(end, range.from()) >= 0 && cmp(pos, range.to()) <= 0)
          return i;
      }
      return -1;
    }
  };

  function Range(anchor, head) {
    this.anchor = anchor; this.head = head;
  }

  Range.prototype = {
    from: function() { return minPos(this.anchor, this.head); },
    to: function() { return maxPos(this.anchor, this.head); },
    empty: function() {
      return this.head.line == this.anchor.line && this.head.ch == this.anchor.ch;
    }
  };

  // Take an unsorted, potentially overlapping set of ranges, and
  // build a selection out of it. 'Consumes' ranges array (modifying
  // it).
  function normalizeSelection(ranges, primIndex) {
    var prim = ranges[primIndex];
    ranges.sort(function(a, b) { return cmp(a.from(), b.from()); });
    primIndex = indexOf(ranges, prim);
    for (var i = 1; i < ranges.length; i++) {
      var cur = ranges[i], prev = ranges[i - 1];
      if (cmp(prev.to(), cur.from()) >= 0) {
        var from = minPos(prev.from(), cur.from()), to = maxPos(prev.to(), cur.to());
        var inv = prev.empty() ? cur.from() == cur.head : prev.from() == prev.head;
        if (i <= primIndex) --primIndex;
        ranges.splice(--i, 2, new Range(inv ? to : from, inv ? from : to));
      }
    }
    return new Selection(ranges, primIndex);
  }

  function simpleSelection(anchor, head) {
    return new Selection([new Range(anchor, head || anchor)], 0);
  }

  // Most of the external API clips given positions to make sure they
  // actually exist within the document.
  function clipLine(doc, n) {return Math.max(doc.first, Math.min(n, doc.first + doc.size - 1));}
  function clipPos(doc, pos) {
    if (pos.line < doc.first) return Pos(doc.first, 0);
    var last = doc.first + doc.size - 1;
    if (pos.line > last) return Pos(last, getLine(doc, last).text.length);
    return clipToLen(pos, getLine(doc, pos.line).text.length);
  }
  function clipToLen(pos, linelen) {
    var ch = pos.ch;
    if (ch == null || ch > linelen) return Pos(pos.line, linelen);
    else if (ch < 0) return Pos(pos.line, 0);
    else return pos;
  }
  function isLine(doc, l) {return l >= doc.first && l < doc.first + doc.size;}
  function clipPosArray(doc, array) {
    for (var out = [], i = 0; i < array.length; i++) out[i] = clipPos(doc, array[i]);
    return out;
  }

  // SELECTION UPDATES

  // The 'scroll' parameter given to many of these indicated whether
  // the new cursor position should be scrolled into view after
  // modifying the selection.

  // If shift is held or the extend flag is set, extends a range to
  // include a given position (and optionally a second position).
  // Otherwise, simply returns the range between the given positions.
  // Used for cursor motion and such.
  function extendRange(doc, range, head, other) {
    if (doc.cm && doc.cm.display.shift || doc.extend) {
      var anchor = range.anchor;
      if (other) {
        var posBefore = cmp(head, anchor) < 0;
        if (posBefore != (cmp(other, anchor) < 0)) {
          anchor = head;
          head = other;
        } else if (posBefore != (cmp(head, other) < 0)) {
          head = other;
        }
      }
      return new Range(anchor, head);
    } else {
      return new Range(other || head, head);
    }
  }

  // Extend the primary selection range, discard the rest.
  function extendSelection(doc, head, other, options) {
    setSelection(doc, new Selection([extendRange(doc, doc.sel.primary(), head, other)], 0), options);
  }

  // Extend all selections (pos is an array of selections with length
  // equal the number of selections)
  function extendSelections(doc, heads, options) {
    for (var out = [], i = 0; i < doc.sel.ranges.length; i++)
      out[i] = extendRange(doc, doc.sel.ranges[i], heads[i], null);
    var newSel = normalizeSelection(out, doc.sel.primIndex);
    setSelection(doc, newSel, options);
  }

  // Updates a single range in the selection.
  function replaceOneSelection(doc, i, range, options) {
    var ranges = doc.sel.ranges.slice(0);
    ranges[i] = range;
    setSelection(doc, normalizeSelection(ranges, doc.sel.primIndex), options);
  }

  // Reset the selection to a single range.
  function setSimpleSelection(doc, anchor, head, options) {
    setSelection(doc, simpleSelection(anchor, head), options);
  }

  // Give beforeSelectionChange handlers a change to influence a
  // selection update.
  function filterSelectionChange(doc, sel, options) {
    var obj = {
      ranges: sel.ranges,
      update: function(ranges) {
        this.ranges = [];
        for (var i = 0; i < ranges.length; i++)
          this.ranges[i] = new Range(clipPos(doc, ranges[i].anchor),
                                     clipPos(doc, ranges[i].head));
      },
      origin: options && options.origin
    };
    signal(doc, "beforeSelectionChange", doc, obj);
    if (doc.cm) signal(doc.cm, "beforeSelectionChange", doc.cm, obj);
    if (obj.ranges != sel.ranges) return normalizeSelection(obj.ranges, obj.ranges.length - 1);
    else return sel;
  }

  function setSelectionReplaceHistory(doc, sel, options) {
    var done = doc.history.done, last = lst(done);
    if (last && last.ranges) {
      done[done.length - 1] = sel;
      setSelectionNoUndo(doc, sel, options);
    } else {
      setSelection(doc, sel, options);
    }
  }

  // Set a new selection.
  function setSelection(doc, sel, options) {
    setSelectionNoUndo(doc, sel, options);
    addSelectionToHistory(doc, doc.sel, doc.cm ? doc.cm.curOp.id : NaN, options);
  }

  function setSelectionNoUndo(doc, sel, options) {
    if (hasHandler(doc, "beforeSelectionChange") || doc.cm && hasHandler(doc.cm, "beforeSelectionChange"))
      sel = filterSelectionChange(doc, sel, options);

    var bias = options && options.bias ||
      (cmp(sel.primary().head, doc.sel.primary().head) < 0 ? -1 : 1);
    setSelectionInner(doc, skipAtomicInSelection(doc, sel, bias, true));

    if (!(options && options.scroll === false) && doc.cm)
      ensureCursorVisible(doc.cm);
  }

  function setSelectionInner(doc, sel) {
    if (sel.equals(doc.sel)) return;

    doc.sel = sel;

    if (doc.cm) {
      doc.cm.curOp.updateInput = doc.cm.curOp.selectionChanged = true;
      signalCursorActivity(doc.cm);
    }
    signalLater(doc, "cursorActivity", doc);
  }

  // Verify that the selection does not partially select any atomic
  // marked ranges.
  function reCheckSelection(doc) {
    setSelectionInner(doc, skipAtomicInSelection(doc, doc.sel, null, false), sel_dontScroll);
  }

  // Return a selection that does not partially select any atomic
  // ranges.
  function skipAtomicInSelection(doc, sel, bias, mayClear) {
    var out;
    for (var i = 0; i < sel.ranges.length; i++) {
      var range = sel.ranges[i];
      var old = sel.ranges.length == doc.sel.ranges.length && doc.sel.ranges[i];
      var newAnchor = skipAtomic(doc, range.anchor, old && old.anchor, bias, mayClear);
      var newHead = skipAtomic(doc, range.head, old && old.head, bias, mayClear);
      if (out || newAnchor != range.anchor || newHead != range.head) {
        if (!out) out = sel.ranges.slice(0, i);
        out[i] = new Range(newAnchor, newHead);
      }
    }
    return out ? normalizeSelection(out, sel.primIndex) : sel;
  }

  function skipAtomicInner(doc, pos, oldPos, dir, mayClear) {
    var line = getLine(doc, pos.line);
    if (line.markedSpans) for (var i = 0; i < line.markedSpans.length; ++i) {
      var sp = line.markedSpans[i], m = sp.marker;
      if ((sp.from == null || (m.inclusiveLeft ? sp.from <= pos.ch : sp.from < pos.ch)) &&
          (sp.to == null || (m.inclusiveRight ? sp.to >= pos.ch : sp.to > pos.ch))) {
        if (mayClear) {
          signal(m, "beforeCursorEnter");
          if (m.explicitlyCleared) {
            if (!line.markedSpans) break;
            else {--i; continue;}
          }
        }
        if (!m.atomic) continue;

        if (oldPos) {
          var near = m.find(dir < 0 ? 1 : -1), diff;
          if (dir < 0 ? m.inclusiveRight : m.inclusiveLeft)
            near = movePos(doc, near, -dir, near && near.line == pos.line ? line : null);
          if (near && near.line == pos.line && (diff = cmp(near, oldPos)) && (dir < 0 ? diff < 0 : diff > 0))
            return skipAtomicInner(doc, near, pos, dir, mayClear);
        }

        var far = m.find(dir < 0 ? -1 : 1);
        if (dir < 0 ? m.inclusiveLeft : m.inclusiveRight)
          far = movePos(doc, far, dir, far.line == pos.line ? line : null);
        return far ? skipAtomicInner(doc, far, pos, dir, mayClear) : null;
      }
    }
    return pos;
  }

  // Ensure a given position is not inside an atomic range.
  function skipAtomic(doc, pos, oldPos, bias, mayClear) {
    var dir = bias || 1;
    var found = skipAtomicInner(doc, pos, oldPos, dir, mayClear) ||
        (!mayClear && skipAtomicInner(doc, pos, oldPos, dir, true)) ||
        skipAtomicInner(doc, pos, oldPos, -dir, mayClear) ||
        (!mayClear && skipAtomicInner(doc, pos, oldPos, -dir, true));
    if (!found) {
      doc.cantEdit = true;
      return Pos(doc.first, 0);
    }
    return found;
  }

  function movePos(doc, pos, dir, line) {
    if (dir < 0 && pos.ch == 0) {
      if (pos.line > doc.first) return clipPos(doc, Pos(pos.line - 1));
      else return null;
    } else if (dir > 0 && pos.ch == (line || getLine(doc, pos.line)).text.length) {
      if (pos.line < doc.first + doc.size - 1) return Pos(pos.line + 1, 0);
      else return null;
    } else {
      return new Pos(pos.line, pos.ch + dir);
    }
  }

  // SELECTION DRAWING

  function updateSelection(cm) {
    cm.display.input.showSelection(cm.display.input.prepareSelection());
  }

  function prepareSelection(cm, primary) {
    var doc = cm.doc, result = {};
    var curFragment = result.cursors = document.createDocumentFragment();
    var selFragment = result.selection = document.createDocumentFragment();

    for (var i = 0; i < doc.sel.ranges.length; i++) {
      if (primary === false && i == doc.sel.primIndex) continue;
      var range = doc.sel.ranges[i];
      if (range.from().line >= cm.display.viewTo || range.to().line < cm.display.viewFrom) continue;
      var collapsed = range.empty();
      if (collapsed || cm.options.showCursorWhenSelecting)
        drawSelectionCursor(cm, range.head, curFragment);
      if (!collapsed)
        drawSelectionRange(cm, range, selFragment);
    }
    return result;
  }

  // Draws a cursor for the given range
  function drawSelectionCursor(cm, head, output) {
    var pos = cursorCoords(cm, head, "div", null, null, !cm.options.singleCursorHeightPerLine);

    var cursor = output.appendChild(elt("div", "\u00a0", "CodeMirror-cursor"));
    cursor.style.left = pos.left + "px";
    cursor.style.top = pos.top + "px";
    cursor.style.height = Math.max(0, pos.bottom - pos.top) * cm.options.cursorHeight + "px";

    if (pos.other) {
      // Secondary cursor, shown when on a 'jump' in bi-directional text
      var otherCursor = output.appendChild(elt("div", "\u00a0", "CodeMirror-cursor CodeMirror-secondarycursor"));
      otherCursor.style.display = "";
      otherCursor.style.left = pos.other.left + "px";
      otherCursor.style.top = pos.other.top + "px";
      otherCursor.style.height = (pos.other.bottom - pos.other.top) * .85 + "px";
    }
  }

  // Draws the given range as a highlighted selection
  function drawSelectionRange(cm, range, output) {
    var display = cm.display, doc = cm.doc;
    var fragment = document.createDocumentFragment();
    var padding = paddingH(cm.display), leftSide = padding.left;
    var rightSide = Math.max(display.sizerWidth, displayWidth(cm) - display.sizer.offsetLeft) - padding.right;

    function add(left, top, width, bottom) {
      if (top < 0) top = 0;
      top = Math.round(top);
      bottom = Math.round(bottom);
      fragment.appendChild(elt("div", null, "CodeMirror-selected", "position: absolute; left: " + left +
                               "px; top: " + top + "px; width: " + (width == null ? rightSide - left : width) +
                               "px; height: " + (bottom - top) + "px"));
    }

    function drawForLine(line, fromArg, toArg) {
      var lineObj = getLine(doc, line);
      var lineLen = lineObj.text.length;
      var start, end;
      function coords(ch, bias) {
        return charCoords(cm, Pos(line, ch), "div", lineObj, bias);
      }

      iterateBidiSections(getOrder(lineObj), fromArg || 0, toArg == null ? lineLen : toArg, function(from, to, dir) {
        var leftPos = coords(from, "left"), rightPos, left, right;
        if (from == to) {
          rightPos = leftPos;
          left = right = leftPos.left;
        } else {
          rightPos = coords(to - 1, "right");
          if (dir == "rtl") { var tmp = leftPos; leftPos = rightPos; rightPos = tmp; }
          left = leftPos.left;
          right = rightPos.right;
        }
        if (fromArg == null && from == 0) left = leftSide;
        if (rightPos.top - leftPos.top > 3) { // Different lines, draw top part
          add(left, leftPos.top, null, leftPos.bottom);
          left = leftSide;
          if (leftPos.bottom < rightPos.top) add(left, leftPos.bottom, null, rightPos.top);
        }
        if (toArg == null && to == lineLen) right = rightSide;
        if (!start || leftPos.top < start.top || leftPos.top == start.top && leftPos.left < start.left)
          start = leftPos;
        if (!end || rightPos.bottom > end.bottom || rightPos.bottom == end.bottom && rightPos.right > end.right)
          end = rightPos;
        if (left < leftSide + 1) left = leftSide;
        add(left, rightPos.top, right - left, rightPos.bottom);
      });
      return {start: start, end: end};
    }

    var sFrom = range.from(), sTo = range.to();
    if (sFrom.line == sTo.line) {
      drawForLine(sFrom.line, sFrom.ch, sTo.ch);
    } else {
      var fromLine = getLine(doc, sFrom.line), toLine = getLine(doc, sTo.line);
      var singleVLine = visualLine(fromLine) == visualLine(toLine);
      var leftEnd = drawForLine(sFrom.line, sFrom.ch, singleVLine ? fromLine.text.length + 1 : null).end;
      var rightStart = drawForLine(sTo.line, singleVLine ? 0 : null, sTo.ch).start;
      if (singleVLine) {
        if (leftEnd.top < rightStart.top - 2) {
          add(leftEnd.right, leftEnd.top, null, leftEnd.bottom);
          add(leftSide, rightStart.top, rightStart.left, rightStart.bottom);
        } else {
          add(leftEnd.right, leftEnd.top, rightStart.left - leftEnd.right, leftEnd.bottom);
        }
      }
      if (leftEnd.bottom < rightStart.top)
        add(leftSide, leftEnd.bottom, null, rightStart.top);
    }

    output.appendChild(fragment);
  }

  // Cursor-blinking
  function restartBlink(cm) {
    if (!cm.state.focused) return;
    var display = cm.display;
    clearInterval(display.blinker);
    var on = true;
    display.cursorDiv.style.visibility = "";
    if (cm.options.cursorBlinkRate > 0)
      display.blinker = setInterval(function() {
        display.cursorDiv.style.visibility = (on = !on) ? "" : "hidden";
      }, cm.options.cursorBlinkRate);
    else if (cm.options.cursorBlinkRate < 0)
      display.cursorDiv.style.visibility = "hidden";
  }

  // HIGHLIGHT WORKER

  function startWorker(cm, time) {
    if (cm.doc.mode.startState && cm.doc.frontier < cm.display.viewTo)
      cm.state.highlight.set(time, bind(highlightWorker, cm));
  }

  function highlightWorker(cm) {
    var doc = cm.doc;
    if (doc.frontier < doc.first) doc.frontier = doc.first;
    if (doc.frontier >= cm.display.viewTo) return;
    var end = +new Date + cm.options.workTime;
    var state = copyState(doc.mode, getStateBefore(cm, doc.frontier));
    var changedLines = [];

    doc.iter(doc.frontier, Math.min(doc.first + doc.size, cm.display.viewTo + 500), function(line) {
      if (doc.frontier >= cm.display.viewFrom) { // Visible
        var oldStyles = line.styles, tooLong = line.text.length > cm.options.maxHighlightLength;
        var highlighted = highlightLine(cm, line, tooLong ? copyState(doc.mode, state) : state, true);
        line.styles = highlighted.styles;
        var oldCls = line.styleClasses, newCls = highlighted.classes;
        if (newCls) line.styleClasses = newCls;
        else if (oldCls) line.styleClasses = null;
        var ischange = !oldStyles || oldStyles.length != line.styles.length ||
          oldCls != newCls && (!oldCls || !newCls || oldCls.bgClass != newCls.bgClass || oldCls.textClass != newCls.textClass);
        for (var i = 0; !ischange && i < oldStyles.length; ++i) ischange = oldStyles[i] != line.styles[i];
        if (ischange) changedLines.push(doc.frontier);
        line.stateAfter = tooLong ? state : copyState(doc.mode, state);
      } else {
        if (line.text.length <= cm.options.maxHighlightLength)
          processLine(cm, line.text, state);
        line.stateAfter = doc.frontier % 5 == 0 ? copyState(doc.mode, state) : null;
      }
      ++doc.frontier;
      if (+new Date > end) {
        startWorker(cm, cm.options.workDelay);
        return true;
      }
    });
    if (changedLines.length) runInOp(cm, function() {
      for (var i = 0; i < changedLines.length; i++)
        regLineChange(cm, changedLines[i], "text");
    });
  }

  // Finds the line to start with when starting a parse. Tries to
  // find a line with a stateAfter, so that it can start with a
  // valid state. If that fails, it returns the line with the
  // smallest indentation, which tends to need the least context to
  // parse correctly.
  function findStartLine(cm, n, precise) {
    var minindent, minline, doc = cm.doc;
    var lim = precise ? -1 : n - (cm.doc.mode.innerMode ? 1000 : 100);
    for (var search = n; search > lim; --search) {
      if (search <= doc.first) return doc.first;
      var line = getLine(doc, search - 1);
      if (line.stateAfter && (!precise || search <= doc.frontier)) return search;
      var indented = countColumn(line.text, null, cm.options.tabSize);
      if (minline == null || minindent > indented) {
        minline = search - 1;
        minindent = indented;
      }
    }
    return minline;
  }

  function getStateBefore(cm, n, precise) {
    var doc = cm.doc, display = cm.display;
    if (!doc.mode.startState) return true;
    var pos = findStartLine(cm, n, precise), state = pos > doc.first && getLine(doc, pos-1).stateAfter;
    if (!state) state = startState(doc.mode);
    else state = copyState(doc.mode, state);
    doc.iter(pos, n, function(line) {
      processLine(cm, line.text, state);
      var save = pos == n - 1 || pos % 5 == 0 || pos >= display.viewFrom && pos < display.viewTo;
      line.stateAfter = save ? copyState(doc.mode, state) : null;
      ++pos;
    });
    if (precise) doc.frontier = pos;
    return state;
  }

  // POSITION MEASUREMENT

  function paddingTop(display) {return display.lineSpace.offsetTop;}
  function paddingVert(display) {return display.mover.offsetHeight - display.lineSpace.offsetHeight;}
  function paddingH(display) {
    if (display.cachedPaddingH) return display.cachedPaddingH;
    var e = removeChildrenAndAdd(display.measure, elt("pre", "x"));
    var style = window.getComputedStyle ? window.getComputedStyle(e) : e.currentStyle;
    var data = {left: parseInt(style.paddingLeft), right: parseInt(style.paddingRight)};
    if (!isNaN(data.left) && !isNaN(data.right)) display.cachedPaddingH = data;
    return data;
  }

  function scrollGap(cm) { return scrollerGap - cm.display.nativeBarWidth; }
  function displayWidth(cm) {
    return cm.display.scroller.clientWidth - scrollGap(cm) - cm.display.barWidth;
  }
  function displayHeight(cm) {
    return cm.display.scroller.clientHeight - scrollGap(cm) - cm.display.barHeight;
  }

  // Ensure the lineView.wrapping.heights array is populated. This is
  // an array of bottom offsets for the lines that make up a drawn
  // line. When lineWrapping is on, there might be more than one
  // height.
  function ensureLineHeights(cm, lineView, rect) {
    var wrapping = cm.options.lineWrapping;
    var curWidth = wrapping && displayWidth(cm);
    if (!lineView.measure.heights || wrapping && lineView.measure.width != curWidth) {
      var heights = lineView.measure.heights = [];
      if (wrapping) {
        lineView.measure.width = curWidth;
        var rects = lineView.text.firstChild.getClientRects();
        for (var i = 0; i < rects.length - 1; i++) {
          var cur = rects[i], next = rects[i + 1];
          if (Math.abs(cur.bottom - next.bottom) > 2)
            heights.push((cur.bottom + next.top) / 2 - rect.top);
        }
      }
      heights.push(rect.bottom - rect.top);
    }
  }

  // Find a line map (mapping character offsets to text nodes) and a
  // measurement cache for the given line number. (A line view might
  // contain multiple lines when collapsed ranges are present.)
  function mapFromLineView(lineView, line, lineN) {
    if (lineView.line == line)
      return {map: lineView.measure.map, cache: lineView.measure.cache};
    for (var i = 0; i < lineView.rest.length; i++)
      if (lineView.rest[i] == line)
        return {map: lineView.measure.maps[i], cache: lineView.measure.caches[i]};
    for (var i = 0; i < lineView.rest.length; i++)
      if (lineNo(lineView.rest[i]) > lineN)
        return {map: lineView.measure.maps[i], cache: lineView.measure.caches[i], before: true};
  }

  // Render a line into the hidden node display.externalMeasured. Used
  // when measurement is needed for a line that's not in the viewport.
  function updateExternalMeasurement(cm, line) {
    line = visualLine(line);
    var lineN = lineNo(line);
    var view = cm.display.externalMeasured = new LineView(cm.doc, line, lineN);
    view.lineN = lineN;
    var built = view.built = buildLineContent(cm, view);
    view.text = built.pre;
    removeChildrenAndAdd(cm.display.lineMeasure, built.pre);
    return view;
  }

  // Get a {top, bottom, left, right} box (in line-local coordinates)
  // for a given character.
  function measureChar(cm, line, ch, bias) {
    return measureCharPrepared(cm, prepareMeasureForLine(cm, line), ch, bias);
  }

  // Find a line view that corresponds to the given line number.
  function findViewForLine(cm, lineN) {
    if (lineN >= cm.display.viewFrom && lineN < cm.display.viewTo)
      return cm.display.view[findViewIndex(cm, lineN)];
    var ext = cm.display.externalMeasured;
    if (ext && lineN >= ext.lineN && lineN < ext.lineN + ext.size)
      return ext;
  }

  // Measurement can be split in two steps, the set-up work that
  // applies to the whole line, and the measurement of the actual
  // character. Functions like coordsChar, that need to do a lot of
  // measurements in a row, can thus ensure that the set-up work is
  // only done once.
  function prepareMeasureForLine(cm, line) {
    var lineN = lineNo(line);
    var view = findViewForLine(cm, lineN);
    if (view && !view.text) {
      view = null;
    } else if (view && view.changes) {
      updateLineForChanges(cm, view, lineN, getDimensions(cm));
      cm.curOp.forceUpdate = true;
    }
    if (!view)
      view = updateExternalMeasurement(cm, line);

    var info = mapFromLineView(view, line, lineN);
    return {
      line: line, view: view, rect: null,
      map: info.map, cache: info.cache, before: info.before,
      hasHeights: false
    };
  }

  // Given a prepared measurement object, measures the position of an
  // actual character (or fetches it from the cache).
  function measureCharPrepared(cm, prepared, ch, bias, varHeight) {
    if (prepared.before) ch = -1;
    var key = ch + (bias || ""), found;
    if (prepared.cache.hasOwnProperty(key)) {
      found = prepared.cache[key];
    } else {
      if (!prepared.rect)
        prepared.rect = prepared.view.text.getBoundingClientRect();
      if (!prepared.hasHeights) {
        ensureLineHeights(cm, prepared.view, prepared.rect);
        prepared.hasHeights = true;
      }
      found = measureCharInner(cm, prepared, ch, bias);
      if (!found.bogus) prepared.cache[key] = found;
    }
    return {left: found.left, right: found.right,
            top: varHeight ? found.rtop : found.top,
            bottom: varHeight ? found.rbottom : found.bottom};
  }

  var nullRect = {left: 0, right: 0, top: 0, bottom: 0};

  function nodeAndOffsetInLineMap(map, ch, bias) {
    var node, start, end, collapse;
    // First, search the line map for the text node corresponding to,
    // or closest to, the target character.
    for (var i = 0; i < map.length; i += 3) {
      var mStart = map[i], mEnd = map[i + 1];
      if (ch < mStart) {
        start = 0; end = 1;
        collapse = "left";
      } else if (ch < mEnd) {
        start = ch - mStart;
        end = start + 1;
      } else if (i == map.length - 3 || ch == mEnd && map[i + 3] > ch) {
        end = mEnd - mStart;
        start = end - 1;
        if (ch >= mEnd) collapse = "right";
      }
      if (start != null) {
        node = map[i + 2];
        if (mStart == mEnd && bias == (node.insertLeft ? "left" : "right"))
          collapse = bias;
        if (bias == "left" && start == 0)
          while (i && map[i - 2] == map[i - 3] && map[i - 1].insertLeft) {
            node = map[(i -= 3) + 2];
            collapse = "left";
          }
        if (bias == "right" && start == mEnd - mStart)
          while (i < map.length - 3 && map[i + 3] == map[i + 4] && !map[i + 5].insertLeft) {
            node = map[(i += 3) + 2];
            collapse = "right";
          }
        break;
      }
    }
    return {node: node, start: start, end: end, collapse: collapse, coverStart: mStart, coverEnd: mEnd};
  }

  function measureCharInner(cm, prepared, ch, bias) {
    var place = nodeAndOffsetInLineMap(prepared.map, ch, bias);
    var node = place.node, start = place.start, end = place.end, collapse = place.collapse;

    var rect;
    if (node.nodeType == 3) { // If it is a text node, use a range to retrieve the coordinates.
      for (var i = 0; i < 4; i++) { // Retry a maximum of 4 times when nonsense rectangles are returned
        while (start && isExtendingChar(prepared.line.text.charAt(place.coverStart + start))) --start;
        while (place.coverStart + end < place.coverEnd && isExtendingChar(prepared.line.text.charAt(place.coverStart + end))) ++end;
        if (ie && ie_version < 9 && start == 0 && end == place.coverEnd - place.coverStart) {
          rect = node.parentNode.getBoundingClientRect();
        } else if (ie && cm.options.lineWrapping) {
          var rects = range(node, start, end).getClientRects();
          if (rects.length)
            rect = rects[bias == "right" ? rects.length - 1 : 0];
          else
            rect = nullRect;
        } else {
          rect = range(node, start, end).getBoundingClientRect() || nullRect;
        }
        if (rect.left || rect.right || start == 0) break;
        end = start;
        start = start - 1;
        collapse = "right";
      }
      if (ie && ie_version < 11) rect = maybeUpdateRectForZooming(cm.display.measure, rect);
    } else { // If it is a widget, simply get the box for the whole widget.
      if (start > 0) collapse = bias = "right";
      var rects;
      if (cm.options.lineWrapping && (rects = node.getClientRects()).length > 1)
        rect = rects[bias == "right" ? rects.length - 1 : 0];
      else
        rect = node.getBoundingClientRect();
    }
    if (ie && ie_version < 9 && !start && (!rect || !rect.left && !rect.right)) {
      var rSpan = node.parentNode.getClientRects()[0];
      if (rSpan)
        rect = {left: rSpan.left, right: rSpan.left + charWidth(cm.display), top: rSpan.top, bottom: rSpan.bottom};
      else
        rect = nullRect;
    }

    var rtop = rect.top - prepared.rect.top, rbot = rect.bottom - prepared.rect.top;
    var mid = (rtop + rbot) / 2;
    var heights = prepared.view.measure.heights;
    for (var i = 0; i < heights.length - 1; i++)
      if (mid < heights[i]) break;
    var top = i ? heights[i - 1] : 0, bot = heights[i];
    var result = {left: (collapse == "right" ? rect.right : rect.left) - prepared.rect.left,
                  right: (collapse == "left" ? rect.left : rect.right) - prepared.rect.left,
                  top: top, bottom: bot};
    if (!rect.left && !rect.right) result.bogus = true;
    if (!cm.options.singleCursorHeightPerLine) { result.rtop = rtop; result.rbottom = rbot; }

    return result;
  }

  // Work around problem with bounding client rects on ranges being
  // returned incorrectly when zoomed on IE10 and below.
  function maybeUpdateRectForZooming(measure, rect) {
    if (!window.screen || screen.logicalXDPI == null ||
        screen.logicalXDPI == screen.deviceXDPI || !hasBadZoomedRects(measure))
      return rect;
    var scaleX = screen.logicalXDPI / screen.deviceXDPI;
    var scaleY = screen.logicalYDPI / screen.deviceYDPI;
    return {left: rect.left * scaleX, right: rect.right * scaleX,
            top: rect.top * scaleY, bottom: rect.bottom * scaleY};
  }

  function clearLineMeasurementCacheFor(lineView) {
    if (lineView.measure) {
      lineView.measure.cache = {};
      lineView.measure.heights = null;
      if (lineView.rest) for (var i = 0; i < lineView.rest.length; i++)
        lineView.measure.caches[i] = {};
    }
  }

  function clearLineMeasurementCache(cm) {
    cm.display.externalMeasure = null;
    removeChildren(cm.display.lineMeasure);
    for (var i = 0; i < cm.display.view.length; i++)
      clearLineMeasurementCacheFor(cm.display.view[i]);
  }

  function clearCaches(cm) {
    clearLineMeasurementCache(cm);
    cm.display.cachedCharWidth = cm.display.cachedTextHeight = cm.display.cachedPaddingH = null;
    if (!cm.options.lineWrapping) cm.display.maxLineChanged = true;
    cm.display.lineNumChars = null;
  }

  function pageScrollX() { return window.pageXOffset || (document.documentElement || document.body).scrollLeft; }
  function pageScrollY() { return window.pageYOffset || (document.documentElement || document.body).scrollTop; }

  // Converts a {top, bottom, left, right} box from line-local
  // coordinates into another coordinate system. Context may be one of
  // "line", "div" (display.lineDiv), "local"/null (editor), "window",
  // or "page".
  function intoCoordSystem(cm, lineObj, rect, context) {
    if (lineObj.widgets) for (var i = 0; i < lineObj.widgets.length; ++i) if (lineObj.widgets[i].above) {
      var size = widgetHeight(lineObj.widgets[i]);
      rect.top += size; rect.bottom += size;
    }
    if (context == "line") return rect;
    if (!context) context = "local";
    var yOff = heightAtLine(lineObj);
    if (context == "local") yOff += paddingTop(cm.display);
    else yOff -= cm.display.viewOffset;
    if (context == "page" || context == "window") {
      var lOff = cm.display.lineSpace.getBoundingClientRect();
      yOff += lOff.top + (context == "window" ? 0 : pageScrollY());
      var xOff = lOff.left + (context == "window" ? 0 : pageScrollX());
      rect.left += xOff; rect.right += xOff;
    }
    rect.top += yOff; rect.bottom += yOff;
    return rect;
  }

  // Coverts a box from "div" coords to another coordinate system.
  // Context may be "window", "page", "div", or "local"/null.
  function fromCoordSystem(cm, coords, context) {
    if (context == "div") return coords;
    var left = coords.left, top = coords.top;
    // First move into "page" coordinate system
    if (context == "page") {
      left -= pageScrollX();
      top -= pageScrollY();
    } else if (context == "local" || !context) {
      var localBox = cm.display.sizer.getBoundingClientRect();
      left += localBox.left;
      top += localBox.top;
    }

    var lineSpaceBox = cm.display.lineSpace.getBoundingClientRect();
    return {left: left - lineSpaceBox.left, top: top - lineSpaceBox.top};
  }

  function charCoords(cm, pos, context, lineObj, bias) {
    if (!lineObj) lineObj = getLine(cm.doc, pos.line);
    return intoCoordSystem(cm, lineObj, measureChar(cm, lineObj, pos.ch, bias), context);
  }

  // Returns a box for a given cursor position, which may have an
  // 'other' property containing the position of the secondary cursor
  // on a bidi boundary.
  function cursorCoords(cm, pos, context, lineObj, preparedMeasure, varHeight) {
    lineObj = lineObj || getLine(cm.doc, pos.line);
    if (!preparedMeasure) preparedMeasure = prepareMeasureForLine(cm, lineObj);
    function get(ch, right) {
      var m = measureCharPrepared(cm, preparedMeasure, ch, right ? "right" : "left", varHeight);
      if (right) m.left = m.right; else m.right = m.left;
      return intoCoordSystem(cm, lineObj, m, context);
    }
    function getBidi(ch, partPos) {
      var part = order[partPos], right = part.level % 2;
      if (ch == bidiLeft(part) && partPos && part.level < order[partPos - 1].level) {
        part = order[--partPos];
        ch = bidiRight(part) - (part.level % 2 ? 0 : 1);
        right = true;
      } else if (ch == bidiRight(part) && partPos < order.length - 1 && part.level < order[partPos + 1].level) {
        part = order[++partPos];
        ch = bidiLeft(part) - part.level % 2;
        right = false;
      }
      if (right && ch == part.to && ch > part.from) return get(ch - 1);
      return get(ch, right);
    }
    var order = getOrder(lineObj), ch = pos.ch;
    if (!order) return get(ch);
    var partPos = getBidiPartAt(order, ch);
    var val = getBidi(ch, partPos);
    if (bidiOther != null) val.other = getBidi(ch, bidiOther);
    return val;
  }

  // Used to cheaply estimate the coordinates for a position. Used for
  // intermediate scroll updates.
  function estimateCoords(cm, pos) {
    var left = 0, pos = clipPos(cm.doc, pos);
    if (!cm.options.lineWrapping) left = charWidth(cm.display) * pos.ch;
    var lineObj = getLine(cm.doc, pos.line);
    var top = heightAtLine(lineObj) + paddingTop(cm.display);
    return {left: left, right: left, top: top, bottom: top + lineObj.height};
  }

  // Positions returned by coordsChar contain some extra information.
  // xRel is the relative x position of the input coordinates compared
  // to the found position (so xRel > 0 means the coordinates are to
  // the right of the character position, for example). When outside
  // is true, that means the coordinates lie outside the line's
  // vertical range.
  function PosWithInfo(line, ch, outside, xRel) {
    var pos = Pos(line, ch);
    pos.xRel = xRel;
    if (outside) pos.outside = true;
    return pos;
  }

  // Compute the character position closest to the given coordinates.
  // Input must be lineSpace-local ("div" coordinate system).
  function coordsChar(cm, x, y) {
    var doc = cm.doc;
    y += cm.display.viewOffset;
    if (y < 0) return PosWithInfo(doc.first, 0, true, -1);
    var lineN = lineAtHeight(doc, y), last = doc.first + doc.size - 1;
    if (lineN > last)
      return PosWithInfo(doc.first + doc.size - 1, getLine(doc, last).text.length, true, 1);
    if (x < 0) x = 0;

    var lineObj = getLine(doc, lineN);
    for (;;) {
      var found = coordsCharInner(cm, lineObj, lineN, x, y);
      var merged = collapsedSpanAtEnd(lineObj);
      var mergedPos = merged && merged.find(0, true);
      if (merged && (found.ch > mergedPos.from.ch || found.ch == mergedPos.from.ch && found.xRel > 0))
        lineN = lineNo(lineObj = mergedPos.to.line);
      else
        return found;
    }
  }

  function coordsCharInner(cm, lineObj, lineNo, x, y) {
    var innerOff = y - heightAtLine(lineObj);
    var wrongLine = false, adjust = 2 * cm.display.wrapper.clientWidth;
    var preparedMeasure = prepareMeasureForLine(cm, lineObj);

    function getX(ch) {
      var sp = cursorCoords(cm, Pos(lineNo, ch), "line", lineObj, preparedMeasure);
      wrongLine = true;
      if (innerOff > sp.bottom) return sp.left - adjust;
      else if (innerOff < sp.top) return sp.left + adjust;
      else wrongLine = false;
      return sp.left;
    }

    var bidi = getOrder(lineObj), dist = lineObj.text.length;
    var from = lineLeft(lineObj), to = lineRight(lineObj);
    var fromX = getX(from), fromOutside = wrongLine, toX = getX(to), toOutside = wrongLine;

    if (x > toX) return PosWithInfo(lineNo, to, toOutside, 1);
    // Do a binary search between these bounds.
    for (;;) {
      if (bidi ? to == from || to == moveVisually(lineObj, from, 1) : to - from <= 1) {
        var ch = x < fromX || x - fromX <= toX - x ? from : to;
        var xDiff = x - (ch == from ? fromX : toX);
        while (isExtendingChar(lineObj.text.charAt(ch))) ++ch;
        var pos = PosWithInfo(lineNo, ch, ch == from ? fromOutside : toOutside,
                              xDiff < -1 ? -1 : xDiff > 1 ? 1 : 0);
        return pos;
      }
      var step = Math.ceil(dist / 2), middle = from + step;
      if (bidi) {
        middle = from;
        for (var i = 0; i < step; ++i) middle = moveVisually(lineObj, middle, 1);
      }
      var middleX = getX(middle);
      if (middleX > x) {to = middle; toX = middleX; if (toOutside = wrongLine) toX += 1000; dist = step;}
      else {from = middle; fromX = middleX; fromOutside = wrongLine; dist -= step;}
    }
  }

  var measureText;
  // Compute the default text height.
  function textHeight(display) {
    if (display.cachedTextHeight != null) return display.cachedTextHeight;
    if (measureText == null) {
      measureText = elt("pre");
      // Measure a bunch of lines, for browsers that compute
      // fractional heights.
      for (var i = 0; i < 49; ++i) {
        measureText.appendChild(document.createTextNode("x"));
        measureText.appendChild(elt("br"));
      }
      measureText.appendChild(document.createTextNode("x"));
    }
    removeChildrenAndAdd(display.measure, measureText);
    var height = measureText.offsetHeight / 50;
    if (height > 3) display.cachedTextHeight = height;
    removeChildren(display.measure);
    return height || 1;
  }

  // Compute the default character width.
  function charWidth(display) {
    if (display.cachedCharWidth != null) return display.cachedCharWidth;
    var anchor = elt("span", "xxxxxxxxxx");
    var pre = elt("pre", [anchor]);
    removeChildrenAndAdd(display.measure, pre);
    var rect = anchor.getBoundingClientRect(), width = (rect.right - rect.left) / 10;
    if (width > 2) display.cachedCharWidth = width;
    return width || 10;
  }

  // OPERATIONS

  // Operations are used to wrap a series of changes to the editor
  // state in such a way that each change won't have to update the
  // cursor and display (which would be awkward, slow, and
  // error-prone). Instead, display updates are batched and then all
  // combined and executed at once.

  var operationGroup = null;

  var nextOpId = 0;
  // Start a new operation.
  function startOperation(cm) {
    cm.curOp = {
      cm: cm,
      viewChanged: false,      // Flag that indicates that lines might need to be redrawn
      startHeight: cm.doc.height, // Used to detect need to update scrollbar
      forceUpdate: false,      // Used to force a redraw
      updateInput: null,       // Whether to reset the input textarea
      typing: false,           // Whether this reset should be careful to leave existing text (for compositing)
      changeObjs: null,        // Accumulated changes, for firing change events
      cursorActivityHandlers: null, // Set of handlers to fire cursorActivity on
      cursorActivityCalled: 0, // Tracks which cursorActivity handlers have been called already
      selectionChanged: false, // Whether the selection needs to be redrawn
      updateMaxLine: false,    // Set when the widest line needs to be determined anew
      scrollLeft: null, scrollTop: null, // Intermediate scroll position, not pushed to DOM yet
      scrollToPos: null,       // Used to scroll to a specific position
      focus: false,
      id: ++nextOpId           // Unique ID
    };
    if (operationGroup) {
      operationGroup.ops.push(cm.curOp);
    } else {
      cm.curOp.ownsGroup = operationGroup = {
        ops: [cm.curOp],
        delayedCallbacks: []
      };
    }
  }

  function fireCallbacksForOps(group) {
    // Calls delayed callbacks and cursorActivity handlers until no
    // new ones appear
    var callbacks = group.delayedCallbacks, i = 0;
    do {
      for (; i < callbacks.length; i++)
        callbacks[i].call(null);
      for (var j = 0; j < group.ops.length; j++) {
        var op = group.ops[j];
        if (op.cursorActivityHandlers)
          while (op.cursorActivityCalled < op.cursorActivityHandlers.length)
            op.cursorActivityHandlers[op.cursorActivityCalled++].call(null, op.cm);
      }
    } while (i < callbacks.length);
  }

  // Finish an operation, updating the display and signalling delayed events
  function endOperation(cm) {
    var op = cm.curOp, group = op.ownsGroup;
    if (!group) return;

    try { fireCallbacksForOps(group); }
    finally {
      operationGroup = null;
      for (var i = 0; i < group.ops.length; i++)
        group.ops[i].cm.curOp = null;
      endOperations(group);
    }
  }

  // The DOM updates done when an operation finishes are batched so
  // that the minimum number of relayouts are required.
  function endOperations(group) {
    var ops = group.ops;
    for (var i = 0; i < ops.length; i++) // Read DOM
      endOperation_R1(ops[i]);
    for (var i = 0; i < ops.length; i++) // Write DOM (maybe)
      endOperation_W1(ops[i]);
    for (var i = 0; i < ops.length; i++) // Read DOM
      endOperation_R2(ops[i]);
    for (var i = 0; i < ops.length; i++) // Write DOM (maybe)
      endOperation_W2(ops[i]);
    for (var i = 0; i < ops.length; i++) // Read DOM
      endOperation_finish(ops[i]);
  }

  function endOperation_R1(op) {
    var cm = op.cm, display = cm.display;
    maybeClipScrollbars(cm);
    if (op.updateMaxLine) findMaxLine(cm);

    op.mustUpdate = op.viewChanged || op.forceUpdate || op.scrollTop != null ||
      op.scrollToPos && (op.scrollToPos.from.line < display.viewFrom ||
                         op.scrollToPos.to.line >= display.viewTo) ||
      display.maxLineChanged && cm.options.lineWrapping;
    op.update = op.mustUpdate &&
      new DisplayUpdate(cm, op.mustUpdate && {top: op.scrollTop, ensure: op.scrollToPos}, op.forceUpdate);
  }

  function endOperation_W1(op) {
    op.updatedDisplay = op.mustUpdate && updateDisplayIfNeeded(op.cm, op.update);
  }

  function endOperation_R2(op) {
    var cm = op.cm, display = cm.display;
    if (op.updatedDisplay) updateHeightsInViewport(cm);

    op.barMeasure = measureForScrollbars(cm);

    // If the max line changed since it was last measured, measure it,
    // and ensure the document's width matches it.
    // updateDisplay_W2 will use these properties to do the actual resizing
    if (display.maxLineChanged && !cm.options.lineWrapping) {
      op.adjustWidthTo = measureChar(cm, display.maxLine, display.maxLine.text.length).left + 3;
      cm.display.sizerWidth = op.adjustWidthTo;
      op.barMeasure.scrollWidth =
        Math.max(display.scroller.clientWidth, display.sizer.offsetLeft + op.adjustWidthTo + scrollGap(cm) + cm.display.barWidth);
      op.maxScrollLeft = Math.max(0, display.sizer.offsetLeft + op.adjustWidthTo - displayWidth(cm));
    }

    if (op.updatedDisplay || op.selectionChanged)
      op.preparedSelection = display.input.prepareSelection(op.focus);
  }

  function endOperation_W2(op) {
    var cm = op.cm;

    if (op.adjustWidthTo != null) {
      cm.display.sizer.style.minWidth = op.adjustWidthTo + "px";
      if (op.maxScrollLeft < cm.doc.scrollLeft)
        setScrollLeft(cm, Math.min(cm.display.scroller.scrollLeft, op.maxScrollLeft), true);
      cm.display.maxLineChanged = false;
    }

    var takeFocus = op.focus && op.focus == activeElt() && (!document.hasFocus || document.hasFocus())
    if (op.preparedSelection)
      cm.display.input.showSelection(op.preparedSelection, takeFocus);
    if (op.updatedDisplay || op.startHeight != cm.doc.height)
      updateScrollbars(cm, op.barMeasure);
    if (op.updatedDisplay)
      setDocumentHeight(cm, op.barMeasure);

    if (op.selectionChanged) restartBlink(cm);

    if (cm.state.focused && op.updateInput)
      cm.display.input.reset(op.typing);
    if (takeFocus) ensureFocus(op.cm);
  }

  function endOperation_finish(op) {
    var cm = op.cm, display = cm.display, doc = cm.doc;

    if (op.updatedDisplay) postUpdateDisplay(cm, op.update);

    // Abort mouse wheel delta measurement, when scrolling explicitly
    if (display.wheelStartX != null && (op.scrollTop != null || op.scrollLeft != null || op.scrollToPos))
      display.wheelStartX = display.wheelStartY = null;

    // Propagate the scroll position to the actual DOM scroller
    if (op.scrollTop != null && (display.scroller.scrollTop != op.scrollTop || op.forceScroll)) {
      doc.scrollTop = Math.max(0, Math.min(display.scroller.scrollHeight - display.scroller.clientHeight, op.scrollTop));
      display.scrollbars.setScrollTop(doc.scrollTop);
      display.scroller.scrollTop = doc.scrollTop;
    }
    if (op.scrollLeft != null && (display.scroller.scrollLeft != op.scrollLeft || op.forceScroll)) {
      doc.scrollLeft = Math.max(0, Math.min(display.scroller.scrollWidth - display.scroller.clientWidth, op.scrollLeft));
      display.scrollbars.setScrollLeft(doc.scrollLeft);
      display.scroller.scrollLeft = doc.scrollLeft;
      alignHorizontally(cm);
    }
    // If we need to scroll a specific position into view, do so.
    if (op.scrollToPos) {
      var coords = scrollPosIntoView(cm, clipPos(doc, op.scrollToPos.from),
                                     clipPos(doc, op.scrollToPos.to), op.scrollToPos.margin);
      if (op.scrollToPos.isCursor && cm.state.focused) maybeScrollWindow(cm, coords);
    }

    // Fire events for markers that are hidden/unidden by editing or
    // undoing
    var hidden = op.maybeHiddenMarkers, unhidden = op.maybeUnhiddenMarkers;
    if (hidden) for (var i = 0; i < hidden.length; ++i)
      if (!hidden[i].lines.length) signal(hidden[i], "hide");
    if (unhidden) for (var i = 0; i < unhidden.length; ++i)
      if (unhidden[i].lines.length) signal(unhidden[i], "unhide");

    if (display.wrapper.offsetHeight)
      doc.scrollTop = cm.display.scroller.scrollTop;

    // Fire change events, and delayed event handlers
    if (op.changeObjs)
      signal(cm, "changes", cm, op.changeObjs);
    if (op.update)
      op.update.finish();
  }

  // Run the given function in an operation
  function runInOp(cm, f) {
    if (cm.curOp) return f();
    startOperation(cm);
    try { return f(); }
    finally { endOperation(cm); }
  }
  // Wraps a function in an operation. Returns the wrapped function.
  function operation(cm, f) {
    return function() {
      if (cm.curOp) return f.apply(cm, arguments);
      startOperation(cm);
      try { return f.apply(cm, arguments); }
      finally { endOperation(cm); }
    };
  }
  // Used to add methods to editor and doc instances, wrapping them in
  // operations.
  function methodOp(f) {
    return function() {
      if (this.curOp) return f.apply(this, arguments);
      startOperation(this);
      try { return f.apply(this, arguments); }
      finally { endOperation(this); }
    };
  }
  function docMethodOp(f) {
    return function() {
      var cm = this.cm;
      if (!cm || cm.curOp) return f.apply(this, arguments);
      startOperation(cm);
      try { return f.apply(this, arguments); }
      finally { endOperation(cm); }
    };
  }

  // VIEW TRACKING

  // These objects are used to represent the visible (currently drawn)
  // part of the document. A LineView may correspond to multiple
  // logical lines, if those are connected by collapsed ranges.
  function LineView(doc, line, lineN) {
    // The starting line
    this.line = line;
    // Continuing lines, if any
    this.rest = visualLineContinued(line);
    // Number of logical lines in this visual line
    this.size = this.rest ? lineNo(lst(this.rest)) - lineN + 1 : 1;
    this.node = this.text = null;
    this.hidden = lineIsHidden(doc, line);
  }

  // Create a range of LineView objects for the given lines.
  function buildViewArray(cm, from, to) {
    var array = [], nextPos;
    for (var pos = from; pos < to; pos = nextPos) {
      var view = new LineView(cm.doc, getLine(cm.doc, pos), pos);
      nextPos = pos + view.size;
      array.push(view);
    }
    return array;
  }

  // Updates the display.view data structure for a given change to the
  // document. From and to are in pre-change coordinates. Lendiff is
  // the amount of lines added or subtracted by the change. This is
  // used for changes that span multiple lines, or change the way
  // lines are divided into visual lines. regLineChange (below)
  // registers single-line changes.
  function regChange(cm, from, to, lendiff) {
    if (from == null) from = cm.doc.first;
    if (to == null) to = cm.doc.first + cm.doc.size;
    if (!lendiff) lendiff = 0;

    var display = cm.display;
    if (lendiff && to < display.viewTo &&
        (display.updateLineNumbers == null || display.updateLineNumbers > from))
      display.updateLineNumbers = from;

    cm.curOp.viewChanged = true;

    if (from >= display.viewTo) { // Change after
      if (sawCollapsedSpans && visualLineNo(cm.doc, from) < display.viewTo)
        resetView(cm);
    } else if (to <= display.viewFrom) { // Change before
      if (sawCollapsedSpans && visualLineEndNo(cm.doc, to + lendiff) > display.viewFrom) {
        resetView(cm);
      } else {
        display.viewFrom += lendiff;
        display.viewTo += lendiff;
      }
    } else if (from <= display.viewFrom && to >= display.viewTo) { // Full overlap
      resetView(cm);
    } else if (from <= display.viewFrom) { // Top overlap
      var cut = viewCuttingPoint(cm, to, to + lendiff, 1);
      if (cut) {
        display.view = display.view.slice(cut.index);
        display.viewFrom = cut.lineN;
        display.viewTo += lendiff;
      } else {
        resetView(cm);
      }
    } else if (to >= display.viewTo) { // Bottom overlap
      var cut = viewCuttingPoint(cm, from, from, -1);
      if (cut) {
        display.view = display.view.slice(0, cut.index);
        display.viewTo = cut.lineN;
      } else {
        resetView(cm);
      }
    } else { // Gap in the middle
      var cutTop = viewCuttingPoint(cm, from, from, -1);
      var cutBot = viewCuttingPoint(cm, to, to + lendiff, 1);
      if (cutTop && cutBot) {
        display.view = display.view.slice(0, cutTop.index)
          .concat(buildViewArray(cm, cutTop.lineN, cutBot.lineN))
          .concat(display.view.slice(cutBot.index));
        display.viewTo += lendiff;
      } else {
        resetView(cm);
      }
    }

    var ext = display.externalMeasured;
    if (ext) {
      if (to < ext.lineN)
        ext.lineN += lendiff;
      else if (from < ext.lineN + ext.size)
        display.externalMeasured = null;
    }
  }

  // Register a change to a single line. Type must be one of "text",
  // "gutter", "class", "widget"
  function regLineChange(cm, line, type) {
    cm.curOp.viewChanged = true;
    var display = cm.display, ext = cm.display.externalMeasured;
    if (ext && line >= ext.lineN && line < ext.lineN + ext.size)
      display.externalMeasured = null;

    if (line < display.viewFrom || line >= display.viewTo) return;
    var lineView = display.view[findViewIndex(cm, line)];
    if (lineView.node == null) return;
    var arr = lineView.changes || (lineView.changes = []);
    if (indexOf(arr, type) == -1) arr.push(type);
  }

  // Clear the view.
  function resetView(cm) {
    cm.display.viewFrom = cm.display.viewTo = cm.doc.first;
    cm.display.view = [];
    cm.display.viewOffset = 0;
  }

  // Find the view element corresponding to a given line. Return null
  // when the line isn't visible.
  function findViewIndex(cm, n) {
    if (n >= cm.display.viewTo) return null;
    n -= cm.display.viewFrom;
    if (n < 0) return null;
    var view = cm.display.view;
    for (var i = 0; i < view.length; i++) {
      n -= view[i].size;
      if (n < 0) return i;
    }
  }

  function viewCuttingPoint(cm, oldN, newN, dir) {
    var index = findViewIndex(cm, oldN), diff, view = cm.display.view;
    if (!sawCollapsedSpans || newN == cm.doc.first + cm.doc.size)
      return {index: index, lineN: newN};
    for (var i = 0, n = cm.display.viewFrom; i < index; i++)
      n += view[i].size;
    if (n != oldN) {
      if (dir > 0) {
        if (index == view.length - 1) return null;
        diff = (n + view[index].size) - oldN;
        index++;
      } else {
        diff = n - oldN;
      }
      oldN += diff; newN += diff;
    }
    while (visualLineNo(cm.doc, newN) != newN) {
      if (index == (dir < 0 ? 0 : view.length - 1)) return null;
      newN += dir * view[index - (dir < 0 ? 1 : 0)].size;
      index += dir;
    }
    return {index: index, lineN: newN};
  }

  // Force the view to cover a given range, adding empty view element
  // or clipping off existing ones as needed.
  function adjustView(cm, from, to) {
    var display = cm.display, view = display.view;
    if (view.length == 0 || from >= display.viewTo || to <= display.viewFrom) {
      display.view = buildViewArray(cm, from, to);
      display.viewFrom = from;
    } else {
      if (display.viewFrom > from)
        display.view = buildViewArray(cm, from, display.viewFrom).concat(display.view);
      else if (display.viewFrom < from)
        display.view = display.view.slice(findViewIndex(cm, from));
      display.viewFrom = from;
      if (display.viewTo < to)
        display.view = display.view.concat(buildViewArray(cm, display.viewTo, to));
      else if (display.viewTo > to)
        display.view = display.view.slice(0, findViewIndex(cm, to));
    }
    display.viewTo = to;
  }

  // Count the number of lines in the view whose DOM representation is
  // out of date (or nonexistent).
  function countDirtyView(cm) {
    var view = cm.display.view, dirty = 0;
    for (var i = 0; i < view.length; i++) {
      var lineView = view[i];
      if (!lineView.hidden && (!lineView.node || lineView.changes)) ++dirty;
    }
    return dirty;
  }

  // EVENT HANDLERS

  // Attach the necessary event handlers when initializing the editor
  function registerEventHandlers(cm) {
    var d = cm.display;
    on(d.scroller, "mousedown", operation(cm, onMouseDown));
    // Older IE's will not fire a second mousedown for a double click
    if (ie && ie_version < 11)
      on(d.scroller, "dblclick", operation(cm, function(e) {
        if (signalDOMEvent(cm, e)) return;
        var pos = posFromMouse(cm, e);
        if (!pos || clickInGutter(cm, e) || eventInWidget(cm.display, e)) return;
        e_preventDefault(e);
        var word = cm.findWordAt(pos);
        extendSelection(cm.doc, word.anchor, word.head);
      }));
    else
      on(d.scroller, "dblclick", function(e) { signalDOMEvent(cm, e) || e_preventDefault(e); });
    // Some browsers fire contextmenu *after* opening the menu, at
    // which point we can't mess with it anymore. Context menu is
    // handled in onMouseDown for these browsers.
    if (!captureRightClick) on(d.scroller, "contextmenu", function(e) {onContextMenu(cm, e);});

    // Used to suppress mouse event handling when a touch happens
    var touchFinished, prevTouch = {end: 0};
    function finishTouch() {
      if (d.activeTouch) {
        touchFinished = setTimeout(function() {d.activeTouch = null;}, 1000);
        prevTouch = d.activeTouch;
        prevTouch.end = +new Date;
      }
    };
    function isMouseLikeTouchEvent(e) {
      if (e.touches.length != 1) return false;
      var touch = e.touches[0];
      return touch.radiusX <= 1 && touch.radiusY <= 1;
    }
    function farAway(touch, other) {
      if (other.left == null) return true;
      var dx = other.left - touch.left, dy = other.top - touch.top;
      return dx * dx + dy * dy > 20 * 20;
    }
    on(d.scroller, "touchstart", function(e) {
      if (!signalDOMEvent(cm, e) && !isMouseLikeTouchEvent(e)) {
        clearTimeout(touchFinished);
        var now = +new Date;
        d.activeTouch = {start: now, moved: false,
                         prev: now - prevTouch.end <= 300 ? prevTouch : null};
        if (e.touches.length == 1) {
          d.activeTouch.left = e.touches[0].pageX;
          d.activeTouch.top = e.touches[0].pageY;
        }
      }
    });
    on(d.scroller, "touchmove", function() {
      if (d.activeTouch) d.activeTouch.moved = true;
    });
    on(d.scroller, "touchend", function(e) {
      var touch = d.activeTouch;
      if (touch && !eventInWidget(d, e) && touch.left != null &&
          !touch.moved && new Date - touch.start < 300) {
        var pos = cm.coordsChar(d.activeTouch, "page"), range;
        if (!touch.prev || farAway(touch, touch.prev)) // Single tap
          range = new Range(pos, pos);
        else if (!touch.prev.prev || farAway(touch, touch.prev.prev)) // Double tap
          range = cm.findWordAt(pos);
        else // Triple tap
          range = new Range(Pos(pos.line, 0), clipPos(cm.doc, Pos(pos.line + 1, 0)));
        cm.setSelection(range.anchor, range.head);
        cm.focus();
        e_preventDefault(e);
      }
      finishTouch();
    });
    on(d.scroller, "touchcancel", finishTouch);

    // Sync scrolling between fake scrollbars and real scrollable
    // area, ensure viewport is updated when scrolling.
    on(d.scroller, "scroll", function() {
      if (d.scroller.clientHeight) {
        setScrollTop(cm, d.scroller.scrollTop);
        setScrollLeft(cm, d.scroller.scrollLeft, true);
        signal(cm, "scroll", cm);
      }
    });

    // Listen to wheel events in order to try and update the viewport on time.
    on(d.scroller, "mousewheel", function(e){onScrollWheel(cm, e);});
    on(d.scroller, "DOMMouseScroll", function(e){onScrollWheel(cm, e);});

    // Prevent wrapper from ever scrolling
    on(d.wrapper, "scroll", function() { d.wrapper.scrollTop = d.wrapper.scrollLeft = 0; });

    d.dragFunctions = {
      enter: function(e) {if (!signalDOMEvent(cm, e)) e_stop(e);},
      over: function(e) {if (!signalDOMEvent(cm, e)) { onDragOver(cm, e); e_stop(e); }},
      start: function(e){onDragStart(cm, e);},
      drop: operation(cm, onDrop),
      leave: function(e) {if (!signalDOMEvent(cm, e)) { clearDragCursor(cm); }}
    };

    var inp = d.input.getField();
    on(inp, "keyup", function(e) { onKeyUp.call(cm, e); });
    on(inp, "keydown", operation(cm, onKeyDown));
    on(inp, "keypress", operation(cm, onKeyPress));
    on(inp, "focus", bind(onFocus, cm));
    on(inp, "blur", bind(onBlur, cm));
  }

  function dragDropChanged(cm, value, old) {
    var wasOn = old && old != CodeMirror.Init;
    if (!value != !wasOn) {
      var funcs = cm.display.dragFunctions;
      var toggle = value ? on : off;
      toggle(cm.display.scroller, "dragstart", funcs.start);
      toggle(cm.display.scroller, "dragenter", funcs.enter);
      toggle(cm.display.scroller, "dragover", funcs.over);
      toggle(cm.display.scroller, "dragleave", funcs.leave);
      toggle(cm.display.scroller, "drop", funcs.drop);
    }
  }

  // Called when the window resizes
  function onResize(cm) {
    var d = cm.display;
    if (d.lastWrapHeight == d.wrapper.clientHeight && d.lastWrapWidth == d.wrapper.clientWidth)
      return;
    // Might be a text scaling operation, clear size caches.
    d.cachedCharWidth = d.cachedTextHeight = d.cachedPaddingH = null;
    d.scrollbarsClipped = false;
    cm.setSize();
  }

  // MOUSE EVENTS

  // Return true when the given mouse event happened in a widget
  function eventInWidget(display, e) {
    for (var n = e_target(e); n != display.wrapper; n = n.parentNode) {
      if (!n || (n.nodeType == 1 && n.getAttribute("cm-ignore-events") == "true") ||
          (n.parentNode == display.sizer && n != display.mover))
        return true;
    }
  }

  // Given a mouse event, find the corresponding position. If liberal
  // is false, it checks whether a gutter or scrollbar was clicked,
  // and returns null if it was. forRect is used by rectangular
  // selections, and tries to estimate a character position even for
  // coordinates beyond the right of the text.
  function posFromMouse(cm, e, liberal, forRect) {
    var display = cm.display;
    if (!liberal && e_target(e).getAttribute("cm-not-content") == "true") return null;

    var x, y, space = display.lineSpace.getBoundingClientRect();
    // Fails unpredictably on IE[67] when mouse is dragged around quickly.
    try { x = e.clientX - space.left; y = e.clientY - space.top; }
    catch (e) { return null; }
    var coords = coordsChar(cm, x, y), line;
    if (forRect && coords.xRel == 1 && (line = getLine(cm.doc, coords.line).text).length == coords.ch) {
      var colDiff = countColumn(line, line.length, cm.options.tabSize) - line.length;
      coords = Pos(coords.line, Math.max(0, Math.round((x - paddingH(cm.display).left) / charWidth(cm.display)) - colDiff));
    }
    return coords;
  }

  // A mouse down can be a single click, double click, triple click,
  // start of selection drag, start of text drag, new cursor
  // (ctrl-click), rectangle drag (alt-drag), or xwin
  // middle-click-paste. Or it might be a click on something we should
  // not interfere with, such as a scrollbar or widget.
  function onMouseDown(e) {
    var cm = this, display = cm.display;
    if (signalDOMEvent(cm, e) || display.activeTouch && display.input.supportsTouch()) return;
    display.shift = e.shiftKey;

    if (eventInWidget(display, e)) {
      if (!webkit) {
        // Briefly turn off draggability, to allow widgets to do
        // normal dragging things.
        display.scroller.draggable = false;
        setTimeout(function(){display.scroller.draggable = true;}, 100);
      }
      return;
    }
    if (clickInGutter(cm, e)) return;
    var start = posFromMouse(cm, e);
    window.focus();

    switch (e_button(e)) {
    case 1:
      // #3261: make sure, that we're not starting a second selection
      if (cm.state.selectingText)
        cm.state.selectingText(e);
      else if (start)
        leftButtonDown(cm, e, start);
      else if (e_target(e) == display.scroller)
        e_preventDefault(e);
      break;
    case 2:
      if (webkit) cm.state.lastMiddleDown = +new Date;
      if (start) extendSelection(cm.doc, start);
      setTimeout(function() {display.input.focus();}, 20);
      e_preventDefault(e);
      break;
    case 3:
      if (captureRightClick) onContextMenu(cm, e);
      else delayBlurEvent(cm);
      break;
    }
  }

  var lastClick, lastDoubleClick;
  function leftButtonDown(cm, e, start) {
    if (ie) setTimeout(bind(ensureFocus, cm), 0);
    else cm.curOp.focus = activeElt();

    var now = +new Date, type;
    if (lastDoubleClick && lastDoubleClick.time > now - 400 && cmp(lastDoubleClick.pos, start) == 0) {
      type = "triple";
    } else if (lastClick && lastClick.time > now - 400 && cmp(lastClick.pos, start) == 0) {
      type = "double";
      lastDoubleClick = {time: now, pos: start};
    } else {
      type = "single";
      lastClick = {time: now, pos: start};
    }

    var sel = cm.doc.sel, modifier = mac ? e.metaKey : e.ctrlKey, contained;
    if (cm.options.dragDrop && dragAndDrop && !cm.isReadOnly() &&
        type == "single" && (contained = sel.contains(start)) > -1 &&
        (cmp((contained = sel.ranges[contained]).from(), start) < 0 || start.xRel > 0) &&
        (cmp(contained.to(), start) > 0 || start.xRel < 0))
      leftButtonStartDrag(cm, e, start, modifier);
    else
      leftButtonSelect(cm, e, start, type, modifier);
  }

  // Start a text drag. When it ends, see if any dragging actually
  // happen, and treat as a click if it didn't.
  function leftButtonStartDrag(cm, e, start, modifier) {
    var display = cm.display, startTime = +new Date;
    var dragEnd = operation(cm, function(e2) {
      if (webkit) display.scroller.draggable = false;
      cm.state.draggingText = false;
      off(document, "mouseup", dragEnd);
      off(display.scroller, "drop", dragEnd);
      if (Math.abs(e.clientX - e2.clientX) + Math.abs(e.clientY - e2.clientY) < 10) {
        e_preventDefault(e2);
        if (!modifier && +new Date - 200 < startTime)
          extendSelection(cm.doc, start);
        // Work around unexplainable focus problem in IE9 (#2127) and Chrome (#3081)
        if (webkit || ie && ie_version == 9)
          setTimeout(function() {document.body.focus(); display.input.focus();}, 20);
        else
          display.input.focus();
      }
    });
    // Let the drag handler handle this.
    if (webkit) display.scroller.draggable = true;
    cm.state.draggingText = dragEnd;
    dragEnd.copy = mac ? e.altKey : e.ctrlKey
    // IE's approach to draggable
    if (display.scroller.dragDrop) display.scroller.dragDrop();
    on(document, "mouseup", dragEnd);
    on(display.scroller, "drop", dragEnd);
  }

  // Normal selection, as opposed to text dragging.
  function leftButtonSelect(cm, e, start, type, addNew) {
    var display = cm.display, doc = cm.doc;
    e_preventDefault(e);

    var ourRange, ourIndex, startSel = doc.sel, ranges = startSel.ranges;
    if (addNew && !e.shiftKey) {
      ourIndex = doc.sel.contains(start);
      if (ourIndex > -1)
        ourRange = ranges[ourIndex];
      else
        ourRange = new Range(start, start);
    } else {
      ourRange = doc.sel.primary();
      ourIndex = doc.sel.primIndex;
    }

    if (chromeOS ? e.shiftKey && e.metaKey : e.altKey) {
      type = "rect";
      if (!addNew) ourRange = new Range(start, start);
      start = posFromMouse(cm, e, true, true);
      ourIndex = -1;
    } else if (type == "double") {
      var word = cm.findWordAt(start);
      if (cm.display.shift || doc.extend)
        ourRange = extendRange(doc, ourRange, word.anchor, word.head);
      else
        ourRange = word;
    } else if (type == "triple") {
      var line = new Range(Pos(start.line, 0), clipPos(doc, Pos(start.line + 1, 0)));
      if (cm.display.shift || doc.extend)
        ourRange = extendRange(doc, ourRange, line.anchor, line.head);
      else
        ourRange = line;
    } else {
      ourRange = extendRange(doc, ourRange, start);
    }

    if (!addNew) {
      ourIndex = 0;
      setSelection(doc, new Selection([ourRange], 0), sel_mouse);
      startSel = doc.sel;
    } else if (ourIndex == -1) {
      ourIndex = ranges.length;
      setSelection(doc, normalizeSelection(ranges.concat([ourRange]), ourIndex),
                   {scroll: false, origin: "*mouse"});
    } else if (ranges.length > 1 && ranges[ourIndex].empty() && type == "single" && !e.shiftKey) {
      setSelection(doc, normalizeSelection(ranges.slice(0, ourIndex).concat(ranges.slice(ourIndex + 1)), 0),
                   {scroll: false, origin: "*mouse"});
      startSel = doc.sel;
    } else {
      replaceOneSelection(doc, ourIndex, ourRange, sel_mouse);
    }

    var lastPos = start;
    function extendTo(pos) {
      if (cmp(lastPos, pos) == 0) return;
      lastPos = pos;

      if (type == "rect") {
        var ranges = [], tabSize = cm.options.tabSize;
        var startCol = countColumn(getLine(doc, start.line).text, start.ch, tabSize);
        var posCol = countColumn(getLine(doc, pos.line).text, pos.ch, tabSize);
        var left = Math.min(startCol, posCol), right = Math.max(startCol, posCol);
        for (var line = Math.min(start.line, pos.line), end = Math.min(cm.lastLine(), Math.max(start.line, pos.line));
             line <= end; line++) {
          var text = getLine(doc, line).text, leftPos = findColumn(text, left, tabSize);
          if (left == right)
            ranges.push(new Range(Pos(line, leftPos), Pos(line, leftPos)));
          else if (text.length > leftPos)
            ranges.push(new Range(Pos(line, leftPos), Pos(line, findColumn(text, right, tabSize))));
        }
        if (!ranges.length) ranges.push(new Range(start, start));
        setSelection(doc, normalizeSelection(startSel.ranges.slice(0, ourIndex).concat(ranges), ourIndex),
                     {origin: "*mouse", scroll: false});
        cm.scrollIntoView(pos);
      } else {
        var oldRange = ourRange;
        var anchor = oldRange.anchor, head = pos;
        if (type != "single") {
          if (type == "double")
            var range = cm.findWordAt(pos);
          else
            var range = new Range(Pos(pos.line, 0), clipPos(doc, Pos(pos.line + 1, 0)));
          if (cmp(range.anchor, anchor) > 0) {
            head = range.head;
            anchor = minPos(oldRange.from(), range.anchor);
          } else {
            head = range.anchor;
            anchor = maxPos(oldRange.to(), range.head);
          }
        }
        var ranges = startSel.ranges.slice(0);
        ranges[ourIndex] = new Range(clipPos(doc, anchor), head);
        setSelection(doc, normalizeSelection(ranges, ourIndex), sel_mouse);
      }
    }

    var editorSize = display.wrapper.getBoundingClientRect();
    // Used to ensure timeout re-tries don't fire when another extend
    // happened in the meantime (clearTimeout isn't reliable -- at
    // least on Chrome, the timeouts still happen even when cleared,
    // if the clear happens after their scheduled firing time).
    var counter = 0;

    function extend(e) {
      var curCount = ++counter;
      var cur = posFromMouse(cm, e, true, type == "rect");
      if (!cur) return;
      if (cmp(cur, lastPos) != 0) {
        cm.curOp.focus = activeElt();
        extendTo(cur);
        var visible = visibleLines(display, doc);
        if (cur.line >= visible.to || cur.line < visible.from)
          setTimeout(operation(cm, function(){if (counter == curCount) extend(e);}), 150);
      } else {
        var outside = e.clientY < editorSize.top ? -20 : e.clientY > editorSize.bottom ? 20 : 0;
        if (outside) setTimeout(operation(cm, function() {
          if (counter != curCount) return;
          display.scroller.scrollTop += outside;
          extend(e);
        }), 50);
      }
    }

    function done(e) {
      cm.state.selectingText = false;
      counter = Infinity;
      e_preventDefault(e);
      display.input.focus();
      off(document, "mousemove", move);
      off(document, "mouseup", up);
      doc.history.lastSelOrigin = null;
    }

    var move = operation(cm, function(e) {
      if (!e_button(e)) done(e);
      else extend(e);
    });
    var up = operation(cm, done);
    cm.state.selectingText = up;
    on(document, "mousemove", move);
    on(document, "mouseup", up);
  }

  // Determines whether an event happened in the gutter, and fires the
  // handlers for the corresponding event.
  function gutterEvent(cm, e, type, prevent) {
    try { var mX = e.clientX, mY = e.clientY; }
    catch(e) { return false; }
    if (mX >= Math.floor(cm.display.gutters.getBoundingClientRect().right)) return false;
    if (prevent) e_preventDefault(e);

    var display = cm.display;
    var lineBox = display.lineDiv.getBoundingClientRect();

    if (mY > lineBox.bottom || !hasHandler(cm, type)) return e_defaultPrevented(e);
    mY -= lineBox.top - display.viewOffset;

    for (var i = 0; i < cm.options.gutters.length; ++i) {
      var g = display.gutters.childNodes[i];
      if (g && g.getBoundingClientRect().right >= mX) {
        var line = lineAtHeight(cm.doc, mY);
        var gutter = cm.options.gutters[i];
        signal(cm, type, cm, line, gutter, e);
        return e_defaultPrevented(e);
      }
    }
  }

  function clickInGutter(cm, e) {
    return gutterEvent(cm, e, "gutterClick", true);
  }

  // Kludge to work around strange IE behavior where it'll sometimes
  // re-fire a series of drag-related events right after the drop (#1551)
  var lastDrop = 0;

  function onDrop(e) {
    var cm = this;
    clearDragCursor(cm);
    if (signalDOMEvent(cm, e) || eventInWidget(cm.display, e))
      return;
    e_preventDefault(e);
    if (ie) lastDrop = +new Date;
    var pos = posFromMouse(cm, e, true), files = e.dataTransfer.files;
    if (!pos || cm.isReadOnly()) return;
    // Might be a file drop, in which case we simply extract the text
    // and insert it.
    if (files && files.length && window.FileReader && window.File) {
      var n = files.length, text = Array(n), read = 0;
      var loadFile = function(file, i) {
        if (cm.options.allowDropFileTypes &&
            indexOf(cm.options.allowDropFileTypes, file.type) == -1)
          return;

        var reader = new FileReader;
        reader.onload = operation(cm, function() {
          var content = reader.result;
          if (/[\x00-\x08\x0e-\x1f]{2}/.test(content)) content = "";
          text[i] = content;
          if (++read == n) {
            pos = clipPos(cm.doc, pos);
            var change = {from: pos, to: pos,
                          text: cm.doc.splitLines(text.join(cm.doc.lineSeparator())),
                          origin: "paste"};
            makeChange(cm.doc, change);
            setSelectionReplaceHistory(cm.doc, simpleSelection(pos, changeEnd(change)));
          }
        });
        reader.readAsText(file);
      };
      for (var i = 0; i < n; ++i) loadFile(files[i], i);
    } else { // Normal drop
      // Don't do a replace if the drop happened inside of the selected text.
      if (cm.state.draggingText && cm.doc.sel.contains(pos) > -1) {
        cm.state.draggingText(e);
        // Ensure the editor is re-focused
        setTimeout(function() {cm.display.input.focus();}, 20);
        return;
      }
      try {
        var text = e.dataTransfer.getData("Text");
        if (text) {
          if (cm.state.draggingText && !cm.state.draggingText.copy)
            var selected = cm.listSelections();
          setSelectionNoUndo(cm.doc, simpleSelection(pos, pos));
          if (selected) for (var i = 0; i < selected.length; ++i)
            replaceRange(cm.doc, "", selected[i].anchor, selected[i].head, "drag");
          cm.replaceSelection(text, "around", "paste");
          cm.display.input.focus();
        }
      }
      catch(e){}
    }
  }

  function onDragStart(cm, e) {
    if (ie && (!cm.state.draggingText || +new Date - lastDrop < 100)) { e_stop(e); return; }
    if (signalDOMEvent(cm, e) || eventInWidget(cm.display, e)) return;

    e.dataTransfer.setData("Text", cm.getSelection());
    e.dataTransfer.effectAllowed = "copyMove"

    // Use dummy image instead of default browsers image.
    // Recent Safari (~6.0.2) have a tendency to segfault when this happens, so we don't do it there.
    if (e.dataTransfer.setDragImage && !safari) {
      var img = elt("img", null, null, "position: fixed; left: 0; top: 0;");
      img.src = "data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==";
      if (presto) {
        img.width = img.height = 1;
        cm.display.wrapper.appendChild(img);
        // Force a relayout, or Opera won't use our image for some obscure reason
        img._top = img.offsetTop;
      }
      e.dataTransfer.setDragImage(img, 0, 0);
      if (presto) img.parentNode.removeChild(img);
    }
  }

  function onDragOver(cm, e) {
    var pos = posFromMouse(cm, e);
    if (!pos) return;
    var frag = document.createDocumentFragment();
    drawSelectionCursor(cm, pos, frag);
    if (!cm.display.dragCursor) {
      cm.display.dragCursor = elt("div", null, "CodeMirror-cursors CodeMirror-dragcursors");
      cm.display.lineSpace.insertBefore(cm.display.dragCursor, cm.display.cursorDiv);
    }
    removeChildrenAndAdd(cm.display.dragCursor, frag);
  }

  function clearDragCursor(cm) {
    if (cm.display.dragCursor) {
      cm.display.lineSpace.removeChild(cm.display.dragCursor);
      cm.display.dragCursor = null;
    }
  }

  // SCROLL EVENTS

  // Sync the scrollable area and scrollbars, ensure the viewport
  // covers the visible area.
  function setScrollTop(cm, val) {
    if (Math.abs(cm.doc.scrollTop - val) < 2) return;
    cm.doc.scrollTop = val;
    if (!gecko) updateDisplaySimple(cm, {top: val});
    if (cm.display.scroller.scrollTop != val) cm.display.scroller.scrollTop = val;
    cm.display.scrollbars.setScrollTop(val);
    if (gecko) updateDisplaySimple(cm);
    startWorker(cm, 100);
  }
  // Sync scroller and scrollbar, ensure the gutter elements are
  // aligned.
  function setScrollLeft(cm, val, isScroller) {
    if (isScroller ? val == cm.doc.scrollLeft : Math.abs(cm.doc.scrollLeft - val) < 2) return;
    val = Math.min(val, cm.display.scroller.scrollWidth - cm.display.scroller.clientWidth);
    cm.doc.scrollLeft = val;
    alignHorizontally(cm);
    if (cm.display.scroller.scrollLeft != val) cm.display.scroller.scrollLeft = val;
    cm.display.scrollbars.setScrollLeft(val);
  }

  // Since the delta values reported on mouse wheel events are
  // unstandardized between browsers and even browser versions, and
  // generally horribly unpredictable, this code starts by measuring
  // the scroll effect that the first few mouse wheel events have,
  // and, from that, detects the way it can convert deltas to pixel
  // offsets afterwards.
  //
  // The reason we want to know the amount a wheel event will scroll
  // is that it gives us a chance to update the display before the
  // actual scrolling happens, reducing flickering.

  var wheelSamples = 0, wheelPixelsPerUnit = null;
  // Fill in a browser-detected starting value on browsers where we
  // know one. These don't have to be accurate -- the result of them
  // being wrong would just be a slight flicker on the first wheel
  // scroll (if it is large enough).
  if (ie) wheelPixelsPerUnit = -.53;
  else if (gecko) wheelPixelsPerUnit = 15;
  else if (chrome) wheelPixelsPerUnit = -.7;
  else if (safari) wheelPixelsPerUnit = -1/3;

  var wheelEventDelta = function(e) {
    var dx = e.wheelDeltaX, dy = e.wheelDeltaY;
    if (dx == null && e.detail && e.axis == e.HORIZONTAL_AXIS) dx = e.detail;
    if (dy == null && e.detail && e.axis == e.VERTICAL_AXIS) dy = e.detail;
    else if (dy == null) dy = e.wheelDelta;
    return {x: dx, y: dy};
  };
  CodeMirror.wheelEventPixels = function(e) {
    var delta = wheelEventDelta(e);
    delta.x *= wheelPixelsPerUnit;
    delta.y *= wheelPixelsPerUnit;
    return delta;
  };

  function onScrollWheel(cm, e) {
    var delta = wheelEventDelta(e), dx = delta.x, dy = delta.y;

    var display = cm.display, scroll = display.scroller;
    // Quit if there's nothing to scroll here
    var canScrollX = scroll.scrollWidth > scroll.clientWidth;
    var canScrollY = scroll.scrollHeight > scroll.clientHeight;
    if (!(dx && canScrollX || dy && canScrollY)) return;

    // Webkit browsers on OS X abort momentum scrolls when the target
    // of the scroll event is removed from the scrollable element.
    // This hack (see related code in patchDisplay) makes sure the
    // element is kept around.
    if (dy && mac && webkit) {
      outer: for (var cur = e.target, view = display.view; cur != scroll; cur = cur.parentNode) {
        for (var i = 0; i < view.length; i++) {
          if (view[i].node == cur) {
            cm.display.currentWheelTarget = cur;
            break outer;
          }
        }
      }
    }

    // On some browsers, horizontal scrolling will cause redraws to
    // happen before the gutter has been realigned, causing it to
    // wriggle around in a most unseemly way. When we have an
    // estimated pixels/delta value, we just handle horizontal
    // scrolling entirely here. It'll be slightly off from native, but
    // better than glitching out.
    if (dx && !gecko && !presto && wheelPixelsPerUnit != null) {
      if (dy && canScrollY)
        setScrollTop(cm, Math.max(0, Math.min(scroll.scrollTop + dy * wheelPixelsPerUnit, scroll.scrollHeight - scroll.clientHeight)));
      setScrollLeft(cm, Math.max(0, Math.min(scroll.scrollLeft + dx * wheelPixelsPerUnit, scroll.scrollWidth - scroll.clientWidth)));
      // Only prevent default scrolling if vertical scrolling is
      // actually possible. Otherwise, it causes vertical scroll
      // jitter on OSX trackpads when deltaX is small and deltaY
      // is large (issue #3579)
      if (!dy || (dy && canScrollY))
        e_preventDefault(e);
      display.wheelStartX = null; // Abort measurement, if in progress
      return;
    }

    // 'Project' the visible viewport to cover the area that is being
    // scrolled into view (if we know enough to estimate it).
    if (dy && wheelPixelsPerUnit != null) {
      var pixels = dy * wheelPixelsPerUnit;
      var top = cm.doc.scrollTop, bot = top + display.wrapper.clientHeight;
      if (pixels < 0) top = Math.max(0, top + pixels - 50);
      else bot = Math.min(cm.doc.height, bot + pixels + 50);
      updateDisplaySimple(cm, {top: top, bottom: bot});
    }

    if (wheelSamples < 20) {
      if (display.wheelStartX == null) {
        display.wheelStartX = scroll.scrollLeft; display.wheelStartY = scroll.scrollTop;
        display.wheelDX = dx; display.wheelDY = dy;
        setTimeout(function() {
          if (display.wheelStartX == null) return;
          var movedX = scroll.scrollLeft - display.wheelStartX;
          var movedY = scroll.scrollTop - display.wheelStartY;
          var sample = (movedY && display.wheelDY && movedY / display.wheelDY) ||
            (movedX && display.wheelDX && movedX / display.wheelDX);
          display.wheelStartX = display.wheelStartY = null;
          if (!sample) return;
          wheelPixelsPerUnit = (wheelPixelsPerUnit * wheelSamples + sample) / (wheelSamples + 1);
          ++wheelSamples;
        }, 200);
      } else {
        display.wheelDX += dx; display.wheelDY += dy;
      }
    }
  }

  // KEY EVENTS

  // Run a handler that was bound to a key.
  function doHandleBinding(cm, bound, dropShift) {
    if (typeof bound == "string") {
      bound = commands[bound];
      if (!bound) return false;
    }
    // Ensure previous input has been read, so that the handler sees a
    // consistent view of the document
    cm.display.input.ensurePolled();
    var prevShift = cm.display.shift, done = false;
    try {
      if (cm.isReadOnly()) cm.state.suppressEdits = true;
      if (dropShift) cm.display.shift = false;
      done = bound(cm) != Pass;
    } finally {
      cm.display.shift = prevShift;
      cm.state.suppressEdits = false;
    }
    return done;
  }

  function lookupKeyForEditor(cm, name, handle) {
    for (var i = 0; i < cm.state.keyMaps.length; i++) {
      var result = lookupKey(name, cm.state.keyMaps[i], handle, cm);
      if (result) return result;
    }
    return (cm.options.extraKeys && lookupKey(name, cm.options.extraKeys, handle, cm))
      || lookupKey(name, cm.options.keyMap, handle, cm);
  }

  var stopSeq = new Delayed;
  function dispatchKey(cm, name, e, handle) {
    var seq = cm.state.keySeq;
    if (seq) {
      if (isModifierKey(name)) return "handled";
      stopSeq.set(50, function() {
        if (cm.state.keySeq == seq) {
          cm.state.keySeq = null;
          cm.display.input.reset();
        }
      });
      name = seq + " " + name;
    }
    var result = lookupKeyForEditor(cm, name, handle);

    if (result == "multi")
      cm.state.keySeq = name;
    if (result == "handled")
      signalLater(cm, "keyHandled", cm, name, e);

    if (result == "handled" || result == "multi") {
      e_preventDefault(e);
      restartBlink(cm);
    }

    if (seq && !result && /\'$/.test(name)) {
      e_preventDefault(e);
      return true;
    }
    return !!result;
  }

  // Handle a key from the keydown event.
  function handleKeyBinding(cm, e) {
    var name = keyName(e, true);
    if (!name) return false;

    if (e.shiftKey && !cm.state.keySeq) {
      // First try to resolve full name (including 'Shift-'). Failing
      // that, see if there is a cursor-motion command (starting with
      // 'go') bound to the keyname without 'Shift-'.
      return dispatchKey(cm, "Shift-" + name, e, function(b) {return doHandleBinding(cm, b, true);})
          || dispatchKey(cm, name, e, function(b) {
               if (typeof b == "string" ? /^go[A-Z]/.test(b) : b.motion)
                 return doHandleBinding(cm, b);
             });
    } else {
      return dispatchKey(cm, name, e, function(b) { return doHandleBinding(cm, b); });
    }
  }

  // Handle a key from the keypress event
  function handleCharBinding(cm, e, ch) {
    return dispatchKey(cm, "'" + ch + "'", e,
                       function(b) { return doHandleBinding(cm, b, true); });
  }

  var lastStoppedKey = null;
  function onKeyDown(e) {
    var cm = this;
    cm.curOp.focus = activeElt();
    if (signalDOMEvent(cm, e)) return;
    // IE does strange things with escape.
    if (ie && ie_version < 11 && e.keyCode == 27) e.returnValue = false;
    var code = e.keyCode;
    cm.display.shift = code == 16 || e.shiftKey;
    var handled = handleKeyBinding(cm, e);
    if (presto) {
      lastStoppedKey = handled ? code : null;
      // Opera has no cut event... we try to at least catch the key combo
      if (!handled && code == 88 && !hasCopyEvent && (mac ? e.metaKey : e.ctrlKey))
        cm.replaceSelection("", null, "cut");
    }

    // Turn mouse into crosshair when Alt is held on Mac.
    if (code == 18 && !/\bCodeMirror-crosshair\b/.test(cm.display.lineDiv.className))
      showCrossHair(cm);
  }

  function showCrossHair(cm) {
    var lineDiv = cm.display.lineDiv;
    addClass(lineDiv, "CodeMirror-crosshair");

    function up(e) {
      if (e.keyCode == 18 || !e.altKey) {
        rmClass(lineDiv, "CodeMirror-crosshair");
        off(document, "keyup", up);
        off(document, "mouseover", up);
      }
    }
    on(document, "keyup", up);
    on(document, "mouseover", up);
  }

  function onKeyUp(e) {
    if (e.keyCode == 16) this.doc.sel.shift = false;
    signalDOMEvent(this, e);
  }

  function onKeyPress(e) {
    var cm = this;
    if (eventInWidget(cm.display, e) || signalDOMEvent(cm, e) || e.ctrlKey && !e.altKey || mac && e.metaKey) return;
    var keyCode = e.keyCode, charCode = e.charCode;
    if (presto && keyCode == lastStoppedKey) {lastStoppedKey = null; e_preventDefault(e); return;}
    if ((presto && (!e.which || e.which < 10)) && handleKeyBinding(cm, e)) return;
    var ch = String.fromCharCode(charCode == null ? keyCode : charCode);
    if (handleCharBinding(cm, e, ch)) return;
    cm.display.input.onKeyPress(e);
  }

  // FOCUS/BLUR EVENTS

  function delayBlurEvent(cm) {
    cm.state.delayingBlurEvent = true;
    setTimeout(function() {
      if (cm.state.delayingBlurEvent) {
        cm.state.delayingBlurEvent = false;
        onBlur(cm);
      }
    }, 100);
  }

  function onFocus(cm) {
    if (cm.state.delayingBlurEvent) cm.state.delayingBlurEvent = false;

    if (cm.options.readOnly == "nocursor") return;
    if (!cm.state.focused) {
      signal(cm, "focus", cm);
      cm.state.focused = true;
      addClass(cm.display.wrapper, "CodeMirror-focused");
      // This test prevents this from firing when a context
      // menu is closed (since the input reset would kill the
      // select-all detection hack)
      if (!cm.curOp && cm.display.selForContextMenu != cm.doc.sel) {
        cm.display.input.reset();
        if (webkit) setTimeout(function() { cm.display.input.reset(true); }, 20); // Issue #1730
      }
      cm.display.input.receivedFocus();
    }
    restartBlink(cm);
  }
  function onBlur(cm) {
    if (cm.state.delayingBlurEvent) return;

    if (cm.state.focused) {
      signal(cm, "blur", cm);
      cm.state.focused = false;
      rmClass(cm.display.wrapper, "CodeMirror-focused");
    }
    clearInterval(cm.display.blinker);
    setTimeout(function() {if (!cm.state.focused) cm.display.shift = false;}, 150);
  }

  // CONTEXT MENU HANDLING

  // To make the context menu work, we need to briefly unhide the
  // textarea (making it as unobtrusive as possible) to let the
  // right-click take effect on it.
  function onContextMenu(cm, e) {
    if (eventInWidget(cm.display, e) || contextMenuInGutter(cm, e)) return;
    if (signalDOMEvent(cm, e, "contextmenu")) return;
    cm.display.input.onContextMenu(e);
  }

  function contextMenuInGutter(cm, e) {
    if (!hasHandler(cm, "gutterContextMenu")) return false;
    return gutterEvent(cm, e, "gutterContextMenu", false);
  }

  // UPDATING

  // Compute the position of the end of a change (its 'to' property
  // refers to the pre-change end).
  var changeEnd = CodeMirror.changeEnd = function(change) {
    if (!change.text) return change.to;
    return Pos(change.from.line + change.text.length - 1,
               lst(change.text).length + (change.text.length == 1 ? change.from.ch : 0));
  };

  // Adjust a position to refer to the post-change position of the
  // same text, or the end of the change if the change covers it.
  function adjustForChange(pos, change) {
    if (cmp(pos, change.from) < 0) return pos;
    if (cmp(pos, change.to) <= 0) return changeEnd(change);

    var line = pos.line + change.text.length - (change.to.line - change.from.line) - 1, ch = pos.ch;
    if (pos.line == change.to.line) ch += changeEnd(change).ch - change.to.ch;
    return Pos(line, ch);
  }

  function computeSelAfterChange(doc, change) {
    var out = [];
    for (var i = 0; i < doc.sel.ranges.length; i++) {
      var range = doc.sel.ranges[i];
      out.push(new Range(adjustForChange(range.anchor, change),
                         adjustForChange(range.head, change)));
    }
    return normalizeSelection(out, doc.sel.primIndex);
  }

  function offsetPos(pos, old, nw) {
    if (pos.line == old.line)
      return Pos(nw.line, pos.ch - old.ch + nw.ch);
    else
      return Pos(nw.line + (pos.line - old.line), pos.ch);
  }

  // Used by replaceSelections to allow moving the selection to the
  // start or around the replaced test. Hint may be "start" or "around".
  function computeReplacedSel(doc, changes, hint) {
    var out = [];
    var oldPrev = Pos(doc.first, 0), newPrev = oldPrev;
    for (var i = 0; i < changes.length; i++) {
      var change = changes[i];
      var from = offsetPos(change.from, oldPrev, newPrev);
      var to = offsetPos(changeEnd(change), oldPrev, newPrev);
      oldPrev = change.to;
      newPrev = to;
      if (hint == "around") {
        var range = doc.sel.ranges[i], inv = cmp(range.head, range.anchor) < 0;
        out[i] = new Range(inv ? to : from, inv ? from : to);
      } else {
        out[i] = new Range(from, from);
      }
    }
    return new Selection(out, doc.sel.primIndex);
  }

  // Allow "beforeChange" event handlers to influence a change
  function filterChange(doc, change, update) {
    var obj = {
      canceled: false,
      from: change.from,
      to: change.to,
      text: change.text,
      origin: change.origin,
      cancel: function() { this.canceled = true; }
    };
    if (update) obj.update = function(from, to, text, origin) {
      if (from) this.from = clipPos(doc, from);
      if (to) this.to = clipPos(doc, to);
      if (text) this.text = text;
      if (origin !== undefined) this.origin = origin;
    };
    signal(doc, "beforeChange", doc, obj);
    if (doc.cm) signal(doc.cm, "beforeChange", doc.cm, obj);

    if (obj.canceled) return null;
    return {from: obj.from, to: obj.to, text: obj.text, origin: obj.origin};
  }

  // Apply a change to a document, and add it to the document's
  // history, and propagating it to all linked documents.
  function makeChange(doc, change, ignoreReadOnly) {
    if (doc.cm) {
      if (!doc.cm.curOp) return operation(doc.cm, makeChange)(doc, change, ignoreReadOnly);
      if (doc.cm.state.suppressEdits) return;
    }

    if (hasHandler(doc, "beforeChange") || doc.cm && hasHandler(doc.cm, "beforeChange")) {
      change = filterChange(doc, change, true);
      if (!change) return;
    }

    // Possibly split or suppress the update based on the presence
    // of read-only spans in its range.
    var split = sawReadOnlySpans && !ignoreReadOnly && removeReadOnlyRanges(doc, change.from, change.to);
    if (split) {
      for (var i = split.length - 1; i >= 0; --i)
        makeChangeInner(doc, {from: split[i].from, to: split[i].to, text: i ? [""] : change.text});
    } else {
      makeChangeInner(doc, change);
    }
  }

  function makeChangeInner(doc, change) {
    if (change.text.length == 1 && change.text[0] == "" && cmp(change.from, change.to) == 0) return;
    var selAfter = computeSelAfterChange(doc, change);
    addChangeToHistory(doc, change, selAfter, doc.cm ? doc.cm.curOp.id : NaN);

    makeChangeSingleDoc(doc, change, selAfter, stretchSpansOverChange(doc, change));
    var rebased = [];

    linkedDocs(doc, function(doc, sharedHist) {
      if (!sharedHist && indexOf(rebased, doc.history) == -1) {
        rebaseHist(doc.history, change);
        rebased.push(doc.history);
      }
      makeChangeSingleDoc(doc, change, null, stretchSpansOverChange(doc, change));
    });
  }

  // Revert a change stored in a document's history.
  function makeChangeFromHistory(doc, type, allowSelectionOnly) {
    if (doc.cm && doc.cm.state.suppressEdits) return;

    var hist = doc.history, event, selAfter = doc.sel;
    var source = type == "undo" ? hist.done : hist.undone, dest = type == "undo" ? hist.undone : hist.done;

    // Verify that there is a useable event (so that ctrl-z won't
    // needlessly clear selection events)
    for (var i = 0; i < source.length; i++) {
      event = source[i];
      if (allowSelectionOnly ? event.ranges && !event.equals(doc.sel) : !event.ranges)
        break;
    }
    if (i == source.length) return;
    hist.lastOrigin = hist.lastSelOrigin = null;

    for (;;) {
      event = source.pop();
      if (event.ranges) {
        pushSelectionToHistory(event, dest);
        if (allowSelectionOnly && !event.equals(doc.sel)) {
          setSelection(doc, event, {clearRedo: false});
          return;
        }
        selAfter = event;
      }
      else break;
    }

    // Build up a reverse change object to add to the opposite history
    // stack (redo when undoing, and vice versa).
    var antiChanges = [];
    pushSelectionToHistory(selAfter, dest);
    dest.push({changes: antiChanges, generation: hist.generation});
    hist.generation = event.generation || ++hist.maxGeneration;

    var filter = hasHandler(doc, "beforeChange") || doc.cm && hasHandler(doc.cm, "beforeChange");

    for (var i = event.changes.length - 1; i >= 0; --i) {
      var change = event.changes[i];
      change.origin = type;
      if (filter && !filterChange(doc, change, false)) {
        source.length = 0;
        return;
      }

      antiChanges.push(historyChangeFromChange(doc, change));

      var after = i ? computeSelAfterChange(doc, change) : lst(source);
      makeChangeSingleDoc(doc, change, after, mergeOldSpans(doc, change));
      if (!i && doc.cm) doc.cm.scrollIntoView({from: change.from, to: changeEnd(change)});
      var rebased = [];

      // Propagate to the linked documents
      linkedDocs(doc, function(doc, sharedHist) {
        if (!sharedHist && indexOf(rebased, doc.history) == -1) {
          rebaseHist(doc.history, change);
          rebased.push(doc.history);
        }
        makeChangeSingleDoc(doc, change, null, mergeOldSpans(doc, change));
      });
    }
  }

  // Sub-views need their line numbers shifted when text is added
  // above or below them in the parent document.
  function shiftDoc(doc, distance) {
    if (distance == 0) return;
    doc.first += distance;
    doc.sel = new Selection(map(doc.sel.ranges, function(range) {
      return new Range(Pos(range.anchor.line + distance, range.anchor.ch),
                       Pos(range.head.line + distance, range.head.ch));
    }), doc.sel.primIndex);
    if (doc.cm) {
      regChange(doc.cm, doc.first, doc.first - distance, distance);
      for (var d = doc.cm.display, l = d.viewFrom; l < d.viewTo; l++)
        regLineChange(doc.cm, l, "gutter");
    }
  }

  // More lower-level change function, handling only a single document
  // (not linked ones).
  function makeChangeSingleDoc(doc, change, selAfter, spans) {
    if (doc.cm && !doc.cm.curOp)
      return operation(doc.cm, makeChangeSingleDoc)(doc, change, selAfter, spans);

    if (change.to.line < doc.first) {
      shiftDoc(doc, change.text.length - 1 - (change.to.line - change.from.line));
      return;
    }
    if (change.from.line > doc.lastLine()) return;

    // Clip the change to the size of this doc
    if (change.from.line < doc.first) {
      var shift = change.text.length - 1 - (doc.first - change.from.line);
      shiftDoc(doc, shift);
      change = {from: Pos(doc.first, 0), to: Pos(change.to.line + shift, change.to.ch),
                text: [lst(change.text)], origin: change.origin};
    }
    var last = doc.lastLine();
    if (change.to.line > last) {
      change = {from: change.from, to: Pos(last, getLine(doc, last).text.length),
                text: [change.text[0]], origin: change.origin};
    }

    change.removed = getBetween(doc, change.from, change.to);

    if (!selAfter) selAfter = computeSelAfterChange(doc, change);
    if (doc.cm) makeChangeSingleDocInEditor(doc.cm, change, spans);
    else updateDoc(doc, change, spans);
    setSelectionNoUndo(doc, selAfter, sel_dontScroll);
  }

  // Handle the interaction of a change to a document with the editor
  // that this document is part of.
  function makeChangeSingleDocInEditor(cm, change, spans) {
    var doc = cm.doc, display = cm.display, from = change.from, to = change.to;

    var recomputeMaxLength = false, checkWidthStart = from.line;
    if (!cm.options.lineWrapping) {
      checkWidthStart = lineNo(visualLine(getLine(doc, from.line)));
      doc.iter(checkWidthStart, to.line + 1, function(line) {
        if (line == display.maxLine) {
          recomputeMaxLength = true;
          return true;
        }
      });
    }

    if (doc.sel.contains(change.from, change.to) > -1)
      signalCursorActivity(cm);

    updateDoc(doc, change, spans, estimateHeight(cm));

    if (!cm.options.lineWrapping) {
      doc.iter(checkWidthStart, from.line + change.text.length, function(line) {
        var len = lineLength(line);
        if (len > display.maxLineLength) {
          display.maxLine = line;
          display.maxLineLength = len;
          display.maxLineChanged = true;
          recomputeMaxLength = false;
        }
      });
      if (recomputeMaxLength) cm.curOp.updateMaxLine = true;
    }

    // Adjust frontier, schedule worker
    doc.frontier = Math.min(doc.frontier, from.line);
    startWorker(cm, 400);

    var lendiff = change.text.length - (to.line - from.line) - 1;
    // Remember that these lines changed, for updating the display
    if (change.full)
      regChange(cm);
    else if (from.line == to.line && change.text.length == 1 && !isWholeLineUpdate(cm.doc, change))
      regLineChange(cm, from.line, "text");
    else
      regChange(cm, from.line, to.line + 1, lendiff);

    var changesHandler = hasHandler(cm, "changes"), changeHandler = hasHandler(cm, "change");
    if (changeHandler || changesHandler) {
      var obj = {
        from: from, to: to,
        text: change.text,
        removed: change.removed,
        origin: change.origin
      };
      if (changeHandler) signalLater(cm, "change", cm, obj);
      if (changesHandler) (cm.curOp.changeObjs || (cm.curOp.changeObjs = [])).push(obj);
    }
    cm.display.selForContextMenu = null;
  }

  function replaceRange(doc, code, from, to, origin) {
    if (!to) to = from;
    if (cmp(to, from) < 0) { var tmp = to; to = from; from = tmp; }
    if (typeof code == "string") code = doc.splitLines(code);
    makeChange(doc, {from: from, to: to, text: code, origin: origin});
  }

  // SCROLLING THINGS INTO VIEW

  // If an editor sits on the top or bottom of the window, partially
  // scrolled out of view, this ensures that the cursor is visible.
  function maybeScrollWindow(cm, coords) {
    if (signalDOMEvent(cm, "scrollCursorIntoView")) return;

    var display = cm.display, box = display.sizer.getBoundingClientRect(), doScroll = null;
    if (coords.top + box.top < 0) doScroll = true;
    else if (coords.bottom + box.top > (window.innerHeight || document.documentElement.clientHeight)) doScroll = false;
    if (doScroll != null && !phantom) {
      var scrollNode = elt("div", "\u200b", null, "position: absolute; top: " +
                           (coords.top - display.viewOffset - paddingTop(cm.display)) + "px; height: " +
                           (coords.bottom - coords.top + scrollGap(cm) + display.barHeight) + "px; left: " +
                           coords.left + "px; width: 2px;");
      cm.display.lineSpace.appendChild(scrollNode);
      scrollNode.scrollIntoView(doScroll);
      cm.display.lineSpace.removeChild(scrollNode);
    }
  }

  // Scroll a given position into view (immediately), verifying that
  // it actually became visible (as line heights are accurately
  // measured, the position of something may 'drift' during drawing).
  function scrollPosIntoView(cm, pos, end, margin) {
    if (margin == null) margin = 0;
    for (var limit = 0; limit < 5; limit++) {
      var changed = false, coords = cursorCoords(cm, pos);
      var endCoords = !end || end == pos ? coords : cursorCoords(cm, end);
      var scrollPos = calculateScrollPos(cm, Math.min(coords.left, endCoords.left),
                                         Math.min(coords.top, endCoords.top) - margin,
                                         Math.max(coords.left, endCoords.left),
                                         Math.max(coords.bottom, endCoords.bottom) + margin);
      var startTop = cm.doc.scrollTop, startLeft = cm.doc.scrollLeft;
      if (scrollPos.scrollTop != null) {
        setScrollTop(cm, scrollPos.scrollTop);
        if (Math.abs(cm.doc.scrollTop - startTop) > 1) changed = true;
      }
      if (scrollPos.scrollLeft != null) {
        setScrollLeft(cm, scrollPos.scrollLeft);
        if (Math.abs(cm.doc.scrollLeft - startLeft) > 1) changed = true;
      }
      if (!changed) break;
    }
    return coords;
  }

  // Scroll a given set of coordinates into view (immediately).
  function scrollIntoView(cm, x1, y1, x2, y2) {
    var scrollPos = calculateScrollPos(cm, x1, y1, x2, y2);
    if (scrollPos.scrollTop != null) setScrollTop(cm, scrollPos.scrollTop);
    if (scrollPos.scrollLeft != null) setScrollLeft(cm, scrollPos.scrollLeft);
  }

  // Calculate a new scroll position needed to scroll the given
  // rectangle into view. Returns an object with scrollTop and
  // scrollLeft properties. When these are undefined, the
  // vertical/horizontal position does not need to be adjusted.
  function calculateScrollPos(cm, x1, y1, x2, y2) {
    var display = cm.display, snapMargin = textHeight(cm.display);
    if (y1 < 0) y1 = 0;
    var screentop = cm.curOp && cm.curOp.scrollTop != null ? cm.curOp.scrollTop : display.scroller.scrollTop;
    var screen = displayHeight(cm), result = {};
    if (y2 - y1 > screen) y2 = y1 + screen;
    var docBottom = cm.doc.height + paddingVert(display);
    var atTop = y1 < snapMargin, atBottom = y2 > docBottom - snapMargin;
    if (y1 < screentop) {
      result.scrollTop = atTop ? 0 : y1;
    } else if (y2 > screentop + screen) {
      var newTop = Math.min(y1, (atBottom ? docBottom : y2) - screen);
      if (newTop != screentop) result.scrollTop = newTop;
    }

    var screenleft = cm.curOp && cm.curOp.scrollLeft != null ? cm.curOp.scrollLeft : display.scroller.scrollLeft;
    var screenw = displayWidth(cm) - (cm.options.fixedGutter ? display.gutters.offsetWidth : 0);
    var tooWide = x2 - x1 > screenw;
    if (tooWide) x2 = x1 + screenw;
    if (x1 < 10)
      result.scrollLeft = 0;
    else if (x1 < screenleft)
      result.scrollLeft = Math.max(0, x1 - (tooWide ? 0 : 10));
    else if (x2 > screenw + screenleft - 3)
      result.scrollLeft = x2 + (tooWide ? 0 : 10) - screenw;
    return result;
  }

  // Store a relative adjustment to the scroll position in the current
  // operation (to be applied when the operation finishes).
  function addToScrollPos(cm, left, top) {
    if (left != null || top != null) resolveScrollToPos(cm);
    if (left != null)
      cm.curOp.scrollLeft = (cm.curOp.scrollLeft == null ? cm.doc.scrollLeft : cm.curOp.scrollLeft) + left;
    if (top != null)
      cm.curOp.scrollTop = (cm.curOp.scrollTop == null ? cm.doc.scrollTop : cm.curOp.scrollTop) + top;
  }

  // Make sure that at the end of the operation the current cursor is
  // shown.
  function ensureCursorVisible(cm) {
    resolveScrollToPos(cm);
    var cur = cm.getCursor(), from = cur, to = cur;
    if (!cm.options.lineWrapping) {
      from = cur.ch ? Pos(cur.line, cur.ch - 1) : cur;
      to = Pos(cur.line, cur.ch + 1);
    }
    cm.curOp.scrollToPos = {from: from, to: to, margin: cm.options.cursorScrollMargin, isCursor: true};
  }

  // When an operation has its scrollToPos property set, and another
  // scroll action is applied before the end of the operation, this
  // 'simulates' scrolling that position into view in a cheap way, so
  // that the effect of intermediate scroll commands is not ignored.
  function resolveScrollToPos(cm) {
    var range = cm.curOp.scrollToPos;
    if (range) {
      cm.curOp.scrollToPos = null;
      var from = estimateCoords(cm, range.from), to = estimateCoords(cm, range.to);
      var sPos = calculateScrollPos(cm, Math.min(from.left, to.left),
                                    Math.min(from.top, to.top) - range.margin,
                                    Math.max(from.right, to.right),
                                    Math.max(from.bottom, to.bottom) + range.margin);
      cm.scrollTo(sPos.scrollLeft, sPos.scrollTop);
    }
  }

  // API UTILITIES

  // Indent the given line. The how parameter can be "smart",
  // "add"/null, "subtract", or "prev". When aggressive is false
  // (typically set to true for forced single-line indents), empty
  // lines are not indented, and places where the mode returns Pass
  // are left alone.
  function indentLine(cm, n, how, aggressive) {
    var doc = cm.doc, state;
    if (how == null) how = "add";
    if (how == "smart") {
      // Fall back to "prev" when the mode doesn't have an indentation
      // method.
      if (!doc.mode.indent) how = "prev";
      else state = getStateBefore(cm, n);
    }

    var tabSize = cm.options.tabSize;
    var line = getLine(doc, n), curSpace = countColumn(line.text, null, tabSize);
    if (line.stateAfter) line.stateAfter = null;
    var curSpaceString = line.text.match(/^\s*/)[0], indentation;
    if (!aggressive && !/\S/.test(line.text)) {
      indentation = 0;
      how = "not";
    } else if (how == "smart") {
      indentation = doc.mode.indent(state, line.text.slice(curSpaceString.length), line.text);
      if (indentation == Pass || indentation > 150) {
        if (!aggressive) return;
        how = "prev";
      }
    }
    if (how == "prev") {
      if (n > doc.first) indentation = countColumn(getLine(doc, n-1).text, null, tabSize);
      else indentation = 0;
    } else if (how == "add") {
      indentation = curSpace + cm.options.indentUnit;
    } else if (how == "subtract") {
      indentation = curSpace - cm.options.indentUnit;
    } else if (typeof how == "number") {
      indentation = curSpace + how;
    }
    indentation = Math.max(0, indentation);

    var indentString = "", pos = 0;
    if (cm.options.indentWithTabs)
      for (var i = Math.floor(indentation / tabSize); i; --i) {pos += tabSize; indentString += "\t";}
    if (pos < indentation) indentString += spaceStr(indentation - pos);

    if (indentString != curSpaceString) {
      replaceRange(doc, indentString, Pos(n, 0), Pos(n, curSpaceString.length), "+input");
      line.stateAfter = null;
      return true;
    } else {
      // Ensure that, if the cursor was in the whitespace at the start
      // of the line, it is moved to the end of that space.
      for (var i = 0; i < doc.sel.ranges.length; i++) {
        var range = doc.sel.ranges[i];
        if (range.head.line == n && range.head.ch < curSpaceString.length) {
          var pos = Pos(n, curSpaceString.length);
          replaceOneSelection(doc, i, new Range(pos, pos));
          break;
        }
      }
    }
  }

  // Utility for applying a change to a line by handle or number,
  // returning the number and optionally registering the line as
  // changed.
  function changeLine(doc, handle, changeType, op) {
    var no = handle, line = handle;
    if (typeof handle == "number") line = getLine(doc, clipLine(doc, handle));
    else no = lineNo(handle);
    if (no == null) return null;
    if (op(line, no) && doc.cm) regLineChange(doc.cm, no, changeType);
    return line;
  }

  // Helper for deleting text near the selection(s), used to implement
  // backspace, delete, and similar functionality.
  function deleteNearSelection(cm, compute) {
    var ranges = cm.doc.sel.ranges, kill = [];
    // Build up a set of ranges to kill first, merging overlapping
    // ranges.
    for (var i = 0; i < ranges.length; i++) {
      var toKill = compute(ranges[i]);
      while (kill.length && cmp(toKill.from, lst(kill).to) <= 0) {
        var replaced = kill.pop();
        if (cmp(replaced.from, toKill.from) < 0) {
          toKill.from = replaced.from;
          break;
        }
      }
      kill.push(toKill);
    }
    // Next, remove those actual ranges.
    runInOp(cm, function() {
      for (var i = kill.length - 1; i >= 0; i--)
        replaceRange(cm.doc, "", kill[i].from, kill[i].to, "+delete");
      ensureCursorVisible(cm);
    });
  }

  // Used for horizontal relative motion. Dir is -1 or 1 (left or
  // right), unit can be "char", "column" (like char, but doesn't
  // cross line boundaries), "word" (across next word), or "group" (to
  // the start of next group of word or non-word-non-whitespace
  // chars). The visually param controls whether, in right-to-left
  // text, direction 1 means to move towards the next index in the
  // string, or towards the character to the right of the current
  // position. The resulting position will have a hitSide=true
  // property if it reached the end of the document.
  function findPosH(doc, pos, dir, unit, visually) {
    var line = pos.line, ch = pos.ch, origDir = dir;
    var lineObj = getLine(doc, line);
    function findNextLine() {
      var l = line + dir;
      if (l < doc.first || l >= doc.first + doc.size) return false
      line = l;
      return lineObj = getLine(doc, l);
    }
    function moveOnce(boundToLine) {
      var next = (visually ? moveVisually : moveLogically)(lineObj, ch, dir, true);
      if (next == null) {
        if (!boundToLine && findNextLine()) {
          if (visually) ch = (dir < 0 ? lineRight : lineLeft)(lineObj);
          else ch = dir < 0 ? lineObj.text.length : 0;
        } else return false
      } else ch = next;
      return true;
    }

    if (unit == "char") {
      moveOnce()
    } else if (unit == "column") {
      moveOnce(true)
    } else if (unit == "word" || unit == "group") {
      var sawType = null, group = unit == "group";
      var helper = doc.cm && doc.cm.getHelper(pos, "wordChars");
      for (var first = true;; first = false) {
        if (dir < 0 && !moveOnce(!first)) break;
        var cur = lineObj.text.charAt(ch) || "\n";
        var type = isWordChar(cur, helper) ? "w"
          : group && cur == "\n" ? "n"
          : !group || /\s/.test(cur) ? null
          : "p";
        if (group && !first && !type) type = "s";
        if (sawType && sawType != type) {
          if (dir < 0) {dir = 1; moveOnce();}
          break;
        }

        if (type) sawType = type;
        if (dir > 0 && !moveOnce(!first)) break;
      }
    }
    var result = skipAtomic(doc, Pos(line, ch), pos, origDir, true);
    if (!cmp(pos, result)) result.hitSide = true;
    return result;
  }

  // For relative vertical movement. Dir may be -1 or 1. Unit can be
  // "page" or "line". The resulting position will have a hitSide=true
  // property if it reached the end of the document.
  function findPosV(cm, pos, dir, unit) {
    var doc = cm.doc, x = pos.left, y;
    if (unit == "page") {
      var pageSize = Math.min(cm.display.wrapper.clientHeight, window.innerHeight || document.documentElement.clientHeight);
      y = pos.top + dir * (pageSize - (dir < 0 ? 1.5 : .5) * textHeight(cm.display));
    } else if (unit == "line") {
      y = dir > 0 ? pos.bottom + 3 : pos.top - 3;
    }
    for (;;) {
      var target = coordsChar(cm, x, y);
      if (!target.outside) break;
      if (dir < 0 ? y <= 0 : y >= doc.height) { target.hitSide = true; break; }
      y += dir * 5;
    }
    return target;
  }

  // EDITOR METHODS

  // The publicly visible API. Note that methodOp(f) means
  // 'wrap f in an operation, performed on its `this` parameter'.

  // This is not the complete set of editor methods. Most of the
  // methods defined on the Doc type are also injected into
  // CodeMirror.prototype, for backwards compatibility and
  // convenience.

  CodeMirror.prototype = {
    constructor: CodeMirror,
    focus: function(){window.focus(); this.display.input.focus();},

    setOption: function(option, value) {
      var options = this.options, old = options[option];
      if (options[option] == value && option != "mode") return;
      options[option] = value;
      if (optionHandlers.hasOwnProperty(option))
        operation(this, optionHandlers[option])(this, value, old);
    },

    getOption: function(option) {return this.options[option];},
    getDoc: function() {return this.doc;},

    addKeyMap: function(map, bottom) {
      this.state.keyMaps[bottom ? "push" : "unshift"](getKeyMap(map));
    },
    removeKeyMap: function(map) {
      var maps = this.state.keyMaps;
      for (var i = 0; i < maps.length; ++i)
        if (maps[i] == map || maps[i].name == map) {
          maps.splice(i, 1);
          return true;
        }
    },

    addOverlay: methodOp(function(spec, options) {
      var mode = spec.token ? spec : CodeMirror.getMode(this.options, spec);
      if (mode.startState) throw new Error("Overlays may not be stateful.");
      this.state.overlays.push({mode: mode, modeSpec: spec, opaque: options && options.opaque});
      this.state.modeGen++;
      regChange(this);
    }),
    removeOverlay: methodOp(function(spec) {
      var overlays = this.state.overlays;
      for (var i = 0; i < overlays.length; ++i) {
        var cur = overlays[i].modeSpec;
        if (cur == spec || typeof spec == "string" && cur.name == spec) {
          overlays.splice(i, 1);
          this.state.modeGen++;
          regChange(this);
          return;
        }
      }
    }),

    indentLine: methodOp(function(n, dir, aggressive) {
      if (typeof dir != "string" && typeof dir != "number") {
        if (dir == null) dir = this.options.smartIndent ? "smart" : "prev";
        else dir = dir ? "add" : "subtract";
      }
      if (isLine(this.doc, n)) indentLine(this, n, dir, aggressive);
    }),
    indentSelection: methodOp(function(how) {
      var ranges = this.doc.sel.ranges, end = -1;
      for (var i = 0; i < ranges.length; i++) {
        var range = ranges[i];
        if (!range.empty()) {
          var from = range.from(), to = range.to();
          var start = Math.max(end, from.line);
          end = Math.min(this.lastLine(), to.line - (to.ch ? 0 : 1)) + 1;
          for (var j = start; j < end; ++j)
            indentLine(this, j, how);
          var newRanges = this.doc.sel.ranges;
          if (from.ch == 0 && ranges.length == newRanges.length && newRanges[i].from().ch > 0)
            replaceOneSelection(this.doc, i, new Range(from, newRanges[i].to()), sel_dontScroll);
        } else if (range.head.line > end) {
          indentLine(this, range.head.line, how, true);
          end = range.head.line;
          if (i == this.doc.sel.primIndex) ensureCursorVisible(this);
        }
      }
    }),

    // Fetch the parser token for a given character. Useful for hacks
    // that want to inspect the mode state (say, for completion).
    getTokenAt: function(pos, precise) {
      return takeToken(this, pos, precise);
    },

    getLineTokens: function(line, precise) {
      return takeToken(this, Pos(line), precise, true);
    },

    getTokenTypeAt: function(pos) {
      pos = clipPos(this.doc, pos);
      var styles = getLineStyles(this, getLine(this.doc, pos.line));
      var before = 0, after = (styles.length - 1) / 2, ch = pos.ch;
      var type;
      if (ch == 0) type = styles[2];
      else for (;;) {
        var mid = (before + after) >> 1;
        if ((mid ? styles[mid * 2 - 1] : 0) >= ch) after = mid;
        else if (styles[mid * 2 + 1] < ch) before = mid + 1;
        else { type = styles[mid * 2 + 2]; break; }
      }
      var cut = type ? type.indexOf("cm-overlay ") : -1;
      return cut < 0 ? type : cut == 0 ? null : type.slice(0, cut - 1);
    },

    getModeAt: function(pos) {
      var mode = this.doc.mode;
      if (!mode.innerMode) return mode;
      return CodeMirror.innerMode(mode, this.getTokenAt(pos).state).mode;
    },

    getHelper: function(pos, type) {
      return this.getHelpers(pos, type)[0];
    },

    getHelpers: function(pos, type) {
      var found = [];
      if (!helpers.hasOwnProperty(type)) return found;
      var help = helpers[type], mode = this.getModeAt(pos);
      if (typeof mode[type] == "string") {
        if (help[mode[type]]) found.push(help[mode[type]]);
      } else if (mode[type]) {
        for (var i = 0; i < mode[type].length; i++) {
          var val = help[mode[type][i]];
          if (val) found.push(val);
        }
      } else if (mode.helperType && help[mode.helperType]) {
        found.push(help[mode.helperType]);
      } else if (help[mode.name]) {
        found.push(help[mode.name]);
      }
      for (var i = 0; i < help._global.length; i++) {
        var cur = help._global[i];
        if (cur.pred(mode, this) && indexOf(found, cur.val) == -1)
          found.push(cur.val);
      }
      return found;
    },

    getStateAfter: function(line, precise) {
      var doc = this.doc;
      line = clipLine(doc, line == null ? doc.first + doc.size - 1: line);
      return getStateBefore(this, line + 1, precise);
    },

    cursorCoords: function(start, mode) {
      var pos, range = this.doc.sel.primary();
      if (start == null) pos = range.head;
      else if (typeof start == "object") pos = clipPos(this.doc, start);
      else pos = start ? range.from() : range.to();
      return cursorCoords(this, pos, mode || "page");
    },

    charCoords: function(pos, mode) {
      return charCoords(this, clipPos(this.doc, pos), mode || "page");
    },

    coordsChar: function(coords, mode) {
      coords = fromCoordSystem(this, coords, mode || "page");
      return coordsChar(this, coords.left, coords.top);
    },

    lineAtHeight: function(height, mode) {
      height = fromCoordSystem(this, {top: height, left: 0}, mode || "page").top;
      return lineAtHeight(this.doc, height + this.display.viewOffset);
    },
    heightAtLine: function(line, mode) {
      var end = false, lineObj;
      if (typeof line == "number") {
        var last = this.doc.first + this.doc.size - 1;
        if (line < this.doc.first) line = this.doc.first;
        else if (line > last) { line = last; end = true; }
        lineObj = getLine(this.doc, line);
      } else {
        lineObj = line;
      }
      return intoCoordSystem(this, lineObj, {top: 0, left: 0}, mode || "page").top +
        (end ? this.doc.height - heightAtLine(lineObj) : 0);
    },

    defaultTextHeight: function() { return textHeight(this.display); },
    defaultCharWidth: function() { return charWidth(this.display); },

    setGutterMarker: methodOp(function(line, gutterID, value) {
      return changeLine(this.doc, line, "gutter", function(line) {
        var markers = line.gutterMarkers || (line.gutterMarkers = {});
        markers[gutterID] = value;
        if (!value && isEmpty(markers)) line.gutterMarkers = null;
        return true;
      });
    }),

    clearGutter: methodOp(function(gutterID) {
      var cm = this, doc = cm.doc, i = doc.first;
      doc.iter(function(line) {
        if (line.gutterMarkers && line.gutterMarkers[gutterID]) {
          line.gutterMarkers[gutterID] = null;
          regLineChange(cm, i, "gutter");
          if (isEmpty(line.gutterMarkers)) line.gutterMarkers = null;
        }
        ++i;
      });
    }),

    lineInfo: function(line) {
      if (typeof line == "number") {
        if (!isLine(this.doc, line)) return null;
        var n = line;
        line = getLine(this.doc, line);
        if (!line) return null;
      } else {
        var n = lineNo(line);
        if (n == null) return null;
      }
      return {line: n, handle: line, text: line.text, gutterMarkers: line.gutterMarkers,
              textClass: line.textClass, bgClass: line.bgClass, wrapClass: line.wrapClass,
              widgets: line.widgets};
    },

    getViewport: function() { return {from: this.display.viewFrom, to: this.display.viewTo};},

    addWidget: function(pos, node, scroll, vert, horiz) {
      var display = this.display;
      pos = cursorCoords(this, clipPos(this.doc, pos));
      var top = pos.bottom, left = pos.left;
      node.style.position = "absolute";
      node.setAttribute("cm-ignore-events", "true");
      this.display.input.setUneditable(node);
      display.sizer.appendChild(node);
      if (vert == "over") {
        top = pos.top;
      } else if (vert == "above" || vert == "near") {
        var vspace = Math.max(display.wrapper.clientHeight, this.doc.height),
        hspace = Math.max(display.sizer.clientWidth, display.lineSpace.clientWidth);
        // Default to positioning above (if specified and possible); otherwise default to positioning below
        if ((vert == 'above' || pos.bottom + node.offsetHeight > vspace) && pos.top > node.offsetHeight)
          top = pos.top - node.offsetHeight;
        else if (pos.bottom + node.offsetHeight <= vspace)
          top = pos.bottom;
        if (left + node.offsetWidth > hspace)
          left = hspace - node.offsetWidth;
      }
      node.style.top = top + "px";
      node.style.left = node.style.right = "";
      if (horiz == "right") {
        left = display.sizer.clientWidth - node.offsetWidth;
        node.style.right = "0px";
      } else {
        if (horiz == "left") left = 0;
        else if (horiz == "middle") left = (display.sizer.clientWidth - node.offsetWidth) / 2;
        node.style.left = left + "px";
      }
      if (scroll)
        scrollIntoView(this, left, top, left + node.offsetWidth, top + node.offsetHeight);
    },

    triggerOnKeyDown: methodOp(onKeyDown),
    triggerOnKeyPress: methodOp(onKeyPress),
    triggerOnKeyUp: onKeyUp,

    execCommand: function(cmd) {
      if (commands.hasOwnProperty(cmd))
        return commands[cmd].call(null, this);
    },

    triggerElectric: methodOp(function(text) { triggerElectric(this, text); }),

    findPosH: function(from, amount, unit, visually) {
      var dir = 1;
      if (amount < 0) { dir = -1; amount = -amount; }
      for (var i = 0, cur = clipPos(this.doc, from); i < amount; ++i) {
        cur = findPosH(this.doc, cur, dir, unit, visually);
        if (cur.hitSide) break;
      }
      return cur;
    },

    moveH: methodOp(function(dir, unit) {
      var cm = this;
      cm.extendSelectionsBy(function(range) {
        if (cm.display.shift || cm.doc.extend || range.empty())
          return findPosH(cm.doc, range.head, dir, unit, cm.options.rtlMoveVisually);
        else
          return dir < 0 ? range.from() : range.to();
      }, sel_move);
    }),

    deleteH: methodOp(function(dir, unit) {
      var sel = this.doc.sel, doc = this.doc;
      if (sel.somethingSelected())
        doc.replaceSelection("", null, "+delete");
      else
        deleteNearSelection(this, function(range) {
          var other = findPosH(doc, range.head, dir, unit, false);
          return dir < 0 ? {from: other, to: range.head} : {from: range.head, to: other};
        });
    }),

    findPosV: function(from, amount, unit, goalColumn) {
      var dir = 1, x = goalColumn;
      if (amount < 0) { dir = -1; amount = -amount; }
      for (var i = 0, cur = clipPos(this.doc, from); i < amount; ++i) {
        var coords = cursorCoords(this, cur, "div");
        if (x == null) x = coords.left;
        else coords.left = x;
        cur = findPosV(this, coords, dir, unit);
        if (cur.hitSide) break;
      }
      return cur;
    },

    moveV: methodOp(function(dir, unit) {
      var cm = this, doc = this.doc, goals = [];
      var collapse = !cm.display.shift && !doc.extend && doc.sel.somethingSelected();
      doc.extendSelectionsBy(function(range) {
        if (collapse)
          return dir < 0 ? range.from() : range.to();
        var headPos = cursorCoords(cm, range.head, "div");
        if (range.goalColumn != null) headPos.left = range.goalColumn;
        goals.push(headPos.left);
        var pos = findPosV(cm, headPos, dir, unit);
        if (unit == "page" && range == doc.sel.primary())
          addToScrollPos(cm, null, charCoords(cm, pos, "div").top - headPos.top);
        return pos;
      }, sel_move);
      if (goals.length) for (var i = 0; i < doc.sel.ranges.length; i++)
        doc.sel.ranges[i].goalColumn = goals[i];
    }),

    // Find the word at the given position (as returned by coordsChar).
    findWordAt: function(pos) {
      var doc = this.doc, line = getLine(doc, pos.line).text;
      var start = pos.ch, end = pos.ch;
      if (line) {
        var helper = this.getHelper(pos, "wordChars");
        if ((pos.xRel < 0 || end == line.length) && start) --start; else ++end;
        var startChar = line.charAt(start);
        var check = isWordChar(startChar, helper)
          ? function(ch) { return isWordChar(ch, helper); }
          : /\s/.test(startChar) ? function(ch) {return /\s/.test(ch);}
          : function(ch) {return !/\s/.test(ch) && !isWordChar(ch);};
        while (start > 0 && check(line.charAt(start - 1))) --start;
        while (end < line.length && check(line.charAt(end))) ++end;
      }
      return new Range(Pos(pos.line, start), Pos(pos.line, end));
    },

    toggleOverwrite: function(value) {
      if (value != null && value == this.state.overwrite) return;
      if (this.state.overwrite = !this.state.overwrite)
        addClass(this.display.cursorDiv, "CodeMirror-overwrite");
      else
        rmClass(this.display.cursorDiv, "CodeMirror-overwrite");

      signal(this, "overwriteToggle", this, this.state.overwrite);
    },
    hasFocus: function() { return this.display.input.getField() == activeElt(); },
    isReadOnly: function() { return !!(this.options.readOnly || this.doc.cantEdit); },

    scrollTo: methodOp(function(x, y) {
      if (x != null || y != null) resolveScrollToPos(this);
      if (x != null) this.curOp.scrollLeft = x;
      if (y != null) this.curOp.scrollTop = y;
    }),
    getScrollInfo: function() {
      var scroller = this.display.scroller;
      return {left: scroller.scrollLeft, top: scroller.scrollTop,
              height: scroller.scrollHeight - scrollGap(this) - this.display.barHeight,
              width: scroller.scrollWidth - scrollGap(this) - this.display.barWidth,
              clientHeight: displayHeight(this), clientWidth: displayWidth(this)};
    },

    scrollIntoView: methodOp(function(range, margin) {
      if (range == null) {
        range = {from: this.doc.sel.primary().head, to: null};
        if (margin == null) margin = this.options.cursorScrollMargin;
      } else if (typeof range == "number") {
        range = {from: Pos(range, 0), to: null};
      } else if (range.from == null) {
        range = {from: range, to: null};
      }
      if (!range.to) range.to = range.from;
      range.margin = margin || 0;

      if (range.from.line != null) {
        resolveScrollToPos(this);
        this.curOp.scrollToPos = range;
      } else {
        var sPos = calculateScrollPos(this, Math.min(range.from.left, range.to.left),
                                      Math.min(range.from.top, range.to.top) - range.margin,
                                      Math.max(range.from.right, range.to.right),
                                      Math.max(range.from.bottom, range.to.bottom) + range.margin);
        this.scrollTo(sPos.scrollLeft, sPos.scrollTop);
      }
    }),

    setSize: methodOp(function(width, height) {
      var cm = this;
      function interpret(val) {
        return typeof val == "number" || /^\d+$/.test(String(val)) ? val + "px" : val;
      }
      if (width != null) cm.display.wrapper.style.width = interpret(width);
      if (height != null) cm.display.wrapper.style.height = interpret(height);
      if (cm.options.lineWrapping) clearLineMeasurementCache(this);
      var lineNo = cm.display.viewFrom;
      cm.doc.iter(lineNo, cm.display.viewTo, function(line) {
        if (line.widgets) for (var i = 0; i < line.widgets.length; i++)
          if (line.widgets[i].noHScroll) { regLineChange(cm, lineNo, "widget"); break; }
        ++lineNo;
      });
      cm.curOp.forceUpdate = true;
      signal(cm, "refresh", this);
    }),

    operation: function(f){return runInOp(this, f);},

    refresh: methodOp(function() {
      var oldHeight = this.display.cachedTextHeight;
      regChange(this);
      this.curOp.forceUpdate = true;
      clearCaches(this);
      this.scrollTo(this.doc.scrollLeft, this.doc.scrollTop);
      updateGutterSpace(this);
      if (oldHeight == null || Math.abs(oldHeight - textHeight(this.display)) > .5)
        estimateLineHeights(this);
      signal(this, "refresh", this);
    }),

    swapDoc: methodOp(function(doc) {
      var old = this.doc;
      old.cm = null;
      attachDoc(this, doc);
      clearCaches(this);
      this.display.input.reset();
      this.scrollTo(doc.scrollLeft, doc.scrollTop);
      this.curOp.forceScroll = true;
      signalLater(this, "swapDoc", this, old);
      return old;
    }),

    getInputField: function(){return this.display.input.getField();},
    getWrapperElement: function(){return this.display.wrapper;},
    getScrollerElement: function(){return this.display.scroller;},
    getGutterElement: function(){return this.display.gutters;}
  };
  eventMixin(CodeMirror);

  // OPTION DEFAULTS

  // The default configuration options.
  var defaults = CodeMirror.defaults = {};
  // Functions to run when options are changed.
  var optionHandlers = CodeMirror.optionHandlers = {};

  function option(name, deflt, handle, notOnInit) {
    CodeMirror.defaults[name] = deflt;
    if (handle) optionHandlers[name] =
      notOnInit ? function(cm, val, old) {if (old != Init) handle(cm, val, old);} : handle;
  }

  // Passed to option handlers when there is no old value.
  var Init = CodeMirror.Init = {toString: function(){return "CodeMirror.Init";}};

  // These two are, on init, called from the constructor because they
  // have to be initialized before the editor can start at all.
  option("value", "", function(cm, val) {
    cm.setValue(val);
  }, true);
  option("mode", null, function(cm, val) {
    cm.doc.modeOption = val;
    loadMode(cm);
  }, true);

  option("indentUnit", 2, loadMode, true);
  option("indentWithTabs", false);
  option("smartIndent", true);
  option("tabSize", 4, function(cm) {
    resetModeState(cm);
    clearCaches(cm);
    regChange(cm);
  }, true);
  option("lineSeparator", null, function(cm, val) {
    cm.doc.lineSep = val;
    if (!val) return;
    var newBreaks = [], lineNo = cm.doc.first;
    cm.doc.iter(function(line) {
      for (var pos = 0;;) {
        var found = line.text.indexOf(val, pos);
        if (found == -1) break;
        pos = found + val.length;
        newBreaks.push(Pos(lineNo, found));
      }
      lineNo++;
    });
    for (var i = newBreaks.length - 1; i >= 0; i--)
      replaceRange(cm.doc, val, newBreaks[i], Pos(newBreaks[i].line, newBreaks[i].ch + val.length))
  });
  option("specialChars", /[\u0000-\u001f\u007f\u00ad\u200b-\u200f\u2028\u2029\ufeff]/g, function(cm, val, old) {
    cm.state.specialChars = new RegExp(val.source + (val.test("\t") ? "" : "|\t"), "g");
    if (old != CodeMirror.Init) cm.refresh();
  });
  option("specialCharPlaceholder", defaultSpecialCharPlaceholder, function(cm) {cm.refresh();}, true);
  option("electricChars", true);
  option("inputStyle", mobile ? "contenteditable" : "textarea", function() {
    throw new Error("inputStyle can not (yet) be changed in a running editor"); // FIXME
  }, true);
  option("rtlMoveVisually", !windows);
  option("wholeLineUpdateBefore", true);

  option("theme", "default", function(cm) {
    themeChanged(cm);
    guttersChanged(cm);
  }, true);
  option("keyMap", "default", function(cm, val, old) {
    var next = getKeyMap(val);
    var prev = old != CodeMirror.Init && getKeyMap(old);
    if (prev && prev.detach) prev.detach(cm, next);
    if (next.attach) next.attach(cm, prev || null);
  });
  option("extraKeys", null);

  option("lineWrapping", false, wrappingChanged, true);
  option("gutters", [], function(cm) {
    setGuttersForLineNumbers(cm.options);
    guttersChanged(cm);
  }, true);
  option("fixedGutter", true, function(cm, val) {
    cm.display.gutters.style.left = val ? compensateForHScroll(cm.display) + "px" : "0";
    cm.refresh();
  }, true);
  option("coverGutterNextToScrollbar", false, function(cm) {updateScrollbars(cm);}, true);
  option("scrollbarStyle", "native", function(cm) {
    initScrollbars(cm);
    updateScrollbars(cm);
    cm.display.scrollbars.setScrollTop(cm.doc.scrollTop);
    cm.display.scrollbars.setScrollLeft(cm.doc.scrollLeft);
  }, true);
  option("lineNumbers", false, function(cm) {
    setGuttersForLineNumbers(cm.options);
    guttersChanged(cm);
  }, true);
  option("firstLineNumber", 1, guttersChanged, true);
  option("lineNumberFormatter", function(integer) {return integer;}, guttersChanged, true);
  option("showCursorWhenSelecting", false, updateSelection, true);

  option("resetSelectionOnContextMenu", true);
  option("lineWiseCopyCut", true);

  option("readOnly", false, function(cm, val) {
    if (val == "nocursor") {
      onBlur(cm);
      cm.display.input.blur();
      cm.display.disabled = true;
    } else {
      cm.display.disabled = false;
    }
    cm.display.input.readOnlyChanged(val)
  });
  option("disableInput", false, function(cm, val) {if (!val) cm.display.input.reset();}, true);
  option("dragDrop", true, dragDropChanged);
  option("allowDropFileTypes", null);

  option("cursorBlinkRate", 530);
  option("cursorScrollMargin", 0);
  option("cursorHeight", 1, updateSelection, true);
  option("singleCursorHeightPerLine", true, updateSelection, true);
  option("workTime", 100);
  option("workDelay", 100);
  option("flattenSpans", true, resetModeState, true);
  option("addModeClass", false, resetModeState, true);
  option("pollInterval", 100);
  option("undoDepth", 200, function(cm, val){cm.doc.history.undoDepth = val;});
  option("historyEventDelay", 1250);
  option("viewportMargin", 10, function(cm){cm.refresh();}, true);
  option("maxHighlightLength", 10000, resetModeState, true);
  option("moveInputWithCursor", true, function(cm, val) {
    if (!val) cm.display.input.resetPosition();
  });

  option("tabindex", null, function(cm, val) {
    cm.display.input.getField().tabIndex = val || "";
  });
  option("autofocus", null);

  // MODE DEFINITION AND QUERYING

  // Known modes, by name and by MIME
  var modes = CodeMirror.modes = {}, mimeModes = CodeMirror.mimeModes = {};

  // Extra arguments are stored as the mode's dependencies, which is
  // used by (legacy) mechanisms like loadmode.js to automatically
  // load a mode. (Preferred mechanism is the require/define calls.)
  CodeMirror.defineMode = function(name, mode) {
    if (!CodeMirror.defaults.mode && name != "null") CodeMirror.defaults.mode = name;
    if (arguments.length > 2)
      mode.dependencies = Array.prototype.slice.call(arguments, 2);
    modes[name] = mode;
  };

  CodeMirror.defineMIME = function(mime, spec) {
    mimeModes[mime] = spec;
  };

  // Given a MIME type, a {name, ...options} config object, or a name
  // string, return a mode config object.
  CodeMirror.resolveMode = function(spec) {
    if (typeof spec == "string" && mimeModes.hasOwnProperty(spec)) {
      spec = mimeModes[spec];
    } else if (spec && typeof spec.name == "string" && mimeModes.hasOwnProperty(spec.name)) {
      var found = mimeModes[spec.name];
      if (typeof found == "string") found = {name: found};
      spec = createObj(found, spec);
      spec.name = found.name;
    } else if (typeof spec == "string" && /^[\w\-]+\/[\w\-]+\+xml$/.test(spec)) {
      return CodeMirror.resolveMode("application/xml");
    }
    if (typeof spec == "string") return {name: spec};
    else return spec || {name: "null"};
  };

  // Given a mode spec (anything that resolveMode accepts), find and
  // initialize an actual mode object.
  CodeMirror.getMode = function(options, spec) {
    var spec = CodeMirror.resolveMode(spec);
    var mfactory = modes[spec.name];
    if (!mfactory) return CodeMirror.getMode(options, "text/plain");
    var modeObj = mfactory(options, spec);
    if (modeExtensions.hasOwnProperty(spec.name)) {
      var exts = modeExtensions[spec.name];
      for (var prop in exts) {
        if (!exts.hasOwnProperty(prop)) continue;
        if (modeObj.hasOwnProperty(prop)) modeObj["_" + prop] = modeObj[prop];
        modeObj[prop] = exts[prop];
      }
    }
    modeObj.name = spec.name;
    if (spec.helperType) modeObj.helperType = spec.helperType;
    if (spec.modeProps) for (var prop in spec.modeProps)
      modeObj[prop] = spec.modeProps[prop];

    return modeObj;
  };

  // Minimal default mode.
  CodeMirror.defineMode("null", function() {
    return {token: function(stream) {stream.skipToEnd();}};
  });
  CodeMirror.defineMIME("text/plain", "null");

  // This can be used to attach properties to mode objects from
  // outside the actual mode definition.
  var modeExtensions = CodeMirror.modeExtensions = {};
  CodeMirror.extendMode = function(mode, properties) {
    var exts = modeExtensions.hasOwnProperty(mode) ? modeExtensions[mode] : (modeExtensions[mode] = {});
    copyObj(properties, exts);
  };

  // EXTENSIONS

  CodeMirror.defineExtension = function(name, func) {
    CodeMirror.prototype[name] = func;
  };
  CodeMirror.defineDocExtension = function(name, func) {
    Doc.prototype[name] = func;
  };
  CodeMirror.defineOption = option;

  var initHooks = [];
  CodeMirror.defineInitHook = function(f) {initHooks.push(f);};

  var helpers = CodeMirror.helpers = {};
  CodeMirror.registerHelper = function(type, name, value) {
    if (!helpers.hasOwnProperty(type)) helpers[type] = CodeMirror[type] = {_global: []};
    helpers[type][name] = value;
  };
  CodeMirror.registerGlobalHelper = function(type, name, predicate, value) {
    CodeMirror.registerHelper(type, name, value);
    helpers[type]._global.push({pred: predicate, val: value});
  };

  // MODE STATE HANDLING

  // Utility functions for working with state. Exported because nested
  // modes need to do this for their inner modes.

  var copyState = CodeMirror.copyState = function(mode, state) {
    if (state === true) return state;
    if (mode.copyState) return mode.copyState(state);
    var nstate = {};
    for (var n in state) {
      var val = state[n];
      if (val instanceof Array) val = val.concat([]);
      nstate[n] = val;
    }
    return nstate;
  };

  var startState = CodeMirror.startState = function(mode, a1, a2) {
    return mode.startState ? mode.startState(a1, a2) : true;
  };

  // Given a mode and a state (for that mode), find the inner mode and
  // state at the position that the state refers to.
  CodeMirror.innerMode = function(mode, state) {
    while (mode.innerMode) {
      var info = mode.innerMode(state);
      if (!info || info.mode == mode) break;
      state = info.state;
      mode = info.mode;
    }
    return info || {mode: mode, state: state};
  };

  // STANDARD COMMANDS

  // Commands are parameter-less actions that can be performed on an
  // editor, mostly used for keybindings.
  var commands = CodeMirror.commands = {
    selectAll: function(cm) {cm.setSelection(Pos(cm.firstLine(), 0), Pos(cm.lastLine()), sel_dontScroll);},
    singleSelection: function(cm) {
      cm.setSelection(cm.getCursor("anchor"), cm.getCursor("head"), sel_dontScroll);
    },
    killLine: function(cm) {
      deleteNearSelection(cm, function(range) {
        if (range.empty()) {
          var len = getLine(cm.doc, range.head.line).text.length;
          if (range.head.ch == len && range.head.line < cm.lastLine())
            return {from: range.head, to: Pos(range.head.line + 1, 0)};
          else
            return {from: range.head, to: Pos(range.head.line, len)};
        } else {
          return {from: range.from(), to: range.to()};
        }
      });
    },
    deleteLine: function(cm) {
      deleteNearSelection(cm, function(range) {
        return {from: Pos(range.from().line, 0),
                to: clipPos(cm.doc, Pos(range.to().line + 1, 0))};
      });
    },
    delLineLeft: function(cm) {
      deleteNearSelection(cm, function(range) {
        return {from: Pos(range.from().line, 0), to: range.from()};
      });
    },
    delWrappedLineLeft: function(cm) {
      deleteNearSelection(cm, function(range) {
        var top = cm.charCoords(range.head, "div").top + 5;
        var leftPos = cm.coordsChar({left: 0, top: top}, "div");
        return {from: leftPos, to: range.from()};
      });
    },
    delWrappedLineRight: function(cm) {
      deleteNearSelection(cm, function(range) {
        var top = cm.charCoords(range.head, "div").top + 5;
        var rightPos = cm.coordsChar({left: cm.display.lineDiv.offsetWidth + 100, top: top}, "div");
        return {from: range.from(), to: rightPos };
      });
    },
    undo: function(cm) {cm.undo();},
    redo: function(cm) {cm.redo();},
    undoSelection: function(cm) {cm.undoSelection();},
    redoSelection: function(cm) {cm.redoSelection();},
    goDocStart: function(cm) {cm.extendSelection(Pos(cm.firstLine(), 0));},
    goDocEnd: function(cm) {cm.extendSelection(Pos(cm.lastLine()));},
    goLineStart: function(cm) {
      cm.extendSelectionsBy(function(range) { return lineStart(cm, range.head.line); },
                            {origin: "+move", bias: 1});
    },
    goLineStartSmart: function(cm) {
      cm.extendSelectionsBy(function(range) {
        return lineStartSmart(cm, range.head);
      }, {origin: "+move", bias: 1});
    },
    goLineEnd: function(cm) {
      cm.extendSelectionsBy(function(range) { return lineEnd(cm, range.head.line); },
                            {origin: "+move", bias: -1});
    },
    goLineRight: function(cm) {
      cm.extendSelectionsBy(function(range) {
        var top = cm.charCoords(range.head, "div").top + 5;
        return cm.coordsChar({left: cm.display.lineDiv.offsetWidth + 100, top: top}, "div");
      }, sel_move);
    },
    goLineLeft: function(cm) {
      cm.extendSelectionsBy(function(range) {
        var top = cm.charCoords(range.head, "div").top + 5;
        return cm.coordsChar({left: 0, top: top}, "div");
      }, sel_move);
    },
    goLineLeftSmart: function(cm) {
      cm.extendSelectionsBy(function(range) {
        var top = cm.charCoords(range.head, "div").top + 5;
        var pos = cm.coordsChar({left: 0, top: top}, "div");
        if (pos.ch < cm.getLine(pos.line).search(/\S/)) return lineStartSmart(cm, range.head);
        return pos;
      }, sel_move);
    },
    goLineUp: function(cm) {cm.moveV(-1, "line");},
    goLineDown: function(cm) {cm.moveV(1, "line");},
    goPageUp: function(cm) {cm.moveV(-1, "page");},
    goPageDown: function(cm) {cm.moveV(1, "page");},
    goCharLeft: function(cm) {cm.moveH(-1, "char");},
    goCharRight: function(cm) {cm.moveH(1, "char");},
    goColumnLeft: function(cm) {cm.moveH(-1, "column");},
    goColumnRight: function(cm) {cm.moveH(1, "column");},
    goWordLeft: function(cm) {cm.moveH(-1, "word");},
    goGroupRight: function(cm) {cm.moveH(1, "group");},
    goGroupLeft: function(cm) {cm.moveH(-1, "group");},
    goWordRight: function(cm) {cm.moveH(1, "word");},
    delCharBefore: function(cm) {cm.deleteH(-1, "char");},
    delCharAfter: function(cm) {cm.deleteH(1, "char");},
    delWordBefore: function(cm) {cm.deleteH(-1, "word");},
    delWordAfter: function(cm) {cm.deleteH(1, "word");},
    delGroupBefore: function(cm) {cm.deleteH(-1, "group");},
    delGroupAfter: function(cm) {cm.deleteH(1, "group");},
    indentAuto: function(cm) {cm.indentSelection("smart");},
    indentMore: function(cm) {cm.indentSelection("add");},
    indentLess: function(cm) {cm.indentSelection("subtract");},
    insertTab: function(cm) {cm.replaceSelection("\t");},
    insertSoftTab: function(cm) {
      var spaces = [], ranges = cm.listSelections(), tabSize = cm.options.tabSize;
      for (var i = 0; i < ranges.length; i++) {
        var pos = ranges[i].from();
        var col = countColumn(cm.getLine(pos.line), pos.ch, tabSize);
        spaces.push(spaceStr(tabSize - col % tabSize));
      }
      cm.replaceSelections(spaces);
    },
    defaultTab: function(cm) {
      if (cm.somethingSelected()) cm.indentSelection("add");
      else cm.execCommand("insertTab");
    },
    transposeChars: function(cm) {
      runInOp(cm, function() {
        var ranges = cm.listSelections(), newSel = [];
        for (var i = 0; i < ranges.length; i++) {
          var cur = ranges[i].head, line = getLine(cm.doc, cur.line).text;
          if (line) {
            if (cur.ch == line.length) cur = new Pos(cur.line, cur.ch - 1);
            if (cur.ch > 0) {
              cur = new Pos(cur.line, cur.ch + 1);
              cm.replaceRange(line.charAt(cur.ch - 1) + line.charAt(cur.ch - 2),
                              Pos(cur.line, cur.ch - 2), cur, "+transpose");
            } else if (cur.line > cm.doc.first) {
              var prev = getLine(cm.doc, cur.line - 1).text;
              if (prev)
                cm.replaceRange(line.charAt(0) + cm.doc.lineSeparator() +
                                prev.charAt(prev.length - 1),
                                Pos(cur.line - 1, prev.length - 1), Pos(cur.line, 1), "+transpose");
            }
          }
          newSel.push(new Range(cur, cur));
        }
        cm.setSelections(newSel);
      });
    },
    newlineAndIndent: function(cm) {
      runInOp(cm, function() {
        var len = cm.listSelections().length;
        for (var i = 0; i < len; i++) {
          var range = cm.listSelections()[i];
          cm.replaceRange(cm.doc.lineSeparator(), range.anchor, range.head, "+input");
          cm.indentLine(range.from().line + 1, null, true);
        }
        ensureCursorVisible(cm);
      });
    },
    openLine: function(cm) {cm.replaceSelection("\n", "start")},
    toggleOverwrite: function(cm) {cm.toggleOverwrite();}
  };


  // STANDARD KEYMAPS

  var keyMap = CodeMirror.keyMap = {};

  keyMap.basic = {
    "Left": "goCharLeft", "Right": "goCharRight", "Up": "goLineUp", "Down": "goLineDown",
    "End": "goLineEnd", "Home": "goLineStartSmart", "PageUp": "goPageUp", "PageDown": "goPageDown",
    "Delete": "delCharAfter", "Backspace": "delCharBefore", "Shift-Backspace": "delCharBefore",
    "Tab": "defaultTab", "Shift-Tab": "indentAuto",
    "Enter": "newlineAndIndent", "Insert": "toggleOverwrite",
    "Esc": "singleSelection"
  };
  // Note that the save and find-related commands aren't defined by
  // default. User code or addons can define them. Unknown commands
  // are simply ignored.
  keyMap.pcDefault = {
    "Ctrl-A": "selectAll", "Ctrl-D": "deleteLine", "Ctrl-Z": "undo", "Shift-Ctrl-Z": "redo", "Ctrl-Y": "redo",
    "Ctrl-Home": "goDocStart", "Ctrl-End": "goDocEnd", "Ctrl-Up": "goLineUp", "Ctrl-Down": "goLineDown",
    "Ctrl-Left": "goGroupLeft", "Ctrl-Right": "goGroupRight", "Alt-Left": "goLineStart", "Alt-Right": "goLineEnd",
    "Ctrl-Backspace": "delGroupBefore", "Ctrl-Delete": "delGroupAfter", "Ctrl-S": "save", "Ctrl-F": "find",
    "Ctrl-G": "findNext", "Shift-Ctrl-G": "findPrev", "Shift-Ctrl-F": "replace", "Shift-Ctrl-R": "replaceAll",
    "Ctrl-[": "indentLess", "Ctrl-]": "indentMore",
    "Ctrl-U": "undoSelection", "Shift-Ctrl-U": "redoSelection", "Alt-U": "redoSelection",
    fallthrough: "basic"
  };
  // Very basic readline/emacs-style bindings, which are standard on Mac.
  keyMap.emacsy = {
    "Ctrl-F": "goCharRight", "Ctrl-B": "goCharLeft", "Ctrl-P": "goLineUp", "Ctrl-N": "goLineDown",
    "Alt-F": "goWordRight", "Alt-B": "goWordLeft", "Ctrl-A": "goLineStart", "Ctrl-E": "goLineEnd",
    "Ctrl-V": "goPageDown", "Shift-Ctrl-V": "goPageUp", "Ctrl-D": "delCharAfter", "Ctrl-H": "delCharBefore",
    "Alt-D": "delWordAfter", "Alt-Backspace": "delWordBefore", "Ctrl-K": "killLine", "Ctrl-T": "transposeChars",
    "Ctrl-O": "openLine"
  };
  keyMap.macDefault = {
    "Cmd-A": "selectAll", "Cmd-D": "deleteLine", "Cmd-Z": "undo", "Shift-Cmd-Z": "redo", "Cmd-Y": "redo",
    "Cmd-Home": "goDocStart", "Cmd-Up": "goDocStart", "Cmd-End": "goDocEnd", "Cmd-Down": "goDocEnd", "Alt-Left": "goGroupLeft",
    "Alt-Right": "goGroupRight", "Cmd-Left": "goLineLeft", "Cmd-Right": "goLineRight", "Alt-Backspace": "delGroupBefore",
    "Ctrl-Alt-Backspace": "delGroupAfter", "Alt-Delete": "delGroupAfter", "Cmd-S": "save", "Cmd-F": "find",
    "Cmd-G": "findNext", "Shift-Cmd-G": "findPrev", "Cmd-Alt-F": "replace", "Shift-Cmd-Alt-F": "replaceAll",
    "Cmd-[": "indentLess", "Cmd-]": "indentMore", "Cmd-Backspace": "delWrappedLineLeft", "Cmd-Delete": "delWrappedLineRight",
    "Cmd-U": "undoSelection", "Shift-Cmd-U": "redoSelection", "Ctrl-Up": "goDocStart", "Ctrl-Down": "goDocEnd",
    fallthrough: ["basic", "emacsy"]
  };
  keyMap["default"] = mac ? keyMap.macDefault : keyMap.pcDefault;

  // KEYMAP DISPATCH

  function normalizeKeyName(name) {
    var parts = name.split(/-(?!$)/), name = parts[parts.length - 1];
    var alt, ctrl, shift, cmd;
    for (var i = 0; i < parts.length - 1; i++) {
      var mod = parts[i];
      if (/^(cmd|meta|m)$/i.test(mod)) cmd = true;
      else if (/^a(lt)?$/i.test(mod)) alt = true;
      else if (/^(c|ctrl|control)$/i.test(mod)) ctrl = true;
      else if (/^s(hift)$/i.test(mod)) shift = true;
      else throw new Error("Unrecognized modifier name: " + mod);
    }
    if (alt) name = "Alt-" + name;
    if (ctrl) name = "Ctrl-" + name;
    if (cmd) name = "Cmd-" + name;
    if (shift) name = "Shift-" + name;
    return name;
  }

  // This is a kludge to keep keymaps mostly working as raw objects
  // (backwards compatibility) while at the same time support features
  // like normalization and multi-stroke key bindings. It compiles a
  // new normalized keymap, and then updates the old object to reflect
  // this.
  CodeMirror.normalizeKeyMap = function(keymap) {
    var copy = {};
    for (var keyname in keymap) if (keymap.hasOwnProperty(keyname)) {
      var value = keymap[keyname];
      if (/^(name|fallthrough|(de|at)tach)$/.test(keyname)) continue;
      if (value == "...") { delete keymap[keyname]; continue; }

      var keys = map(keyname.split(" "), normalizeKeyName);
      for (var i = 0; i < keys.length; i++) {
        var val, name;
        if (i == keys.length - 1) {
          name = keys.join(" ");
          val = value;
        } else {
          name = keys.slice(0, i + 1).join(" ");
          val = "...";
        }
        var prev = copy[name];
        if (!prev) copy[name] = val;
        else if (prev != val) throw new Error("Inconsistent bindings for " + name);
      }
      delete keymap[keyname];
    }
    for (var prop in copy) keymap[prop] = copy[prop];
    return keymap;
  };

  var lookupKey = CodeMirror.lookupKey = function(key, map, handle, context) {
    map = getKeyMap(map);
    var found = map.call ? map.call(key, context) : map[key];
    if (found === false) return "nothing";
    if (found === "...") return "multi";
    if (found != null && handle(found)) return "handled";

    if (map.fallthrough) {
      if (Object.prototype.toString.call(map.fallthrough) != "[object Array]")
        return lookupKey(key, map.fallthrough, handle, context);
      for (var i = 0; i < map.fallthrough.length; i++) {
        var result = lookupKey(key, map.fallthrough[i], handle, context);
        if (result) return result;
      }
    }
  };

  // Modifier key presses don't count as 'real' key presses for the
  // purpose of keymap fallthrough.
  var isModifierKey = CodeMirror.isModifierKey = function(value) {
    var name = typeof value == "string" ? value : keyNames[value.keyCode];
    return name == "Ctrl" || name == "Alt" || name == "Shift" || name == "Mod";
  };

  // Look up the name of a key as indicated by an event object.
  var keyName = CodeMirror.keyName = function(event, noShift) {
    if (presto && event.keyCode == 34 && event["char"]) return false;
    var base = keyNames[event.keyCode], name = base;
    if (name == null || event.altGraphKey) return false;
    if (event.altKey && base != "Alt") name = "Alt-" + name;
    if ((flipCtrlCmd ? event.metaKey : event.ctrlKey) && base != "Ctrl") name = "Ctrl-" + name;
    if ((flipCtrlCmd ? event.ctrlKey : event.metaKey) && base != "Cmd") name = "Cmd-" + name;
    if (!noShift && event.shiftKey && base != "Shift") name = "Shift-" + name;
    return name;
  };

  function getKeyMap(val) {
    return typeof val == "string" ? keyMap[val] : val;
  }

  // FROMTEXTAREA

  CodeMirror.fromTextArea = function(textarea, options) {
    options = options ? copyObj(options) : {};
    options.value = textarea.value;
    if (!options.tabindex && textarea.tabIndex)
      options.tabindex = textarea.tabIndex;
    if (!options.placeholder && textarea.placeholder)
      options.placeholder = textarea.placeholder;
    // Set autofocus to true if this textarea is focused, or if it has
    // autofocus and no other element is focused.
    if (options.autofocus == null) {
      var hasFocus = activeElt();
      options.autofocus = hasFocus == textarea ||
        textarea.getAttribute("autofocus") != null && hasFocus == document.body;
    }

    function save() {textarea.value = cm.getValue();}
    if (textarea.form) {
      on(textarea.form, "submit", save);
      // Deplorable hack to make the submit method do the right thing.
      if (!options.leaveSubmitMethodAlone) {
        var form = textarea.form, realSubmit = form.submit;
        try {
          var wrappedSubmit = form.submit = function() {
            save();
            form.submit = realSubmit;
            form.submit();
            form.submit = wrappedSubmit;
          };
        } catch(e) {}
      }
    }

    options.finishInit = function(cm) {
      cm.save = save;
      cm.getTextArea = function() { return textarea; };
      cm.toTextArea = function() {
        cm.toTextArea = isNaN; // Prevent this from being ran twice
        save();
        textarea.parentNode.removeChild(cm.getWrapperElement());
        textarea.style.display = "";
        if (textarea.form) {
          off(textarea.form, "submit", save);
          if (typeof textarea.form.submit == "function")
            textarea.form.submit = realSubmit;
        }
      };
    };

    textarea.style.display = "none";
    var cm = CodeMirror(function(node) {
      textarea.parentNode.insertBefore(node, textarea.nextSibling);
    }, options);
    return cm;
  };

  // STRING STREAM

  // Fed to the mode parsers, provides helper functions to make
  // parsers more succinct.

  var StringStream = CodeMirror.StringStream = function(string, tabSize) {
    this.pos = this.start = 0;
    this.string = string;
    this.tabSize = tabSize || 8;
    this.lastColumnPos = this.lastColumnValue = 0;
    this.lineStart = 0;
  };

  StringStream.prototype = {
    eol: function() {return this.pos >= this.string.length;},
    sol: function() {return this.pos == this.lineStart;},
    peek: function() {return this.string.charAt(this.pos) || undefined;},
    next: function() {
      if (this.pos < this.string.length)
        return this.string.charAt(this.pos++);
    },
    eat: function(match) {
      var ch = this.string.charAt(this.pos);
      if (typeof match == "string") var ok = ch == match;
      else var ok = ch && (match.test ? match.test(ch) : match(ch));
      if (ok) {++this.pos; return ch;}
    },
    eatWhile: function(match) {
      var start = this.pos;
      while (this.eat(match)){}
      return this.pos > start;
    },
    eatSpace: function() {
      var start = this.pos;
      while (/[\s\u00a0]/.test(this.string.charAt(this.pos))) ++this.pos;
      return this.pos > start;
    },
    skipToEnd: function() {this.pos = this.string.length;},
    skipTo: function(ch) {
      var found = this.string.indexOf(ch, this.pos);
      if (found > -1) {this.pos = found; return true;}
    },
    backUp: function(n) {this.pos -= n;},
    column: function() {
      if (this.lastColumnPos < this.start) {
        this.lastColumnValue = countColumn(this.string, this.start, this.tabSize, this.lastColumnPos, this.lastColumnValue);
        this.lastColumnPos = this.start;
      }
      return this.lastColumnValue - (this.lineStart ? countColumn(this.string, this.lineStart, this.tabSize) : 0);
    },
    indentation: function() {
      return countColumn(this.string, null, this.tabSize) -
        (this.lineStart ? countColumn(this.string, this.lineStart, this.tabSize) : 0);
    },
    match: function(pattern, consume, caseInsensitive) {
      if (typeof pattern == "string") {
        var cased = function(str) {return caseInsensitive ? str.toLowerCase() : str;};
        var substr = this.string.substr(this.pos, pattern.length);
        if (cased(substr) == cased(pattern)) {
          if (consume !== false) this.pos += pattern.length;
          return true;
        }
      } else {
        var match = this.string.slice(this.pos).match(pattern);
        if (match && match.index > 0) return null;
        if (match && consume !== false) this.pos += match[0].length;
        return match;
      }
    },
    current: function(){return this.string.slice(this.start, this.pos);},
    hideFirstChars: function(n, inner) {
      this.lineStart += n;
      try { return inner(); }
      finally { this.lineStart -= n; }
    }
  };

  // TEXTMARKERS

  // Created with markText and setBookmark methods. A TextMarker is a
  // handle that can be used to clear or find a marked position in the
  // document. Line objects hold arrays (markedSpans) containing
  // {from, to, marker} object pointing to such marker objects, and
  // indicating that such a marker is present on that line. Multiple
  // lines may point to the same marker when it spans across lines.
  // The spans will have null for their from/to properties when the
  // marker continues beyond the start/end of the line. Markers have
  // links back to the lines they currently touch.

  var nextMarkerId = 0;

  var TextMarker = CodeMirror.TextMarker = function(doc, type) {
    this.lines = [];
    this.type = type;
    this.doc = doc;
    this.id = ++nextMarkerId;
  };
  eventMixin(TextMarker);

  // Clear the marker.
  TextMarker.prototype.clear = function() {
    if (this.explicitlyCleared) return;
    var cm = this.doc.cm, withOp = cm && !cm.curOp;
    if (withOp) startOperation(cm);
    if (hasHandler(this, "clear")) {
      var found = this.find();
      if (found) signalLater(this, "clear", found.from, found.to);
    }
    var min = null, max = null;
    for (var i = 0; i < this.lines.length; ++i) {
      var line = this.lines[i];
      var span = getMarkedSpanFor(line.markedSpans, this);
      if (cm && !this.collapsed) regLineChange(cm, lineNo(line), "text");
      else if (cm) {
        if (span.to != null) max = lineNo(line);
        if (span.from != null) min = lineNo(line);
      }
      line.markedSpans = removeMarkedSpan(line.markedSpans, span);
      if (span.from == null && this.collapsed && !lineIsHidden(this.doc, line) && cm)
        updateLineHeight(line, textHeight(cm.display));
    }
    if (cm && this.collapsed && !cm.options.lineWrapping) for (var i = 0; i < this.lines.length; ++i) {
      var visual = visualLine(this.lines[i]), len = lineLength(visual);
      if (len > cm.display.maxLineLength) {
        cm.display.maxLine = visual;
        cm.display.maxLineLength = len;
        cm.display.maxLineChanged = true;
      }
    }

    if (min != null && cm && this.collapsed) regChange(cm, min, max + 1);
    this.lines.length = 0;
    this.explicitlyCleared = true;
    if (this.atomic && this.doc.cantEdit) {
      this.doc.cantEdit = false;
      if (cm) reCheckSelection(cm.doc);
    }
    if (cm) signalLater(cm, "markerCleared", cm, this);
    if (withOp) endOperation(cm);
    if (this.parent) this.parent.clear();
  };

  // Find the position of the marker in the document. Returns a {from,
  // to} object by default. Side can be passed to get a specific side
  // -- 0 (both), -1 (left), or 1 (right). When lineObj is true, the
  // Pos objects returned contain a line object, rather than a line
  // number (used to prevent looking up the same line twice).
  TextMarker.prototype.find = function(side, lineObj) {
    if (side == null && this.type == "bookmark") side = 1;
    var from, to;
    for (var i = 0; i < this.lines.length; ++i) {
      var line = this.lines[i];
      var span = getMarkedSpanFor(line.markedSpans, this);
      if (span.from != null) {
        from = Pos(lineObj ? line : lineNo(line), span.from);
        if (side == -1) return from;
      }
      if (span.to != null) {
        to = Pos(lineObj ? line : lineNo(line), span.to);
        if (side == 1) return to;
      }
    }
    return from && {from: from, to: to};
  };

  // Signals that the marker's widget changed, and surrounding layout
  // should be recomputed.
  TextMarker.prototype.changed = function() {
    var pos = this.find(-1, true), widget = this, cm = this.doc.cm;
    if (!pos || !cm) return;
    runInOp(cm, function() {
      var line = pos.line, lineN = lineNo(pos.line);
      var view = findViewForLine(cm, lineN);
      if (view) {
        clearLineMeasurementCacheFor(view);
        cm.curOp.selectionChanged = cm.curOp.forceUpdate = true;
      }
      cm.curOp.updateMaxLine = true;
      if (!lineIsHidden(widget.doc, line) && widget.height != null) {
        var oldHeight = widget.height;
        widget.height = null;
        var dHeight = widgetHeight(widget) - oldHeight;
        if (dHeight)
          updateLineHeight(line, line.height + dHeight);
      }
    });
  };

  TextMarker.prototype.attachLine = function(line) {
    if (!this.lines.length && this.doc.cm) {
      var op = this.doc.cm.curOp;
      if (!op.maybeHiddenMarkers || indexOf(op.maybeHiddenMarkers, this) == -1)
        (op.maybeUnhiddenMarkers || (op.maybeUnhiddenMarkers = [])).push(this);
    }
    this.lines.push(line);
  };
  TextMarker.prototype.detachLine = function(line) {
    this.lines.splice(indexOf(this.lines, line), 1);
    if (!this.lines.length && this.doc.cm) {
      var op = this.doc.cm.curOp;
      (op.maybeHiddenMarkers || (op.maybeHiddenMarkers = [])).push(this);
    }
  };

  // Collapsed markers have unique ids, in order to be able to order
  // them, which is needed for uniquely determining an outer marker
  // when they overlap (they may nest, but not partially overlap).
  var nextMarkerId = 0;

  // Create a marker, wire it up to the right lines, and
  function markText(doc, from, to, options, type) {
    // Shared markers (across linked documents) are handled separately
    // (markTextShared will call out to this again, once per
    // document).
    if (options && options.shared) return markTextShared(doc, from, to, options, type);
    // Ensure we are in an operation.
    if (doc.cm && !doc.cm.curOp) return operation(doc.cm, markText)(doc, from, to, options, type);

    var marker = new TextMarker(doc, type), diff = cmp(from, to);
    if (options) copyObj(options, marker, false);
    // Don't connect empty markers unless clearWhenEmpty is false
    if (diff > 0 || diff == 0 && marker.clearWhenEmpty !== false)
      return marker;
    if (marker.replacedWith) {
      // Showing up as a widget implies collapsed (widget replaces text)
      marker.collapsed = true;
      marker.widgetNode = elt("span", [marker.replacedWith], "CodeMirror-widget");
      if (!options.handleMouseEvents) marker.widgetNode.setAttribute("cm-ignore-events", "true");
      if (options.insertLeft) marker.widgetNode.insertLeft = true;
    }
    if (marker.collapsed) {
      if (conflictingCollapsedRange(doc, from.line, from, to, marker) ||
          from.line != to.line && conflictingCollapsedRange(doc, to.line, from, to, marker))
        throw new Error("Inserting collapsed marker partially overlapping an existing one");
      sawCollapsedSpans = true;
    }

    if (marker.addToHistory)
      addChangeToHistory(doc, {from: from, to: to, origin: "markText"}, doc.sel, NaN);

    var curLine = from.line, cm = doc.cm, updateMaxLine;
    doc.iter(curLine, to.line + 1, function(line) {
      if (cm && marker.collapsed && !cm.options.lineWrapping && visualLine(line) == cm.display.maxLine)
        updateMaxLine = true;
      if (marker.collapsed && curLine != from.line) updateLineHeight(line, 0);
      addMarkedSpan(line, new MarkedSpan(marker,
                                         curLine == from.line ? from.ch : null,
                                         curLine == to.line ? to.ch : null));
      ++curLine;
    });
    // lineIsHidden depends on the presence of the spans, so needs a second pass
    if (marker.collapsed) doc.iter(from.line, to.line + 1, function(line) {
      if (lineIsHidden(doc, line)) updateLineHeight(line, 0);
    });

    if (marker.clearOnEnter) on(marker, "beforeCursorEnter", function() { marker.clear(); });

    if (marker.readOnly) {
      sawReadOnlySpans = true;
      if (doc.history.done.length || doc.history.undone.length)
        doc.clearHistory();
    }
    if (marker.collapsed) {
      marker.id = ++nextMarkerId;
      marker.atomic = true;
    }
    if (cm) {
      // Sync editor state
      if (updateMaxLine) cm.curOp.updateMaxLine = true;
      if (marker.collapsed)
        regChange(cm, from.line, to.line + 1);
      else if (marker.className || marker.title || marker.startStyle || marker.endStyle || marker.css)
        for (var i = from.line; i <= to.line; i++) regLineChange(cm, i, "text");
      if (marker.atomic) reCheckSelection(cm.doc);
      signalLater(cm, "markerAdded", cm, marker);
    }
    return marker;
  }

  // SHARED TEXTMARKERS

  // A shared marker spans multiple linked documents. It is
  // implemented as a meta-marker-object controlling multiple normal
  // markers.
  var SharedTextMarker = CodeMirror.SharedTextMarker = function(markers, primary) {
    this.markers = markers;
    this.primary = primary;
    for (var i = 0; i < markers.length; ++i)
      markers[i].parent = this;
  };
  eventMixin(SharedTextMarker);

  SharedTextMarker.prototype.clear = function() {
    if (this.explicitlyCleared) return;
    this.explicitlyCleared = true;
    for (var i = 0; i < this.markers.length; ++i)
      this.markers[i].clear();
    signalLater(this, "clear");
  };
  SharedTextMarker.prototype.find = function(side, lineObj) {
    return this.primary.find(side, lineObj);
  };

  function markTextShared(doc, from, to, options, type) {
    options = copyObj(options);
    options.shared = false;
    var markers = [markText(doc, from, to, options, type)], primary = markers[0];
    var widget = options.widgetNode;
    linkedDocs(doc, function(doc) {
      if (widget) options.widgetNode = widget.cloneNode(true);
      markers.push(markText(doc, clipPos(doc, from), clipPos(doc, to), options, type));
      for (var i = 0; i < doc.linked.length; ++i)
        if (doc.linked[i].isParent) return;
      primary = lst(markers);
    });
    return new SharedTextMarker(markers, primary);
  }

  function findSharedMarkers(doc) {
    return doc.findMarks(Pos(doc.first, 0), doc.clipPos(Pos(doc.lastLine())),
                         function(m) { return m.parent; });
  }

  function copySharedMarkers(doc, markers) {
    for (var i = 0; i < markers.length; i++) {
      var marker = markers[i], pos = marker.find();
      var mFrom = doc.clipPos(pos.from), mTo = doc.clipPos(pos.to);
      if (cmp(mFrom, mTo)) {
        var subMark = markText(doc, mFrom, mTo, marker.primary, marker.primary.type);
        marker.markers.push(subMark);
        subMark.parent = marker;
      }
    }
  }

  function detachSharedMarkers(markers) {
    for (var i = 0; i < markers.length; i++) {
      var marker = markers[i], linked = [marker.primary.doc];;
      linkedDocs(marker.primary.doc, function(d) { linked.push(d); });
      for (var j = 0; j < marker.markers.length; j++) {
        var subMarker = marker.markers[j];
        if (indexOf(linked, subMarker.doc) == -1) {
          subMarker.parent = null;
          marker.markers.splice(j--, 1);
        }
      }
    }
  }

  // TEXTMARKER SPANS

  function MarkedSpan(marker, from, to) {
    this.marker = marker;
    this.from = from; this.to = to;
  }

  // Search an array of spans for a span matching the given marker.
  function getMarkedSpanFor(spans, marker) {
    if (spans) for (var i = 0; i < spans.length; ++i) {
      var span = spans[i];
      if (span.marker == marker) return span;
    }
  }
  // Remove a span from an array, returning undefined if no spans are
  // left (we don't store arrays for lines without spans).
  function removeMarkedSpan(spans, span) {
    for (var r, i = 0; i < spans.length; ++i)
      if (spans[i] != span) (r || (r = [])).push(spans[i]);
    return r;
  }
  // Add a span to a line.
  function addMarkedSpan(line, span) {
    line.markedSpans = line.markedSpans ? line.markedSpans.concat([span]) : [span];
    span.marker.attachLine(line);
  }

  // Used for the algorithm that adjusts markers for a change in the
  // document. These functions cut an array of spans at a given
  // character position, returning an array of remaining chunks (or
  // undefined if nothing remains).
  function markedSpansBefore(old, startCh, isInsert) {
    if (old) for (var i = 0, nw; i < old.length; ++i) {
      var span = old[i], marker = span.marker;
      var startsBefore = span.from == null || (marker.inclusiveLeft ? span.from <= startCh : span.from < startCh);
      if (startsBefore || span.from == startCh && marker.type == "bookmark" && (!isInsert || !span.marker.insertLeft)) {
        var endsAfter = span.to == null || (marker.inclusiveRight ? span.to >= startCh : span.to > startCh);
        (nw || (nw = [])).push(new MarkedSpan(marker, span.from, endsAfter ? null : span.to));
      }
    }
    return nw;
  }
  function markedSpansAfter(old, endCh, isInsert) {
    if (old) for (var i = 0, nw; i < old.length; ++i) {
      var span = old[i], marker = span.marker;
      var endsAfter = span.to == null || (marker.inclusiveRight ? span.to >= endCh : span.to > endCh);
      if (endsAfter || span.from == endCh && marker.type == "bookmark" && (!isInsert || span.marker.insertLeft)) {
        var startsBefore = span.from == null || (marker.inclusiveLeft ? span.from <= endCh : span.from < endCh);
        (nw || (nw = [])).push(new MarkedSpan(marker, startsBefore ? null : span.from - endCh,
                                              span.to == null ? null : span.to - endCh));
      }
    }
    return nw;
  }

  // Given a change object, compute the new set of marker spans that
  // cover the line in which the change took place. Removes spans
  // entirely within the change, reconnects spans belonging to the
  // same marker that appear on both sides of the change, and cuts off
  // spans partially within the change. Returns an array of span
  // arrays with one element for each line in (after) the change.
  function stretchSpansOverChange(doc, change) {
    if (change.full) return null;
    var oldFirst = isLine(doc, change.from.line) && getLine(doc, change.from.line).markedSpans;
    var oldLast = isLine(doc, change.to.line) && getLine(doc, change.to.line).markedSpans;
    if (!oldFirst && !oldLast) return null;

    var startCh = change.from.ch, endCh = change.to.ch, isInsert = cmp(change.from, change.to) == 0;
    // Get the spans that 'stick out' on both sides
    var first = markedSpansBefore(oldFirst, startCh, isInsert);
    var last = markedSpansAfter(oldLast, endCh, isInsert);

    // Next, merge those two ends
    var sameLine = change.text.length == 1, offset = lst(change.text).length + (sameLine ? startCh : 0);
    if (first) {
      // Fix up .to properties of first
      for (var i = 0; i < first.length; ++i) {
        var span = first[i];
        if (span.to == null) {
          var found = getMarkedSpanFor(last, span.marker);
          if (!found) span.to = startCh;
          else if (sameLine) span.to = found.to == null ? null : found.to + offset;
        }
      }
    }
    if (last) {
      // Fix up .from in last (or move them into first in case of sameLine)
      for (var i = 0; i < last.length; ++i) {
        var span = last[i];
        if (span.to != null) span.to += offset;
        if (span.from == null) {
          var found = getMarkedSpanFor(first, span.marker);
          if (!found) {
            span.from = offset;
            if (sameLine) (first || (first = [])).push(span);
          }
        } else {
          span.from += offset;
          if (sameLine) (first || (first = [])).push(span);
        }
      }
    }
    // Make sure we didn't create any zero-length spans
    if (first) first = clearEmptySpans(first);
    if (last && last != first) last = clearEmptySpans(last);

    var newMarkers = [first];
    if (!sameLine) {
      // Fill gap with whole-line-spans
      var gap = change.text.length - 2, gapMarkers;
      if (gap > 0 && first)
        for (var i = 0; i < first.length; ++i)
          if (first[i].to == null)
            (gapMarkers || (gapMarkers = [])).push(new MarkedSpan(first[i].marker, null, null));
      for (var i = 0; i < gap; ++i)
        newMarkers.push(gapMarkers);
      newMarkers.push(last);
    }
    return newMarkers;
  }

  // Remove spans that are empty and don't have a clearWhenEmpty
  // option of false.
  function clearEmptySpans(spans) {
    for (var i = 0; i < spans.length; ++i) {
      var span = spans[i];
      if (span.from != null && span.from == span.to && span.marker.clearWhenEmpty !== false)
        spans.splice(i--, 1);
    }
    if (!spans.length) return null;
    return spans;
  }

  // Used for un/re-doing changes from the history. Combines the
  // result of computing the existing spans with the set of spans that
  // existed in the history (so that deleting around a span and then
  // undoing brings back the span).
  function mergeOldSpans(doc, change) {
    var old = getOldSpans(doc, change);
    var stretched = stretchSpansOverChange(doc, change);
    if (!old) return stretched;
    if (!stretched) return old;

    for (var i = 0; i < old.length; ++i) {
      var oldCur = old[i], stretchCur = stretched[i];
      if (oldCur && stretchCur) {
        spans: for (var j = 0; j < stretchCur.length; ++j) {
          var span = stretchCur[j];
          for (var k = 0; k < oldCur.length; ++k)
            if (oldCur[k].marker == span.marker) continue spans;
          oldCur.push(span);
        }
      } else if (stretchCur) {
        old[i] = stretchCur;
      }
    }
    return old;
  }

  // Used to 'clip' out readOnly ranges when making a change.
  function removeReadOnlyRanges(doc, from, to) {
    var markers = null;
    doc.iter(from.line, to.line + 1, function(line) {
      if (line.markedSpans) for (var i = 0; i < line.markedSpans.length; ++i) {
        var mark = line.markedSpans[i].marker;
        if (mark.readOnly && (!markers || indexOf(markers, mark) == -1))
          (markers || (markers = [])).push(mark);
      }
    });
    if (!markers) return null;
    var parts = [{from: from, to: to}];
    for (var i = 0; i < markers.length; ++i) {
      var mk = markers[i], m = mk.find(0);
      for (var j = 0; j < parts.length; ++j) {
        var p = parts[j];
        if (cmp(p.to, m.from) < 0 || cmp(p.from, m.to) > 0) continue;
        var newParts = [j, 1], dfrom = cmp(p.from, m.from), dto = cmp(p.to, m.to);
        if (dfrom < 0 || !mk.inclusiveLeft && !dfrom)
          newParts.push({from: p.from, to: m.from});
        if (dto > 0 || !mk.inclusiveRight && !dto)
          newParts.push({from: m.to, to: p.to});
        parts.splice.apply(parts, newParts);
        j += newParts.length - 1;
      }
    }
    return parts;
  }

  // Connect or disconnect spans from a line.
  function detachMarkedSpans(line) {
    var spans = line.markedSpans;
    if (!spans) return;
    for (var i = 0; i < spans.length; ++i)
      spans[i].marker.detachLine(line);
    line.markedSpans = null;
  }
  function attachMarkedSpans(line, spans) {
    if (!spans) return;
    for (var i = 0; i < spans.length; ++i)
      spans[i].marker.attachLine(line);
    line.markedSpans = spans;
  }

  // Helpers used when computing which overlapping collapsed span
  // counts as the larger one.
  function extraLeft(marker) { return marker.inclusiveLeft ? -1 : 0; }
  function extraRight(marker) { return marker.inclusiveRight ? 1 : 0; }

  // Returns a number indicating which of two overlapping collapsed
  // spans is larger (and thus includes the other). Falls back to
  // comparing ids when the spans cover exactly the same range.
  function compareCollapsedMarkers(a, b) {
    var lenDiff = a.lines.length - b.lines.length;
    if (lenDiff != 0) return lenDiff;
    var aPos = a.find(), bPos = b.find();
    var fromCmp = cmp(aPos.from, bPos.from) || extraLeft(a) - extraLeft(b);
    if (fromCmp) return -fromCmp;
    var toCmp = cmp(aPos.to, bPos.to) || extraRight(a) - extraRight(b);
    if (toCmp) return toCmp;
    return b.id - a.id;
  }

  // Find out whether a line ends or starts in a collapsed span. If
  // so, return the marker for that span.
  function collapsedSpanAtSide(line, start) {
    var sps = sawCollapsedSpans && line.markedSpans, found;
    if (sps) for (var sp, i = 0; i < sps.length; ++i) {
      sp = sps[i];
      if (sp.marker.collapsed && (start ? sp.from : sp.to) == null &&
          (!found || compareCollapsedMarkers(found, sp.marker) < 0))
        found = sp.marker;
    }
    return found;
  }
  function collapsedSpanAtStart(line) { return collapsedSpanAtSide(line, true); }
  function collapsedSpanAtEnd(line) { return collapsedSpanAtSide(line, false); }

  // Test whether there exists a collapsed span that partially
  // overlaps (covers the start or end, but not both) of a new span.
  // Such overlap is not allowed.
  function conflictingCollapsedRange(doc, lineNo, from, to, marker) {
    var line = getLine(doc, lineNo);
    var sps = sawCollapsedSpans && line.markedSpans;
    if (sps) for (var i = 0; i < sps.length; ++i) {
      var sp = sps[i];
      if (!sp.marker.collapsed) continue;
      var found = sp.marker.find(0);
      var fromCmp = cmp(found.from, from) || extraLeft(sp.marker) - extraLeft(marker);
      var toCmp = cmp(found.to, to) || extraRight(sp.marker) - extraRight(marker);
      if (fromCmp >= 0 && toCmp <= 0 || fromCmp <= 0 && toCmp >= 0) continue;
      if (fromCmp <= 0 && (sp.marker.inclusiveRight && marker.inclusiveLeft ? cmp(found.to, from) >= 0 : cmp(found.to, from) > 0) ||
          fromCmp >= 0 && (sp.marker.inclusiveRight && marker.inclusiveLeft ? cmp(found.from, to) <= 0 : cmp(found.from, to) < 0))
        return true;
    }
  }

  // A visual line is a line as drawn on the screen. Folding, for
  // example, can cause multiple logical lines to appear on the same
  // visual line. This finds the start of the visual line that the
  // given line is part of (usually that is the line itself).
  function visualLine(line) {
    var merged;
    while (merged = collapsedSpanAtStart(line))
      line = merged.find(-1, true).line;
    return line;
  }

  // Returns an array of logical lines that continue the visual line
  // started by the argument, or undefined if there are no such lines.
  function visualLineContinued(line) {
    var merged, lines;
    while (merged = collapsedSpanAtEnd(line)) {
      line = merged.find(1, true).line;
      (lines || (lines = [])).push(line);
    }
    return lines;
  }

  // Get the line number of the start of the visual line that the
  // given line number is part of.
  function visualLineNo(doc, lineN) {
    var line = getLine(doc, lineN), vis = visualLine(line);
    if (line == vis) return lineN;
    return lineNo(vis);
  }
  // Get the line number of the start of the next visual line after
  // the given line.
  function visualLineEndNo(doc, lineN) {
    if (lineN > doc.lastLine()) return lineN;
    var line = getLine(doc, lineN), merged;
    if (!lineIsHidden(doc, line)) return lineN;
    while (merged = collapsedSpanAtEnd(line))
      line = merged.find(1, true).line;
    return lineNo(line) + 1;
  }

  // Compute whether a line is hidden. Lines count as hidden when they
  // are part of a visual line that starts with another line, or when
  // they are entirely covered by collapsed, non-widget span.
  function lineIsHidden(doc, line) {
    var sps = sawCollapsedSpans && line.markedSpans;
    if (sps) for (var sp, i = 0; i < sps.length; ++i) {
      sp = sps[i];
      if (!sp.marker.collapsed) continue;
      if (sp.from == null) return true;
      if (sp.marker.widgetNode) continue;
      if (sp.from == 0 && sp.marker.inclusiveLeft && lineIsHiddenInner(doc, line, sp))
        return true;
    }
  }
  function lineIsHiddenInner(doc, line, span) {
    if (span.to == null) {
      var end = span.marker.find(1, true);
      return lineIsHiddenInner(doc, end.line, getMarkedSpanFor(end.line.markedSpans, span.marker));
    }
    if (span.marker.inclusiveRight && span.to == line.text.length)
      return true;
    for (var sp, i = 0; i < line.markedSpans.length; ++i) {
      sp = line.markedSpans[i];
      if (sp.marker.collapsed && !sp.marker.widgetNode && sp.from == span.to &&
          (sp.to == null || sp.to != span.from) &&
          (sp.marker.inclusiveLeft || span.marker.inclusiveRight) &&
          lineIsHiddenInner(doc, line, sp)) return true;
    }
  }

  // LINE WIDGETS

  // Line widgets are block elements displayed above or below a line.

  var LineWidget = CodeMirror.LineWidget = function(doc, node, options) {
    if (options) for (var opt in options) if (options.hasOwnProperty(opt))
      this[opt] = options[opt];
    this.doc = doc;
    this.node = node;
  };
  eventMixin(LineWidget);

  function adjustScrollWhenAboveVisible(cm, line, diff) {
    if (heightAtLine(line) < ((cm.curOp && cm.curOp.scrollTop) || cm.doc.scrollTop))
      addToScrollPos(cm, null, diff);
  }

  LineWidget.prototype.clear = function() {
    var cm = this.doc.cm, ws = this.line.widgets, line = this.line, no = lineNo(line);
    if (no == null || !ws) return;
    for (var i = 0; i < ws.length; ++i) if (ws[i] == this) ws.splice(i--, 1);
    if (!ws.length) line.widgets = null;
    var height = widgetHeight(this);
    updateLineHeight(line, Math.max(0, line.height - height));
    if (cm) runInOp(cm, function() {
      adjustScrollWhenAboveVisible(cm, line, -height);
      regLineChange(cm, no, "widget");
    });
  };
  LineWidget.prototype.changed = function() {
    var oldH = this.height, cm = this.doc.cm, line = this.line;
    this.height = null;
    var diff = widgetHeight(this) - oldH;
    if (!diff) return;
    updateLineHeight(line, line.height + diff);
    if (cm) runInOp(cm, function() {
      cm.curOp.forceUpdate = true;
      adjustScrollWhenAboveVisible(cm, line, diff);
    });
  };

  function widgetHeight(widget) {
    if (widget.height != null) return widget.height;
    var cm = widget.doc.cm;
    if (!cm) return 0;
    if (!contains(document.body, widget.node)) {
      var parentStyle = "position: relative;";
      if (widget.coverGutter)
        parentStyle += "margin-left: -" + cm.display.gutters.offsetWidth + "px;";
      if (widget.noHScroll)
        parentStyle += "width: " + cm.display.wrapper.clientWidth + "px;";
      removeChildrenAndAdd(cm.display.measure, elt("div", [widget.node], null, parentStyle));
    }
    return widget.height = widget.node.parentNode.offsetHeight;
  }

  function addLineWidget(doc, handle, node, options) {
    var widget = new LineWidget(doc, node, options);
    var cm = doc.cm;
    if (cm && widget.noHScroll) cm.display.alignWidgets = true;
    changeLine(doc, handle, "widget", function(line) {
      var widgets = line.widgets || (line.widgets = []);
      if (widget.insertAt == null) widgets.push(widget);
      else widgets.splice(Math.min(widgets.length - 1, Math.max(0, widget.insertAt)), 0, widget);
      widget.line = line;
      if (cm && !lineIsHidden(doc, line)) {
        var aboveVisible = heightAtLine(line) < doc.scrollTop;
        updateLineHeight(line, line.height + widgetHeight(widget));
        if (aboveVisible) addToScrollPos(cm, null, widget.height);
        cm.curOp.forceUpdate = true;
      }
      return true;
    });
    return widget;
  }

  // LINE DATA STRUCTURE

  // Line objects. These hold state related to a line, including
  // highlighting info (the styles array).
  var Line = CodeMirror.Line = function(text, markedSpans, estimateHeight) {
    this.text = text;
    attachMarkedSpans(this, markedSpans);
    this.height = estimateHeight ? estimateHeight(this) : 1;
  };
  eventMixin(Line);
  Line.prototype.lineNo = function() { return lineNo(this); };

  // Change the content (text, markers) of a line. Automatically
  // invalidates cached information and tries to re-estimate the
  // line's height.
  function updateLine(line, text, markedSpans, estimateHeight) {
    line.text = text;
    if (line.stateAfter) line.stateAfter = null;
    if (line.styles) line.styles = null;
    if (line.order != null) line.order = null;
    detachMarkedSpans(line);
    attachMarkedSpans(line, markedSpans);
    var estHeight = estimateHeight ? estimateHeight(line) : 1;
    if (estHeight != line.height) updateLineHeight(line, estHeight);
  }

  // Detach a line from the document tree and its markers.
  function cleanUpLine(line) {
    line.parent = null;
    detachMarkedSpans(line);
  }

  function extractLineClasses(type, output) {
    if (type) for (;;) {
      var lineClass = type.match(/(?:^|\s+)line-(background-)?(\S+)/);
      if (!lineClass) break;
      type = type.slice(0, lineClass.index) + type.slice(lineClass.index + lineClass[0].length);
      var prop = lineClass[1] ? "bgClass" : "textClass";
      if (output[prop] == null)
        output[prop] = lineClass[2];
      else if (!(new RegExp("(?:^|\s)" + lineClass[2] + "(?:$|\s)")).test(output[prop]))
        output[prop] += " " + lineClass[2];
    }
    return type;
  }

  function callBlankLine(mode, state) {
    if (mode.blankLine) return mode.blankLine(state);
    if (!mode.innerMode) return;
    var inner = CodeMirror.innerMode(mode, state);
    if (inner.mode.blankLine) return inner.mode.blankLine(inner.state);
  }

  function readToken(mode, stream, state, inner) {
    for (var i = 0; i < 10; i++) {
      if (inner) inner[0] = CodeMirror.innerMode(mode, state).mode;
      var style = mode.token(stream, state);
      if (stream.pos > stream.start) return style;
    }
    throw new Error("Mode " + mode.name + " failed to advance stream.");
  }

  // Utility for getTokenAt and getLineTokens
  function takeToken(cm, pos, precise, asArray) {
    function getObj(copy) {
      return {start: stream.start, end: stream.pos,
              string: stream.current(),
              type: style || null,
              state: copy ? copyState(doc.mode, state) : state};
    }

    var doc = cm.doc, mode = doc.mode, style;
    pos = clipPos(doc, pos);
    var line = getLine(doc, pos.line), state = getStateBefore(cm, pos.line, precise);
    var stream = new StringStream(line.text, cm.options.tabSize), tokens;
    if (asArray) tokens = [];
    while ((asArray || stream.pos < pos.ch) && !stream.eol()) {
      stream.start = stream.pos;
      style = readToken(mode, stream, state);
      if (asArray) tokens.push(getObj(true));
    }
    return asArray ? tokens : getObj();
  }

  // Run the given mode's parser over a line, calling f for each token.
  function runMode(cm, text, mode, state, f, lineClasses, forceToEnd) {
    var flattenSpans = mode.flattenSpans;
    if (flattenSpans == null) flattenSpans = cm.options.flattenSpans;
    var curStart = 0, curStyle = null;
    var stream = new StringStream(text, cm.options.tabSize), style;
    var inner = cm.options.addModeClass && [null];
    if (text == "") extractLineClasses(callBlankLine(mode, state), lineClasses);
    while (!stream.eol()) {
      if (stream.pos > cm.options.maxHighlightLength) {
        flattenSpans = false;
        if (forceToEnd) processLine(cm, text, state, stream.pos);
        stream.pos = text.length;
        style = null;
      } else {
        style = extractLineClasses(readToken(mode, stream, state, inner), lineClasses);
      }
      if (inner) {
        var mName = inner[0].name;
        if (mName) style = "m-" + (style ? mName + " " + style : mName);
      }
      if (!flattenSpans || curStyle != style) {
        while (curStart < stream.start) {
          curStart = Math.min(stream.start, curStart + 50000);
          f(curStart, curStyle);
        }
        curStyle = style;
      }
      stream.start = stream.pos;
    }
    while (curStart < stream.pos) {
      // Webkit seems to refuse to render text nodes longer than 57444 characters
      var pos = Math.min(stream.pos, curStart + 50000);
      f(pos, curStyle);
      curStart = pos;
    }
  }

  // Compute a style array (an array starting with a mode generation
  // -- for invalidation -- followed by pairs of end positions and
  // style strings), which is used to highlight the tokens on the
  // line.
  function highlightLine(cm, line, state, forceToEnd) {
    // A styles array always starts with a number identifying the
    // mode/overlays that it is based on (for easy invalidation).
    var st = [cm.state.modeGen], lineClasses = {};
    // Compute the base array of styles
    runMode(cm, line.text, cm.doc.mode, state, function(end, style) {
      st.push(end, style);
    }, lineClasses, forceToEnd);

    // Run overlays, adjust style array.
    for (var o = 0; o < cm.state.overlays.length; ++o) {
      var overlay = cm.state.overlays[o], i = 1, at = 0;
      runMode(cm, line.text, overlay.mode, true, function(end, style) {
        var start = i;
        // Ensure there's a token end at the current position, and that i points at it
        while (at < end) {
          var i_end = st[i];
          if (i_end > end)
            st.splice(i, 1, end, st[i+1], i_end);
          i += 2;
          at = Math.min(end, i_end);
        }
        if (!style) return;
        if (overlay.opaque) {
          st.splice(start, i - start, end, "cm-overlay " + style);
          i = start + 2;
        } else {
          for (; start < i; start += 2) {
            var cur = st[start+1];
            st[start+1] = (cur ? cur + " " : "") + "cm-overlay " + style;
          }
        }
      }, lineClasses);
    }

    return {styles: st, classes: lineClasses.bgClass || lineClasses.textClass ? lineClasses : null};
  }

  function getLineStyles(cm, line, updateFrontier) {
    if (!line.styles || line.styles[0] != cm.state.modeGen) {
      var state = getStateBefore(cm, lineNo(line));
      var result = highlightLine(cm, line, line.text.length > cm.options.maxHighlightLength ? copyState(cm.doc.mode, state) : state);
      line.stateAfter = state;
      line.styles = result.styles;
      if (result.classes) line.styleClasses = result.classes;
      else if (line.styleClasses) line.styleClasses = null;
      if (updateFrontier === cm.doc.frontier) cm.doc.frontier++;
    }
    return line.styles;
  }

  // Lightweight form of highlight -- proceed over this line and
  // update state, but don't save a style array. Used for lines that
  // aren't currently visible.
  function processLine(cm, text, state, startAt) {
    var mode = cm.doc.mode;
    var stream = new StringStream(text, cm.options.tabSize);
    stream.start = stream.pos = startAt || 0;
    if (text == "") callBlankLine(mode, state);
    while (!stream.eol()) {
      readToken(mode, stream, state);
      stream.start = stream.pos;
    }
  }

  // Convert a style as returned by a mode (either null, or a string
  // containing one or more styles) to a CSS style. This is cached,
  // and also looks for line-wide styles.
  var styleToClassCache = {}, styleToClassCacheWithMode = {};
  function interpretTokenStyle(style, options) {
    if (!style || /^\s*$/.test(style)) return null;
    var cache = options.addModeClass ? styleToClassCacheWithMode : styleToClassCache;
    return cache[style] ||
      (cache[style] = style.replace(/\S+/g, "cm-$&"));
  }

  // Render the DOM representation of the text of a line. Also builds
  // up a 'line map', which points at the DOM nodes that represent
  // specific stretches of text, and is used by the measuring code.
  // The returned object contains the DOM node, this map, and
  // information about line-wide styles that were set by the mode.
  function buildLineContent(cm, lineView) {
    // The padding-right forces the element to have a 'border', which
    // is needed on Webkit to be able to get line-level bounding
    // rectangles for it (in measureChar).
    var content = elt("span", null, null, webkit ? "padding-right: .1px" : null);
    var builder = {pre: elt("pre", [content], "CodeMirror-line"), content: content,
                   col: 0, pos: 0, cm: cm,
                   splitSpaces: (ie || webkit) && cm.getOption("lineWrapping")};
    lineView.measure = {};

    // Iterate over the logical lines that make up this visual line.
    for (var i = 0; i <= (lineView.rest ? lineView.rest.length : 0); i++) {
      var line = i ? lineView.rest[i - 1] : lineView.line, order;
      builder.pos = 0;
      builder.addToken = buildToken;
      // Optionally wire in some hacks into the token-rendering
      // algorithm, to deal with browser quirks.
      if (hasBadBidiRects(cm.display.measure) && (order = getOrder(line)))
        builder.addToken = buildTokenBadBidi(builder.addToken, order);
      builder.map = [];
      var allowFrontierUpdate = lineView != cm.display.externalMeasured && lineNo(line);
      insertLineContent(line, builder, getLineStyles(cm, line, allowFrontierUpdate));
      if (line.styleClasses) {
        if (line.styleClasses.bgClass)
          builder.bgClass = joinClasses(line.styleClasses.bgClass, builder.bgClass || "");
        if (line.styleClasses.textClass)
          builder.textClass = joinClasses(line.styleClasses.textClass, builder.textClass || "");
      }

      // Ensure at least a single node is present, for measuring.
      if (builder.map.length == 0)
        builder.map.push(0, 0, builder.content.appendChild(zeroWidthElement(cm.display.measure)));

      // Store the map and a cache object for the current logical line
      if (i == 0) {
        lineView.measure.map = builder.map;
        lineView.measure.cache = {};
      } else {
        (lineView.measure.maps || (lineView.measure.maps = [])).push(builder.map);
        (lineView.measure.caches || (lineView.measure.caches = [])).push({});
      }
    }

    // See issue #2901
    if (webkit) {
      var last = builder.content.lastChild
      if (/\bcm-tab\b/.test(last.className) || (last.querySelector && last.querySelector(".cm-tab")))
        builder.content.className = "cm-tab-wrap-hack";
    }

    signal(cm, "renderLine", cm, lineView.line, builder.pre);
    if (builder.pre.className)
      builder.textClass = joinClasses(builder.pre.className, builder.textClass || "");

    return builder;
  }

  function defaultSpecialCharPlaceholder(ch) {
    var token = elt("span", "\u2022", "cm-invalidchar");
    token.title = "\\u" + ch.charCodeAt(0).toString(16);
    token.setAttribute("aria-label", token.title);
    return token;
  }

  // Build up the DOM representation for a single token, and add it to
  // the line map. Takes care to render special characters separately.
  function buildToken(builder, text, style, startStyle, endStyle, title, css) {
    if (!text) return;
    var displayText = builder.splitSpaces ? text.replace(/ {3,}/g, splitSpaces) : text;
    var special = builder.cm.state.specialChars, mustWrap = false;
    if (!special.test(text)) {
      builder.col += text.length;
      var content = document.createTextNode(displayText);
      builder.map.push(builder.pos, builder.pos + text.length, content);
      if (ie && ie_version < 9) mustWrap = true;
      builder.pos += text.length;
    } else {
      var content = document.createDocumentFragment(), pos = 0;
      while (true) {
        special.lastIndex = pos;
        var m = special.exec(text);
        var skipped = m ? m.index - pos : text.length - pos;
        if (skipped) {
          var txt = document.createTextNode(displayText.slice(pos, pos + skipped));
          if (ie && ie_version < 9) content.appendChild(elt("span", [txt]));
          else content.appendChild(txt);
          builder.map.push(builder.pos, builder.pos + skipped, txt);
          builder.col += skipped;
          builder.pos += skipped;
        }
        if (!m) break;
        pos += skipped + 1;
        if (m[0] == "\t") {
          var tabSize = builder.cm.options.tabSize, tabWidth = tabSize - builder.col % tabSize;
          var txt = content.appendChild(elt("span", spaceStr(tabWidth), "cm-tab"));
          txt.setAttribute("role", "presentation");
          txt.setAttribute("cm-text", "\t");
          builder.col += tabWidth;
        } else if (m[0] == "\r" || m[0] == "\n") {
          var txt = content.appendChild(elt("span", m[0] == "\r" ? "\u240d" : "\u2424", "cm-invalidchar"));
          txt.setAttribute("cm-text", m[0]);
          builder.col += 1;
        } else {
          var txt = builder.cm.options.specialCharPlaceholder(m[0]);
          txt.setAttribute("cm-text", m[0]);
          if (ie && ie_version < 9) content.appendChild(elt("span", [txt]));
          else content.appendChild(txt);
          builder.col += 1;
        }
        builder.map.push(builder.pos, builder.pos + 1, txt);
        builder.pos++;
      }
    }
    if (style || startStyle || endStyle || mustWrap || css) {
      var fullStyle = style || "";
      if (startStyle) fullStyle += startStyle;
      if (endStyle) fullStyle += endStyle;
      var token = elt("span", [content], fullStyle, css);
      if (title) token.title = title;
      return builder.content.appendChild(token);
    }
    builder.content.appendChild(content);
  }

  function splitSpaces(old) {
    var out = " ";
    for (var i = 0; i < old.length - 2; ++i) out += i % 2 ? " " : "\u00a0";
    out += " ";
    return out;
  }

  // Work around nonsense dimensions being reported for stretches of
  // right-to-left text.
  function buildTokenBadBidi(inner, order) {
    return function(builder, text, style, startStyle, endStyle, title, css) {
      style = style ? style + " cm-force-border" : "cm-force-border";
      var start = builder.pos, end = start + text.length;
      for (;;) {
        // Find the part that overlaps with the start of this text
        for (var i = 0; i < order.length; i++) {
          var part = order[i];
          if (part.to > start && part.from <= start) break;
        }
        if (part.to >= end) return inner(builder, text, style, startStyle, endStyle, title, css);
        inner(builder, text.slice(0, part.to - start), style, startStyle, null, title, css);
        startStyle = null;
        text = text.slice(part.to - start);
        start = part.to;
      }
    };
  }

  function buildCollapsedSpan(builder, size, marker, ignoreWidget) {
    var widget = !ignoreWidget && marker.widgetNode;
    if (widget) builder.map.push(builder.pos, builder.pos + size, widget);
    if (!ignoreWidget && builder.cm.display.input.needsContentAttribute) {
      if (!widget)
        widget = builder.content.appendChild(document.createElement("span"));
      widget.setAttribute("cm-marker", marker.id);
    }
    if (widget) {
      builder.cm.display.input.setUneditable(widget);
      builder.content.appendChild(widget);
    }
    builder.pos += size;
  }

  // Outputs a number of spans to make up a line, taking highlighting
  // and marked text into account.
  function insertLineContent(line, builder, styles) {
    var spans = line.markedSpans, allText = line.text, at = 0;
    if (!spans) {
      for (var i = 1; i < styles.length; i+=2)
        builder.addToken(builder, allText.slice(at, at = styles[i]), interpretTokenStyle(styles[i+1], builder.cm.options));
      return;
    }

    var len = allText.length, pos = 0, i = 1, text = "", style, css;
    var nextChange = 0, spanStyle, spanEndStyle, spanStartStyle, title, collapsed;
    for (;;) {
      if (nextChange == pos) { // Update current marker set
        spanStyle = spanEndStyle = spanStartStyle = title = css = "";
        collapsed = null; nextChange = Infinity;
        var foundBookmarks = [], endStyles
        for (var j = 0; j < spans.length; ++j) {
          var sp = spans[j], m = sp.marker;
          if (m.type == "bookmark" && sp.from == pos && m.widgetNode) {
            foundBookmarks.push(m);
          } else if (sp.from <= pos && (sp.to == null || sp.to > pos || m.collapsed && sp.to == pos && sp.from == pos)) {
            if (sp.to != null && sp.to != pos && nextChange > sp.to) {
              nextChange = sp.to;
              spanEndStyle = "";
            }
            if (m.className) spanStyle += " " + m.className;
            if (m.css) css = (css ? css + ";" : "") + m.css;
            if (m.startStyle && sp.from == pos) spanStartStyle += " " + m.startStyle;
            if (m.endStyle && sp.to == nextChange) (endStyles || (endStyles = [])).push(m.endStyle, sp.to)
            if (m.title && !title) title = m.title;
            if (m.collapsed && (!collapsed || compareCollapsedMarkers(collapsed.marker, m) < 0))
              collapsed = sp;
          } else if (sp.from > pos && nextChange > sp.from) {
            nextChange = sp.from;
          }
        }
        if (endStyles) for (var j = 0; j < endStyles.length; j += 2)
          if (endStyles[j + 1] == nextChange) spanEndStyle += " " + endStyles[j]

        if (!collapsed || collapsed.from == pos) for (var j = 0; j < foundBookmarks.length; ++j)
          buildCollapsedSpan(builder, 0, foundBookmarks[j]);
        if (collapsed && (collapsed.from || 0) == pos) {
          buildCollapsedSpan(builder, (collapsed.to == null ? len + 1 : collapsed.to) - pos,
                             collapsed.marker, collapsed.from == null);
          if (collapsed.to == null) return;
          if (collapsed.to == pos) collapsed = false;
        }
      }
      if (pos >= len) break;

      var upto = Math.min(len, nextChange);
      while (true) {
        if (text) {
          var end = pos + text.length;
          if (!collapsed) {
            var tokenText = end > upto ? text.slice(0, upto - pos) : text;
            builder.addToken(builder, tokenText, style ? style + spanStyle : spanStyle,
                             spanStartStyle, pos + tokenText.length == nextChange ? spanEndStyle : "", title, css);
          }
          if (end >= upto) {text = text.slice(upto - pos); pos = upto; break;}
          pos = end;
          spanStartStyle = "";
        }
        text = allText.slice(at, at = styles[i++]);
        style = interpretTokenStyle(styles[i++], builder.cm.options);
      }
    }
  }

  // DOCUMENT DATA STRUCTURE

  // By default, updates that start and end at the beginning of a line
  // are treated specially, in order to make the association of line
  // widgets and marker elements with the text behave more intuitive.
  function isWholeLineUpdate(doc, change) {
    return change.from.ch == 0 && change.to.ch == 0 && lst(change.text) == "" &&
      (!doc.cm || doc.cm.options.wholeLineUpdateBefore);
  }

  // Perform a change on the document data structure.
  function updateDoc(doc, change, markedSpans, estimateHeight) {
    function spansFor(n) {return markedSpans ? markedSpans[n] : null;}
    function update(line, text, spans) {
      updateLine(line, text, spans, estimateHeight);
      signalLater(line, "change", line, change);
    }
    function linesFor(start, end) {
      for (var i = start, result = []; i < end; ++i)
        result.push(new Line(text[i], spansFor(i), estimateHeight));
      return result;
    }

    var from = change.from, to = change.to, text = change.text;
    var firstLine = getLine(doc, from.line), lastLine = getLine(doc, to.line);
    var lastText = lst(text), lastSpans = spansFor(text.length - 1), nlines = to.line - from.line;

    // Adjust the line structure
    if (change.full) {
      doc.insert(0, linesFor(0, text.length));
      doc.remove(text.length, doc.size - text.length);
    } else if (isWholeLineUpdate(doc, change)) {
      // This is a whole-line replace. Treated specially to make
      // sure line objects move the way they are supposed to.
      var added = linesFor(0, text.length - 1);
      update(lastLine, lastLine.text, lastSpans);
      if (nlines) doc.remove(from.line, nlines);
      if (added.length) doc.insert(from.line, added);
    } else if (firstLine == lastLine) {
      if (text.length == 1) {
        update(firstLine, firstLine.text.slice(0, from.ch) + lastText + firstLine.text.slice(to.ch), lastSpans);
      } else {
        var added = linesFor(1, text.length - 1);
        added.push(new Line(lastText + firstLine.text.slice(to.ch), lastSpans, estimateHeight));
        update(firstLine, firstLine.text.slice(0, from.ch) + text[0], spansFor(0));
        doc.insert(from.line + 1, added);
      }
    } else if (text.length == 1) {
      update(firstLine, firstLine.text.slice(0, from.ch) + text[0] + lastLine.text.slice(to.ch), spansFor(0));
      doc.remove(from.line + 1, nlines);
    } else {
      update(firstLine, firstLine.text.slice(0, from.ch) + text[0], spansFor(0));
      update(lastLine, lastText + lastLine.text.slice(to.ch), lastSpans);
      var added = linesFor(1, text.length - 1);
      if (nlines > 1) doc.remove(from.line + 1, nlines - 1);
      doc.insert(from.line + 1, added);
    }

    signalLater(doc, "change", doc, change);
  }

  // The document is represented as a BTree consisting of leaves, with
  // chunk of lines in them, and branches, with up to ten leaves or
  // other branch nodes below them. The top node is always a branch
  // node, and is the document object itself (meaning it has
  // additional methods and properties).
  //
  // All nodes have parent links. The tree is used both to go from
  // line numbers to line objects, and to go from objects to numbers.
  // It also indexes by height, and is used to convert between height
  // and line object, and to find the total height of the document.
  //
  // See also http://marijnhaverbeke.nl/blog/codemirror-line-tree.html

  function LeafChunk(lines) {
    this.lines = lines;
    this.parent = null;
    for (var i = 0, height = 0; i < lines.length; ++i) {
      lines[i].parent = this;
      height += lines[i].height;
    }
    this.height = height;
  }

  LeafChunk.prototype = {
    chunkSize: function() { return this.lines.length; },
    // Remove the n lines at offset 'at'.
    removeInner: function(at, n) {
      for (var i = at, e = at + n; i < e; ++i) {
        var line = this.lines[i];
        this.height -= line.height;
        cleanUpLine(line);
        signalLater(line, "delete");
      }
      this.lines.splice(at, n);
    },
    // Helper used to collapse a small branch into a single leaf.
    collapse: function(lines) {
      lines.push.apply(lines, this.lines);
    },
    // Insert the given array of lines at offset 'at', count them as
    // having the given height.
    insertInner: function(at, lines, height) {
      this.height += height;
      this.lines = this.lines.slice(0, at).concat(lines).concat(this.lines.slice(at));
      for (var i = 0; i < lines.length; ++i) lines[i].parent = this;
    },
    // Used to iterate over a part of the tree.
    iterN: function(at, n, op) {
      for (var e = at + n; at < e; ++at)
        if (op(this.lines[at])) return true;
    }
  };

  function BranchChunk(children) {
    this.children = children;
    var size = 0, height = 0;
    for (var i = 0; i < children.length; ++i) {
      var ch = children[i];
      size += ch.chunkSize(); height += ch.height;
      ch.parent = this;
    }
    this.size = size;
    this.height = height;
    this.parent = null;
  }

  BranchChunk.prototype = {
    chunkSize: function() { return this.size; },
    removeInner: function(at, n) {
      this.size -= n;
      for (var i = 0; i < this.children.length; ++i) {
        var child = this.children[i], sz = child.chunkSize();
        if (at < sz) {
          var rm = Math.min(n, sz - at), oldHeight = child.height;
          child.removeInner(at, rm);
          this.height -= oldHeight - child.height;
          if (sz == rm) { this.children.splice(i--, 1); child.parent = null; }
          if ((n -= rm) == 0) break;
          at = 0;
        } else at -= sz;
      }
      // If the result is smaller than 25 lines, ensure that it is a
      // single leaf node.
      if (this.size - n < 25 &&
          (this.children.length > 1 || !(this.children[0] instanceof LeafChunk))) {
        var lines = [];
        this.collapse(lines);
        this.children = [new LeafChunk(lines)];
        this.children[0].parent = this;
      }
    },
    collapse: function(lines) {
      for (var i = 0; i < this.children.length; ++i) this.children[i].collapse(lines);
    },
    insertInner: function(at, lines, height) {
      this.size += lines.length;
      this.height += height;
      for (var i = 0; i < this.children.length; ++i) {
        var child = this.children[i], sz = child.chunkSize();
        if (at <= sz) {
          child.insertInner(at, lines, height);
          if (child.lines && child.lines.length > 50) {
            // To avoid memory thrashing when child.lines is huge (e.g. first view of a large file), it's never spliced.
            // Instead, small slices are taken. They're taken in order because sequential memory accesses are fastest.
            var remaining = child.lines.length % 25 + 25
            for (var pos = remaining; pos < child.lines.length;) {
              var leaf = new LeafChunk(child.lines.slice(pos, pos += 25));
              child.height -= leaf.height;
              this.children.splice(++i, 0, leaf);
              leaf.parent = this;
            }
            child.lines = child.lines.slice(0, remaining);
            this.maybeSpill();
          }
          break;
        }
        at -= sz;
      }
    },
    // When a node has grown, check whether it should be split.
    maybeSpill: function() {
      if (this.children.length <= 10) return;
      var me = this;
      do {
        var spilled = me.children.splice(me.children.length - 5, 5);
        var sibling = new BranchChunk(spilled);
        if (!me.parent) { // Become the parent node
          var copy = new BranchChunk(me.children);
          copy.parent = me;
          me.children = [copy, sibling];
          me = copy;
       } else {
          me.size -= sibling.size;
          me.height -= sibling.height;
          var myIndex = indexOf(me.parent.children, me);
          me.parent.children.splice(myIndex + 1, 0, sibling);
        }
        sibling.parent = me.parent;
      } while (me.children.length > 10);
      me.parent.maybeSpill();
    },
    iterN: function(at, n, op) {
      for (var i = 0; i < this.children.length; ++i) {
        var child = this.children[i], sz = child.chunkSize();
        if (at < sz) {
          var used = Math.min(n, sz - at);
          if (child.iterN(at, used, op)) return true;
          if ((n -= used) == 0) break;
          at = 0;
        } else at -= sz;
      }
    }
  };

  var nextDocId = 0;
  var Doc = CodeMirror.Doc = function(text, mode, firstLine, lineSep) {
    if (!(this instanceof Doc)) return new Doc(text, mode, firstLine, lineSep);
    if (firstLine == null) firstLine = 0;

    BranchChunk.call(this, [new LeafChunk([new Line("", null)])]);
    this.first = firstLine;
    this.scrollTop = this.scrollLeft = 0;
    this.cantEdit = false;
    this.cleanGeneration = 1;
    this.frontier = firstLine;
    var start = Pos(firstLine, 0);
    this.sel = simpleSelection(start);
    this.history = new History(null);
    this.id = ++nextDocId;
    this.modeOption = mode;
    this.lineSep = lineSep;
    this.extend = false;

    if (typeof text == "string") text = this.splitLines(text);
    updateDoc(this, {from: start, to: start, text: text});
    setSelection(this, simpleSelection(start), sel_dontScroll);
  };

  Doc.prototype = createObj(BranchChunk.prototype, {
    constructor: Doc,
    // Iterate over the document. Supports two forms -- with only one
    // argument, it calls that for each line in the document. With
    // three, it iterates over the range given by the first two (with
    // the second being non-inclusive).
    iter: function(from, to, op) {
      if (op) this.iterN(from - this.first, to - from, op);
      else this.iterN(this.first, this.first + this.size, from);
    },

    // Non-public interface for adding and removing lines.
    insert: function(at, lines) {
      var height = 0;
      for (var i = 0; i < lines.length; ++i) height += lines[i].height;
      this.insertInner(at - this.first, lines, height);
    },
    remove: function(at, n) { this.removeInner(at - this.first, n); },

    // From here, the methods are part of the public interface. Most
    // are also available from CodeMirror (editor) instances.

    getValue: function(lineSep) {
      var lines = getLines(this, this.first, this.first + this.size);
      if (lineSep === false) return lines;
      return lines.join(lineSep || this.lineSeparator());
    },
    setValue: docMethodOp(function(code) {
      var top = Pos(this.first, 0), last = this.first + this.size - 1;
      makeChange(this, {from: top, to: Pos(last, getLine(this, last).text.length),
                        text: this.splitLines(code), origin: "setValue", full: true}, true);
      setSelection(this, simpleSelection(top));
    }),
    replaceRange: function(code, from, to, origin) {
      from = clipPos(this, from);
      to = to ? clipPos(this, to) : from;
      replaceRange(this, code, from, to, origin);
    },
    getRange: function(from, to, lineSep) {
      var lines = getBetween(this, clipPos(this, from), clipPos(this, to));
      if (lineSep === false) return lines;
      return lines.join(lineSep || this.lineSeparator());
    },

    getLine: function(line) {var l = this.getLineHandle(line); return l && l.text;},

    getLineHandle: function(line) {if (isLine(this, line)) return getLine(this, line);},
    getLineNumber: function(line) {return lineNo(line);},

    getLineHandleVisualStart: function(line) {
      if (typeof line == "number") line = getLine(this, line);
      return visualLine(line);
    },

    lineCount: function() {return this.size;},
    firstLine: function() {return this.first;},
    lastLine: function() {return this.first + this.size - 1;},

    clipPos: function(pos) {return clipPos(this, pos);},

    getCursor: function(start) {
      var range = this.sel.primary(), pos;
      if (start == null || start == "head") pos = range.head;
      else if (start == "anchor") pos = range.anchor;
      else if (start == "end" || start == "to" || start === false) pos = range.to();
      else pos = range.from();
      return pos;
    },
    listSelections: function() { return this.sel.ranges; },
    somethingSelected: function() {return this.sel.somethingSelected();},

    setCursor: docMethodOp(function(line, ch, options) {
      setSimpleSelection(this, clipPos(this, typeof line == "number" ? Pos(line, ch || 0) : line), null, options);
    }),
    setSelection: docMethodOp(function(anchor, head, options) {
      setSimpleSelection(this, clipPos(this, anchor), clipPos(this, head || anchor), options);
    }),
    extendSelection: docMethodOp(function(head, other, options) {
      extendSelection(this, clipPos(this, head), other && clipPos(this, other), options);
    }),
    extendSelections: docMethodOp(function(heads, options) {
      extendSelections(this, clipPosArray(this, heads), options);
    }),
    extendSelectionsBy: docMethodOp(function(f, options) {
      var heads = map(this.sel.ranges, f);
      extendSelections(this, clipPosArray(this, heads), options);
    }),
    setSelections: docMethodOp(function(ranges, primary, options) {
      if (!ranges.length) return;
      for (var i = 0, out = []; i < ranges.length; i++)
        out[i] = new Range(clipPos(this, ranges[i].anchor),
                           clipPos(this, ranges[i].head));
      if (primary == null) primary = Math.min(ranges.length - 1, this.sel.primIndex);
      setSelection(this, normalizeSelection(out, primary), options);
    }),
    addSelection: docMethodOp(function(anchor, head, options) {
      var ranges = this.sel.ranges.slice(0);
      ranges.push(new Range(clipPos(this, anchor), clipPos(this, head || anchor)));
      setSelection(this, normalizeSelection(ranges, ranges.length - 1), options);
    }),

    getSelection: function(lineSep) {
      var ranges = this.sel.ranges, lines;
      for (var i = 0; i < ranges.length; i++) {
        var sel = getBetween(this, ranges[i].from(), ranges[i].to());
        lines = lines ? lines.concat(sel) : sel;
      }
      if (lineSep === false) return lines;
      else return lines.join(lineSep || this.lineSeparator());
    },
    getSelections: function(lineSep) {
      var parts = [], ranges = this.sel.ranges;
      for (var i = 0; i < ranges.length; i++) {
        var sel = getBetween(this, ranges[i].from(), ranges[i].to());
        if (lineSep !== false) sel = sel.join(lineSep || this.lineSeparator());
        parts[i] = sel;
      }
      return parts;
    },
    replaceSelection: function(code, collapse, origin) {
      var dup = [];
      for (var i = 0; i < this.sel.ranges.length; i++)
        dup[i] = code;
      this.replaceSelections(dup, collapse, origin || "+input");
    },
    replaceSelections: docMethodOp(function(code, collapse, origin) {
      var changes = [], sel = this.sel;
      for (var i = 0; i < sel.ranges.length; i++) {
        var range = sel.ranges[i];
        changes[i] = {from: range.from(), to: range.to(), text: this.splitLines(code[i]), origin: origin};
      }
      var newSel = collapse && collapse != "end" && computeReplacedSel(this, changes, collapse);
      for (var i = changes.length - 1; i >= 0; i--)
        makeChange(this, changes[i]);
      if (newSel) setSelectionReplaceHistory(this, newSel);
      else if (this.cm) ensureCursorVisible(this.cm);
    }),
    undo: docMethodOp(function() {makeChangeFromHistory(this, "undo");}),
    redo: docMethodOp(function() {makeChangeFromHistory(this, "redo");}),
    undoSelection: docMethodOp(function() {makeChangeFromHistory(this, "undo", true);}),
    redoSelection: docMethodOp(function() {makeChangeFromHistory(this, "redo", true);}),

    setExtending: function(val) {this.extend = val;},
    getExtending: function() {return this.extend;},

    historySize: function() {
      var hist = this.history, done = 0, undone = 0;
      for (var i = 0; i < hist.done.length; i++) if (!hist.done[i].ranges) ++done;
      for (var i = 0; i < hist.undone.length; i++) if (!hist.undone[i].ranges) ++undone;
      return {undo: done, redo: undone};
    },
    clearHistory: function() {this.history = new History(this.history.maxGeneration);},

    markClean: function() {
      this.cleanGeneration = this.changeGeneration(true);
    },
    changeGeneration: function(forceSplit) {
      if (forceSplit)
        this.history.lastOp = this.history.lastSelOp = this.history.lastOrigin = null;
      return this.history.generation;
    },
    isClean: function (gen) {
      return this.history.generation == (gen || this.cleanGeneration);
    },

    getHistory: function() {
      return {done: copyHistoryArray(this.history.done),
              undone: copyHistoryArray(this.history.undone)};
    },
    setHistory: function(histData) {
      var hist = this.history = new History(this.history.maxGeneration);
      hist.done = copyHistoryArray(histData.done.slice(0), null, true);
      hist.undone = copyHistoryArray(histData.undone.slice(0), null, true);
    },

    addLineClass: docMethodOp(function(handle, where, cls) {
      return changeLine(this, handle, where == "gutter" ? "gutter" : "class", function(line) {
        var prop = where == "text" ? "textClass"
                 : where == "background" ? "bgClass"
                 : where == "gutter" ? "gutterClass" : "wrapClass";
        if (!line[prop]) line[prop] = cls;
        else if (classTest(cls).test(line[prop])) return false;
        else line[prop] += " " + cls;
        return true;
      });
    }),
    removeLineClass: docMethodOp(function(handle, where, cls) {
      return changeLine(this, handle, where == "gutter" ? "gutter" : "class", function(line) {
        var prop = where == "text" ? "textClass"
                 : where == "background" ? "bgClass"
                 : where == "gutter" ? "gutterClass" : "wrapClass";
        var cur = line[prop];
        if (!cur) return false;
        else if (cls == null) line[prop] = null;
        else {
          var found = cur.match(classTest(cls));
          if (!found) return false;
          var end = found.index + found[0].length;
          line[prop] = cur.slice(0, found.index) + (!found.index || end == cur.length ? "" : " ") + cur.slice(end) || null;
        }
        return true;
      });
    }),

    addLineWidget: docMethodOp(function(handle, node, options) {
      return addLineWidget(this, handle, node, options);
    }),
    removeLineWidget: function(widget) { widget.clear(); },

    markText: function(from, to, options) {
      return markText(this, clipPos(this, from), clipPos(this, to), options, options && options.type || "range");
    },
    setBookmark: function(pos, options) {
      var realOpts = {replacedWith: options && (options.nodeType == null ? options.widget : options),
                      insertLeft: options && options.insertLeft,
                      clearWhenEmpty: false, shared: options && options.shared,
                      handleMouseEvents: options && options.handleMouseEvents};
      pos = clipPos(this, pos);
      return markText(this, pos, pos, realOpts, "bookmark");
    },
    findMarksAt: function(pos) {
      pos = clipPos(this, pos);
      var markers = [], spans = getLine(this, pos.line).markedSpans;
      if (spans) for (var i = 0; i < spans.length; ++i) {
        var span = spans[i];
        if ((span.from == null || span.from <= pos.ch) &&
            (span.to == null || span.to >= pos.ch))
          markers.push(span.marker.parent || span.marker);
      }
      return markers;
    },
    findMarks: function(from, to, filter) {
      from = clipPos(this, from); to = clipPos(this, to);
      var found = [], lineNo = from.line;
      this.iter(from.line, to.line + 1, function(line) {
        var spans = line.markedSpans;
        if (spans) for (var i = 0; i < spans.length; i++) {
          var span = spans[i];
          if (!(span.to != null && lineNo == from.line && from.ch >= span.to ||
                span.from == null && lineNo != from.line ||
                span.from != null && lineNo == to.line && span.from >= to.ch) &&
              (!filter || filter(span.marker)))
            found.push(span.marker.parent || span.marker);
        }
        ++lineNo;
      });
      return found;
    },
    getAllMarks: function() {
      var markers = [];
      this.iter(function(line) {
        var sps = line.markedSpans;
        if (sps) for (var i = 0; i < sps.length; ++i)
          if (sps[i].from != null) markers.push(sps[i].marker);
      });
      return markers;
    },

    posFromIndex: function(off) {
      var ch, lineNo = this.first, sepSize = this.lineSeparator().length;
      this.iter(function(line) {
        var sz = line.text.length + sepSize;
        if (sz > off) { ch = off; return true; }
        off -= sz;
        ++lineNo;
      });
      return clipPos(this, Pos(lineNo, ch));
    },
    indexFromPos: function (coords) {
      coords = clipPos(this, coords);
      var index = coords.ch;
      if (coords.line < this.first || coords.ch < 0) return 0;
      var sepSize = this.lineSeparator().length;
      this.iter(this.first, coords.line, function (line) {
        index += line.text.length + sepSize;
      });
      return index;
    },

    copy: function(copyHistory) {
      var doc = new Doc(getLines(this, this.first, this.first + this.size),
                        this.modeOption, this.first, this.lineSep);
      doc.scrollTop = this.scrollTop; doc.scrollLeft = this.scrollLeft;
      doc.sel = this.sel;
      doc.extend = false;
      if (copyHistory) {
        doc.history.undoDepth = this.history.undoDepth;
        doc.setHistory(this.getHistory());
      }
      return doc;
    },

    linkedDoc: function(options) {
      if (!options) options = {};
      var from = this.first, to = this.first + this.size;
      if (options.from != null && options.from > from) from = options.from;
      if (options.to != null && options.to < to) to = options.to;
      var copy = new Doc(getLines(this, from, to), options.mode || this.modeOption, from, this.lineSep);
      if (options.sharedHist) copy.history = this.history;
      (this.linked || (this.linked = [])).push({doc: copy, sharedHist: options.sharedHist});
      copy.linked = [{doc: this, isParent: true, sharedHist: options.sharedHist}];
      copySharedMarkers(copy, findSharedMarkers(this));
      return copy;
    },
    unlinkDoc: function(other) {
      if (other instanceof CodeMirror) other = other.doc;
      if (this.linked) for (var i = 0; i < this.linked.length; ++i) {
        var link = this.linked[i];
        if (link.doc != other) continue;
        this.linked.splice(i, 1);
        other.unlinkDoc(this);
        detachSharedMarkers(findSharedMarkers(this));
        break;
      }
      // If the histories were shared, split them again
      if (other.history == this.history) {
        var splitIds = [other.id];
        linkedDocs(other, function(doc) {splitIds.push(doc.id);}, true);
        other.history = new History(null);
        other.history.done = copyHistoryArray(this.history.done, splitIds);
        other.history.undone = copyHistoryArray(this.history.undone, splitIds);
      }
    },
    iterLinkedDocs: function(f) {linkedDocs(this, f);},

    getMode: function() {return this.mode;},
    getEditor: function() {return this.cm;},

    splitLines: function(str) {
      if (this.lineSep) return str.split(this.lineSep);
      return splitLinesAuto(str);
    },
    lineSeparator: function() { return this.lineSep || "\n"; }
  });

  // Public alias.
  Doc.prototype.eachLine = Doc.prototype.iter;

  // Set up methods on CodeMirror's prototype to redirect to the editor's document.
  var dontDelegate = "iter insert remove copy getEditor constructor".split(" ");
  for (var prop in Doc.prototype) if (Doc.prototype.hasOwnProperty(prop) && indexOf(dontDelegate, prop) < 0)
    CodeMirror.prototype[prop] = (function(method) {
      return function() {return method.apply(this.doc, arguments);};
    })(Doc.prototype[prop]);

  eventMixin(Doc);

  // Call f for all linked documents.
  function linkedDocs(doc, f, sharedHistOnly) {
    function propagate(doc, skip, sharedHist) {
      if (doc.linked) for (var i = 0; i < doc.linked.length; ++i) {
        var rel = doc.linked[i];
        if (rel.doc == skip) continue;
        var shared = sharedHist && rel.sharedHist;
        if (sharedHistOnly && !shared) continue;
        f(rel.doc, shared);
        propagate(rel.doc, doc, shared);
      }
    }
    propagate(doc, null, true);
  }

  // Attach a document to an editor.
  function attachDoc(cm, doc) {
    if (doc.cm) throw new Error("This document is already in use.");
    cm.doc = doc;
    doc.cm = cm;
    estimateLineHeights(cm);
    loadMode(cm);
    if (!cm.options.lineWrapping) findMaxLine(cm);
    cm.options.mode = doc.modeOption;
    regChange(cm);
  }

  // LINE UTILITIES

  // Find the line object corresponding to the given line number.
  function getLine(doc, n) {
    n -= doc.first;
    if (n < 0 || n >= doc.size) throw new Error("There is no line " + (n + doc.first) + " in the document.");
    for (var chunk = doc; !chunk.lines;) {
      for (var i = 0;; ++i) {
        var child = chunk.children[i], sz = child.chunkSize();
        if (n < sz) { chunk = child; break; }
        n -= sz;
      }
    }
    return chunk.lines[n];
  }

  // Get the part of a document between two positions, as an array of
  // strings.
  function getBetween(doc, start, end) {
    var out = [], n = start.line;
    doc.iter(start.line, end.line + 1, function(line) {
      var text = line.text;
      if (n == end.line) text = text.slice(0, end.ch);
      if (n == start.line) text = text.slice(start.ch);
      out.push(text);
      ++n;
    });
    return out;
  }
  // Get the lines between from and to, as array of strings.
  function getLines(doc, from, to) {
    var out = [];
    doc.iter(from, to, function(line) { out.push(line.text); });
    return out;
  }

  // Update the height of a line, propagating the height change
  // upwards to parent nodes.
  function updateLineHeight(line, height) {
    var diff = height - line.height;
    if (diff) for (var n = line; n; n = n.parent) n.height += diff;
  }

  // Given a line object, find its line number by walking up through
  // its parent links.
  function lineNo(line) {
    if (line.parent == null) return null;
    var cur = line.parent, no = indexOf(cur.lines, line);
    for (var chunk = cur.parent; chunk; cur = chunk, chunk = chunk.parent) {
      for (var i = 0;; ++i) {
        if (chunk.children[i] == cur) break;
        no += chunk.children[i].chunkSize();
      }
    }
    return no + cur.first;
  }

  // Find the line at the given vertical position, using the height
  // information in the document tree.
  function lineAtHeight(chunk, h) {
    var n = chunk.first;
    outer: do {
      for (var i = 0; i < chunk.children.length; ++i) {
        var child = chunk.children[i], ch = child.height;
        if (h < ch) { chunk = child; continue outer; }
        h -= ch;
        n += child.chunkSize();
      }
      return n;
    } while (!chunk.lines);
    for (var i = 0; i < chunk.lines.length; ++i) {
      var line = chunk.lines[i], lh = line.height;
      if (h < lh) break;
      h -= lh;
    }
    return n + i;
  }


  // Find the height above the given line.
  function heightAtLine(lineObj) {
    lineObj = visualLine(lineObj);

    var h = 0, chunk = lineObj.parent;
    for (var i = 0; i < chunk.lines.length; ++i) {
      var line = chunk.lines[i];
      if (line == lineObj) break;
      else h += line.height;
    }
    for (var p = chunk.parent; p; chunk = p, p = chunk.parent) {
      for (var i = 0; i < p.children.length; ++i) {
        var cur = p.children[i];
        if (cur == chunk) break;
        else h += cur.height;
      }
    }
    return h;
  }

  // Get the bidi ordering for the given line (and cache it). Returns
  // false for lines that are fully left-to-right, and an array of
  // BidiSpan objects otherwise.
  function getOrder(line) {
    var order = line.order;
    if (order == null) order = line.order = bidiOrdering(line.text);
    return order;
  }

  // HISTORY

  function History(startGen) {
    // Arrays of change events and selections. Doing something adds an
    // event to done and clears undo. Undoing moves events from done
    // to undone, redoing moves them in the other direction.
    this.done = []; this.undone = [];
    this.undoDepth = Infinity;
    // Used to track when changes can be merged into a single undo
    // event
    this.lastModTime = this.lastSelTime = 0;
    this.lastOp = this.lastSelOp = null;
    this.lastOrigin = this.lastSelOrigin = null;
    // Used by the isClean() method
    this.generation = this.maxGeneration = startGen || 1;
  }

  // Create a history change event from an updateDoc-style change
  // object.
  function historyChangeFromChange(doc, change) {
    var histChange = {from: copyPos(change.from), to: changeEnd(change), text: getBetween(doc, change.from, change.to)};
    attachLocalSpans(doc, histChange, change.from.line, change.to.line + 1);
    linkedDocs(doc, function(doc) {attachLocalSpans(doc, histChange, change.from.line, change.to.line + 1);}, true);
    return histChange;
  }

  // Pop all selection events off the end of a history array. Stop at
  // a change event.
  function clearSelectionEvents(array) {
    while (array.length) {
      var last = lst(array);
      if (last.ranges) array.pop();
      else break;
    }
  }

  // Find the top change event in the history. Pop off selection
  // events that are in the way.
  function lastChangeEvent(hist, force) {
    if (force) {
      clearSelectionEvents(hist.done);
      return lst(hist.done);
    } else if (hist.done.length && !lst(hist.done).ranges) {
      return lst(hist.done);
    } else if (hist.done.length > 1 && !hist.done[hist.done.length - 2].ranges) {
      hist.done.pop();
      return lst(hist.done);
    }
  }

  // Register a change in the history. Merges changes that are within
  // a single operation, ore are close together with an origin that
  // allows merging (starting with "+") into a single event.
  function addChangeToHistory(doc, change, selAfter, opId) {
    var hist = doc.history;
    hist.undone.length = 0;
    var time = +new Date, cur;

    if ((hist.lastOp == opId ||
         hist.lastOrigin == change.origin && change.origin &&
         ((change.origin.charAt(0) == "+" && doc.cm && hist.lastModTime > time - doc.cm.options.historyEventDelay) ||
          change.origin.charAt(0) == "*")) &&
        (cur = lastChangeEvent(hist, hist.lastOp == opId))) {
      // Merge this change into the last event
      var last = lst(cur.changes);
      if (cmp(change.from, change.to) == 0 && cmp(change.from, last.to) == 0) {
        // Optimized case for simple insertion -- don't want to add
        // new changesets for every character typed
        last.to = changeEnd(change);
      } else {
        // Add new sub-event
        cur.changes.push(historyChangeFromChange(doc, change));
      }
    } else {
      // Can not be merged, start a new event.
      var before = lst(hist.done);
      if (!before || !before.ranges)
        pushSelectionToHistory(doc.sel, hist.done);
      cur = {changes: [historyChangeFromChange(doc, change)],
             generation: hist.generation};
      hist.done.push(cur);
      while (hist.done.length > hist.undoDepth) {
        hist.done.shift();
        if (!hist.done[0].ranges) hist.done.shift();
      }
    }
    hist.done.push(selAfter);
    hist.generation = ++hist.maxGeneration;
    hist.lastModTime = hist.lastSelTime = time;
    hist.lastOp = hist.lastSelOp = opId;
    hist.lastOrigin = hist.lastSelOrigin = change.origin;

    if (!last) signal(doc, "historyAdded");
  }

  function selectionEventCanBeMerged(doc, origin, prev, sel) {
    var ch = origin.charAt(0);
    return ch == "*" ||
      ch == "+" &&
      prev.ranges.length == sel.ranges.length &&
      prev.somethingSelected() == sel.somethingSelected() &&
      new Date - doc.history.lastSelTime <= (doc.cm ? doc.cm.options.historyEventDelay : 500);
  }

  // Called whenever the selection changes, sets the new selection as
  // the pending selection in the history, and pushes the old pending
  // selection into the 'done' array when it was significantly
  // different (in number of selected ranges, emptiness, or time).
  function addSelectionToHistory(doc, sel, opId, options) {
    var hist = doc.history, origin = options && options.origin;

    // A new event is started when the previous origin does not match
    // the current, or the origins don't allow matching. Origins
    // starting with * are always merged, those starting with + are
    // merged when similar and close together in time.
    if (opId == hist.lastSelOp ||
        (origin && hist.lastSelOrigin == origin &&
         (hist.lastModTime == hist.lastSelTime && hist.lastOrigin == origin ||
          selectionEventCanBeMerged(doc, origin, lst(hist.done), sel))))
      hist.done[hist.done.length - 1] = sel;
    else
      pushSelectionToHistory(sel, hist.done);

    hist.lastSelTime = +new Date;
    hist.lastSelOrigin = origin;
    hist.lastSelOp = opId;
    if (options && options.clearRedo !== false)
      clearSelectionEvents(hist.undone);
  }

  function pushSelectionToHistory(sel, dest) {
    var top = lst(dest);
    if (!(top && top.ranges && top.equals(sel)))
      dest.push(sel);
  }

  // Used to store marked span information in the history.
  function attachLocalSpans(doc, change, from, to) {
    var existing = change["spans_" + doc.id], n = 0;
    doc.iter(Math.max(doc.first, from), Math.min(doc.first + doc.size, to), function(line) {
      if (line.markedSpans)
        (existing || (existing = change["spans_" + doc.id] = {}))[n] = line.markedSpans;
      ++n;
    });
  }

  // When un/re-doing restores text containing marked spans, those
  // that have been explicitly cleared should not be restored.
  function removeClearedSpans(spans) {
    if (!spans) return null;
    for (var i = 0, out; i < spans.length; ++i) {
      if (spans[i].marker.explicitlyCleared) { if (!out) out = spans.slice(0, i); }
      else if (out) out.push(spans[i]);
    }
    return !out ? spans : out.length ? out : null;
  }

  // Retrieve and filter the old marked spans stored in a change event.
  function getOldSpans(doc, change) {
    var found = change["spans_" + doc.id];
    if (!found) return null;
    for (var i = 0, nw = []; i < change.text.length; ++i)
      nw.push(removeClearedSpans(found[i]));
    return nw;
  }

  // Used both to provide a JSON-safe object in .getHistory, and, when
  // detaching a document, to split the history in two
  function copyHistoryArray(events, newGroup, instantiateSel) {
    for (var i = 0, copy = []; i < events.length; ++i) {
      var event = events[i];
      if (event.ranges) {
        copy.push(instantiateSel ? Selection.prototype.deepCopy.call(event) : event);
        continue;
      }
      var changes = event.changes, newChanges = [];
      copy.push({changes: newChanges});
      for (var j = 0; j < changes.length; ++j) {
        var change = changes[j], m;
        newChanges.push({from: change.from, to: change.to, text: change.text});
        if (newGroup) for (var prop in change) if (m = prop.match(/^spans_(\d+)$/)) {
          if (indexOf(newGroup, Number(m[1])) > -1) {
            lst(newChanges)[prop] = change[prop];
            delete change[prop];
          }
        }
      }
    }
    return copy;
  }

  // Rebasing/resetting history to deal with externally-sourced changes

  function rebaseHistSelSingle(pos, from, to, diff) {
    if (to < pos.line) {
      pos.line += diff;
    } else if (from < pos.line) {
      pos.line = from;
      pos.ch = 0;
    }
  }

  // Tries to rebase an array of history events given a change in the
  // document. If the change touches the same lines as the event, the
  // event, and everything 'behind' it, is discarded. If the change is
  // before the event, the event's positions are updated. Uses a
  // copy-on-write scheme for the positions, to avoid having to
  // reallocate them all on every rebase, but also avoid problems with
  // shared position objects being unsafely updated.
  function rebaseHistArray(array, from, to, diff) {
    for (var i = 0; i < array.length; ++i) {
      var sub = array[i], ok = true;
      if (sub.ranges) {
        if (!sub.copied) { sub = array[i] = sub.deepCopy(); sub.copied = true; }
        for (var j = 0; j < sub.ranges.length; j++) {
          rebaseHistSelSingle(sub.ranges[j].anchor, from, to, diff);
          rebaseHistSelSingle(sub.ranges[j].head, from, to, diff);
        }
        continue;
      }
      for (var j = 0; j < sub.changes.length; ++j) {
        var cur = sub.changes[j];
        if (to < cur.from.line) {
          cur.from = Pos(cur.from.line + diff, cur.from.ch);
          cur.to = Pos(cur.to.line + diff, cur.to.ch);
        } else if (from <= cur.to.line) {
          ok = false;
          break;
        }
      }
      if (!ok) {
        array.splice(0, i + 1);
        i = 0;
      }
    }
  }

  function rebaseHist(hist, change) {
    var from = change.from.line, to = change.to.line, diff = change.text.length - (to - from) - 1;
    rebaseHistArray(hist.done, from, to, diff);
    rebaseHistArray(hist.undone, from, to, diff);
  }

  // EVENT UTILITIES

  // Due to the fact that we still support jurassic IE versions, some
  // compatibility wrappers are needed.

  var e_preventDefault = CodeMirror.e_preventDefault = function(e) {
    if (e.preventDefault) e.preventDefault();
    else e.returnValue = false;
  };
  var e_stopPropagation = CodeMirror.e_stopPropagation = function(e) {
    if (e.stopPropagation) e.stopPropagation();
    else e.cancelBubble = true;
  };
  function e_defaultPrevented(e) {
    return e.defaultPrevented != null ? e.defaultPrevented : e.returnValue == false;
  }
  var e_stop = CodeMirror.e_stop = function(e) {e_preventDefault(e); e_stopPropagation(e);};

  function e_target(e) {return e.target || e.srcElement;}
  function e_button(e) {
    var b = e.which;
    if (b == null) {
      if (e.button & 1) b = 1;
      else if (e.button & 2) b = 3;
      else if (e.button & 4) b = 2;
    }
    if (mac && e.ctrlKey && b == 1) b = 3;
    return b;
  }

  // EVENT HANDLING

  // Lightweight event framework. on/off also work on DOM nodes,
  // registering native DOM handlers.

  var on = CodeMirror.on = function(emitter, type, f) {
    if (emitter.addEventListener)
      emitter.addEventListener(type, f, false);
    else if (emitter.attachEvent)
      emitter.attachEvent("on" + type, f);
    else {
      var map = emitter._handlers || (emitter._handlers = {});
      var arr = map[type] || (map[type] = []);
      arr.push(f);
    }
  };

  var noHandlers = []
  function getHandlers(emitter, type, copy) {
    var arr = emitter._handlers && emitter._handlers[type]
    if (copy) return arr && arr.length > 0 ? arr.slice() : noHandlers
    else return arr || noHandlers
  }

  var off = CodeMirror.off = function(emitter, type, f) {
    if (emitter.removeEventListener)
      emitter.removeEventListener(type, f, false);
    else if (emitter.detachEvent)
      emitter.detachEvent("on" + type, f);
    else {
      var handlers = getHandlers(emitter, type, false)
      for (var i = 0; i < handlers.length; ++i)
        if (handlers[i] == f) { handlers.splice(i, 1); break; }
    }
  };

  var signal = CodeMirror.signal = function(emitter, type /*, values...*/) {
    var handlers = getHandlers(emitter, type, true)
    if (!handlers.length) return;
    var args = Array.prototype.slice.call(arguments, 2);
    for (var i = 0; i < handlers.length; ++i) handlers[i].apply(null, args);
  };

  var orphanDelayedCallbacks = null;

  // Often, we want to signal events at a point where we are in the
  // middle of some work, but don't want the handler to start calling
  // other methods on the editor, which might be in an inconsistent
  // state or simply not expect any other events to happen.
  // signalLater looks whether there are any handlers, and schedules
  // them to be executed when the last operation ends, or, if no
  // operation is active, when a timeout fires.
  function signalLater(emitter, type /*, values...*/) {
    var arr = getHandlers(emitter, type, false)
    if (!arr.length) return;
    var args = Array.prototype.slice.call(arguments, 2), list;
    if (operationGroup) {
      list = operationGroup.delayedCallbacks;
    } else if (orphanDelayedCallbacks) {
      list = orphanDelayedCallbacks;
    } else {
      list = orphanDelayedCallbacks = [];
      setTimeout(fireOrphanDelayed, 0);
    }
    function bnd(f) {return function(){f.apply(null, args);};};
    for (var i = 0; i < arr.length; ++i)
      list.push(bnd(arr[i]));
  }

  function fireOrphanDelayed() {
    var delayed = orphanDelayedCallbacks;
    orphanDelayedCallbacks = null;
    for (var i = 0; i < delayed.length; ++i) delayed[i]();
  }

  // The DOM events that CodeMirror handles can be overridden by
  // registering a (non-DOM) handler on the editor for the event name,
  // and preventDefault-ing the event in that handler.
  function signalDOMEvent(cm, e, override) {
    if (typeof e == "string")
      e = {type: e, preventDefault: function() { this.defaultPrevented = true; }};
    signal(cm, override || e.type, cm, e);
    return e_defaultPrevented(e) || e.codemirrorIgnore;
  }

  function signalCursorActivity(cm) {
    var arr = cm._handlers && cm._handlers.cursorActivity;
    if (!arr) return;
    var set = cm.curOp.cursorActivityHandlers || (cm.curOp.cursorActivityHandlers = []);
    for (var i = 0; i < arr.length; ++i) if (indexOf(set, arr[i]) == -1)
      set.push(arr[i]);
  }

  function hasHandler(emitter, type) {
    return getHandlers(emitter, type).length > 0
  }

  // Add on and off methods to a constructor's prototype, to make
  // registering events on such objects more convenient.
  function eventMixin(ctor) {
    ctor.prototype.on = function(type, f) {on(this, type, f);};
    ctor.prototype.off = function(type, f) {off(this, type, f);};
  }

  // MISC UTILITIES

  // Number of pixels added to scroller and sizer to hide scrollbar
  var scrollerGap = 30;

  // Returned or thrown by various protocols to signal 'I'm not
  // handling this'.
  var Pass = CodeMirror.Pass = {toString: function(){return "CodeMirror.Pass";}};

  // Reused option objects for setSelection & friends
  var sel_dontScroll = {scroll: false}, sel_mouse = {origin: "*mouse"}, sel_move = {origin: "+move"};

  function Delayed() {this.id = null;}
  Delayed.prototype.set = function(ms, f) {
    clearTimeout(this.id);
    this.id = setTimeout(f, ms);
  };

  // Counts the column offset in a string, taking tabs into account.
  // Used mostly to find indentation.
  var countColumn = CodeMirror.countColumn = function(string, end, tabSize, startIndex, startValue) {
    if (end == null) {
      end = string.search(/[^\s\u00a0]/);
      if (end == -1) end = string.length;
    }
    for (var i = startIndex || 0, n = startValue || 0;;) {
      var nextTab = string.indexOf("\t", i);
      if (nextTab < 0 || nextTab >= end)
        return n + (end - i);
      n += nextTab - i;
      n += tabSize - (n % tabSize);
      i = nextTab + 1;
    }
  };

  // The inverse of countColumn -- find the offset that corresponds to
  // a particular column.
  var findColumn = CodeMirror.findColumn = function(string, goal, tabSize) {
    for (var pos = 0, col = 0;;) {
      var nextTab = string.indexOf("\t", pos);
      if (nextTab == -1) nextTab = string.length;
      var skipped = nextTab - pos;
      if (nextTab == string.length || col + skipped >= goal)
        return pos + Math.min(skipped, goal - col);
      col += nextTab - pos;
      col += tabSize - (col % tabSize);
      pos = nextTab + 1;
      if (col >= goal) return pos;
    }
  }

  var spaceStrs = [""];
  function spaceStr(n) {
    while (spaceStrs.length <= n)
      spaceStrs.push(lst(spaceStrs) + " ");
    return spaceStrs[n];
  }

  function lst(arr) { return arr[arr.length-1]; }

  var selectInput = function(node) { node.select(); };
  if (ios) // Mobile Safari apparently has a bug where select() is broken.
    selectInput = function(node) { node.selectionStart = 0; node.selectionEnd = node.value.length; };
  else if (ie) // Suppress mysterious IE10 errors
    selectInput = function(node) { try { node.select(); } catch(_e) {} };

  function indexOf(array, elt) {
    for (var i = 0; i < array.length; ++i)
      if (array[i] == elt) return i;
    return -1;
  }
  function map(array, f) {
    var out = [];
    for (var i = 0; i < array.length; i++) out[i] = f(array[i], i);
    return out;
  }

  function nothing() {}

  function createObj(base, props) {
    var inst;
    if (Object.create) {
      inst = Object.create(base);
    } else {
      nothing.prototype = base;
      inst = new nothing();
    }
    if (props) copyObj(props, inst);
    return inst;
  };

  function copyObj(obj, target, overwrite) {
    if (!target) target = {};
    for (var prop in obj)
      if (obj.hasOwnProperty(prop) && (overwrite !== false || !target.hasOwnProperty(prop)))
        target[prop] = obj[prop];
    return target;
  }

  function bind(f) {
    var args = Array.prototype.slice.call(arguments, 1);
    return function(){return f.apply(null, args);};
  }

  var nonASCIISingleCaseWordChar = /[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;
  var isWordCharBasic = CodeMirror.isWordChar = function(ch) {
    return /\w/.test(ch) || ch > "\x80" &&
      (ch.toUpperCase() != ch.toLowerCase() || nonASCIISingleCaseWordChar.test(ch));
  };
  function isWordChar(ch, helper) {
    if (!helper) return isWordCharBasic(ch);
    if (helper.source.indexOf("\\w") > -1 && isWordCharBasic(ch)) return true;
    return helper.test(ch);
  }

  function isEmpty(obj) {
    for (var n in obj) if (obj.hasOwnProperty(n) && obj[n]) return false;
    return true;
  }

  // Extending unicode characters. A series of a non-extending char +
  // any number of extending chars is treated as a single unit as far
  // as editing and measuring is concerned. This is not fully correct,
  // since some scripts/fonts/browsers also treat other configurations
  // of code points as a group.
  var extendingChars = /[\u0300-\u036f\u0483-\u0489\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u065e\u0670\u06d6-\u06dc\u06de-\u06e4\u06e7\u06e8\u06ea-\u06ed\u0711\u0730-\u074a\u07a6-\u07b0\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0900-\u0902\u093c\u0941-\u0948\u094d\u0951-\u0955\u0962\u0963\u0981\u09bc\u09be\u09c1-\u09c4\u09cd\u09d7\u09e2\u09e3\u0a01\u0a02\u0a3c\u0a41\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a70\u0a71\u0a75\u0a81\u0a82\u0abc\u0ac1-\u0ac5\u0ac7\u0ac8\u0acd\u0ae2\u0ae3\u0b01\u0b3c\u0b3e\u0b3f\u0b41-\u0b44\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b82\u0bbe\u0bc0\u0bcd\u0bd7\u0c3e-\u0c40\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0cbc\u0cbf\u0cc2\u0cc6\u0ccc\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0d3e\u0d41-\u0d44\u0d4d\u0d57\u0d62\u0d63\u0dca\u0dcf\u0dd2-\u0dd4\u0dd6\u0ddf\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0f18\u0f19\u0f35\u0f37\u0f39\u0f71-\u0f7e\u0f80-\u0f84\u0f86\u0f87\u0f90-\u0f97\u0f99-\u0fbc\u0fc6\u102d-\u1030\u1032-\u1037\u1039\u103a\u103d\u103e\u1058\u1059\u105e-\u1060\u1071-\u1074\u1082\u1085\u1086\u108d\u109d\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b7-\u17bd\u17c6\u17c9-\u17d3\u17dd\u180b-\u180d\u18a9\u1920-\u1922\u1927\u1928\u1932\u1939-\u193b\u1a17\u1a18\u1a56\u1a58-\u1a5e\u1a60\u1a62\u1a65-\u1a6c\u1a73-\u1a7c\u1a7f\u1b00-\u1b03\u1b34\u1b36-\u1b3a\u1b3c\u1b42\u1b6b-\u1b73\u1b80\u1b81\u1ba2-\u1ba5\u1ba8\u1ba9\u1c2c-\u1c33\u1c36\u1c37\u1cd0-\u1cd2\u1cd4-\u1ce0\u1ce2-\u1ce8\u1ced\u1dc0-\u1de6\u1dfd-\u1dff\u200c\u200d\u20d0-\u20f0\u2cef-\u2cf1\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua66f-\ua672\ua67c\ua67d\ua6f0\ua6f1\ua802\ua806\ua80b\ua825\ua826\ua8c4\ua8e0-\ua8f1\ua926-\ua92d\ua947-\ua951\ua980-\ua982\ua9b3\ua9b6-\ua9b9\ua9bc\uaa29-\uaa2e\uaa31\uaa32\uaa35\uaa36\uaa43\uaa4c\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uabe5\uabe8\uabed\udc00-\udfff\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\uff9e\uff9f]/;
  function isExtendingChar(ch) { return ch.charCodeAt(0) >= 768 && extendingChars.test(ch); }

  // DOM UTILITIES

  function elt(tag, content, className, style) {
    var e = document.createElement(tag);
    if (className) e.className = className;
    if (style) e.style.cssText = style;
    if (typeof content == "string") e.appendChild(document.createTextNode(content));
    else if (content) for (var i = 0; i < content.length; ++i) e.appendChild(content[i]);
    return e;
  }

  var range;
  if (document.createRange) range = function(node, start, end, endNode) {
    var r = document.createRange();
    r.setEnd(endNode || node, end);
    r.setStart(node, start);
    return r;
  };
  else range = function(node, start, end) {
    var r = document.body.createTextRange();
    try { r.moveToElementText(node.parentNode); }
    catch(e) { return r; }
    r.collapse(true);
    r.moveEnd("character", end);
    r.moveStart("character", start);
    return r;
  };

  function removeChildren(e) {
    for (var count = e.childNodes.length; count > 0; --count)
      e.removeChild(e.firstChild);
    return e;
  }

  function removeChildrenAndAdd(parent, e) {
    return removeChildren(parent).appendChild(e);
  }

  var contains = CodeMirror.contains = function(parent, child) {
    if (child.nodeType == 3) // Android browser always returns false when child is a textnode
      child = child.parentNode;
    if (parent.contains)
      return parent.contains(child);
    do {
      if (child.nodeType == 11) child = child.host;
      if (child == parent) return true;
    } while (child = child.parentNode);
  };

  function activeElt() {
    var activeElement = document.activeElement;
    while (activeElement && activeElement.root && activeElement.root.activeElement)
      activeElement = activeElement.root.activeElement;
    return activeElement;
  }
  // Older versions of IE throws unspecified error when touching
  // document.activeElement in some cases (during loading, in iframe)
  if (ie && ie_version < 11) activeElt = function() {
    try { return document.activeElement; }
    catch(e) { return document.body; }
  };

  function classTest(cls) { return new RegExp("(^|\\s)" + cls + "(?:$|\\s)\\s*"); }
  var rmClass = CodeMirror.rmClass = function(node, cls) {
    var current = node.className;
    var match = classTest(cls).exec(current);
    if (match) {
      var after = current.slice(match.index + match[0].length);
      node.className = current.slice(0, match.index) + (after ? match[1] + after : "");
    }
  };
  var addClass = CodeMirror.addClass = function(node, cls) {
    var current = node.className;
    if (!classTest(cls).test(current)) node.className += (current ? " " : "") + cls;
  };
  function joinClasses(a, b) {
    var as = a.split(" ");
    for (var i = 0; i < as.length; i++)
      if (as[i] && !classTest(as[i]).test(b)) b += " " + as[i];
    return b;
  }

  // WINDOW-WIDE EVENTS

  // These must be handled carefully, because naively registering a
  // handler for each editor will cause the editors to never be
  // garbage collected.

  function forEachCodeMirror(f) {
    if (!document.body.getElementsByClassName) return;
    var byClass = document.body.getElementsByClassName("CodeMirror");
    for (var i = 0; i < byClass.length; i++) {
      var cm = byClass[i].CodeMirror;
      if (cm) f(cm);
    }
  }

  var globalsRegistered = false;
  function ensureGlobalHandlers() {
    if (globalsRegistered) return;
    registerGlobalHandlers();
    globalsRegistered = true;
  }
  function registerGlobalHandlers() {
    // When the window resizes, we need to refresh active editors.
    var resizeTimer;
    on(window, "resize", function() {
      if (resizeTimer == null) resizeTimer = setTimeout(function() {
        resizeTimer = null;
        forEachCodeMirror(onResize);
      }, 100);
    });
    // When the window loses focus, we want to show the editor as blurred
    on(window, "blur", function() {
      forEachCodeMirror(onBlur);
    });
  }

  // FEATURE DETECTION

  // Detect drag-and-drop
  var dragAndDrop = function() {
    // There is *some* kind of drag-and-drop support in IE6-8, but I
    // couldn't get it to work yet.
    if (ie && ie_version < 9) return false;
    var div = elt('div');
    return "draggable" in div || "dragDrop" in div;
  }();

  var zwspSupported;
  function zeroWidthElement(measure) {
    if (zwspSupported == null) {
      var test = elt("span", "\u200b");
      removeChildrenAndAdd(measure, elt("span", [test, document.createTextNode("x")]));
      if (measure.firstChild.offsetHeight != 0)
        zwspSupported = test.offsetWidth <= 1 && test.offsetHeight > 2 && !(ie && ie_version < 8);
    }
    var node = zwspSupported ? elt("span", "\u200b") :
      elt("span", "\u00a0", null, "display: inline-block; width: 1px; margin-right: -1px");
    node.setAttribute("cm-text", "");
    return node;
  }

  // Feature-detect IE's crummy client rect reporting for bidi text
  var badBidiRects;
  function hasBadBidiRects(measure) {
    if (badBidiRects != null) return badBidiRects;
    var txt = removeChildrenAndAdd(measure, document.createTextNode("A\u062eA"));
    var r0 = range(txt, 0, 1).getBoundingClientRect();
    if (!r0 || r0.left == r0.right) return false; // Safari returns null in some cases (#2780)
    var r1 = range(txt, 1, 2).getBoundingClientRect();
    return badBidiRects = (r1.right - r0.right < 3);
  }

  // See if "".split is the broken IE version, if so, provide an
  // alternative way to split lines.
  var splitLinesAuto = CodeMirror.splitLines = "\n\nb".split(/\n/).length != 3 ? function(string) {
    var pos = 0, result = [], l = string.length;
    while (pos <= l) {
      var nl = string.indexOf("\n", pos);
      if (nl == -1) nl = string.length;
      var line = string.slice(pos, string.charAt(nl - 1) == "\r" ? nl - 1 : nl);
      var rt = line.indexOf("\r");
      if (rt != -1) {
        result.push(line.slice(0, rt));
        pos += rt + 1;
      } else {
        result.push(line);
        pos = nl + 1;
      }
    }
    return result;
  } : function(string){return string.split(/\r\n?|\n/);};

  var hasSelection = window.getSelection ? function(te) {
    try { return te.selectionStart != te.selectionEnd; }
    catch(e) { return false; }
  } : function(te) {
    try {var range = te.ownerDocument.selection.createRange();}
    catch(e) {}
    if (!range || range.parentElement() != te) return false;
    return range.compareEndPoints("StartToEnd", range) != 0;
  };

  var hasCopyEvent = (function() {
    var e = elt("div");
    if ("oncopy" in e) return true;
    e.setAttribute("oncopy", "return;");
    return typeof e.oncopy == "function";
  })();

  var badZoomedRects = null;
  function hasBadZoomedRects(measure) {
    if (badZoomedRects != null) return badZoomedRects;
    var node = removeChildrenAndAdd(measure, elt("span", "x"));
    var normal = node.getBoundingClientRect();
    var fromRange = range(node, 0, 1).getBoundingClientRect();
    return badZoomedRects = Math.abs(normal.left - fromRange.left) > 1;
  }

  // KEY NAMES

  var keyNames = CodeMirror.keyNames = {
    3: "Enter", 8: "Backspace", 9: "Tab", 13: "Enter", 16: "Shift", 17: "Ctrl", 18: "Alt",
    19: "Pause", 20: "CapsLock", 27: "Esc", 32: "Space", 33: "PageUp", 34: "PageDown", 35: "End",
    36: "Home", 37: "Left", 38: "Up", 39: "Right", 40: "Down", 44: "PrintScrn", 45: "Insert",
    46: "Delete", 59: ";", 61: "=", 91: "Mod", 92: "Mod", 93: "Mod",
    106: "*", 107: "=", 109: "-", 110: ".", 111: "/", 127: "Delete",
    173: "-", 186: ";", 187: "=", 188: ",", 189: "-", 190: ".", 191: "/", 192: "`", 219: "[", 220: "\\",
    221: "]", 222: "'", 63232: "Up", 63233: "Down", 63234: "Left", 63235: "Right", 63272: "Delete",
    63273: "Home", 63275: "End", 63276: "PageUp", 63277: "PageDown", 63302: "Insert"
  };
  (function() {
    // Number keys
    for (var i = 0; i < 10; i++) keyNames[i + 48] = keyNames[i + 96] = String(i);
    // Alphabetic keys
    for (var i = 65; i <= 90; i++) keyNames[i] = String.fromCharCode(i);
    // Function keys
    for (var i = 1; i <= 12; i++) keyNames[i + 111] = keyNames[i + 63235] = "F" + i;
  })();

  // BIDI HELPERS

  function iterateBidiSections(order, from, to, f) {
    if (!order) return f(from, to, "ltr");
    var found = false;
    for (var i = 0; i < order.length; ++i) {
      var part = order[i];
      if (part.from < to && part.to > from || from == to && part.to == from) {
        f(Math.max(part.from, from), Math.min(part.to, to), part.level == 1 ? "rtl" : "ltr");
        found = true;
      }
    }
    if (!found) f(from, to, "ltr");
  }

  function bidiLeft(part) { return part.level % 2 ? part.to : part.from; }
  function bidiRight(part) { return part.level % 2 ? part.from : part.to; }

  function lineLeft(line) { var order = getOrder(line); return order ? bidiLeft(order[0]) : 0; }
  function lineRight(line) {
    var order = getOrder(line);
    if (!order) return line.text.length;
    return bidiRight(lst(order));
  }

  function lineStart(cm, lineN) {
    var line = getLine(cm.doc, lineN);
    var visual = visualLine(line);
    if (visual != line) lineN = lineNo(visual);
    var order = getOrder(visual);
    var ch = !order ? 0 : order[0].level % 2 ? lineRight(visual) : lineLeft(visual);
    return Pos(lineN, ch);
  }
  function lineEnd(cm, lineN) {
    var merged, line = getLine(cm.doc, lineN);
    while (merged = collapsedSpanAtEnd(line)) {
      line = merged.find(1, true).line;
      lineN = null;
    }
    var order = getOrder(line);
    var ch = !order ? line.text.length : order[0].level % 2 ? lineLeft(line) : lineRight(line);
    return Pos(lineN == null ? lineNo(line) : lineN, ch);
  }
  function lineStartSmart(cm, pos) {
    var start = lineStart(cm, pos.line);
    var line = getLine(cm.doc, start.line);
    var order = getOrder(line);
    if (!order || order[0].level == 0) {
      var firstNonWS = Math.max(0, line.text.search(/\S/));
      var inWS = pos.line == start.line && pos.ch <= firstNonWS && pos.ch;
      return Pos(start.line, inWS ? 0 : firstNonWS);
    }
    return start;
  }

  function compareBidiLevel(order, a, b) {
    var linedir = order[0].level;
    if (a == linedir) return true;
    if (b == linedir) return false;
    return a < b;
  }
  var bidiOther;
  function getBidiPartAt(order, pos) {
    bidiOther = null;
    for (var i = 0, found; i < order.length; ++i) {
      var cur = order[i];
      if (cur.from < pos && cur.to > pos) return i;
      if ((cur.from == pos || cur.to == pos)) {
        if (found == null) {
          found = i;
        } else if (compareBidiLevel(order, cur.level, order[found].level)) {
          if (cur.from != cur.to) bidiOther = found;
          return i;
        } else {
          if (cur.from != cur.to) bidiOther = i;
          return found;
        }
      }
    }
    return found;
  }

  function moveInLine(line, pos, dir, byUnit) {
    if (!byUnit) return pos + dir;
    do pos += dir;
    while (pos > 0 && isExtendingChar(line.text.charAt(pos)));
    return pos;
  }

  // This is needed in order to move 'visually' through bi-directional
  // text -- i.e., pressing left should make the cursor go left, even
  // when in RTL text. The tricky part is the 'jumps', where RTL and
  // LTR text touch each other. This often requires the cursor offset
  // to move more than one unit, in order to visually move one unit.
  function moveVisually(line, start, dir, byUnit) {
    var bidi = getOrder(line);
    if (!bidi) return moveLogically(line, start, dir, byUnit);
    var pos = getBidiPartAt(bidi, start), part = bidi[pos];
    var target = moveInLine(line, start, part.level % 2 ? -dir : dir, byUnit);

    for (;;) {
      if (target > part.from && target < part.to) return target;
      if (target == part.from || target == part.to) {
        if (getBidiPartAt(bidi, target) == pos) return target;
        part = bidi[pos += dir];
        return (dir > 0) == part.level % 2 ? part.to : part.from;
      } else {
        part = bidi[pos += dir];
        if (!part) return null;
        if ((dir > 0) == part.level % 2)
          target = moveInLine(line, part.to, -1, byUnit);
        else
          target = moveInLine(line, part.from, 1, byUnit);
      }
    }
  }

  function moveLogically(line, start, dir, byUnit) {
    var target = start + dir;
    if (byUnit) while (target > 0 && isExtendingChar(line.text.charAt(target))) target += dir;
    return target < 0 || target > line.text.length ? null : target;
  }

  // Bidirectional ordering algorithm
  // See http://unicode.org/reports/tr9/tr9-13.html for the algorithm
  // that this (partially) implements.

  // One-char codes used for character types:
  // L (L):   Left-to-Right
  // R (R):   Right-to-Left
  // r (AL):  Right-to-Left Arabic
  // 1 (EN):  European Number
  // + (ES):  European Number Separator
  // % (ET):  European Number Terminator
  // n (AN):  Arabic Number
  // , (CS):  Common Number Separator
  // m (NSM): Non-Spacing Mark
  // b (BN):  Boundary Neutral
  // s (B):   Paragraph Separator
  // t (S):   Segment Separator
  // w (WS):  Whitespace
  // N (ON):  Other Neutrals

  // Returns null if characters are ordered as they appear
  // (left-to-right), or an array of sections ({from, to, level}
  // objects) in the order in which they occur visually.
  var bidiOrdering = (function() {
    // Character types for codepoints 0 to 0xff
    var lowTypes = "bbbbbbbbbtstwsbbbbbbbbbbbbbbssstwNN%%%NNNNNN,N,N1111111111NNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNbbbbbbsbbbbbbbbbbbbbbbbbbbbbbbbbb,N%%%%NNNNLNNNNN%%11NLNNN1LNNNNNLLLLLLLLLLLLLLLLLLLLLLLNLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLN";
    // Character types for codepoints 0x600 to 0x6ff
    var arabicTypes = "rrrrrrrrrrrr,rNNmmmmmmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmrrrrrrrnnnnnnnnnn%nnrrrmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmmmmmmNmmmm";
    function charType(code) {
      if (code <= 0xf7) return lowTypes.charAt(code);
      else if (0x590 <= code && code <= 0x5f4) return "R";
      else if (0x600 <= code && code <= 0x6ed) return arabicTypes.charAt(code - 0x600);
      else if (0x6ee <= code && code <= 0x8ac) return "r";
      else if (0x2000 <= code && code <= 0x200b) return "w";
      else if (code == 0x200c) return "b";
      else return "L";
    }

    var bidiRE = /[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/;
    var isNeutral = /[stwN]/, isStrong = /[LRr]/, countsAsLeft = /[Lb1n]/, countsAsNum = /[1n]/;
    // Browsers seem to always treat the boundaries of block elements as being L.
    var outerType = "L";

    function BidiSpan(level, from, to) {
      this.level = level;
      this.from = from; this.to = to;
    }

    return function(str) {
      if (!bidiRE.test(str)) return false;
      var len = str.length, types = [];
      for (var i = 0, type; i < len; ++i)
        types.push(type = charType(str.charCodeAt(i)));

      // W1. Examine each non-spacing mark (NSM) in the level run, and
      // change the type of the NSM to the type of the previous
      // character. If the NSM is at the start of the level run, it will
      // get the type of sor.
      for (var i = 0, prev = outerType; i < len; ++i) {
        var type = types[i];
        if (type == "m") types[i] = prev;
        else prev = type;
      }

      // W2. Search backwards from each instance of a European number
      // until the first strong type (R, L, AL, or sor) is found. If an
      // AL is found, change the type of the European number to Arabic
      // number.
      // W3. Change all ALs to R.
      for (var i = 0, cur = outerType; i < len; ++i) {
        var type = types[i];
        if (type == "1" && cur == "r") types[i] = "n";
        else if (isStrong.test(type)) { cur = type; if (type == "r") types[i] = "R"; }
      }

      // W4. A single European separator between two European numbers
      // changes to a European number. A single common separator between
      // two numbers of the same type changes to that type.
      for (var i = 1, prev = types[0]; i < len - 1; ++i) {
        var type = types[i];
        if (type == "+" && prev == "1" && types[i+1] == "1") types[i] = "1";
        else if (type == "," && prev == types[i+1] &&
                 (prev == "1" || prev == "n")) types[i] = prev;
        prev = type;
      }

      // W5. A sequence of European terminators adjacent to European
      // numbers changes to all European numbers.
      // W6. Otherwise, separators and terminators change to Other
      // Neutral.
      for (var i = 0; i < len; ++i) {
        var type = types[i];
        if (type == ",") types[i] = "N";
        else if (type == "%") {
          for (var end = i + 1; end < len && types[end] == "%"; ++end) {}
          var replace = (i && types[i-1] == "!") || (end < len && types[end] == "1") ? "1" : "N";
          for (var j = i; j < end; ++j) types[j] = replace;
          i = end - 1;
        }
      }

      // W7. Search backwards from each instance of a European number
      // until the first strong type (R, L, or sor) is found. If an L is
      // found, then change the type of the European number to L.
      for (var i = 0, cur = outerType; i < len; ++i) {
        var type = types[i];
        if (cur == "L" && type == "1") types[i] = "L";
        else if (isStrong.test(type)) cur = type;
      }

      // N1. A sequence of neutrals takes the direction of the
      // surrounding strong text if the text on both sides has the same
      // direction. European and Arabic numbers act as if they were R in
      // terms of their influence on neutrals. Start-of-level-run (sor)
      // and end-of-level-run (eor) are used at level run boundaries.
      // N2. Any remaining neutrals take the embedding direction.
      for (var i = 0; i < len; ++i) {
        if (isNeutral.test(types[i])) {
          for (var end = i + 1; end < len && isNeutral.test(types[end]); ++end) {}
          var before = (i ? types[i-1] : outerType) == "L";
          var after = (end < len ? types[end] : outerType) == "L";
          var replace = before || after ? "L" : "R";
          for (var j = i; j < end; ++j) types[j] = replace;
          i = end - 1;
        }
      }

      // Here we depart from the documented algorithm, in order to avoid
      // building up an actual levels array. Since there are only three
      // levels (0, 1, 2) in an implementation that doesn't take
      // explicit embedding into account, we can build up the order on
      // the fly, without following the level-based algorithm.
      var order = [], m;
      for (var i = 0; i < len;) {
        if (countsAsLeft.test(types[i])) {
          var start = i;
          for (++i; i < len && countsAsLeft.test(types[i]); ++i) {}
          order.push(new BidiSpan(0, start, i));
        } else {
          var pos = i, at = order.length;
          for (++i; i < len && types[i] != "L"; ++i) {}
          for (var j = pos; j < i;) {
            if (countsAsNum.test(types[j])) {
              if (pos < j) order.splice(at, 0, new BidiSpan(1, pos, j));
              var nstart = j;
              for (++j; j < i && countsAsNum.test(types[j]); ++j) {}
              order.splice(at, 0, new BidiSpan(2, nstart, j));
              pos = j;
            } else ++j;
          }
          if (pos < i) order.splice(at, 0, new BidiSpan(1, pos, i));
        }
      }
      if (order[0].level == 1 && (m = str.match(/^\s+/))) {
        order[0].from = m[0].length;
        order.unshift(new BidiSpan(0, 0, m[0].length));
      }
      if (lst(order).level == 1 && (m = str.match(/\s+$/))) {
        lst(order).to -= m[0].length;
        order.push(new BidiSpan(0, len - m[0].length, len));
      }
      if (order[0].level == 2)
        order.unshift(new BidiSpan(1, order[0].to, order[0].to));
      if (order[0].level != lst(order).level)
        order.push(new BidiSpan(order[0].level, len, len));

      return order;
    };
  })();

  // THE END

  CodeMirror.version = "5.15.3";

  return CodeMirror;
});



/*******************
 * From src: modules/lsfilter/src/js/codemirror/lsfilter-hint.js
 *******************/
// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE

(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    mod(require("../../lib/codemirror"), require("../../mode/lsfilter/lsfilter"));
  else if (typeof define == "function" && define.amd) // AMD
    define(["../../lib/codemirror", "../../mode/lsfitler/lsfilter"], mod);
  else // Plain browser env
    mod(CodeMirror);
})(function(CodeMirror) {
  "use strict";

	var LSFilterHint = {

		_structure: {},
		_table: '',
		_fields: [],
		_tables: [],

		set_structure: function (structure) {
			LSFilterHint._structure = structure;
			LSFilterHint._tables = Object.keys(LSFilterHint._structure);
		},

		set_table: function (table) {
			LSFilterHint._table = table;
			LSFilterHint._fields = Object.keys(LSFilterHint._structure[table]);
		},

		is_object: function (field) {
			if (LSFilterHint._table && LSFilterHint._structure[LSFilterHint._table][field]) {
				return (LSFilterHint._structure[LSFilterHint._table][field][0] === 'object');
			} else {
				return false;
			}
		},

		get_field_object_type: function (field) {
			if (LSFilterHint.is_object(field)) {
				return LSFilterHint._structure[LSFilterHint._table][field][1];
			} else {
				return false;
			}
		},

		get_operator_match: function (search, field, formatter) {

			var operators = [];
			if (!LSFilterHint._table || !LSFilterHint._structure[LSFilterHint._table][field]) {

				operators = Object.keys(lsfilter_graphics_visitor.operators).reduce(function (ops, type) {

					var type_ops = Object.keys(lsfilter_graphics_visitor.operators[type]).map(function (operator_name) {
						return lsfilter_dom_to_query.operators[operator_name];
					});

					type_ops.forEach(function (type_op) {
						if (ops.indexOf(type_op) < 0) {
							ops.push({
								operator: type_op,
								lhs_type: type
							});
						}
					});

					return ops;

				}, []);

			} else {

				var type = LSFilterHint._structure[LSFilterHint._table][field][0];
				operators = Object.keys(lsfilter_graphics_visitor.operators[type]).map(function (operator_name) {
					return {
						operator: lsfilter_dom_to_query.operators[operator_name],
						lhs_type: type
					};
				});

				if (type==="string") {
					formatter = function (value) { return value + ' ""'}
				}

			}

			return operators.reduce(function (matches, op) {
				if (op.operator.toUpperCase().indexOf(search.toUpperCase()) >= 0)
					matches.push({
						operator: formatter ? formatter(op.operator) : op.operator,
						lhs_type: op.lhs_type
					});
				return matches;
			}, []).sort(function (a, b) {
				if (a.length > b.length) return 1;
				else if (a.length < b.length) return -1;
				else return 0;
			});

		},

		get_table_match: function (search, formatter) {

			return LSFilterHint._tables.reduce(function (matches, table) {
				var matcher = new RegExp(search, 'i')
				if (table.toUpperCase().indexOf(search.toUpperCase()) >= 0)
					matches.push(formatter ? formatter(table) : table);
				return matches;
			}, []).sort(function (a, b) {
				if (a.length > b.length) return 1;
				else if (a.length < b.length) return -1;
				else return 0;
			});

		},

		get_field_match: function (search, formatter) {

			return LSFilterHint._fields.reduce(function (matches, field) {
				if (field.toUpperCase().indexOf(search.toUpperCase()) >= 0)
					matches.push(formatter ? formatter(field) : field);
				return matches;
			}, []).sort(function (a, b) {
				if (a.length > b.length) return 1;
				else if (a.length < b.length) return -1;
				else return 0;
			});
		},

		get_foreign_field_match: function (search, field, formatter) {

			if (!LSFilterHint.is_object(field)) {
				return [];
			}

			search = search.substring(1);
			var foreign_table = LSFilterHint.get_field_object_type(field);
			var fields = Object.keys(LSFilterHint._structure[foreign_table]);

			return fields.reduce(function (matches, field) {
				if (field.toUpperCase().indexOf(search.toUpperCase()) >= 0)
					matches.push(formatter ? formatter(field) : field);
				return matches;
			}, []).sort(function (a, b) {
				if (a.length > b.length) return 1;
				else if (a.length < b.length) return -1;
				else return 0;
			});
		}


	};

  CodeMirror.registerHelper("hint", "lsfilter", function(editor, options) {

		LSFilterHint.set_structure(options.structure);
		LSFilterHint.cursor = editor.getCursor();

    var result = [];
    var cursor = editor.getCursor();
    var token = editor.getTokenAt(cursor);
		var search;

		var mode = CodeMirror.resolveMode("text/x-lsfilter");
		var keywords = Object.keys(mode.keywords);
		var operator_matcher = mode.operatorChars;

		var select_table = false;

		if (token.string[0] == '[' && token.start == 0) {

			if (token.string[token.string.length - 1] === ']') {
				search = token.string.substring(1, token.string.length - 1);
			} else search = token.string.substring(1);

			select_table = true;
			result = LSFilterHint.get_table_match(search, function (value) {
				return "[" + value + "]";
			});

		} else if (token.string.match(operator_matcher)) {

			search = token.string;
			var last_token = editor.getTokenAt(CodeMirror.Pos(cursor.line, token.start - 2));
			var operators = LSFilterHint.get_operator_match(search, last_token.string);

			result = operators.map(function (op) {
				return op.operator;
			});

			var token_id = token.type.split(' ')[1];
			var viewport = editor.getViewport();
			console.log(viewport, token_id);

		} else {

			search = token.string;

			if (search.charAt(0) == ".") { /* Dot member syntax */
				var last_token = editor.getTokenAt(CodeMirror.Pos(cursor.line, token.start - 2));
				result = LSFilterHint.get_foreign_field_match(search, last_token.string, function (value) {
					return "." + value;
				});
			} else {
				result = LSFilterHint.get_field_match(search).concat(
					keywords.reduce(function (words, word) {
						if (word.toUpperCase().indexOf(search.toUpperCase()) >= 0)
							words.push(word);
						return words;
					}, [])
				);
			}
		}

		var completion = {
			list: result,
			from: CodeMirror.Pos(cursor.line, token.start),
			to: CodeMirror.Pos(cursor.line, token.end)
		};

		CodeMirror.on(completion, 'pick', function (value) {

			if (select_table) {
				LSFilterHint.set_table(value.substring(1, value.length - 1));
			}

			if (operators) {
				var operator = operators.reduce(function (found, op) {
					if (op.operator === value)
						return op;
					return found;
				}, null);
				if (operator && operator.lhs_type === 'string') {
					editor.setCursor(
						cursor.line,
						cursor.ch + 2
					);
				}
			}

		});

		return completion;

  });
});



/*******************
 * From src: modules/lsfilter/src/js/codemirror/mode/lsfilter/lsfilter.js
 *******************/
// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE

(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    mod(require("../../lib/codemirror"));
  else if (typeof define == "function" && define.amd) // AMD
    define(["../../lib/codemirror"], mod);
  else // Plain browser env
    mod(CodeMirror);
})(function(CodeMirror) {
"use strict";

CodeMirror.defineMode("lsfilter", function(config, parserConfig) {
  "use strict";

  var client         = parserConfig.client || {},
      atoms          = parserConfig.atoms || {"false": true, "true": true, "null": true},
      builtin        = parserConfig.builtin || {},
      keywords       = parserConfig.keywords || {},
      operatorChars  = parserConfig.operatorChars || /^[*+\-%<>!=&|~^\{\}\[\]]/;

	var unique_id_index = 0;
	function unique_id () {
		unique_id_index++;
		return 'token-id-' + unique_id_index;
	}

  function tokenBase(stream, state) {

		var ch = stream.next();
		if (ch == '[') {
			return (state.tokenize = function(stream, state) {
				var ch;
				while ((ch = stream.next()) != null) {
					if (ch == ']') {
						state.tokenize = tokenBase;
						break;
					}
				}
				return "keyword";
			})(stream, state);
		} else if (ch.charCodeAt(0) > 47 && ch.charCodeAt(0) < 58) {
      stream.match(/^[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?/);
      return "number";
    } else if (ch == "'" || (ch == '"')) {
			return (state.tokenize = function(stream, state) {
				var escaped = false, ich;
				while ((ich = stream.next()) != null) {
					if (ich == ch && !escaped) {
						state.tokenize = tokenBase;
						break;
					}
					escaped = !escaped && ich == "\\";
				}
				return "string";
			})(stream, state);
    } else if (/^[\(\),\;\[\]]/.test(ch)) {
      // no highlighting
      return null;
    } else if (ch == "/" && stream.eat("*")) {
      return (state.tokenize = function (stream, state) {
				while (true) {
					if (stream.skipTo("*")) {
						stream.next();
						if (stream.eat("/")) {
							state.tokenize = tokenBase;
							break;
						}
					} else {
						stream.skipToEnd();
						break;
					}
				}
				return "comment";
			})(stream, state);
    } else if (ch == ".") {
      return "variable-2";
    } else if (operatorChars.test(ch)) {
      stream.eatWhile(operatorChars);
      return "operator " + unique_id();
    } else {
      stream.eatWhile(/^[_\w\d]/);
      return "variable-3";
    }

  }

  function pushContext(stream, state, type) {
    state.context = {
      prev: state.context,
      indent: stream.indentation(),
      col: stream.column(),
      type: type
    };
  }

  function popContext(state) {
    state.indent = state.context.indent;
    state.context = state.context.prev;
  }

  var ls_filter_mode = {

		startState: function() {
      return {
				tokenize: tokenBase,
				context: null
			};
    },

    token: function(stream, state) {
      if (stream.sol()) {
        if (state.context && state.context.align == null)
          state.context.align = false;
      }
      if (stream.eatSpace()) return null;

      var style = state.tokenize(stream, state);
      if (style == "comment") return style;

      if (state.context && state.context.align == null)
        state.context.align = true;

      var tok = stream.current();
      if (tok == "(")
        pushContext(stream, state, ")");
      else if (tok == "[")
        pushContext(stream, state, "]");
      else if (state.context && state.context.type == tok)
        popContext(state);
      return style;
    },

    indent: function(state, textAfter) {
      var cx = state.context;
      if (!cx) return CodeMirror.Pass;
      var closing = textAfter.charAt(0) == cx.type;
      if (cx.align) return cx.col + (closing ? 0 : 1);
      else return cx.indent + (closing ? 0 : config.indentUnit);
    },

    blockCommentStart: "/*",
    blockCommentEnd: "*/"
  };

	return ls_filter_mode;

});

(function() {
  "use strict";

  // turn a space-separated list into an array
  function set(str) {
    var obj = {}, words = str.split(" ");
    for (var i = 0; i < words.length; ++i) obj[words[i]] = true;
    return obj;
  }

  CodeMirror.defineMIME("text/x-lsfilter", {
    name: "lsfilter",
    keywords: set("and or"),
    builtin: set("date"),
    atoms: set("false true"),
    operatorChars: /^[*+\-%<>!=]/
  });

}());

});




/*******************
 * From src: modules/lsfilter/src/js/codemirror/show-hint.js
 *******************/
// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE

(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    mod(require("../../lib/codemirror"));
  else if (typeof define == "function" && define.amd) // AMD
    define(["../../lib/codemirror"], mod);
  else // Plain browser env
    mod(CodeMirror);
})(function(CodeMirror) {
  "use strict";

  var HINT_ELEMENT_CLASS        = "CodeMirror-hint";
  var ACTIVE_HINT_ELEMENT_CLASS = "CodeMirror-hint-active";

  // This is the old interface, kept around for now to stay
  // backwards-compatible.
  CodeMirror.showHint = function(cm, getHints, options) {
    if (!getHints) return cm.showHint(options);
    if (options && options.async) getHints.async = true;
    var newOpts = {hint: getHints};
    if (options) for (var prop in options) newOpts[prop] = options[prop];
    return cm.showHint(newOpts);
  };

  CodeMirror.defineExtension("showHint", function(options) {
    options = parseOptions(this, this.getCursor("start"), options);
    var selections = this.listSelections()
    if (selections.length > 1) return;
    // By default, don't allow completion when something is selected.
    // A hint function can have a `supportsSelection` property to
    // indicate that it can handle selections.
    if (this.somethingSelected()) {
      if (!options.hint.supportsSelection) return;
      // Don't try with cross-line selections
      for (var i = 0; i < selections.length; i++)
        if (selections[i].head.line != selections[i].anchor.line) return;
    }

    if (this.state.completionActive) this.state.completionActive.close();
    var completion = this.state.completionActive = new Completion(this, options);
    if (!completion.options.hint) return;

    CodeMirror.signal(this, "startCompletion", this);
    completion.update(true);
  });

  function Completion(cm, options) {
    this.cm = cm;
    this.options = options;
    this.widget = null;
    this.debounce = 0;
    this.tick = 0;
    this.startPos = this.cm.getCursor("start");
    this.startLen = this.cm.getLine(this.startPos.line).length - this.cm.getSelection().length;

    var self = this;
    cm.on("cursorActivity", this.activityFunc = function() { self.cursorActivity(); });
  }

  var requestAnimationFrame = window.requestAnimationFrame || function(fn) {
    return setTimeout(fn, 1000/60);
  };
  var cancelAnimationFrame = window.cancelAnimationFrame || clearTimeout;

  Completion.prototype = {
    close: function() {
      if (!this.active()) return;
      this.cm.state.completionActive = null;
      this.tick = null;
      this.cm.off("cursorActivity", this.activityFunc);

      if (this.widget && this.data) CodeMirror.signal(this.data, "close");
      if (this.widget) this.widget.close();
      CodeMirror.signal(this.cm, "endCompletion", this.cm);
    },

    active: function() {
      return this.cm.state.completionActive == this;
    },

    pick: function(data, i) {
      var completion = data.list[i];
      if (completion.hint) completion.hint(this.cm, data, completion);
      else this.cm.replaceRange(getText(completion), completion.from || data.from,
                                completion.to || data.to, "complete");
      CodeMirror.signal(data, "pick", completion);
      this.close();
    },

    cursorActivity: function() {
      if (this.debounce) {
        cancelAnimationFrame(this.debounce);
        this.debounce = 0;
      }

      var pos = this.cm.getCursor(), line = this.cm.getLine(pos.line);
      if (pos.line != this.startPos.line || line.length - pos.ch != this.startLen - this.startPos.ch ||
          pos.ch < this.startPos.ch || this.cm.somethingSelected() ||
          (pos.ch && this.options.closeCharacters.test(line.charAt(pos.ch - 1)))) {
        this.close();
      } else {
        var self = this;
        this.debounce = requestAnimationFrame(function() {self.update();});
        if (this.widget) this.widget.disable();
      }
    },

    update: function(first) {
      if (this.tick == null) return
      var self = this, myTick = ++this.tick
      fetchHints(this.options.hint, this.cm, this.options, function(data) {
        if (self.tick == myTick) self.finishUpdate(data, first)
      })
    },

    finishUpdate: function(data, first) {
      if (this.data) CodeMirror.signal(this.data, "update");

      var picked = (this.widget && this.widget.picked) || (first && this.options.completeSingle);
      if (this.widget) this.widget.close();

      if (data && this.data && isNewCompletion(this.data, data)) return;
      this.data = data;

      if (data && data.list.length) {
        if (picked && data.list.length == 1) {
          this.pick(data, 0);
        } else {
          this.widget = new Widget(this, data);
          CodeMirror.signal(data, "shown");
        }
      }
    }
  };

  function isNewCompletion(old, nw) {
    var moved = CodeMirror.cmpPos(nw.from, old.from)
    return moved > 0 && old.to.ch - old.from.ch != nw.to.ch - nw.from.ch
  }

  function parseOptions(cm, pos, options) {
    var editor = cm.options.hintOptions;
    var out = {};
    for (var prop in defaultOptions) out[prop] = defaultOptions[prop];
    if (editor) for (var prop in editor)
      if (editor[prop] !== undefined) out[prop] = editor[prop];
    if (options) for (var prop in options)
      if (options[prop] !== undefined) out[prop] = options[prop];
    if (out.hint.resolve) out.hint = out.hint.resolve(cm, pos)
    return out;
  }

  function getText(completion) {
    if (typeof completion == "string") return completion;
    else return completion.text;
  }

  function buildKeyMap(completion, handle) {
    var baseMap = {
      Up: function() {handle.moveFocus(-1);},
      Down: function() {handle.moveFocus(1);},
      PageUp: function() {handle.moveFocus(-handle.menuSize() + 1, true);},
      PageDown: function() {handle.moveFocus(handle.menuSize() - 1, true);},
      Home: function() {handle.setFocus(0);},
      End: function() {handle.setFocus(handle.length - 1);},
      Enter: handle.pick,
      Tab: handle.pick,
      Esc: handle.close
    };
    var custom = completion.options.customKeys;
    var ourMap = custom ? {} : baseMap;
    function addBinding(key, val) {
      var bound;
      if (typeof val != "string")
        bound = function(cm) { return val(cm, handle); };
      // This mechanism is deprecated
      else if (baseMap.hasOwnProperty(val))
        bound = baseMap[val];
      else
        bound = val;
      ourMap[key] = bound;
    }
    if (custom)
      for (var key in custom) if (custom.hasOwnProperty(key))
        addBinding(key, custom[key]);
    var extra = completion.options.extraKeys;
    if (extra)
      for (var key in extra) if (extra.hasOwnProperty(key))
        addBinding(key, extra[key]);
    return ourMap;
  }

  function getHintElement(hintsElement, el) {
    while (el && el != hintsElement) {
      if (el.nodeName.toUpperCase() === "LI" && el.parentNode == hintsElement) return el;
      el = el.parentNode;
    }
  }

  function Widget(completion, data) {
    this.completion = completion;
    this.data = data;
    this.picked = false;
    var widget = this, cm = completion.cm;

    var hints = this.hints = document.createElement("ul");
    hints.className = "CodeMirror-hints";
    this.selectedHint = data.selectedHint || 0;

    var completions = data.list;
    for (var i = 0; i < completions.length; ++i) {
      var elt = hints.appendChild(document.createElement("li")), cur = completions[i];
      var className = HINT_ELEMENT_CLASS + (i != this.selectedHint ? "" : " " + ACTIVE_HINT_ELEMENT_CLASS);
      if (cur.className != null) className = cur.className + " " + className;
      elt.className = className;
      if (cur.render) cur.render(elt, data, cur);
      else elt.appendChild(document.createTextNode(cur.displayText || getText(cur)));
      elt.hintId = i;
    }

    var pos = cm.cursorCoords(completion.options.alignWithWord ? data.from : null);
    var left = pos.left, top = pos.bottom, below = true;
    hints.style.left = left + "px";
    hints.style.top = top + "px";
    // If we're at the edge of the screen, then we want the menu to appear on the left of the cursor.
    var winW = window.innerWidth || Math.max(document.body.offsetWidth, document.documentElement.offsetWidth);
    var winH = window.innerHeight || Math.max(document.body.offsetHeight, document.documentElement.offsetHeight);
    (completion.options.container || document.body).appendChild(hints);
    var box = hints.getBoundingClientRect(), overlapY = box.bottom - winH;
    var scrolls = hints.scrollHeight > hints.clientHeight + 1
    if (overlapY > 0) {
      var height = box.bottom - box.top, curTop = pos.top - (pos.bottom - box.top);
      if (curTop - height > 0) { // Fits above cursor
        hints.style.top = (top = pos.top - height) + "px";
        below = false;
      } else if (height > winH) {
        hints.style.height = (winH - 5) + "px";
        hints.style.top = (top = pos.bottom - box.top) + "px";
        var cursor = cm.getCursor();
        if (data.from.ch != cursor.ch) {
          pos = cm.cursorCoords(cursor);
          hints.style.left = (left = pos.left) + "px";
          box = hints.getBoundingClientRect();
        }
      }
    }
    var overlapX = box.right - winW;
    if (overlapX > 0) {
      if (box.right - box.left > winW) {
        hints.style.width = (winW - 5) + "px";
        overlapX -= (box.right - box.left) - winW;
      }
      hints.style.left = (left = pos.left - overlapX) + "px";
    }
    if (scrolls) for (var node = hints.firstChild; node; node = node.nextSibling)
      node.style.paddingRight = cm.display.nativeBarWidth + "px"

    cm.addKeyMap(this.keyMap = buildKeyMap(completion, {
      moveFocus: function(n, avoidWrap) { widget.changeActive(widget.selectedHint + n, avoidWrap); },
      setFocus: function(n) { widget.changeActive(n); },
      menuSize: function() { return widget.screenAmount(); },
      length: completions.length,
      close: function() { completion.close(); },
      pick: function() { widget.pick(); },
      data: data
    }));

    if (completion.options.closeOnUnfocus) {
      var closingOnBlur;
      cm.on("blur", this.onBlur = function() { closingOnBlur = setTimeout(function() { completion.close(); }, 100); });
      cm.on("focus", this.onFocus = function() { clearTimeout(closingOnBlur); });
    }

    var startScroll = cm.getScrollInfo();
    cm.on("scroll", this.onScroll = function() {
      var curScroll = cm.getScrollInfo(), editor = cm.getWrapperElement().getBoundingClientRect();
      var newTop = top + startScroll.top - curScroll.top;
      var point = newTop - (window.pageYOffset || (document.documentElement || document.body).scrollTop);
      if (!below) point += hints.offsetHeight;
      if (point <= editor.top || point >= editor.bottom) return completion.close();
      hints.style.top = newTop + "px";
      hints.style.left = (left + startScroll.left - curScroll.left) + "px";
    });

    CodeMirror.on(hints, "dblclick", function(e) {
      var t = getHintElement(hints, e.target || e.srcElement);
      if (t && t.hintId != null) {widget.changeActive(t.hintId); widget.pick();}
    });

    CodeMirror.on(hints, "click", function(e) {
      var t = getHintElement(hints, e.target || e.srcElement);
      if (t && t.hintId != null) {
        widget.changeActive(t.hintId);
        if (completion.options.completeOnSingleClick) widget.pick();
      }
    });

    CodeMirror.on(hints, "mousedown", function() {
      setTimeout(function(){cm.focus();}, 20);
    });

    CodeMirror.signal(data, "select", completions[0], hints.firstChild);
    return true;
  }

  Widget.prototype = {
    close: function() {
      if (this.completion.widget != this) return;
      this.completion.widget = null;
      this.hints.parentNode.removeChild(this.hints);
      this.completion.cm.removeKeyMap(this.keyMap);

      var cm = this.completion.cm;
      if (this.completion.options.closeOnUnfocus) {
        cm.off("blur", this.onBlur);
        cm.off("focus", this.onFocus);
      }
      cm.off("scroll", this.onScroll);
    },

    disable: function() {
      this.completion.cm.removeKeyMap(this.keyMap);
      var widget = this;
      this.keyMap = {Enter: function() { widget.picked = true; }};
      this.completion.cm.addKeyMap(this.keyMap);
    },

    pick: function() {
      this.completion.pick(this.data, this.selectedHint);
    },

    changeActive: function(i, avoidWrap) {
      if (i >= this.data.list.length)
        i = avoidWrap ? this.data.list.length - 1 : 0;
      else if (i < 0)
        i = avoidWrap ? 0  : this.data.list.length - 1;
      if (this.selectedHint == i) return;
      var node = this.hints.childNodes[this.selectedHint];
      node.className = node.className.replace(" " + ACTIVE_HINT_ELEMENT_CLASS, "");
      node = this.hints.childNodes[this.selectedHint = i];
      node.className += " " + ACTIVE_HINT_ELEMENT_CLASS;
      if (node.offsetTop < this.hints.scrollTop)
        this.hints.scrollTop = node.offsetTop - 3;
      else if (node.offsetTop + node.offsetHeight > this.hints.scrollTop + this.hints.clientHeight)
        this.hints.scrollTop = node.offsetTop + node.offsetHeight - this.hints.clientHeight + 3;
      CodeMirror.signal(this.data, "select", this.data.list[this.selectedHint], node);
    },

    screenAmount: function() {
      return Math.floor(this.hints.clientHeight / this.hints.firstChild.offsetHeight) || 1;
    }
  };

  function applicableHelpers(cm, helpers) {
    if (!cm.somethingSelected()) return helpers
    var result = []
    for (var i = 0; i < helpers.length; i++)
      if (helpers[i].supportsSelection) result.push(helpers[i])
    return result
  }

  function fetchHints(hint, cm, options, callback) {
    if (hint.async) {
      hint(cm, callback, options)
    } else {
      var result = hint(cm, options)
      if (result && result.then) result.then(callback)
      else callback(result)
    }
  }

  function resolveAutoHints(cm, pos) {
    var helpers = cm.getHelpers(pos, "hint"), words
    if (helpers.length) {
      var resolved = function(cm, callback, options) {
        var app = applicableHelpers(cm, helpers);
        function run(i) {
          if (i == app.length) return callback(null)
          fetchHints(app[i], cm, options, function(result) {
            if (result && result.list.length > 0) callback(result)
            else run(i + 1)
          })
        }
        run(0)
      }
      resolved.async = true
      resolved.supportsSelection = true
      return resolved
    } else if (words = cm.getHelper(cm.getCursor(), "hintWords")) {
      return function(cm) { return CodeMirror.hint.fromList(cm, {words: words}) }
    } else if (CodeMirror.hint.anyword) {
      return function(cm, options) { return CodeMirror.hint.anyword(cm, options) }
    } else {
      return function() {}
    }
  }

  CodeMirror.registerHelper("hint", "auto", {
    resolve: resolveAutoHints
  });

  CodeMirror.registerHelper("hint", "fromList", function(cm, options) {
    var cur = cm.getCursor(), token = cm.getTokenAt(cur);
    var to = CodeMirror.Pos(cur.line, token.end);
    if (token.string && /\w/.test(token.string[token.string.length - 1])) {
      var term = token.string, from = CodeMirror.Pos(cur.line, token.start);
    } else {
      var term = "", from = to;
    }
    var found = [];
    for (var i = 0; i < options.words.length; i++) {
      var word = options.words[i];
      if (word.slice(0, term.length) == term)
        found.push(word);
    }

    if (found.length) return {list: found, from: from, to: to};
  });

  CodeMirror.commands.autocomplete = CodeMirror.showHint;

  var defaultOptions = {
    hint: CodeMirror.hint.auto,
    completeSingle: true,
    alignWithWord: true,
    closeCharacters: /[\s()\[\]{};:>,]/,
    closeOnUnfocus: true,
    completeOnSingleClick: true,
    container: null,
    customKeys: null,
    extraKeys: null
  };

  CodeMirror.defineOption("hintOptions", null);
});



/*******************
 * From src: modules/lsfilter/src/js/form.autocomplete.js
 *******************/
(function () {

	"use strict";

	/**
	 * This should not be here, but instead have its own module
	 * but it is so tightly knit into the autocomplete currently
	 * that  I don't want to touch it.
	 */
	$(document).on('autocompleted', '.nj-form-field input', function(e) {
		var field = $(this);
		var form = field.closest('form');
		var field_name = field.attr('name');
		/* trim away possible array values, because if we're listening
		 * for "host", we don't want to rely on specific rendering
		 * details such as host[value], in the case of ORMObject form
		 * fields */
			field_name = field_name.replace(/\[.*/, "");
		/* only handling <select/> elements for now */

		form.find(".nj-form-field select[data-njform-target='"+field_name+"']").each(function() {
			var select = $(this);
			select.empty();
			$.ajax(_site_domain + _index_page + '/perfdata/perf_data_sources', {
				data: {
					table: form.find('input[name="'+field_name+'[table]"]').val(),
					key: field.val()
				},
				success: function(data) {
					if(!data.result.length) {
						Notify.message('No performance data available for this object', {
							type: "info"
						});
						return;
					}
					$.each(data.result, function(key, value) {
						select.append($("<option/>").text(value).val(value));
					});
				},
				error: function(jqXHR) {
					var msg = '';
					try {
						msg = JSON.parse(jqXHR.responseText).message;
					} catch (e) {
						msg = 'Something went wrong, perhaps you could try again';
					}
					Notify.message(msg, {'type': 'error'});
				},
				method: "GET"
			});

		});

	});


	var Autocomplete = function (form_field) {

		this.key_index = 1;
		this.timeout = null;

		this.input = form_field.find('.nj-form-field-autocomplete-input');
		this.items = form_field.find('.nj-form-field-autocomplete-items');
		this.dropper = form_field.find('.nj-form-field-autocomplete-dropper');
		this.table_element = form_field.find('.nj-form-field-autocomplete-table');
		this.placeholder = this.input.attr('placeholder');

		this.tables = form_field.attr('data-autocomplete').split(',');
		var self = this;

		this.input.on('keyup', function (e) {
			e.preventDefault();
			if (e.key !== 'ArrowDown' && e.keyCode !== 40 && e.key !== 'ArrowUp' && e.keyCode !== 38) {
				self.update(self.input.val());
			}
			return false;
		});

		this.input.on('keydown', function (e) {

			if (e.key === 'Enter' || e.keyCode === 13) {

				self.set_value(
					self.get_current().attr('data-autocomplete-value'),
					self.get_current().attr('data-autocomplete-table')
				);
				self.hide();

				e.preventDefault();
				return false;

			} else if (e.key === 'ArrowDown' || e.keyCode === 40) {

				var items = self.items.children('li.nj-form-field-autocomplete-item');
				self.key_index = (self.key_index < items.length) ? self.key_index + 1 : items.length;
				self.reselect();

			} else if (e.key === 'ArrowUp' || e.keyCode === 38) {

				self.key_index = (self.key_index > 1) ? self.key_index - 1 : 1;
				self.reselect();

			} else if (e.key === 'Tab' || e.keyCode === 9) {

				self.set_value(
					self.get_current().attr('data-autocomplete-value'),
					self.get_current().attr('data-autocomplete-table')
				);

				self.hide();

				e.preventDefault();
				return false;

			}

		});

		this.items.on('mousedown', 'li.nj-form-field-autocomplete-item', function (e) {

			var item = $(e.target);
			self.set_value(
				item.attr('data-autocomplete-value'),
				item.attr('data-autocomplete-table')
			);

			self.hide();

			e.preventDefault();
			return false;

		});

		this.input.on('focus', function (e) {
			self.update(self.input.val());
			self.input.attr('placeholder', "");
		});

		this.input.on('blur', function (e) {
			self.hide();
			self.input.attr('placeholder', self.placeholder);
			e.preventDefault();
			return false;
		});

	};

	Autocomplete.prototype = {

		get_current: function () {
			return this.items
				.children('li.nj-form-field-autocomplete-item')
				.eq(this.key_index - 1);
		},

		set_value: function (value, table) {
			if(value === this.input.val()) {
				return;
			}
			this.input.val(value);
			// make sure that those listening on events don't get
			// left in the cold
			this.input.trigger('autocompleted');
			this.table_element.val(table);
		},

		reselect: function () {

			var items = this.items.children('.nj-form-field-autocomplete-item');
			var item = items.eq(this.key_index - 1);

			items.removeClass('nj-form-field-autocomplete-item-selected');
			item.addClass('nj-form-field-autocomplete-item-selected');

		},

		fetch: function (term, callback) {

			$.ajax(_site_domain + _index_page + '/autocomplete/autocomplete', {
				data: {
					tables: this.tables,
					term: term
				},
				success: callback,
				method: "GET",
			});

		},

		update: function (term) {

			clearTimeout(this.timeout);
			this.timeout = setTimeout(function () {
				this.fetch(term, function (data) {

					if (data.length === 1 && data[0].name === term) {
						this.table_element.val(data[0].table);
						this.hide();
						return;
					}


					this.key_index = 1;

					this.items.empty();
					this.populate(data);
					this.items.show();

				}.bind(this));
			}.bind(this), 150);

		},

		populate: function (key_values) {

			var fragment = document.createDocumentFragment();
			var current_table;

			key_values.forEach(function (item, index) {

				if (current_table != item.table) {
					var title = $('<li class="nj-form-field-autocomplete-items-title">').text("Suggestions");
					fragment.appendChild(title.get(0));
					current_table = item.table;
				}

				var item_element = $('<li class="nj-form-field-autocomplete-item">').attr({
					'data-autocomplete-value': item.key,
					'data-autocomplete-table': item.table
				}).text(item.name);

				fragment.appendChild(item_element.get(0));

			});

			this.items.append(fragment);

		},

		hide: function () {
			this.items.hide();
			this.items.empty();
		},

		show: function () {
			this.items.show();
		}

	};

	FormModule.register(function (form) {
		form.find('.nj-form-field-autocomplete').each(function (index, field) {
			new Autocomplete($(field));
		});
	});

}());



/*******************
 * From src: modules/lsfilter/src/js/form.codemirror.js
 *******************/
(function () {
	"use strict";
	$(document).ready(function () {
		$('.nj-form-field-listview-query').each(function () {

			var field = $(this);
			var textarea = field.find('textarea').get(0);
			var validator_to = null;

			var editor = CodeMirror.fromTextArea(textarea, {
				lineNumber: true,
				mode: "text/x-lsfilter",
				hintOptions: {
					structure: orm_structure,
					completeSingle: false
				}
			});

			editor.on('keyup', function (e, s) {

				var mode = CodeMirror.resolveMode("text/x-lsfilter");
				var operator_matcher = mode.operatorChars;

				if (s.key.match(/^[\d\w\{\}\[\]\.\_]$/) || s.key.match(operator_matcher))	{
					CodeMirror.commands.autocomplete(editor);
				}

			});

			editor.on('change', function () {

				clearTimeout(validator_to);
				validator_to = setTimeout(function () {

					var query = editor.getValue();

					var visitor = new LSFilterMetadataVisitor();
					var parser = new LSFilterParser(visitor);
					var preprocessor = new LSFilterPreprocessor(parser);
					var lexer = new LSFilterLexer(query, preprocessor);

					try {
						var result = parser.parse(lexer);
						field.find('.CodeMirror').addClass('success');
						field.find('.CodeMirror').removeClass('error');
					} catch (error) {
						field.find('.CodeMirror').removeClass('success');
						field.find('.CodeMirror').addClass('error');
					}

				}, 150);

			});

		});


	});
}());



/*******************
 * From src: modules/lsfilter/src/js/generated/LSColumns.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSColumns = function LSColumns(preprocessor, visitor){
	this.preprocessor = preprocessor;
	this.visitor = visitor;
	this.parse = function(string) {
		var lexer = new LSColumnsLexer( string, this.preprocessor );
		var parser = new LSColumnsParser( this.visitor );
		return parser.parse( lexer );
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSColumnsLexer.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSColumnsLexer = function LSColumnsLexer(buffer, visitor){
	this.buffer = buffer;
	this.visitor = visitor;
	this.expression = buffer;
	this.position = 0;
	this.fetch_token = function() {
		do {
			var length = -1;
			var token = 0;
			var value = 0;
			var token_pos = this.position;
			
			// Match end token
			if( this.buffer.length == 0 ) {
				length = 0;
				token = "end";
			}
			
			// Match token: _whitespace
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\s+)/);
				if( matches != null ) {
					length = matches[1].length;
				}
			}
			
			// Match token: string
			if( length < 0 ) {
				var matches = this.buffer.match(/^("(?:[^"\\]|\\.)*")/);
				if( matches != null ) {
					length = matches[1].length;
					token = "string";
					value = this.visitor.preprocess_string(matches[1]);
				}
			}
			
			// Match token: float
			if( length < 0 ) {
				var matches = this.buffer.match(/^([0-9]+\.[0-9]*)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "float";
					value = this.visitor.preprocess_float(matches[1]);
				}
			}
			
			// Match token: integer
			if( length < 0 ) {
				var matches = this.buffer.match(/^([0-9]+)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "integer";
					value = this.visitor.preprocess_integer(matches[1]);
				}
			}
			
			// Match token: then
			if( length < 0 ) {
				var matches = this.buffer.match(/^(then)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "then";
					value = this.visitor.preprocess_then(matches[1]);
				}
			}
			
			// Match token: else
			if( length < 0 ) {
				var matches = this.buffer.match(/^(else)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "else";
					value = this.visitor.preprocess_else(matches[1]);
				}
			}
			
			// Match token: all
			if( length < 0 ) {
				var matches = this.buffer.match(/^(all)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "all";
					value = this.visitor.preprocess_all(matches[1]);
				}
			}
			
			// Match token: default
			if( length < 0 ) {
				var matches = this.buffer.match(/^(default)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "default";
					value = this.visitor.preprocess_default(matches[1]);
				}
			}
			
			// Match token: for
			if( length < 0 ) {
				var matches = this.buffer.match(/^(for)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "for";
					value = this.visitor.preprocess_for(matches[1]);
				}
			}
			
			// Match token: in
			if( length < 0 ) {
				var matches = this.buffer.match(/^(in)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "in";
					value = this.visitor.preprocess_in(matches[1]);
				}
			}
			
			// Match token: if
			if( length < 0 ) {
				var matches = this.buffer.match(/^(if)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "if";
					value = this.visitor.preprocess_if(matches[1]);
				}
			}
			
			// Match token: name
			if( length < 0 ) {
				var matches = this.buffer.match(/^([a-z0-9_]+)/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "name";
					value = this.visitor.preprocess_name(matches[1]);
				}
			}
			
			// Match token: op_sub
			if( length < 0 ) {
				var matches = this.buffer.match(/^(-)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "op_sub";
					value = this.visitor.preprocess_op_sub(matches[1]);
				}
			}
			
			// Match token: op_add
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\+)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "op_add";
					value = this.visitor.preprocess_op_add(matches[1]);
				}
			}
			
			// Match token: op_mult
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\*)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "op_mult";
					value = this.visitor.preprocess_op_mult(matches[1]);
				}
			}
			
			// Match token: op_div
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\/)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "op_div";
					value = this.visitor.preprocess_op_div(matches[1]);
				}
			}
			
			// Match token: comma
			if( length < 0 ) {
				var matches = this.buffer.match(/^(,)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "comma";
					value = this.visitor.preprocess_comma(matches[1]);
				}
			}
			
			// Match token: dot
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\.)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "dot";
					value = this.visitor.preprocess_dot(matches[1]);
				}
			}
			
			// Match token: eq
			if( length < 0 ) {
				var matches = this.buffer.match(/^(=)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "eq";
					value = this.visitor.preprocess_eq(matches[1]);
				}
			}
			
			// Match token: par_l
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\()/);
				if( matches != null ) {
					length = matches[1].length;
					token = "par_l";
					value = this.visitor.preprocess_par_l(matches[1]);
				}
			}
			
			// Match token: par_r
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\))/);
				if( matches != null ) {
					length = matches[1].length;
					token = "par_r";
					value = this.visitor.preprocess_par_r(matches[1]);
				}
			}
			
			// Match token: sq_l
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\[)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "sq_l";
					value = this.visitor.preprocess_sq_l(matches[1]);
				}
			}
			
			// Match token: sq_r
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\])/);
				if( matches != null ) {
					length = matches[1].length;
					token = "sq_r";
					value = this.visitor.preprocess_sq_r(matches[1]);
				}
			}
			
			// Match token: char
			if( length < 0 ) {
				var matches = this.buffer.match(/^(.)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "char";
					value = this.visitor.preprocess_char(matches[1]);
				}
			}
			
			// Error if no match
			if( length < 0 ) {
				throw "Unknown token: " + this.buffer.substring(0,30);
			}
			
			// Remove token from buffer, and move length forward
			this.buffer = this.buffer.substr( length );
			this.position += length; 
		} while( token == 0 );
		return [token, value, token_pos, length ];
	};
	
	this.tokens_to_string = function(tokens) {
		var start=false;
		var end=false;
		for( var i=0; i<tokens.length; i++ ) {
			if(start === false || start > tokens[i][2]) {
				start = tokens[i][2];
			}
			var tokend = tokens[i][2] + tokens[i][3];
			if(end === false || end < tokend) {
				end = tokend;
			}
		}
		return this.expression.substring(start,end);
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSColumnsParser.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSColumnsParser = function LSColumnsParser(visitor){
	this.visitor = null;
	this.stack = [];
	this.cont = false;
	this.done = false;
	this.lexer = false;
	this.visitor = visitor;
	this.parse = function(lexer) {
		this.stack = new Array();
		this.stack.push( [0,"start"] );
		this.done = false;
		this.lexer = lexer;
		var result = false;
		do {
			this.token = lexer.fetch_token();
			do {
				this.cont = false;
				var head = this.stack[this.stack.length-1];
				this.tmp = this.states[head[0]];
				result = this.tmp();
			} while( this.cont );
		} while( !this.done );
		return result;
	};
	
	this.states = [
	function() {
		// State: 0
		switch( this.token[0] ) {
			case "all":
				// shift:104
				this.stack.push( [104,this.token] );
				return null;
			case "default":
				// shift:103
				this.stack.push( [103,this.token] );
				return null;
			case "name":
				// shift:102
				this.stack.push( [102,this.token] );
				return null;
			case "op_sub":
				// shift:99
				this.stack.push( [99,this.token] );
				return null;
			case "string":
				// shift:1
				this.stack.push( [1,this.token] );
				return null;
			case "end":
			case "comma":
				// error:column_list_error
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 1
		switch( this.token[0] ) {
			case "eq":
				// reduce:_custom_name_str
				this.reduce__custom_name_str();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 2
		switch( this.token[0] ) {
			case "eq":
				// shift:3
				this.stack.push( [3,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 3
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
			case "end":
			case "comma":
				// error:custom_content_error
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 4
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 5
		switch( this.token[0] ) {
			case "sq_l":
			case "dot":
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:var_var
				this.reduce_var_var();
				return null;
			case "par_l":
				// shift:6
				this.stack.push( [6,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 6
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 7
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 8
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 9
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_float
				this.reduce_expr_float();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 10
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_int
				this.reduce_expr_int();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 11
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_string
				this.reduce_expr_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 12
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_var
				this.reduce_expr_var();
				return null;
			case "sq_l":
				// shift:15
				this.stack.push( [15,this.token] );
				return null;
			case "dot":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 13
		switch( this.token[0] ) {
			case "name":
				// shift:14
				this.stack.push( [14,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 14
		switch( this.token[0] ) {
			case "sq_l":
			case "dot":
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:var_attr
				this.reduce_var_attr();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 15
		switch( this.token[0] ) {
			case "integer":
				// shift:16
				this.stack.push( [16,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 16
		switch( this.token[0] ) {
			case "sq_r":
				// shift:17
				this.stack.push( [17,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 17
		switch( this.token[0] ) {
			case "sq_l":
			case "dot":
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:var_index
				this.reduce_var_index();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 18
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_3_4
				this.reduce__expr_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 19
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 20
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_neg
				this.reduce_expr_neg();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 21
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_2_3
				this.reduce__expr_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 22
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_1_2
				this.reduce__expr_1_2();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 23
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 24
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_div
				this.reduce_expr_div();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 25
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 26
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_mult
				this.reduce_expr_mult();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 27
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_0_1
				this.reduce__expr_0_1();
				return null;
			case "eq":
				// shift:32
				this.stack.push( [32,this.token] );
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 28
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 29
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_sub
				this.reduce_expr_sub();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 30
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 31
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_add
				this.reduce_expr_add();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 32
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 33
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_eq
				this.reduce_expr_eq();
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 34
		switch( this.token[0] ) {
			case "par_r":
				// shift:35
				this.stack.push( [35,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 35
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_par
				this.reduce__expr_par();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 36
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_float
				this.reduce_expr_float();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 37
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_int
				this.reduce_expr_int();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 38
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_string
				this.reduce_expr_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 39
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_var
				this.reduce_expr_var();
				return null;
			case "sq_l":
				// shift:15
				this.stack.push( [15,this.token] );
				return null;
			case "dot":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 40
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_3_4
				this.reduce__expr_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 41
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 42
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_2_3
				this.reduce__expr_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 43
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_1_2
				this.reduce__expr_1_2();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 44
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_0_1
				this.reduce__expr_0_1();
				return null;
			case "eq":
				// shift:32
				this.stack.push( [32,this.token] );
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 45
		switch( this.token[0] ) {
			case "for":
				// shift:46
				this.stack.push( [46,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 46
		switch( this.token[0] ) {
			case "name":
				// shift:47
				this.stack.push( [47,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 47
		switch( this.token[0] ) {
			case "in":
				// shift:48
				this.stack.push( [48,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 48
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 49
		switch( this.token[0] ) {
			case "sq_r":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "if":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 50
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 51
		switch( this.token[0] ) {
			case "sq_r":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 52
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_list_comp_if
				this.reduce_expr_list_comp_if();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 53
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_list_comp
				this.reduce_expr_list_comp();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 54
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 55
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_float
				this.reduce_expr_float();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 56
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_int
				this.reduce_expr_int();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 57
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_string
				this.reduce_expr_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 58
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_var
				this.reduce_expr_var();
				return null;
			case "sq_l":
				// shift:15
				this.stack.push( [15,this.token] );
				return null;
			case "dot":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 59
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_3_4
				this.reduce__expr_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 60
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 61
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_2_3
				this.reduce__expr_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 62
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_1_2
				this.reduce__expr_1_2();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 63
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_0_1
				this.reduce__expr_0_1();
				return null;
			case "eq":
				// shift:32
				this.stack.push( [32,this.token] );
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 64
		switch( this.token[0] ) {
			case "par_r":
				// reduce:expr_list_end
				this.reduce_expr_list_end();
				return null;
			case "comma":
				// shift:65
				this.stack.push( [65,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 65
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 66
		switch( this.token[0] ) {
			case "par_r":
				// reduce:expr_list
				this.reduce_expr_list();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 67
		switch( this.token[0] ) {
			case "par_r":
				// shift:68
				this.stack.push( [68,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 68
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_func
				this.reduce_expr_func();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 69
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 70
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 71
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_float
				this.reduce_expr_float();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 72
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_int
				this.reduce_expr_int();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 73
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_string
				this.reduce_expr_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 74
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_var
				this.reduce_expr_var();
				return null;
			case "sq_l":
				// shift:15
				this.stack.push( [15,this.token] );
				return null;
			case "dot":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 75
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_3_4
				this.reduce__expr_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 76
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 77
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_2_3
				this.reduce__expr_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 78
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_1_2
				this.reduce__expr_1_2();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 79
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_0_1
				this.reduce__expr_0_1();
				return null;
			case "eq":
				// shift:32
				this.stack.push( [32,this.token] );
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 80
		switch( this.token[0] ) {
			case "then":
				// shift:81
				this.stack.push( [81,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 81
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 82
		switch( this.token[0] ) {
			case "else":
				// shift:83
				this.stack.push( [83,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 83
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 84
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_if
				this.reduce_expr_if();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 85
		switch( this.token[0] ) {
			case "sq_l":
			case "dot":
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:var_var
				this.reduce_var_var();
				return null;
			case "par_l":
				// shift:6
				this.stack.push( [6,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 86
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 87
		switch( this.token[0] ) {
			case "op_sub":
				// shift:19
				this.stack.push( [19,this.token] );
				return null;
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 88
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_float
				this.reduce_expr_float();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 89
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_int
				this.reduce_expr_int();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 90
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_string
				this.reduce_expr_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 91
		switch( this.token[0] ) {
			case "else":
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:expr_var
				this.reduce_expr_var();
				return null;
			case "sq_l":
				// shift:15
				this.stack.push( [15,this.token] );
				return null;
			case "dot":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 92
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_3_4
				this.reduce__expr_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 93
		switch( this.token[0] ) {
			case "string":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "integer":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
			case "float":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
			case "par_l":
				// shift:8
				this.stack.push( [8,this.token] );
				return null;
			case "sq_l":
				// shift:7
				this.stack.push( [7,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
			case "if":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 94
		switch( this.token[0] ) {
			case "op_mult":
			case "op_div":
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_2_3
				this.reduce__expr_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 95
		switch( this.token[0] ) {
			case "eq":
			case "op_add":
			case "op_sub":
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_1_2
				this.reduce__expr_1_2();
				return null;
			case "op_mult":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "op_div":
				// shift:23
				this.stack.push( [23,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 96
		switch( this.token[0] ) {
			case "par_r":
			case "for":
			case "sq_r":
			case "if":
			case "then":
			case "comma":
			case "end":
				// reduce:_expr_0_1
				this.reduce__expr_0_1();
				return null;
			case "eq":
				// shift:32
				this.stack.push( [32,this.token] );
				return null;
			case "op_add":
				// shift:30
				this.stack.push( [30,this.token] );
				return null;
			case "op_sub":
				// shift:28
				this.stack.push( [28,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 97
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:_custom_content_expr
				this.reduce__custom_content_expr();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 98
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_custom
				this.reduce_column_custom();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 99
		switch( this.token[0] ) {
			case "name":
				// shift:101
				this.stack.push( [101,this.token] );
				return null;
			case "string":
				// shift:100
				this.stack.push( [100,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 100
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_disable_str
				this.reduce_column_disable_str();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 101
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_disable
				this.reduce_column_disable();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 102
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_named
				this.reduce_column_named();
				return null;
			case "eq":
				// reduce:_custom_name_name
				this.reduce__custom_name_name();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 103
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_default
				this.reduce_column_default();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 104
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_all
				this.reduce_column_all();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 105
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_list_single
				this.reduce_column_list_single();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 106
		switch( this.token[0] ) {
			case "end":
				// accept:
				var program = this.stack.pop();
				this.done = true;
				return this.visitor.accept(program[1][1]);
			case "comma":
				// shift:107
				this.stack.push( [107,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 107
		switch( this.token[0] ) {
			case "all":
				// shift:104
				this.stack.push( [104,this.token] );
				return null;
			case "default":
				// shift:103
				this.stack.push( [103,this.token] );
				return null;
			case "name":
				// shift:102
				this.stack.push( [102,this.token] );
				return null;
			case "op_sub":
				// shift:99
				this.stack.push( [99,this.token] );
				return null;
			case "string":
				// shift:1
				this.stack.push( [1,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 108
		switch( this.token[0] ) {
			case "end":
			case "comma":
				// reduce:column_list_multi
				this.reduce_column_list_multi();
				return null;
		}
		this.errorhandler();
		return null;
	},
	];
	this.reduce_column_list_single = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["column_list", this.visitor.visit_column_list_single(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([106,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_list_multi = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["column_list", this.visitor.visit_column_list_multi(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([106,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_all = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_all(), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_default = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_default(), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_named = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_named(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_disable = function() {
		this.cont = true;
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_disable(arg1[1][1]), arg0[1][2], arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_disable_str = function() {
		this.cont = true;
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_disable_str(arg1[1][1]), arg0[1][2], arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_column_custom = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["column", this.visitor.visit_column_custom(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([105,new_token]); break;
			case 107:
				this.stack.push([108,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__custom_name_name = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["custom_name", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
			case 107:
				this.stack.push([2,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__custom_name_str = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["custom_name", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
			case 107:
				this.stack.push([2,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__custom_content_expr = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["custom_content", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
				this.stack.push([98,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_eq = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr", this.visitor.visit_expr_eq(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
				this.stack.push([97,new_token]); break;
			case 4:
				this.stack.push([80,new_token]); break;
			case 6:
			case 65:
				this.stack.push([64,new_token]); break;
			case 7:
			case 69:
			case 86:
				this.stack.push([45,new_token]); break;
			case 8:
			case 54:
			case 70:
			case 87:
				this.stack.push([34,new_token]); break;
			case 48:
				this.stack.push([49,new_token]); break;
			case 50:
				this.stack.push([51,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__expr_0_1 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
				this.stack.push([97,new_token]); break;
			case 4:
				this.stack.push([80,new_token]); break;
			case 6:
			case 65:
				this.stack.push([64,new_token]); break;
			case 7:
			case 69:
			case 86:
				this.stack.push([45,new_token]); break;
			case 8:
			case 54:
			case 70:
			case 87:
				this.stack.push([34,new_token]); break;
			case 48:
				this.stack.push([49,new_token]); break;
			case 50:
				this.stack.push([51,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_add = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr1", this.visitor.visit_expr_add(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([27,new_token]); break;
			case 32:
				this.stack.push([33,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_sub = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr1", this.visitor.visit_expr_sub(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([27,new_token]); break;
			case 32:
				this.stack.push([33,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__expr_1_2 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr1", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([27,new_token]); break;
			case 32:
				this.stack.push([33,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_mult = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr2", this.visitor.visit_expr_mult(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([22,new_token]); break;
			case 28:
				this.stack.push([29,new_token]); break;
			case 30:
				this.stack.push([31,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_div = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr2", this.visitor.visit_expr_div(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([22,new_token]); break;
			case 28:
				this.stack.push([29,new_token]); break;
			case 30:
				this.stack.push([31,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__expr_2_3 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr2", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([22,new_token]); break;
			case 28:
				this.stack.push([29,new_token]); break;
			case 30:
				this.stack.push([31,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_neg = function() {
		this.cont = true;
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr3", this.visitor.visit_expr_neg(arg1[1][1]), arg0[1][2], arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([21,new_token]); break;
			case 23:
				this.stack.push([24,new_token]); break;
			case 25:
				this.stack.push([26,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__expr_3_4 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr3", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([21,new_token]); break;
			case 23:
				this.stack.push([24,new_token]); break;
			case 25:
				this.stack.push([26,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_var = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_var(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_string = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_string(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_int = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_int(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_float = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_float(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__expr_par = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr4", arg1[1][1], arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_list_comp = function() {
		this.cont = true;
		var arg6 = this.stack.pop();
		var arg5 = this.stack.pop();
		var arg4 = this.stack.pop();
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_list_comp(arg1[1][1],arg3[1][1],arg5[1][1]), arg0[1][2], arg6[1][3]+arg5[1][3]+arg4[1][3]+arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_list_comp_if = function() {
		this.cont = true;
		var arg8 = this.stack.pop();
		var arg7 = this.stack.pop();
		var arg6 = this.stack.pop();
		var arg5 = this.stack.pop();
		var arg4 = this.stack.pop();
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_list_comp_if(arg1[1][1],arg3[1][1],arg5[1][1],arg7[1][1]), arg0[1][2], arg8[1][3]+arg7[1][3]+arg6[1][3]+arg5[1][3]+arg4[1][3]+arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_func = function() {
		this.cont = true;
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_func(arg0[1][1],arg2[1][1]), arg0[1][2], arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_if = function() {
		this.cont = true;
		var arg5 = this.stack.pop();
		var arg4 = this.stack.pop();
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr4", this.visitor.visit_expr_if(arg1[1][1],arg3[1][1],arg5[1][1]), arg0[1][2], arg5[1][3]+arg4[1][3]+arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 48:
			case 50:
			case 54:
			case 65:
			case 69:
			case 70:
			case 86:
			case 87:
				this.stack.push([18,new_token]); break;
			case 19:
			case 41:
			case 60:
			case 76:
			case 93:
				this.stack.push([20,new_token]); break;
			case 81:
				this.stack.push([82,new_token]); break;
			case 83:
				this.stack.push([84,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_list = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["expr_list", this.visitor.visit_expr_list(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 6:
				this.stack.push([67,new_token]); break;
			case 65:
				this.stack.push([66,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_expr_list_end = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["expr_list", this.visitor.visit_expr_list_end(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 6:
				this.stack.push([67,new_token]); break;
			case 65:
				this.stack.push([66,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_var_var = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["var", this.visitor.visit_var_var(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 19:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 41:
			case 48:
			case 50:
			case 54:
			case 60:
			case 65:
			case 69:
			case 70:
			case 76:
			case 81:
			case 83:
			case 86:
			case 87:
			case 93:
				this.stack.push([12,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_var_index = function() {
		this.cont = true;
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["var", this.visitor.visit_var_index(arg0[1][1],arg2[1][1]), arg0[1][2], arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 19:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 41:
			case 48:
			case 50:
			case 54:
			case 60:
			case 65:
			case 69:
			case 70:
			case 76:
			case 81:
			case 83:
			case 86:
			case 87:
			case 93:
				this.stack.push([12,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_var_attr = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["var", this.visitor.visit_var_attr(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
			case 4:
			case 6:
			case 7:
			case 8:
			case 19:
			case 23:
			case 25:
			case 28:
			case 30:
			case 32:
			case 41:
			case 48:
			case 50:
			case 54:
			case 60:
			case 65:
			case 69:
			case 70:
			case 76:
			case 81:
			case 83:
			case 86:
			case 87:
			case 93:
				this.stack.push([12,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	// column_list
	this.error_column_list_error = function(stack, tokens) {
		if(typeof this.visitor.error_column_list_error == "undefined"){
			throw "Parse error at: " + this.lexer.tokens_to_string(tokens);
		}
		var value = this.visitor.error_column_list_error(stack, tokens, this.lexer);
		var new_token = ["column_list",value,0,0];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([106,new_token]); break;
		}
	};
	
	// custom_content
	this.error_custom_content_error = function(stack, tokens) {
		if(typeof this.visitor.error_custom_content_error == "undefined"){
			throw "Parse error at: " + this.lexer.tokens_to_string(tokens);
		}
		var value = this.visitor.error_custom_content_error(stack, tokens, this.lexer);
		var new_token = ["custom_content",value,0,0];
		switch( this.stack[this.stack.length-1][0] ) {
			case 3:
				this.stack.push([98,new_token]); break;
		}
	};
	
	this.errorhandler = function() {
		var errorstack = [];
		var errortokens = [];
		var running = true;
		while( running ) {
			switch( this.stack[this.stack.length-1][0] ) {
				case 1:
				case 2:
				case 4:
				case 5:
				case 6:
				case 7:
				case 8:
				case 9:
				case 10:
				case 11:
				case 12:
				case 13:
				case 14:
				case 15:
				case 16:
				case 17:
				case 18:
				case 19:
				case 20:
				case 21:
				case 22:
				case 23:
				case 24:
				case 25:
				case 26:
				case 27:
				case 28:
				case 29:
				case 30:
				case 31:
				case 32:
				case 33:
				case 34:
				case 35:
				case 36:
				case 37:
				case 38:
				case 39:
				case 40:
				case 41:
				case 42:
				case 43:
				case 44:
				case 45:
				case 46:
				case 47:
				case 48:
				case 49:
				case 50:
				case 51:
				case 52:
				case 53:
				case 54:
				case 55:
				case 56:
				case 57:
				case 58:
				case 59:
				case 60:
				case 61:
				case 62:
				case 63:
				case 64:
				case 65:
				case 66:
				case 67:
				case 68:
				case 69:
				case 70:
				case 71:
				case 72:
				case 73:
				case 74:
				case 75:
				case 76:
				case 77:
				case 78:
				case 79:
				case 80:
				case 81:
				case 82:
				case 83:
				case 84:
				case 85:
				case 86:
				case 87:
				case 88:
				case 89:
				case 90:
				case 91:
				case 92:
				case 93:
				case 94:
				case 95:
				case 96:
				case 97:
				case 98:
				case 99:
				case 100:
				case 101:
				case 102:
				case 103:
				case 104:
				case 105:
				case 106:
				case 107:
				case 108:
					var errtok = this.stack.pop();
					errorstack.unshift(errtok);
					break;
				default:
					running = false;
			}
		}
		running = true;
		while( running ) {
			switch(this.stack[this.stack.length-1][0]) {
				case 0:
					switch( this.token[0] ) {
						case "end":
						case "comma":
							// error:column_list_error
							this.error_column_list_error(errorstack, errortokens);
							this.cont = true;
							running = false;
							break;
						default:
							errortokens.push(this.token);
							this.token = this.lexer.fetch_token();
					}
					break;
				case 3:
					switch( this.token[0] ) {
						case "end":
						case "comma":
							// error:custom_content_error
							this.error_custom_content_error(errorstack, errortokens);
							this.cont = true;
							running = false;
							break;
						default:
							errortokens.push(this.token);
							this.token = this.lexer.fetch_token();
					}
					break;
				default:
					throw "Internal parser error...";
					return false;
			}
		}
		return true;
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSColumnsPreprocessor.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSColumnsPreprocessor = function LSColumnsPreprocessor(){
	this.preprocess_string = function(value) {
		return value;
	};
	
	this.preprocess_float = function(value) {
		return value;
	};
	
	this.preprocess_integer = function(value) {
		return value;
	};
	
	this.preprocess_then = function(value) {
		return value;
	};
	
	this.preprocess_else = function(value) {
		return value;
	};
	
	this.preprocess_all = function(value) {
		return value;
	};
	
	this.preprocess_default = function(value) {
		return value;
	};
	
	this.preprocess_for = function(value) {
		return value;
	};
	
	this.preprocess_in = function(value) {
		return value;
	};
	
	this.preprocess_if = function(value) {
		return value;
	};
	
	this.preprocess_name = function(value) {
		return value;
	};
	
	this.preprocess_op_sub = function(value) {
		return value;
	};
	
	this.preprocess_op_add = function(value) {
		return value;
	};
	
	this.preprocess_op_mult = function(value) {
		return value;
	};
	
	this.preprocess_op_div = function(value) {
		return value;
	};
	
	this.preprocess_comma = function(value) {
		return value;
	};
	
	this.preprocess_dot = function(value) {
		return value;
	};
	
	this.preprocess_eq = function(value) {
		return value;
	};
	
	this.preprocess_par_l = function(value) {
		return value;
	};
	
	this.preprocess_par_r = function(value) {
		return value;
	};
	
	this.preprocess_sq_l = function(value) {
		return value;
	};
	
	this.preprocess_sq_r = function(value) {
		return value;
	};
	
	this.preprocess_char = function(value) {
		return value;
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSColumnsVisitor.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSColumnsVisitor = function LSColumnsVisitor(){
	// entry: definition := * column_list end
	this.visit_entry = function(column_list0, end1) {
	};
	
	// column_list_single: column_list := * column
	this.visit_column_list_single = function(column0) {
	};
	
	// column_list_multi: column_list := * column_list comma column
	this.visit_column_list_multi = function(column_list0, column2) {
	};
	
	// column_all: column := * all
	this.visit_column_all = function() {
	};
	
	// column_default: column := * default
	this.visit_column_default = function() {
	};
	
	// column_named: column := * name
	this.visit_column_named = function(name0) {
	};
	
	// column_disable: column := * op_sub name
	this.visit_column_disable = function(name1) {
	};
	
	// column_disable_str: column := * op_sub string
	this.visit_column_disable_str = function(string1) {
	};
	
	// column_custom: column := * custom_name eq custom_content
	this.visit_column_custom = function(custom_name0, custom_content2) {
	};
	
	// expr_eq: expr := * expr1 eq expr1
	this.visit_expr_eq = function(expr0, expr2) {
	};
	
	// expr_add: expr1 := * expr1 op_add expr2
	this.visit_expr_add = function(expr0, expr2) {
	};
	
	// expr_sub: expr1 := * expr1 op_sub expr2
	this.visit_expr_sub = function(expr0, expr2) {
	};
	
	// expr_mult: expr2 := * expr2 op_mult expr3
	this.visit_expr_mult = function(expr0, expr2) {
	};
	
	// expr_div: expr2 := * expr2 op_div expr3
	this.visit_expr_div = function(expr0, expr2) {
	};
	
	// expr_neg: expr3 := * op_sub expr4
	this.visit_expr_neg = function(expr1) {
	};
	
	// expr_var: expr4 := * var
	this.visit_expr_var = function(var0) {
	};
	
	// expr_string: expr4 := * string
	this.visit_expr_string = function(string0) {
	};
	
	// expr_int: expr4 := * integer
	this.visit_expr_int = function(integer0) {
	};
	
	// expr_float: expr4 := * float
	this.visit_expr_float = function(float0) {
	};
	
	// expr_list_comp: expr4 := * sq_l expr for name in expr sq_r
	this.visit_expr_list_comp = function(expr1, name3, expr5) {
	};
	
	// expr_list_comp_if: expr4 := * sq_l expr for name in expr if expr sq_r
	this.visit_expr_list_comp_if = function(expr1, name3, expr5, expr7) {
	};
	
	// expr_func: expr4 := * name par_l expr_list par_r
	this.visit_expr_func = function(name0, expr_list2) {
	};
	
	// expr_if: expr4 := * if expr then expr4 else expr4
	this.visit_expr_if = function(expr1, expr3, expr5) {
	};
	
	// expr_list: expr_list := * expr comma expr_list
	this.visit_expr_list = function(expr0, expr_list2) {
	};
	
	// expr_list_end: expr_list := * expr
	this.visit_expr_list_end = function(expr0) {
	};
	
	// var_var: var := * name
	this.visit_var_var = function(name0) {
	};
	
	// var_index: var := * var sq_l integer sq_r
	this.visit_var_index = function(var0, integer2) {
	};
	
	// var_attr: var := * var dot name
	this.visit_var_attr = function(var0, name2) {
	};
	
	this.accept = function(result) {
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSFilter.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSFilter = function LSFilter(preprocessor, visitor){
	this.preprocessor = preprocessor;
	this.visitor = visitor;
	this.parse = function(string) {
		var lexer = new LSFilterLexer( string, this.preprocessor );
		var parser = new LSFilterParser( this.visitor );
		return parser.parse( lexer );
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSFilterLexer.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSFilterLexer = function LSFilterLexer(buffer, visitor){
	this.buffer = buffer;
	this.visitor = visitor;
	this.expression = buffer;
	this.position = 0;
	this.fetch_token = function() {
		do {
			var length = -1;
			var token = 0;
			var value = 0;
			var token_pos = this.position;
			
			// Match end token
			if( this.buffer.length == 0 ) {
				length = 0;
				token = "end";
			}
			
			// Match token: _whitespace
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\s+)/);
				if( matches != null ) {
					length = matches[1].length;
				}
			}
			
			// Match token: _comment
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\/\*(?:[^\*]|\*[^\/])*\*\/)/);
				if( matches != null ) {
					length = matches[1].length;
				}
			}
			
			// Match token: string
			if( length < 0 ) {
				var matches = this.buffer.match(/^("(?:[^"\\]|\\.)*")/);
				if( matches != null ) {
					length = matches[1].length;
					token = "string";
					value = this.visitor.preprocess_string(matches[1]);
				}
			}
			
			// Match token: all
			if( length < 0 ) {
				var matches = this.buffer.match(/^(all)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "all";
					value = this.visitor.preprocess_all(matches[1]);
				}
			}
			
			// Match token: none
			if( length < 0 ) {
				var matches = this.buffer.match(/^(none)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "none";
					value = this.visitor.preprocess_none(matches[1]);
				}
			}
			
			// Match token: and
			if( length < 0 ) {
				var matches = this.buffer.match(/^(and)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "and";
					value = this.visitor.preprocess_and(matches[1]);
				}
			}
			
			// Match token: not
			if( length < 0 ) {
				var matches = this.buffer.match(/^(not)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "not";
					value = this.visitor.preprocess_not(matches[1]);
				}
			}
			
			// Match token: or
			if( length < 0 ) {
				var matches = this.buffer.match(/^(or)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "or";
					value = this.visitor.preprocess_or(matches[1]);
				}
			}
			
			// Match token: in
			if( length < 0 ) {
				var matches = this.buffer.match(/^(in)(?:[^a-z0-9_][\s\S]*|)$/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "in";
					value = this.visitor.preprocess_in(matches[1]);
				}
			}
			
			// Match token: name
			if( length < 0 ) {
				var matches = this.buffer.match(/^([a-z][a-z0-9_]*)/i);
				if( matches != null ) {
					length = matches[1].length;
					token = "name";
					value = this.visitor.preprocess_name(matches[1]);
				}
			}
			
			// Match token: integer
			if( length < 0 ) {
				var matches = this.buffer.match(/^([0-9]+)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "integer";
					value = this.visitor.preprocess_integer(matches[1]);
				}
			}
			
			// Match token: not_re_ci
			if( length < 0 ) {
				var matches = this.buffer.match(/^(!~~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "not_re_ci";
					value = this.visitor.preprocess_not_re_ci(matches[1]);
				}
			}
			
			// Match token: not_re_cs
			if( length < 0 ) {
				var matches = this.buffer.match(/^(!~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "not_re_cs";
					value = this.visitor.preprocess_not_re_cs(matches[1]);
				}
			}
			
			// Match token: re_ci
			if( length < 0 ) {
				var matches = this.buffer.match(/^(~~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "re_ci";
					value = this.visitor.preprocess_re_ci(matches[1]);
				}
			}
			
			// Match token: re_cs
			if( length < 0 ) {
				var matches = this.buffer.match(/^(~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "re_cs";
					value = this.visitor.preprocess_re_cs(matches[1]);
				}
			}
			
			// Match token: not_eq_ci
			if( length < 0 ) {
				var matches = this.buffer.match(/^(!=~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "not_eq_ci";
					value = this.visitor.preprocess_not_eq_ci(matches[1]);
				}
			}
			
			// Match token: eq_ci
			if( length < 0 ) {
				var matches = this.buffer.match(/^(=~)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "eq_ci";
					value = this.visitor.preprocess_eq_ci(matches[1]);
				}
			}
			
			// Match token: not_eq
			if( length < 0 ) {
				var matches = this.buffer.match(/^(!=)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "not_eq";
					value = this.visitor.preprocess_not_eq(matches[1]);
				}
			}
			
			// Match token: gt_eq
			if( length < 0 ) {
				var matches = this.buffer.match(/^(>=)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "gt_eq";
					value = this.visitor.preprocess_gt_eq(matches[1]);
				}
			}
			
			// Match token: lt_eq
			if( length < 0 ) {
				var matches = this.buffer.match(/^(<=)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "lt_eq";
					value = this.visitor.preprocess_lt_eq(matches[1]);
				}
			}
			
			// Match token: gt
			if( length < 0 ) {
				var matches = this.buffer.match(/^(>)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "gt";
					value = this.visitor.preprocess_gt(matches[1]);
				}
			}
			
			// Match token: lt
			if( length < 0 ) {
				var matches = this.buffer.match(/^(<)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "lt";
					value = this.visitor.preprocess_lt(matches[1]);
				}
			}
			
			// Match token: eq
			if( length < 0 ) {
				var matches = this.buffer.match(/^(=)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "eq";
					value = this.visitor.preprocess_eq(matches[1]);
				}
			}
			
			// Match token: colon
			if( length < 0 ) {
				var matches = this.buffer.match(/^(:)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "colon";
					value = this.visitor.preprocess_colon(matches[1]);
				}
			}
			
			// Match token: comma
			if( length < 0 ) {
				var matches = this.buffer.match(/^(,)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "comma";
					value = this.visitor.preprocess_comma(matches[1]);
				}
			}
			
			// Match token: dot
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\.)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "dot";
					value = this.visitor.preprocess_dot(matches[1]);
				}
			}
			
			// Match token: par_l
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\()/);
				if( matches != null ) {
					length = matches[1].length;
					token = "par_l";
					value = this.visitor.preprocess_par_l(matches[1]);
				}
			}
			
			// Match token: par_r
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\))/);
				if( matches != null ) {
					length = matches[1].length;
					token = "par_r";
					value = this.visitor.preprocess_par_r(matches[1]);
				}
			}
			
			// Match token: brace_l
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\[)/);
				if( matches != null ) {
					length = matches[1].length;
					token = "brace_l";
					value = this.visitor.preprocess_brace_l(matches[1]);
				}
			}
			
			// Match token: brace_r
			if( length < 0 ) {
				var matches = this.buffer.match(/^(\])/);
				if( matches != null ) {
					length = matches[1].length;
					token = "brace_r";
					value = this.visitor.preprocess_brace_r(matches[1]);
				}
			}
			
			// Error if no match
			if( length < 0 ) {
				throw "Unknown token: " + this.buffer.substring(0,30);
			}
			
			// Remove token from buffer, and move length forward
			this.buffer = this.buffer.substr( length );
			this.position += length; 
		} while( token == 0 );
		return [token, value, token_pos, length ];
	};
	
	this.tokens_to_string = function(tokens) {
		var start=false;
		var end=false;
		for( var i=0; i<tokens.length; i++ ) {
			if(start === false || start > tokens[i][2]) {
				start = tokens[i][2];
			}
			var tokend = tokens[i][2] + tokens[i][3];
			if(end === false || end < tokend) {
				end = tokend;
			}
		}
		return this.expression.substring(start,end);
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSFilterParser.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSFilterParser = function LSFilterParser(visitor){
	this.visitor = null;
	this.stack = [];
	this.cont = false;
	this.done = false;
	this.lexer = false;
	this.visitor = visitor;
	this.parse = function(lexer) {
		this.stack = new Array();
		this.stack.push( [0,"start"] );
		this.done = false;
		this.lexer = lexer;
		var result = false;
		do {
			this.token = lexer.fetch_token();
			do {
				this.cont = false;
				var head = this.stack[this.stack.length-1];
				this.tmp = this.states[head[0]];
				result = this.tmp();
			} while( this.cont );
		} while( !this.done );
		return result;
	};
	
	this.states = [
	function() {
		// State: 0
		switch( this.token[0] ) {
			case "brace_l":
				// shift:1
				this.stack.push( [1,this.token] );
				return null;
			case "end":
				// error:default_error_handler
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 1
		switch( this.token[0] ) {
			case "name":
				// shift:2
				this.stack.push( [2,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 2
		switch( this.token[0] ) {
			case "brace_r":
				// reduce:table_def_simple
				this.reduce_table_def_simple();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 3
		switch( this.token[0] ) {
			case "brace_r":
				// shift:4
				this.stack.push( [4,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 4
		switch( this.token[0] ) {
			case "not":
				// shift:57
				this.stack.push( [57,this.token] );
				return null;
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 5
		switch( this.token[0] ) {
			case "in":
			case "not_re_ci":
			case "not_re_cs":
			case "re_ci":
			case "re_cs":
			case "not_eq_ci":
			case "eq_ci":
			case "not_eq":
			case "gt_eq":
			case "lt_eq":
			case "gt":
			case "lt":
			case "eq":
				// reduce:field_name
				this.reduce_field_name();
				return null;
			case "dot":
				// shift:6
				this.stack.push( [6,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 6
		switch( this.token[0] ) {
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 7
		switch( this.token[0] ) {
			case "in":
			case "not_re_ci":
			case "not_re_cs":
			case "re_ci":
			case "re_cs":
			case "not_eq_ci":
			case "eq_ci":
			case "not_eq":
			case "gt_eq":
			case "lt_eq":
			case "gt":
			case "lt":
			case "eq":
				// reduce:field_obj
				this.reduce_field_obj();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 8
		switch( this.token[0] ) {
			case "in":
				// shift:47
				this.stack.push( [47,this.token] );
				return null;
			case "not_re_ci":
				// shift:45
				this.stack.push( [45,this.token] );
				return null;
			case "not_re_cs":
				// shift:43
				this.stack.push( [43,this.token] );
				return null;
			case "re_ci":
				// shift:41
				this.stack.push( [41,this.token] );
				return null;
			case "re_cs":
				// shift:39
				this.stack.push( [39,this.token] );
				return null;
			case "not_eq_ci":
				// shift:37
				this.stack.push( [37,this.token] );
				return null;
			case "eq_ci":
				// shift:35
				this.stack.push( [35,this.token] );
				return null;
			case "not_eq":
				// shift:33
				this.stack.push( [33,this.token] );
				return null;
			case "gt_eq":
				// shift:31
				this.stack.push( [31,this.token] );
				return null;
			case "lt_eq":
				// shift:29
				this.stack.push( [29,this.token] );
				return null;
			case "gt":
				// shift:27
				this.stack.push( [27,this.token] );
				return null;
			case "lt":
				// shift:25
				this.stack.push( [25,this.token] );
				return null;
			case "eq":
				// shift:9
				this.stack.push( [9,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 9
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 10
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_string
				this.reduce__arg_string();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 11
		switch( this.token[0] ) {
			case "par_l":
				// shift:12
				this.stack.push( [12,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 12
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 13
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num
				this.reduce__arg_num();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 14
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num_str_str
				this.reduce__arg_num_str_str();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 15
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num_str_num
				this.reduce__arg_num_str_num();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 16
		switch( this.token[0] ) {
			case "par_r":
				// reduce:arg_list_end
				this.reduce_arg_list_end();
				return null;
			case "comma":
				// shift:17
				this.stack.push( [17,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 17
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 18
		switch( this.token[0] ) {
			case "par_r":
				// reduce:arg_list
				this.reduce_arg_list();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 19
		switch( this.token[0] ) {
			case "par_r":
				// shift:20
				this.stack.push( [20,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 20
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:arg_num_func
				this.reduce_arg_num_func();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 21
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num
				this.reduce__arg_num();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 22
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num_str_str
				this.reduce__arg_num_str_str();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 23
		switch( this.token[0] ) {
			case "comma":
			case "par_r":
			case "and":
			case "or":
			case "end":
				// reduce:_arg_num_str_num
				this.reduce__arg_num_str_num();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 24
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_eq
				this.reduce_match_eq();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 25
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 26
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_lt
				this.reduce_match_lt();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 27
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 28
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_gt
				this.reduce_match_gt();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 29
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 30
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_lt_eq
				this.reduce_match_lt_eq();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 31
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 32
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_gt_eq
				this.reduce_match_gt_eq();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 33
		switch( this.token[0] ) {
			case "integer":
				// shift:13
				this.stack.push( [13,this.token] );
				return null;
			case "name":
				// shift:11
				this.stack.push( [11,this.token] );
				return null;
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 34
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_not_eq
				this.reduce_match_not_eq();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 35
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 36
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_eq_ci
				this.reduce_match_eq_ci();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 37
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 38
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_not_eq_ci
				this.reduce_match_not_eq_ci();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 39
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 40
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_re_cs
				this.reduce_match_re_cs();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 41
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 42
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_re_ci
				this.reduce_match_re_ci();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 43
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 44
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_not_re_cs
				this.reduce_match_not_re_cs();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 45
		switch( this.token[0] ) {
			case "string":
				// shift:10
				this.stack.push( [10,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 46
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_not_re_ci
				this.reduce_match_not_re_ci();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 47
		switch( this.token[0] ) {
			case "string":
				// shift:48
				this.stack.push( [48,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 48
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:set_descr_name
				this.reduce_set_descr_name();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 49
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_field_in
				this.reduce_match_field_in();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 50
		switch( this.token[0] ) {
			case "string":
				// shift:48
				this.stack.push( [48,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 51
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_in
				this.reduce_match_in();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 52
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_none
				this.reduce_match_none();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 53
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:match_all
				this.reduce_match_all();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 54
		switch( this.token[0] ) {
			case "not":
				// shift:57
				this.stack.push( [57,this.token] );
				return null;
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 55
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:filter_ok
				this.reduce_filter_ok();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 56
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_3_4
				this.reduce__filter_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 57
		switch( this.token[0] ) {
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 58
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:filter_not
				this.reduce_filter_not();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 59
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_2_3
				this.reduce__filter_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 60
		switch( this.token[0] ) {
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_1_2
				this.reduce__filter_1_2();
				return null;
			case "and":
				// shift:61
				this.stack.push( [61,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 61
		switch( this.token[0] ) {
			case "not":
				// shift:57
				this.stack.push( [57,this.token] );
				return null;
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 62
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:filter_and
				this.reduce_filter_and();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 63
		switch( this.token[0] ) {
			case "par_r":
				// shift:66
				this.stack.push( [66,this.token] );
				return null;
			case "or":
				// shift:64
				this.stack.push( [64,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 64
		switch( this.token[0] ) {
			case "not":
				// shift:57
				this.stack.push( [57,this.token] );
				return null;
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 65
		switch( this.token[0] ) {
			case "or":
			case "par_r":
			case "end":
				// reduce:filter_or
				this.reduce_filter_or();
				return null;
			case "and":
				// shift:61
				this.stack.push( [61,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 66
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_parentesis
				this.reduce__filter_parentesis();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 67
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:filter_ok
				this.reduce_filter_ok();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 68
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_3_4
				this.reduce__filter_3_4();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 69
		switch( this.token[0] ) {
			case "par_l":
				// shift:54
				this.stack.push( [54,this.token] );
				return null;
			case "all":
				// shift:53
				this.stack.push( [53,this.token] );
				return null;
			case "none":
				// shift:52
				this.stack.push( [52,this.token] );
				return null;
			case "in":
				// shift:50
				this.stack.push( [50,this.token] );
				return null;
			case "name":
				// shift:5
				this.stack.push( [5,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 70
		switch( this.token[0] ) {
			case "and":
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_2_3
				this.reduce__filter_2_3();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 71
		switch( this.token[0] ) {
			case "or":
			case "par_r":
			case "end":
				// reduce:_filter_1_2
				this.reduce__filter_1_2();
				return null;
			case "and":
				// shift:61
				this.stack.push( [61,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 72
		switch( this.token[0] ) {
			case "end":
				// reduce:search_query
				this.reduce_search_query();
				return null;
			case "or":
				// shift:64
				this.stack.push( [64,this.token] );
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 73
		switch( this.token[0] ) {
			case "end":
				// reduce:query
				this.reduce_query();
				return null;
		}
		this.errorhandler();
		return null;
	},
	function() {
		// State: 74
		switch( this.token[0] ) {
			case "end":
				// accept:
				var program = this.stack.pop();
				this.done = true;
				return this.visitor.accept(program[1][1]);
		}
		this.errorhandler();
		return null;
	},
	];
	this.reduce_query = function() {
		this.cont = true;
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["query", this.visitor.visit_query(arg1[1][1],arg3[1][1]), arg0[1][2], arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([74,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_table_def_simple = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["table_def", this.visitor.visit_table_def_simple(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 1:
				this.stack.push([3,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_search_query = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["search_query", this.visitor.visit_search_query(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
				this.stack.push([73,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_filter_or = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["filter", this.visitor.visit_filter_or(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
				this.stack.push([72,new_token]); break;
			case 54:
				this.stack.push([63,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__filter_1_2 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["filter", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
				this.stack.push([72,new_token]); break;
			case 54:
				this.stack.push([63,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_filter_and = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["filter2", this.visitor.visit_filter_and(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
				this.stack.push([60,new_token]); break;
			case 64:
				this.stack.push([65,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__filter_2_3 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["filter2", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
				this.stack.push([60,new_token]); break;
			case 64:
				this.stack.push([65,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_filter_not = function() {
		this.cont = true;
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["filter3", this.visitor.visit_filter_not(arg1[1][1]), arg0[1][2], arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 64:
				this.stack.push([59,new_token]); break;
			case 61:
				this.stack.push([62,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__filter_3_4 = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["filter3", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 64:
				this.stack.push([59,new_token]); break;
			case 61:
				this.stack.push([62,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_filter_ok = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["filter4", this.visitor.visit_filter_ok(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 61:
			case 64:
				this.stack.push([56,new_token]); break;
			case 57:
			case 69:
				this.stack.push([58,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__filter_parentesis = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["filter4", arg1[1][1], arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 61:
			case 64:
				this.stack.push([56,new_token]); break;
			case 57:
			case 69:
				this.stack.push([58,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_all = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_all(), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_none = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_none(), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_in = function() {
		this.cont = true;
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_in(arg1[1][1]), arg0[1][2], arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_field_in = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_field_in(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_not_re_ci = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_not_re_ci(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_not_re_cs = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_not_re_cs(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_re_ci = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_re_ci(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_re_cs = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_re_cs(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_not_eq_ci = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_not_eq_ci(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_eq_ci = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_eq_ci(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_not_eq = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_not_eq(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_gt_eq = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_gt_eq(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_lt_eq = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_lt_eq(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_gt = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_gt(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_lt = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_lt(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_match_eq = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["match", this.visitor.visit_match_eq(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([55,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_set_descr_name = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["set_descr", this.visitor.visit_set_descr_name(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 47:
				this.stack.push([49,new_token]); break;
			case 50:
				this.stack.push([51,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_field_name = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["field", this.visitor.visit_field_name(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([8,new_token]); break;
			case 6:
				this.stack.push([7,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_field_obj = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["field", this.visitor.visit_field_obj(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 4:
			case 54:
			case 57:
			case 61:
			case 64:
			case 69:
				this.stack.push([8,new_token]); break;
			case 6:
				this.stack.push([7,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__arg_num_str_num = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["arg_num_string", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 9:
				this.stack.push([24,new_token]); break;
			case 12:
			case 17:
				this.stack.push([16,new_token]); break;
			case 31:
				this.stack.push([32,new_token]); break;
			case 33:
				this.stack.push([34,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__arg_num_str_str = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["arg_num_string", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 9:
				this.stack.push([24,new_token]); break;
			case 12:
			case 17:
				this.stack.push([16,new_token]); break;
			case 31:
				this.stack.push([32,new_token]); break;
			case 33:
				this.stack.push([34,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__arg_num = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["arg_num", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 9:
			case 12:
			case 17:
			case 31:
			case 33:
				this.stack.push([15,new_token]); break;
			case 25:
				this.stack.push([26,new_token]); break;
			case 27:
				this.stack.push([28,new_token]); break;
			case 29:
				this.stack.push([30,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce__arg_string = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["arg_string", arg0[1][1], arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 9:
			case 12:
			case 17:
			case 31:
			case 33:
				this.stack.push([14,new_token]); break;
			case 35:
				this.stack.push([36,new_token]); break;
			case 37:
				this.stack.push([38,new_token]); break;
			case 39:
				this.stack.push([40,new_token]); break;
			case 41:
				this.stack.push([42,new_token]); break;
			case 43:
				this.stack.push([44,new_token]); break;
			case 45:
				this.stack.push([46,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_arg_num_func = function() {
		this.cont = true;
		var arg3 = this.stack.pop();
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["arg_num", this.visitor.visit_arg_num_func(arg0[1][1],arg2[1][1]), arg0[1][2], arg3[1][3]+arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 9:
			case 12:
			case 17:
			case 31:
			case 33:
				this.stack.push([15,new_token]); break;
			case 25:
				this.stack.push([26,new_token]); break;
			case 27:
				this.stack.push([28,new_token]); break;
			case 29:
				this.stack.push([30,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_arg_list = function() {
		this.cont = true;
		var arg2 = this.stack.pop();
		var arg1 = this.stack.pop();
		var arg0 = this.stack.pop();
		var new_token = ["arg_list", this.visitor.visit_arg_list(arg0[1][1],arg2[1][1]), arg0[1][2], arg2[1][3]+arg1[1][3]+arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 12:
				this.stack.push([19,new_token]); break;
			case 17:
				this.stack.push([18,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	this.reduce_arg_list_end = function() {
		this.cont = true;
		var arg0 = this.stack.pop();
		var new_token = ["arg_list", this.visitor.visit_arg_list_end(arg0[1][1]), arg0[1][2], arg0[1][3]];
		switch( this.stack[this.stack.length-1][0] ) {
			case 12:
				this.stack.push([19,new_token]); break;
			case 17:
				this.stack.push([18,new_token]); break;
		}
		// error handler...
		return null;
	};
	
	// query
	this.error_default_error_handler = function(stack, tokens) {
		if(typeof this.visitor.error_default_error_handler == "undefined"){
			throw "Parse error at: " + this.lexer.tokens_to_string(tokens);
		}
		var value = this.visitor.error_default_error_handler(stack, tokens, this.lexer);
		var new_token = ["query",value,0,0];
		switch( this.stack[this.stack.length-1][0] ) {
			case 0:
				this.stack.push([74,new_token]); break;
		}
	};
	
	this.errorhandler = function() {
		var errorstack = [];
		var errortokens = [];
		var running = true;
		while( running ) {
			switch( this.stack[this.stack.length-1][0] ) {
				case 1:
				case 2:
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
				case 8:
				case 9:
				case 10:
				case 11:
				case 12:
				case 13:
				case 14:
				case 15:
				case 16:
				case 17:
				case 18:
				case 19:
				case 20:
				case 21:
				case 22:
				case 23:
				case 24:
				case 25:
				case 26:
				case 27:
				case 28:
				case 29:
				case 30:
				case 31:
				case 32:
				case 33:
				case 34:
				case 35:
				case 36:
				case 37:
				case 38:
				case 39:
				case 40:
				case 41:
				case 42:
				case 43:
				case 44:
				case 45:
				case 46:
				case 47:
				case 48:
				case 49:
				case 50:
				case 51:
				case 52:
				case 53:
				case 54:
				case 55:
				case 56:
				case 57:
				case 58:
				case 59:
				case 60:
				case 61:
				case 62:
				case 63:
				case 64:
				case 65:
				case 66:
				case 67:
				case 68:
				case 69:
				case 70:
				case 71:
				case 72:
				case 73:
				case 74:
					var errtok = this.stack.pop();
					errorstack.unshift(errtok);
					break;
				default:
					running = false;
			}
		}
		running = true;
		while( running ) {
			switch(this.stack[this.stack.length-1][0]) {
				case 0:
					switch( this.token[0] ) {
						case "end":
							// error:default_error_handler
							this.error_default_error_handler(errorstack, errortokens);
							this.cont = true;
							running = false;
							break;
						default:
							errortokens.push(this.token);
							this.token = this.lexer.fetch_token();
					}
					break;
				default:
					throw "Internal parser error...";
					return false;
			}
		}
		return true;
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSFilterPreprocessor.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSFilterPreprocessor = function LSFilterPreprocessor(){
	this.preprocess_string = function(value) {
		return value;
	};
	
	this.preprocess_all = function(value) {
		return value;
	};
	
	this.preprocess_none = function(value) {
		return value;
	};
	
	this.preprocess_and = function(value) {
		return value;
	};
	
	this.preprocess_not = function(value) {
		return value;
	};
	
	this.preprocess_or = function(value) {
		return value;
	};
	
	this.preprocess_in = function(value) {
		return value;
	};
	
	this.preprocess_name = function(value) {
		return value;
	};
	
	this.preprocess_integer = function(value) {
		return value;
	};
	
	this.preprocess_not_re_ci = function(value) {
		return value;
	};
	
	this.preprocess_not_re_cs = function(value) {
		return value;
	};
	
	this.preprocess_re_ci = function(value) {
		return value;
	};
	
	this.preprocess_re_cs = function(value) {
		return value;
	};
	
	this.preprocess_not_eq_ci = function(value) {
		return value;
	};
	
	this.preprocess_eq_ci = function(value) {
		return value;
	};
	
	this.preprocess_not_eq = function(value) {
		return value;
	};
	
	this.preprocess_gt_eq = function(value) {
		return value;
	};
	
	this.preprocess_lt_eq = function(value) {
		return value;
	};
	
	this.preprocess_gt = function(value) {
		return value;
	};
	
	this.preprocess_lt = function(value) {
		return value;
	};
	
	this.preprocess_eq = function(value) {
		return value;
	};
	
	this.preprocess_colon = function(value) {
		return value;
	};
	
	this.preprocess_comma = function(value) {
		return value;
	};
	
	this.preprocess_dot = function(value) {
		return value;
	};
	
	this.preprocess_par_l = function(value) {
		return value;
	};
	
	this.preprocess_par_r = function(value) {
		return value;
	};
	
	this.preprocess_brace_l = function(value) {
		return value;
	};
	
	this.preprocess_brace_r = function(value) {
		return value;
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/LSFilterVisitor.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 

var LSFilterVisitor = function LSFilterVisitor(){
	// entry: program := * query end
	this.visit_entry = function(query0) {
	};
	
	// query: query := * brace_l table_def brace_r search_query
	this.visit_query = function(table_def1, search_query3) {
	};
	
	// table_def_simple: table_def := * name
	this.visit_table_def_simple = function(name0) {
	};
	
	// search_query: search_query := * filter
	this.visit_search_query = function(filter0) {
	};
	
	// filter_or: filter := * filter or filter2
	this.visit_filter_or = function(filter0, filter2) {
	};
	
	// filter_and: filter2 := * filter2 and filter3
	this.visit_filter_and = function(filter0, filter2) {
	};
	
	// filter_not: filter3 := * not filter4
	this.visit_filter_not = function(filter1) {
	};
	
	// filter_ok: filter4 := * match
	this.visit_filter_ok = function(match0) {
	};
	
	// match_all: match := * all
	this.visit_match_all = function() {
	};
	
	// match_none: match := * none
	this.visit_match_none = function() {
	};
	
	// match_in: match := * in set_descr
	this.visit_match_in = function(set_descr1) {
	};
	
	// match_field_in: match := * field in set_descr
	this.visit_match_field_in = function(field0, set_descr2) {
	};
	
	// match_not_re_ci: match := * field not_re_ci arg_string
	this.visit_match_not_re_ci = function(field0, arg_string2) {
	};
	
	// match_not_re_cs: match := * field not_re_cs arg_string
	this.visit_match_not_re_cs = function(field0, arg_string2) {
	};
	
	// match_re_ci: match := * field re_ci arg_string
	this.visit_match_re_ci = function(field0, arg_string2) {
	};
	
	// match_re_cs: match := * field re_cs arg_string
	this.visit_match_re_cs = function(field0, arg_string2) {
	};
	
	// match_not_eq_ci: match := * field not_eq_ci arg_string
	this.visit_match_not_eq_ci = function(field0, arg_string2) {
	};
	
	// match_eq_ci: match := * field eq_ci arg_string
	this.visit_match_eq_ci = function(field0, arg_string2) {
	};
	
	// match_not_eq: match := * field not_eq arg_num_string
	this.visit_match_not_eq = function(field0, arg_num_string2) {
	};
	
	// match_gt_eq: match := * field gt_eq arg_num_string
	this.visit_match_gt_eq = function(field0, arg_num_string2) {
	};
	
	// match_lt_eq: match := * field lt_eq arg_num
	this.visit_match_lt_eq = function(field0, arg_num2) {
	};
	
	// match_gt: match := * field gt arg_num
	this.visit_match_gt = function(field0, arg_num2) {
	};
	
	// match_lt: match := * field lt arg_num
	this.visit_match_lt = function(field0, arg_num2) {
	};
	
	// match_eq: match := * field eq arg_num_string
	this.visit_match_eq = function(field0, arg_num_string2) {
	};
	
	// set_descr_name: set_descr := * string
	this.visit_set_descr_name = function(string0) {
	};
	
	// field_name: field := * name
	this.visit_field_name = function(name0) {
	};
	
	// field_obj: field := * name dot field
	this.visit_field_obj = function(name0, field2) {
	};
	
	// arg_num_func: arg_num := * name par_l arg_list par_r
	this.visit_arg_num_func = function(name0, arg_list2) {
	};
	
	// arg_list: arg_list := * arg_num_string comma arg_list
	this.visit_arg_list = function(arg_num_string0, arg_list2) {
	};
	
	// arg_list_end: arg_list := * arg_num_string
	this.visit_arg_list_end = function(arg_num_string0) {
	};
	
	this.accept = function(result) {
	};
	
};



/*******************
 * From src: modules/lsfilter/src/js/generated/ninja_manifest.js
 *******************/
// 
// NOTE!
// 
// This is an auto generated file. Changes to this file will be overwritten!
// 
if(typeof(ninja_manifest) === "undefined") {
ninja_manifest = {}
}
$.extend(true, ninja_manifest, {"autocomplete":{"saved_filters":[{"display_column":"filter_name","query":"[saved_filters] filter_name ~~ \"%s\""}]},"module_permissions":{"monitor":{"monitoring":{"hosts":{":update":{"commands":{"acknowledge_problem":[],"add_comment":[],"disable_check":[],"disable_service_checks":[],"disable_service_notifications":[],"enable_check":[],"enable_service_checks":[],"enable_service_notifications":[],"process_check_result":[],"remove_acknowledgement":[],"schedule_check":[],"schedule_downtime":[],"schedule_service_checks":[],"send_custom_notification":[]}}},"services":{":update":{"commands":{"add_comment":[],"disable_check":[],"process_check_result":[],"schedule_check":[],"schedule_downtime":[],"send_custom_notification":[]}}},"hostgroup":{":update":{"commands":{"disable_service_checks":[],"disable_service_notifications":[],"enable_service_checks":[],"enable_service_notifications":[],"schedule_host_downtime":[],"schedule_service_downtime":[]}}},"servicegroup":{":update":{"commands":{"disable_service_checks":[],"disable_service_notifications":[],"enable_service_checks":[],"enable_service_notifications":[],"schedule_host_downtime":[],"schedule_service_downtime":[]}}}}}},"orm_doctags":{"savedfilter_model":{"get_scope":{"depend":["username"]},"get_deletable":{"depend":["username"]}}},"orm_structure":{"object":[],"saved_filters":{"id":["int"],"username":["string"],"filter_name":["string"],"filter_table":["string"],"filter":["string"],"filter_description":["string"]},"settings":{"id":["int"],"username":["string"],"type":["string"],"page":["string"],"setting":["string"],"widget_id":["int"]}},"orm_table_classes":{"object":{"object":"Object_Model","set":"ObjectSet_Model","pool":"ObjectPool_Model"},"saved_filters":{"object":"SavedFilter_Model","set":"SavedFilterSet_Model","pool":"SavedFilterPool_Model"},"settings":{"object":"Setting_Model","set":"SettingSet_Model","pool":"SettingPool_Model"}}});



/*******************
 * From src: modules/lsfilter/src/js/lib.js
 *******************/
function helpicon(controller, key)
{
	var link = $('<a class="helptext_target" style="border: 0" href="#"/>');
	link.append(icon12('help'));
	link.attr('data-popover', 'help:' + controller + '.' + key);
	return link;
}
function span_icon(className, title)
{
	var icon = $('<span />');
	icon.addClass((!className) ? "icon-help" : className);
	if (title) icon.attr('title', title);
	return icon;
}
function icon12(name, title, link)
{
	var icon = span_icon('icon-12 x12-' + name, title)
	if (link) {
		icon = link.clone().append(icon);
		icon.css('border', '0');
	}
	return icon;
}
function icon16(name, title, link, base)
{
	if( !base ) base = 'x16';
	var icon = span_icon('icon-16 ' + base + '-' + name, title)
	if (link) {
		icon = link.append(icon);
		icon.css('border', '0');
	}
	return icon;
}
function icon(url, link)
{
	var img = $('<img />');
	img.attr('src', _logo_path + url); // FIXME
	img.css('height', '16px');
	img.css('width', '16px');
	if (link) {
		img = link.append(img);
		img.css('border', '0');
	}
	return img;
}
function link(rel_url, args)
{
	var get_data = "";
	var delim = "?";
	for ( var key in args) {
		get_data += delim + key + "=" + encodeURIComponent(args[key]);
		delim = "&";
	}

	var el = $('<a />');
	el.attr('href', _site_domain + _index_page + "/" + rel_url + get_data);
	return el;
}
function link_fnc(fnc)
{
	return $('<a />').click(fnc);
}
function link_query(query)
{
	var link = $('<a />');
	link.attr('href', _site_domain + _index_page + '/listview?q='
			+ encodeURIComponent(query));
	link.attr('data-query', query);
	link.addClass('query_link');
	return link;
}
function extinfo_link(args)
{
	return link('extinfo/details', args);
}
function format_timestamp(timestamp)
{
	// remember: server's offset is local - UTC, client's offset is UTC - local,
	// so the sign should be the same.
	var timestamp_int = parseInt(timestamp);

	var dateobj = new Date((timestamp_int + _server_utc_offset) * 1000);
	dateobj = new Date(dateobj.getTime() + (dateobj.getTimezoneOffset() * 60000));
	var ret = dateobj.format(_date_format);
	dateobj = null;
	return ret;
}
function format_interval(interval)
{
	if (interval < 0) return _('N/A');
	var str = "";
	if (interval % 60 !== 0) str = (interval % 60) + "s " + str;
	interval = Math.floor(interval / 60);
	if (interval % 60 !== 0) str = (interval % 60) + "m " + str;
	interval = Math.floor(interval / 60);
	if (interval % 24 !== 0) str = (interval % 24) + "h " + str;
	interval = Math.floor(interval / 24);
	if (interval !== 0) str = (interval) + "d " + str;
	return str;
}

function comment_icon( host, service ) {
	var obj_name = service ? host+';'+service : host;
	var query = '[comments] host.name="' + host + '"' + (service?' and service.description="'+service+'"':'');
	var link_data = {};
	link_data.host = host;
	if(service)
		link_data.service = service;

	var ext_link = extinfo_link(link_data)
		.append(icon16('add-comment', _('Comments')))
		.css('border', '0px');

	ext_link.attr({
		'data-popover': 'get:' + _site_domain + _index_page + "/listview_tools/fetch_comments?host=" + obj_name
	});
	return ext_link;

}

jQuery.fn.update_text = function(text) {
	if( _escape_html_tags ) {
		return this.text(text);
	} else {
		return this.html(text);
	}
};

jQuery.fn.link_email = function(email) {
	var link = $('<a />');
	link.attr({
		'href': 'mailto:' + encodeURIComponent(email),
		'target': '_blank'
	});
	link.text(email);
	return this.append(link);
}

jQuery.fn.querylink = function(text) {
	if( _escape_html_tags ) {
		return this.text(text);
	} else {
		return this.html(text);
	}
};

jQuery.fn.replaceContent = function(new_data) {
	this.empty();
	this.append(new_data);
	return this;
};






function render_summary_state(ul, state, stats, substates)
{
	if (stats.stats[state] === 0) return;

	var li = $('<li />').append(
			link_query(stats.queries[state]).append(icon16('shield-' + state))
					.append(
							$('<span />')
									.text(stats.stats[state] + " " + state)));

	var delim = ' ( ';
	var suffix = '';

	for ( var tag in substates) {
		var key = state + tag;
		var type = substates[tag];

		if (stats.stats[key]) {
			li.append(delim);
			li.append(link_query(stats.queries[key]).text(
					stats.stats[key] + ' ' + type));
			delim = ', ';
			suffix = ' ) ';
		}
	}

	li.append(suffix);

	ul.append(li);
}

function render_service_status_summary(stats)
{
	var ul = $('<ul class="listview-summary" />');

	render_summary_state(ul, 'ok', stats, {});
	render_summary_state(ul, 'warning', stats, {
		'_and_ack': _('acknowledged'),
		'_and_disabled_active': _('disabled active'),
		'_and_scheduled': _('scheduled'),
		'_and_unhandled': _('unhandled'),
		'_on_down_host': _('on down host')
	});
	render_summary_state(ul, 'critical', stats, {
		'_and_ack': _('acknowledged'),
		'_and_disabled_active': _('disabled active'),
		'_and_scheduled': _('scheduled'),
		'_and_unhandled': _('unhandled'),
		'_on_down_host': _('on down host')
	});
	render_summary_state(ul, 'unknown', stats, {
		'_and_ack': _('acknowledged'),
		'_and_disabled_active': _('disabled active'),
		'_and_scheduled': _('scheduled'),
		'_and_unhandled': _('unhandled'),
		'_on_down_host': _('on down host')
	});
	render_summary_state(ul, 'pending', stats, {});

	return ul;
}

function render_host_status_summary(stats)
{
	var ul = $('<ul class="listview-summary" />');

	render_summary_state(ul, 'up', stats, {});
	render_summary_state(ul, 'down', stats, {});
	render_summary_state(ul, 'unreachable', stats, {});
	render_summary_state(ul, 'pending', stats, {});

	return ul;
}

$(window).on('load', function () {

	var start;

	$(document).on('click', '.listview_multiselect_checkbox', function (e) {

		if (start && e.shiftKey) {

			var boxes = $(this).parents('form')
				.find('.listview_multiselect_checkbox');

			var checked = start.is(':checked');

			var s_i = boxes.index(start);
			var e_i = boxes.index($(this));

			var high = Math.max(s_i, e_i);
			var low = Math.min(s_i, e_i);

			for (var i = low; i <= high; i++) {
				boxes.eq(i)
					.attr('checked', checked)
					.trigger('change');
			}
		}

		start = $(this);

	});

});



/*******************
 * From src: modules/lsfilter/src/js/lsfilter_renderer_buttons.js
 *******************/
listview_renderer_buttons.all = [
	$('<a href="#" data-drop-down="filter-query-multi-action"/>').attr('title', _('Send multi action')).append($('<span class="icon-16 x16-check-boxes"/>')),
	$('<a href="#" data-drop-down="filter-query-builder"/>').attr('title', _('Show/Edit Text Filter')).append($('<span class="icon-16 x16-filter"/>'))
];


/*******************
 * From src: modules/lsfilter/src/js/lsfilter_renderer_extra_objects.js
 *******************/
var listview_renderer_extra_objects_all = [
	function() {
		var menu;
		if (!lsfilter_multiselect.elem_objtype) {
			menu = $('<div id="filter-query-multi-action"/>');
			menu.text(_('Table doesn\'t support multi-select'));
			lsfilter_multiselect.init(menu);
			lsfilter_multiselect.update({
				'source': false,
				'metadata': {'table': lsfilter_storage.list.request_metadata.table}
			});
		} else {
			menu = $('#filter-query-multi-action')
		}
		return menu;
	}
];



/*******************
 * From src: modules/lsfilter/src/js/lsfilter_renderer_table.js
 *******************/
/*******************************************************************************
 * Table renderer
 ******************************************************************************/

var listview_renderer_table_all = {
	"select" : {
		"order" : 35,
		"header" : $('<input type="checkbox" id="select_all" class="listview_multiselect_checkbox_all" />'),
		"depends" : [ 'key' ],
		"sort" : false,
		"available" : function(args) {
			if(_controller_name != 'listview')
				return false;
			if(!listview_commands[args.table])
				return false;

			var cmd_count = 0;
			for ( var cmdname in listview_commands[args.table]) {
				// Redirect commands can't be applied in multi aciton
				if(!listview_commands[args.table][cmdname]['redirect']) {
					cmd_count++;
				}
			}
			if(cmd_count == 0)
				return false;
			return true;
		},
		"cell" : function(args)
		{
			var checkbox = $(
					'<input type="checkbox" name="object[]" class="listview_multiselect_checkbox" />')
					.attr('value', args.obj.key);
			if ( lsfilter_multiselect.box_selected(args.obj.key) ) {
				checkbox.prop('checked', true);
				if (args.row.hasClass('odd'))
					args.row.addClass('selected_odd');
				else
					args.row.addClass('selected_even');
			}
			return $('<td style="width: 1em; padding: 0 3px" />').append(checkbox);
		}
	},
}

listview_renderer_table.saved_filters = {
	"icon" : {
		"header" : '',
		"depends" : [ 'filter_table' ],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td class="icon" />');

			var icon = false;
			var base = 'menu';
			switch (args.obj.filter_table) {
			case 'hosts':
				icon = 'host';
				break;
			case 'services':
				icon = 'service';
				break;
			case 'hostgroups':
				icon = 'hostgroup';
				break;
			case 'servicegroups':
				icon = 'servicegroup';
				break;
			case 'comments':
				icon = 'comments';
				break;
			case 'notifications':
				icon = 'notifications';
				break;
			default:
				icon = 'eventlog';
			}
			if (icon) {
				cell.append(icon16(icon, false, false, base));
			}
			return cell;

		}
	},
	"scope" : {
		"header" : _('Scope'),
		"depends" : [ 'scope' ],
		"sort" : false,
		"cell" : function(args) {
			return $('<td />').text(args.obj.scope);
		}
	},
	"name" : {
		"header" : _('Name'),
		"depends" : [ 'filter_name', 'filter' ],
		"sort" : [ 'filter_name' ],
		"cell" : function(args) {
			return $('<td />').append(
					link_query(args.obj.filter).text(args.obj.filter_name));

		}
	},
	"filter" : {
		"header" : _('filter string'),
		"depends" : [ 'filter' ],
		"sort" : [ 'filter' ],
		"cell" : function(args) {
			return $('<td />').append(
					link_query(args.obj.filter).text(args.obj.filter));
		}
	},
	"owner" : {
		"header" : _('Owner'),
		"depends" : [ 'username' ],
		"sort" : [ 'username' ],
		"cell" : function(args) {
			var cell = $('<td />');
			if (args.obj.username) {
				cell.text(args.obj.username);
			}
			return cell;

		}
	},
	"actions" : {
		"header" : _('Actions'),
		"depends" : [ 'id', 'deletable' ],
		"sort" : false,
		"cell" : function(args) {
			var cell = $('<td />');

			if (args.obj.deletable) {
				// Delete
				var del_icon = icon16('delete',
						_("Delete/cancel this saved filter"));
				var del_link = _site_domain + _index_page
						+ '/listview/delete_saved_filter?id=' + args.obj.id;

				var del_elem = $('<a />')
						.attr('href', del_link)
						.addClass('link_ajax_refresh')
						.click(
								function(ev) {
									if (!confirm(_("Are you sure you want to delete the filter '")
											+ args.obj.filter_name + "' ?")) {
										return false;
									}
								}).append(del_icon);

				cell.append(del_elem);
			}
			return cell;
		}
	}
};

